# -*-makefile-*-
#
# Copyright (C) 2010 by Juergen Beisert <jbe@pengutronix.de>
#
# See CREDITS for details about who has contributed to this project.
#
# For further information about the PTXdist project and license conditions
# see the README file.
#

#
# We provide this package
#
PACKAGES-$(PTXCONF_BOOTLETS) += bootlets

#
# Paths and names
#
BOOTLETS_VERSION	:= 10.12.01
BOOTLETS		:= imx-bootlets-src-$(BOOTLETS_VERSION)
BOOTLETS_SUFFIX		:= tar.gz
BOOTLETS_URL		:= http://www.pengutronix.de/software/ptxdist/temporary-src/$(BOOTLETS).$(BOOTLETS_SUFFIX)
BOOTLETS_MD5		:= cf0ab3822dca694b930a051501c1d0e4
BOOTLETS_SOURCE		:= $(SRCDIR)/$(BOOTLETS).$(BOOTLETS_SUFFIX)
BOOTLETS_DIR		:= $(BUILDDIR)/$(BOOTLETS)
BOOTLETS_LICENSE	:= GPLv2

# ----------------------------------------------------------------------------
# Get
# ----------------------------------------------------------------------------

$(BOOTLETS_SOURCE):
	@$(call targetinfo)
	@$(call get, BOOTLETS)

# ----------------------------------------------------------------------------
# Prepare (nothing)
# ----------------------------------------------------------------------------

BOOTLETS_COMPILE_DEPS :=

ifdef PTXCONF_BOOTLETS_KERNEL
BOOTLETS_COMPILE_DEPS += $(STATEDIR)/kernel.targetinstall.post
endif

ifdef PTXCONF_BOOTLETS_BOOTLOADER
BOOTLETS_COMPILE_DEPS += $(STATEDIR)/barebox.targetinstall
endif

$(STATEDIR)/bootlets.prepare: $(BOOTLETS_COMPILE_DEPS)
	@$(call targetinfo)
	@$(call touch)

# ----------------------------------------------------------------------------
# Compile
# ----------------------------------------------------------------------------

# do not use ':=' here!
BOOTLETS_MAKE_OPT = \
	BOARD=iMX28_EVK \
	CROSS_COMPILE=$(PTXCONF_GNU_TARGET)- \
	ARCH=mx28 \
#	ELFTOSB="$(PTXDIST_SYSROOT_HOST)/bin/elftosb"

ifdef PTXCONF_BOOTLETS_DEBUG
BOOTLETS_MAKE_OPT += DEBUG=1
endif

ifdef PTXCONF_BOOTLETS_KERNEL
#BOOTLETS_MAKE_OPT += linux_stream DFT_IMAGE=$(IMAGEDIR)/linuximage
BOOTLETS_MAKE_OPT += DFT_IMAGE=$(IMAGEDIR)/linuximage_dtb

BOOTLETS_CMDLINE1 = "console=ttyAMA0,115200 mtdparts=gpmi-nand:20M(nand0.bcb),80M(nand0.root0),80M(nand0.root1),-(nand0.data) root=ubi0:rootfs0 rw ubi.mtd=1 ubi.mtd=2 ubi.mtd=3 rootfstype=ubifs rootflags=sync selinux=1 security=selinux"
BOOTLETS_CMDLINE2 = $(BOOTLETS_CMDLINE1)

BOOTLETS_MAKE_OPT += CMDLINE1=$(BOOTLETS_CMDLINE1)
BOOTLETS_MAKE_OPT += CMDLINE2=$(BOOTLETS_CMDLINE2)
endif

ifdef PTXCONF_BOOTLETS_BOOTLOADER
BOOTLETS_MAKE_OPT += bootloader_stream DFT_UBOOT=$(BAREBOX_DIR)/barebox
endif

ifdef PTXCONF_BOOTLETS_MOBILESDRAM
BOOTLETS_MAKE_OPT += MEM_TYPE=MEM_MDDR
else
BOOTLETS_MAKE_OPT += MEM_TYPE=MEM_DDR1
endif

ifdef PTXCONF_BOOTLETS_ENCRYPT

ifdef PTXCONF_BOOTLETS_ZERO_ENCRYPT
BOOTLETS_MAKE_OPT += ELFTOSB_KEY_PARAM="-z"
endif

ifdef PTXCONF_BOOTLETS_KEY_ENCRYPT
BOOTLETS_MAKE_OPT += ELFTOSB_KEY_PARAM="-k $(PTXCONF_BOOTLETS_KEYFILE_ENCRYPT)"
endif

else
BOOTLETS_MAKE_OPT += ELFTOSB_KEY_PARAM=
endif

# Package's build system is broken
BOOTLETS_MAKE_PAR := NO

# ----------------------------------------------------------------------------
# Install
# ----------------------------------------------------------------------------

$(STATEDIR)/bootlets.install:
	@$(call targetinfo)
ifdef PTXCONF_BOOTLETS_KERNEL
ifdef PTXCONF_BOOTLETS_CPU_T1
ifdef PTXCONF_BOOTLETS_ENCRYPT
ifdef PTXCONF_BOOTLETS_KEY_ENCRYPT
	@cp $(BOOTLETS_DIR)/imx28_aes_linux.sb $(IMAGEDIR)/imx28_aes_linux.sb
else
	@cp $(BOOTLETS_DIR)/imx28_ivt_linux.sb $(IMAGEDIR)/imx28_ivt_linux.sb
endif
else
	@cp $(BOOTLETS_DIR)/imx28_nokey_linux.sb $(IMAGEDIR)/imx28_nokey_linux.sb
endif
else
	@cp $(BOOTLETS_DIR)/linux_stream $(IMAGEDIR)/sd_card_linux_bootstream
endif
endif
ifdef PTXCONF_BOOTLETS_BOOTLOADER
ifdef PTXCONF_BOOTLETS_CPU_T1
	@cp $(BOOTLETS_DIR)/bootloader_stream_ivt $(IMAGEDIR)/sd_card_bootloader_bootstream
else
	@cp $(BOOTLETS_DIR)/bootloader_stream $(IMAGEDIR)/sd_card_bootloader_bootstream
endif
endif

ifdef PTXCONF_BOOTLETS_KEY_ENCRYPT
	@cp $(PTXCONF_BOOTLETS_KEYFILE_ENCRYPT) $(PTXCONF_BOOTLETS_HAB4_CST_SOURCE)/customkey.txt
endif

ifdef PTXCONF_BOOTLETS_HAB4_CST
	@sh $(BOOTLETS_DIR)/hab4-cst.sh $(PTXCONF_BOOTLETS_HAB4_CST_SOURCE) "$(BOOTLETS_DIR)" $(IMAGEDIR)
endif

ifdef PTXCONF_BOOTLETS_KEY_ENCRYPT
	@ rm -rf $(PTXCONF_BOOTLETS_HAB4_CST_SOURCE)/customkey.txt
endif

ifdef PTXCONF_BOOTLETS_KERNEL_CREATE_IPK
	@sh $(PTXDIST_PLATFORMCONFIGDIR)/local_src/script/bootstream2ipk.sh $(IMAGEDIR)
endif

	@$(call touch)

# ----------------------------------------------------------------------------
# Target-Install (nothing)
# ----------------------------------------------------------------------------

$(STATEDIR)/bootlets.targetinstall:
	@$(call targetinfo)
	@$(call touch)

# ----------------------------------------------------------------------------
# Clean
# ----------------------------------------------------------------------------

#$(STATEDIR)/bootlets.clean:
#	@$(call targetinfo)
#	@$(call clean_pkg, BOOTLETS)

# vim: syntax=make
