#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <map>

#include "hal.h"

#ifdef WIN32
#define UNICODE 1
#define _UNICODE 1

#include <windows.h>
#include <strsafe.h>
#endif

using namespace hal;

void outputHelpscreen()
{
	printf("Usage:\n");
	printf("halcl led<0|1|2|3|4|5|s> <on|off>\n");	
	printf("halcl blinkled<0|1|2|3|4|5|s>\n");
	printf("halcl delayled<0|1|2|3|4|5|s> <Offtime in ms> <Ontime in ms>\n");
	printf("halcl wdt\n");
	printf("halcl ipaddr <eth-device> <IPv4-Adresse>\n");
	printf("halcl hwaddr <eth-device> <MAC-Adresse>\n");
	printf("halcl netmask <eth-device> <Netzmaske>\n");
	printf("halcl broadcast <eth-device> <Broadcast-Adresse>\n");
	printf("halcl gw <Default-Gateway-Adresse>\n");
	printf("halcl rmgw <Remove-Default-Gateway-Adresse>\n");
	printf("halcl ip <eth-device> <up|down>\n");
	printf("halcl link <eth-device>\n");
	printf("halcl ipstatus <eth-device>\n");
	printf("halcl resetsw|resetgsm|resetwmbus\n");
	printf("halcl configsw\n");
	printf("halcl powgsm <on|off>\n");
	printf("halcl powwmbus <on|off>\n");
	printf("halcl pow12v <on|off>\n");
	printf("halcl powport<1|2> <on|off>\n");
	printf("halcl iptblockin <eth-device>\n");
	printf("halcl iptopenport <any|eth-device> <Port-Nr> <tcp|udp>\n");
	printf("halcl iptcloseport <any|eth-device> <Port-Nr> <tcp|udp>\n");
	printf("halcl iptflush\n");
	printf("halcl eeprom <clean|read|write> [-ver <ver> |& -hwid <hwid> |& -swid <swid> |& -hwver <hwver>]\n");
	printf("                                [-osc <osc> |& -bc <bc> |& -mac0 <mac0> |& -mac1 <mac1> |& -mac2 <mac2>]\n");
	printf("                                [-sn <sn> |& -devid <devid> |& -pdate <pdate> |& -key <key> |& -oem <oem>]\n");
	printf("                                [-wmbus <wmbus> |& -mnterrinit <err_cnt> |& -mnterrrun <err_cnt> |& -romsig <signature>]\n");
	printf("halcl eeprom <bootread|bootwrite> [<byte_value(dec)>]\n");
	printf("halcl ver [-os |& -kernel |& -fw |& -patch |& -<paket> |& -all\n");	
	printf("halcl diskfree <pathfilename>\n");	
	printf("halcl disk <pathfilename>\n");	
	printf("halcl diskuse <pathfilename>\n");
	printf("halcl tamper\n");
	printf("halcl tamperreboot\n");
	printf("halcl setdtr\n");
	printf("halcl pppdpid\n");
	printf("halcl getdsr\n");
	printf("halcl getbatt\n");
	printf("halcl getcurr12v\n");
	printf("halcl ntpsync [<NTP_port>] [<Offset_Limit>]\n");
	printf("halcl bootconf\n");
	printf("halcl bootbit <set|clr> <bit_no>\n");
	printf("halcl setcpuboot <byte_value(dec)>\n");
	printf("halcl tz <get|set> [timezoneinfo]\n");	
}

int LEDSwitch(char *ledarg[])
{	
	int ret = 0;
	bool onoff;

	// berprfung Vorhandensein des On-Off Kommandos
	if (ledarg[2]==NULL)
	{
		printf("fehlendes Steuerkommando LED\n");
		return 1;
	}

	// berprfung des On-Off Kommandos
	if (strcmp(ledarg[2],"on")==0)
	{
		onoff=ON;
	} 
	else if (strcmp(ledarg[2],"off")==0)
	{
		onoff=OFF;
	}
	else
	{
		printf("falsches Steuerkommando LED\n");
		return 1;
	}

	// Festestellen, welche LED angesteuert werden soll und Ausfhrung
	if (strcmp(ledarg[1],"led0")==0)
	{
		hal::SwitchLed(0,onoff);
	}
	else if (strcmp(ledarg[1],"led1")==0)
	{
		hal::SwitchLed(1, onoff);
	} 
	else if (strcmp(ledarg[1],"led2")==0)
	{
		hal::SwitchLed(2,onoff);
	} 
	else if (strcmp(ledarg[1],"led3")==0)
	{
		hal::SwitchLed(3,onoff);
	} 
	else if (strcmp(ledarg[1],"led4")==0)
	{
		hal::SwitchLed(4,onoff);
	} 
	else if (strcmp(ledarg[1],"led5")==0)
	{
		hal::SwitchLed(5,onoff);
	} 
	else
	{
		hal::SwitchLed(0,onoff);
		hal::SwitchLed(1,onoff);
		hal::SwitchLed(2,onoff);
		hal::SwitchLed(3,onoff);
		hal::SwitchLed(4,onoff);
		hal::SwitchLed(5,onoff);
	}

	return ret;
}

int LEDBlinking(char *ledarg[])
{
	int ret = 0;
	bool rc = true;

	// Feststellen, welche LED zum Blinken getriggert werden soll
	if (strcmp(ledarg[1],"blinkled0")==0)
	{
		rc = hal::BlinkLed(0);
	}
	else if (strcmp(ledarg[1],"blinkled1")==0)
	{
		rc = hal::BlinkLed(1);
	} 
	else if (strcmp(ledarg[1],"blinkled2")==0)
	{
		rc = hal::BlinkLed(2);
	} 
	else if (strcmp(ledarg[1],"blinkled3")==0)
	{
		rc = hal::BlinkLed(3);
	} 
	else if (strcmp(ledarg[1],"blinkled4")==0)
	{
		rc = hal::BlinkLed(4);
	} 
	else if (strcmp(ledarg[1],"blinkled5")==0)
	{
		rc = hal::BlinkLed(5);
	} 
	else
	{
		rc =	hal::BlinkLed(0) && 
				hal::BlinkLed(1) &&
				hal::BlinkLed(2) &&
				hal::BlinkLed(3) &&
				hal::BlinkLed(4) &&
				hal::BlinkLed(5);
	}

	if (rc == false) ret = -1;
	return ret;
}

int setLEDBlinkingdelay(char *ledarg[])
{
	int ret = 0;
	bool rc = true;

	// Festestellen, welche LED mit dem Blinkverhalten modifiziert werden soll
	if (strcmp(ledarg[1],"delayled0")==0)
	{
		rc = hal::setLedDelay(0,ledarg[2],ledarg[3]);
	}
	else if (strcmp(ledarg[1],"delayled1")==0)
	{
		rc = hal::setLedDelay(1,ledarg[2],ledarg[3]);
	} 
	else if (strcmp(ledarg[1],"delayled2")==0)
	{
		rc = hal::setLedDelay(2,ledarg[2],ledarg[3]);
	} 
	else if (strcmp(ledarg[1],"delayled3")==0)
	{
		rc = hal::setLedDelay(3,ledarg[2],ledarg[3]);
	} 
	else if (strcmp(ledarg[1],"delayled4")==0)
	{
		rc = hal::setLedDelay(4,ledarg[2],ledarg[3]);
	} 
	else if (strcmp(ledarg[1],"delayled5")==0)
	{
		rc = hal::setLedDelay(5,ledarg[2],ledarg[3]);
	} 
	else
	{
		rc =	hal::setLedDelay(0,ledarg[2],ledarg[3]) && 
				hal::setLedDelay(1,ledarg[2],ledarg[3]) &&
				hal::setLedDelay(2,ledarg[2],ledarg[3]) &&
				hal::setLedDelay(3,ledarg[2],ledarg[3]) &&
				hal::setLedDelay(4,ledarg[2],ledarg[3]) &&
				hal::setLedDelay(5,ledarg[2],ledarg[3]);
	}

	if (rc == false) ret = -1;
	return ret;
}

int WatchdogTrigger()
{
	int ret = 0;
	ret = hal::TriggerSysWD();
	return ret;
}

int setIPAddr(char* iparg[])
{
	int ret = 0;
	ret = hal::ConfigureIpAddr(iparg[2], iparg[3]);
	return ret;
}

int setHwAddr(char* iparg[])
{
	int ret = 0;
	ret = hal::ConfigureIpHw(iparg[2], iparg[3]);
	return ret;
}

int setNetmask(char* iparg[])
{
	int ret = 0;
	ret = hal::ConfigureIpNetmask(iparg[2], iparg[3]);
	return ret;
}

int setBroadcast(char* iparg[])
{
	int ret = 0;
	ret = hal::ConfigureIpBroadcast(iparg[2], iparg[3]);
	return ret;
}

int32_t setGateway(char* gwarg[])
{
	int32_t ret = 0;
	std::string errortext;

	if (!hal::ConfigureIpDefaultGateway(gwarg[2], errortext))
	{
		printf("ERROR: %s\n", errortext.c_str());
		ret = -1;
	}
	return ret;
}

int32_t remove_gw(char* gwarg[])
{
	int32_t ret = 0;
	std::string errortext;

	if (!hal::RemoveIPDefaultGateway(gwarg[2],  errortext))
	{
		printf("ERROR: %s\n", errortext.c_str());
		ret = -1;
	}
	return ret;
}

int getLinkState(char* iparg[])
{
	int ret = 0;
	bool state;

	state = hal::GetLinkState(iparg[2]);

	if (state==UP)
	{
		printf("Link for device %s is UP !\n",iparg[2]);
	} 
	else
	{
		printf("Link for device %s is DOWN !\n",iparg[2]);
	}

	return ret;
}

int getIPStatus(char* iparg[])
{
	int ret = 0;
	std::string ipv4addr="";
	std::string netmask="";
	std::string broadcast="";

	bool isUpDown = hal::GetIPDeviceState(iparg[2]);
	
	if (isUpDown==UP)
	{
		printf("Device %s is UP !\n",iparg[2]);
	} 
	else
	{
		printf("Device %s is DOWN !\n",iparg[2]);
	}

	ret = hal::GetIpv4Addr(iparg[2], ipv4addr);
	printf("IP-Addr for device %s:		%s		with rc: %d\n", iparg[2], ipv4addr.c_str(),ret);

	ret = hal::GetNetmask(iparg[2], netmask);
	printf("Netmask for device %s:		%s		with rc: %d\n", iparg[2], netmask.c_str(),ret);

	ret = hal::GetBroadcast(iparg[2], broadcast);
	printf("Broadcast for device %s:		%s		with rc: %d\n", iparg[2], broadcast.c_str(),ret);

	return ret;
}


int setIpUpDown(char* iparg[])
{
	int ret = 0;
	if (strcmp(iparg[3],"up")==0)
	{
		hal::ConfigureIpUpDown(iparg[2],UP);
	} 
	else
	{
		hal::ConfigureIpUpDown(iparg[2],DOWN);
	}
	return ret;
}

int powergsm(char* powerarg[])
{
	bool ret = true;
	int retint = 0;

	if (strcmp(powerarg[2],"on")==0)
	{
		ret = hal::PowerGSM(ON);
	} 
	else
	{
		ret = hal::PowerGSM(OFF);
	}

	if (!ret) retint = -1;
	return retint;
}

int powerwmbus(char* powerarg[])
{
	bool ret = true;
	int retint = 0;

	if (strcmp(powerarg[2],"on")==0)
	{
		ret = hal::PowerWMBUS(ON);
	} 
	else 
	{
		ret = hal::PowerWMBUS(OFF);
	}
	
	if (!ret) retint = -1;
	return retint;
}

int power12v(char* powerarg[])
{
	bool ret = true;
	int retint = 0;

	if (strcmp(powerarg[2],"on")==0)
	{
		ret = hal::PowerMBUS12V(ON);
	} 
	else 
	{
		ret = hal::PowerMBUS12V(OFF);
	}
	if (!ret) retint = -1;
	return retint;	
}

int ipt_blockinputdevice(char* iptarg[])
{
	int ret = 0;
	
	ret = hal::IPTablesBlockInputDevice(iptarg[2]);

	return ret;
}

int ipt_openport(char* iptarg[])
{
	int32_t ret = 0;
	std::string errortext = "";
		
	//if (!hal::IPTablesOpenPort(iptarg[2], std::strtol(iptarg[3], NULL, 10), iptarg[4], errortext)) ret = -1;
		
	if (!hal::IPTablesOpenPort(iptarg[2], atoi(iptarg[3]), iptarg[4], errortext)) ret = -1;
	if (errortext != "") printf("%s\n", errortext.c_str());
	//hal::IPTablesOpenPort("eth1.1", "25", "udp");
	//hal::IPTablesOpenPort("eth1.2", "26", "tcp");
	//hal::IPTablesOpenPort("eth0", "27", "udp");
	//return 0;
	
	return ret;
}

int ipt_closeport(char* iptarg[])
{
	int32_t ret = 0;
	std::string errortext = "";
	
	if (!hal::IPTablesClosePort(iptarg[2], atoi(iptarg[3]), iptarg[4], errortext)) ret = -1;
	if (errortext != "") printf("%s\n", errortext.c_str());
	
	return ret;
}

int ipt_flush()
{	
	return hal::IPTablesFlush();
}

int eeprom_boot(int argcnt, char* romarg[])
{
	uint8_t boot_byte;

	if (strcmp(romarg[2],"bootread") == 0)
	{
		if (hal::EepromReadBootConf(boot_byte))
		{
			printf("Bootbyte: 0x%02X\n", boot_byte);
		}
		else 
		{
			return -1;
		}
	}
	else if ((strcmp(romarg[2],"bootwrite") == 0) && (argcnt >= 4))
	{
		boot_byte = (uint8_t)atoi(romarg[3]);
		if (hal::EepromWriteBootConf(boot_byte))
		{
			printf("written Bootbyte: 0x%02X\n", boot_byte);
		}
		else 
		{
			return -1;
		}
	}
	return (int)boot_byte;
}

int eeprom(int argcnt, char* romarg[])
{
	int i;
	bool ret = false;

	std::map<std::string, std::string> data;

	if (strcmp(romarg[2],"clean") == 0)
	{
		ret = hal::EepromErase();
	}
	else if (strcmp(romarg[2],"read") == 0)
	{		
		if ( argcnt==3 )				// read all values
		{
			data["ver"]="";	data["hwid"]=""; data["swid"]=""; data["hwver"]=""; data["osc"]="";
			data["bc"]=""; data["mac0"]=""; data["mac1"]=""; data["mac2"]=""; data["sn"]="";
			data["devid"]=""; data["pdate"]=""; data["key"]=""; data["oem"]=""; data["wmbus"]="";
			data["mnterrinit"]=""; data["mnterrrun"]=""; data["romsig"];			
		}
		else if ( argcnt > 3 )			// selektives Lesen einer oder mehrerer values
		{
			for (i=3; i<argcnt; i++)										
			{
				if (strcmp(romarg[i],"-ver")==0)				data["ver"]="";
				else if (strcmp(romarg[i],"-hwid")==0)			data["hwid"]="";
				else if (strcmp(romarg[i],"-swid")==0)			data["swid"]="";				
				else if (strcmp(romarg[i],"-hwver")==0)			data["hwver"]="";
				else if (strcmp(romarg[i],"-osc")==0)			data["osc"]="";
				else if (strcmp(romarg[i],"-bc")==0)			data["bc"]="";
				else if (strcmp(romarg[i],"-mac0")==0)			data["mac0"]="";
				else if (strcmp(romarg[i],"-mac1")==0)			data["mac1"]="";
				else if (strcmp(romarg[i],"-mac2")==0)			data["mac2"]="";
				else if (strcmp(romarg[i],"-sn")==0)			data["sn"]="";
				else if (strcmp(romarg[i],"-devid")==0)			data["devid"]="";
				else if (strcmp(romarg[i],"-pdate")==0)			data["pdate"]="";
				else if (strcmp(romarg[i],"-key")==0)			data["key"]="";
				else if (strcmp(romarg[i],"-oem")==0)			data["oem"]="";
				else if (strcmp(romarg[i],"-wmbus")==0)			data["wmbus"]="";
				else if (strcmp(romarg[i],"-mnterrinit")==0)	data["mnterrinit"]="";
				else if (strcmp(romarg[i],"-mnterrrun")==0)		data["mnterrrun"]="";
				else if (strcmp(romarg[i],"-romsig")==0)		data["romsig"]="";
			}			
		}
		ret = hal::EepromRead(data);

		for(std::map<std::string, std::string>::iterator it=data.begin();it!=data.end();it++)
		{			
			printf("%s\t%s\n",it->first.c_str(),it->second.c_str());
		}
	}
	else if (strcmp(romarg[2],"write") == 0)
	{
		for (i=3; i<argcnt-1; i++)
		{
			if (strcmp(romarg[i],"-ver")==0)
			{
				data["ver"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-hwid")==0)
			{
				data["hwid"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-swid")==0)
			{
				data["swid"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-hwver")==0)
			{
				data["hwver"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-osc")==0)
			{
				data["osc"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-bc")==0)
			{
				data["bc"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-mac0")==0)
			{
				data["mac0"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-mac1")==0)
			{
				data["mac1"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-mac2")==0)
			{
				data["mac2"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-sn")==0)
			{
				data["sn"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-devid")==0)
			{
				data["devid"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-pdate")==0)
			{
				data["pdate"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-key")==0)
			{
				data["key"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-oem")==0)
			{
				data["oem"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-wmbus")==0)
			{
				data["wmbus"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-mnterrinit")==0)
			{
				data["mnterrinit"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-mnterrrun")==0)
			{
				data["mnterrrun"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-romsig")==0)
			{
				data["romsig"] = romarg[i+1];
			}
		}

		ret = hal::EepromWrite(data);
	}

	return ((ret) ? 0 : -1);
}

int VersionRequest(int argcnt, char* verarg[])
{
	int i;
	int ret = 0;
	bool retbool = true;

	std::string requestmode = "single";
	std::map<std::string, std::string> data;

	for (i=2; i<argcnt; i++)
	{
		if (strcmp(verarg[i],"-os")==0)
		{
			data["os"]="";			
		}
		else if (strcmp(verarg[i],"-kernel")==0)
		{
			data["kernel"]="";			
		}
		else if (strcmp(verarg[i],"-fw")==0)
		{
			data["Application"]="";			
		}
		else if (strcmp(verarg[i],"-all")==0)
		{
			// Abbruch: mode wechseln, map clearen, Paketliste soll geliefert werden
			requestmode = "allpackages";
			data.clear();
			std::string error;
			hal::QuerySystemVersion(requestmode, data, error);
			break;
		}
		else if (strcmp(verarg[i],"-patch")==0)
		{
			// Abbruch: mode wechseln, map clearen, Liste der patche soll geliefert werden
			requestmode = "patches";
			data.clear();
			std::string error;
			hal::QuerySystemVersion(requestmode, data, error);
			break;
		}
		else
		{
			// irgendein Paket ? Fhrendes '-' vorausgesetzt
			if ( verarg[i][0] == '-' ) data[verarg[i]+1]="";			
		}		
	}

	if (requestmode == "single")
	{		
		std::string error;
		retbool = hal::QuerySystemVersion(requestmode, data, error);
	}

	std::map<std::string, std::string>::const_iterator j;

	for (j = data.begin(); j!=data.end(); j++)
	{
		printf("%s\t%s\n",j->first.c_str(),j->second.c_str());
	}

	if (!retbool)
	{
		ret = -1;
	}

	return ret;
}

int diskfree(int argcnt, char* verarg[])
{
	int ret = 0;
	long long int value;


	if (argcnt<3)
	{
		value = hal::GetFreeDiskSpace("/");		// root
		printf("Free disk space for path: root (/) is [bytes]: %lli\n", value);
	}
	else
	{
		value = hal::GetFreeDiskSpace(verarg[2]);
		printf("Free disk space for path: %s is [bytes]: %lli\n", verarg[2], value);
	}

	return ret;
}

int disk(int argcnt, char* verarg[])
{
	int ret = 0;
	long long int value;

	if (argcnt<3)
	{
		value = hal::GetDiskSize("/");		// root
		printf("Disk size for path: root (/) is [bytes]: %lli\n", value);
	}
	else
	{
		value = hal::GetDiskSize(verarg[2]);
		printf("Disk size for path: %s is [bytes]: %lli\n", verarg[2], value);
	}
	return ret;
}

int tamper()
{
	int ret = 0;

	ret = hal::ReadTamperStatus();

	switch (ret)
	{
	case 1:	printf("Tamper OK\n");
		break;
	case 2:	printf("Tamper released\n");
		break;
	case 3:	printf("Tamper OK, but released in the past\n");
		break;
	default: printf("Error getting tamper status\n");
		break;
	}

	return ret;
}

int tamperreboot()
{
	int ret = 0;

	ret = hal::TamperReboot();

	if (ret>=0)
	{
		printf("Tamper Firmware successfully rebooted...\n");
	}
	else
	{
		printf("Tamper Firmware reboot failed...\n");
	}

	return ret;
}


int usedDisk(int argcnt, char* verarg[])
{
	int ret = 0;
	long long int value;

	if (argcnt<3)
	{
		value = hal::GetUsedDiskSpace("/");		// root
		printf("Used disk size for path: root (/) is [bytes]: %lli\n", value);
	}
	else
	{
		value = hal::GetUsedDiskSpace(verarg[2]);
		printf("Used disk size for path: %s is [bytes]: %lli\n", verarg[2], value);
	}
	return ret;
}

int setdtr()
{
	int ret = 0;
	hal::SetDTR();

	return ret;
}

int get_pppd_pid()
{
	int ret = 0;

	ret = hal::getPppdPid();

	if (ret == 0)
	{
		ret = -1;
		printf("Error reading file /var/run/ppp0.pid\n");
	}
	else
	{
		printf("PID for pppd is %d\n", ret);
	}

	return ret;
}

int power_port(int argcnt, char* pwarg[])
{
	int ret = 0;
	unsigned int portnr;
	bool onoff;

	if (strcmp(pwarg[1],"powport1")==0)
	{
		portnr = 1;
	}
	else
	{
		portnr = 2;
	}

	if (strcmp(pwarg[2],"on")==0)
	{
		onoff = ON;
	}
	else
	{
		onoff = OFF;
	}

	if (!hal::PowerSwitchPort(portnr, onoff))
	{
		ret = -1;
	}

	return ret;
}

int resetwmbu()
{
	int ret = -1;
	if (hal::ResetWMBUS())
	{
		ret = 0;
	}
	return ret;
}

int resetgsm()
{
	int ret = -1;
	if (hal::ResetGSM())
	{
		ret = 0;
	}
	return ret;
}

int get_DSR()
{
	int ret = -1;

	if (hal::GetDSRPegel())
	{
		ret = 1;
		printf("Gsm module is in Command Mode, DSR Level is HIGH\n");
	}
	else
	{
		ret = 0;
		printf("Gsm module is in Data Mode, DSR Level is LOW\n");
	}
	return ret;
}

int get_BattVoltage()
{
	int ret = 0;
	unsigned int battVolt_mv = hal::GetBattVoltage_mv();
	unsigned int battVolt_percent = hal::GetBattVoltage_percent();
	
	printf("Battery voltage: %d mV (%d %%)\n", battVolt_mv, battVolt_percent);

	return ret;
}

int get_ntpstat(int argcnt, char* ntparg[])
{		
	uint16_t port = 123;
	uint32_t offset = 2000;

	if (argcnt == 4)
	{
		offset = atol(ntparg[3]);
	}

	if (argcnt > 2)
	{
		port = atoi(ntparg[2]);
	}	

	int ret = hal::IsNTPsynchronized(port, offset) ? 0 : -1;

	return ret;
}

int get_bootconf()
{
	//  Legende der persistenten Bits (Bootkonfiguration im EEPROM, BIT0 = LSB)
	//		Bit0 0 = rootfs0, 1 = rootfs1
	//		Bit1 Updated rootfs
	//		Bit2 Updated BootStream
	//		Bit3 Update Failed
	//		Bit4 previous rootsfs updated
	//		Bit5 previous BootStream updated
	//		Bit6 BootSelector Soll
	
	uint8_t boot_byte_eeprom, boot_byte_imx28;
	bool match = true;

	if (!hal::GetBootConf(boot_byte_eeprom, boot_byte_imx28)) return -1;
	
	if (boot_byte_eeprom == boot_byte_imx28)
	{
		printf("Bootconf match:\t\tEeprom-Bootconf=0x%02X\t\tiMX28-Bootconf=0x%02X\n", boot_byte_eeprom, boot_byte_imx28);
	}
	else
	{
		printf("Bootconf dismatch:\t\tEeprom-Bootconf=0x%02X\t\tiMX28-Bootconf=0x%02X\n", boot_byte_eeprom, boot_byte_imx28);
		match = false;
	}

	printf("iMx28 Register settings:\n");
	printf(((boot_byte_imx28 & (0x01)) == 0x01 ? "Active rootfs: rootfs1\n" : "Active rootfs: rootfs0\n"));
	printf(((boot_byte_imx28 & (0x02)) == 0x02 ? "rootfs updated\n" : "rootfs not updated\n"));
	printf(((boot_byte_imx28 & (0x04)) == 0x04 ? "bootstream updated\n" : "bootstream not updated\n"));
	printf(((boot_byte_imx28 & (0x08)) == 0x08 ? "Update failed\n" : "Update didnt failed\n"));
	printf(((boot_byte_imx28 & (0x10)) == 0x10 ? "previous rootsfs updated\n" : "previous rootsfs not updated\n"));
	printf(((boot_byte_imx28 & (0x20)) == 0x20 ? "previous BootStream updated\n" : "previous BootStream not updated\n"));
	printf(((boot_byte_imx28 & (0x40)) == 0x40 ? "Should be bootstream: 1\n" : "Should be bootstream: 0\n"));
	printf(((boot_byte_imx28 & (0x80)) == 0x80 ? "Active bootstream: 1\n" : "Active bootstream: 0\n"));

	if (!match)
	{
		printf("\nEEPROM Register settings:\n");
		printf(((boot_byte_eeprom & (0x01)) == 0x01 ? "Active rootfs: rootfs1\n" : "Active rootfs: rootfs0\n"));
		printf(((boot_byte_eeprom & (0x02)) == 0x02 ? "rootfs updated\n" : "rootfs not updated\n"));
		printf(((boot_byte_eeprom & (0x04)) == 0x04 ? "bootstream updated\n" : "bootstream not updated\n"));
		printf(((boot_byte_eeprom & (0x08)) == 0x08 ? "Update failed\n" : "Update didnt failed\n"));
		printf(((boot_byte_eeprom & (0x10)) == 0x10 ? "previous rootsfs updated\n" : "previous rootsfs not updated\n"));
		printf(((boot_byte_eeprom & (0x20)) == 0x20 ? "previous BootStream updated\n" : "previous BootStream not updated\n"));
		printf(((boot_byte_imx28 & (0x40)) == 0x40 ? "Should be bootstream: 1\n" : "Should be bootstream: 0\n"));
		printf(((boot_byte_eeprom & (0x80)) == 0x80 ? "Active bootstream: 1\n" : "Active bootstream: 0\n"));
	}

	return 0;
}

int BootBitManipulate(char* bootarg[])
{	
	bool ret;
	
	uint8_t bitnr = atoi(bootarg[3]);
	
	if (strcmp(bootarg[2],"set")==0)
	{
		ret = hal::EepromBootConfSetBit(bitnr);
	}
	else
	{
		ret = hal::EepromBootConfClrBit(bitnr);
	}
	return (ret ? 0 : -1);
}

int setcpuboot(char* bootarg[])
{
	uint8_t boot_byte_imx28 = atoi(bootarg[2]);

	if (!hal::WriteBootConfToCPU(boot_byte_imx28))
	{
		return -1;
	}
	
	return 0;
}

int timezoneinfo(int argcnt, char* tzarg[])
{
	std::string tzinfo = "";
	std::string errortext;
	
	if (strcmp(tzarg[2],"get")==0)
	{
		if (hal::GetTimeZone(tzinfo, errortext))
		{
			printf("current local timezone: %s\n", tzinfo.c_str());			
		}
		else
		{
			printf("Error determining current local timezone info, Error: %s !\n", errortext.c_str());
			return -1;
		}
	}
	else if ((strcmp(tzarg[2],"set")==0) && (argcnt>=4))
	{
		tzinfo = tzarg[3];
		if (hal::SetTimeZone(tzinfo, errortext))
		{
			printf("Set timezone to %s successfully !\n", tzinfo.c_str());
		}
		else
		{
			printf("Error setting timezone: %s, Error: %s !\n", tzinfo.c_str(), errortext.c_str());
			return -1;
		}
	}
	else
	{
		printf("Error: unknown command !\n");
		return -1;
	}
	return 0;
}

int get_current_12v()
{
	printf("Meter bus current (U=12V): %d\n", hal::GetCurrent12v_mA());
	return 0;
}

int main (int argc, char *argv[])
{
	int ret = -1;

#ifdef WIN32
	// Enable the required privilege
	HANDLE hToken;
	TOKEN_PRIVILEGES tkp;

	OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES|TOKEN_QUERY, &hToken);
	LookupPrivilegeValue(NULL, SE_TIME_ZONE_NAME, &tkp.Privileges[0].Luid);
	tkp.PrivilegeCount = 1;
	tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
	AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, (PTOKEN_PRIVILEGES)NULL, 0);
#endif
	
	// Wenn kein Kommando oder -? bergeben wurde, Anzeige der Hilfeseite
	if ((argc<=1) || (strcmp(argv[1],"-?")==0))
	{
		outputHelpscreen();
		return 0;
	}

	// Initialisierung der libhal
	halInit();

	if ((strcmp(argv[1],"led0")==0) || (strcmp(argv[1],"led1")==0) || 
		(strcmp(argv[1],"led2")==0) || (strcmp(argv[1],"led3")==0) || 
		(strcmp(argv[1],"led4")==0) || (strcmp(argv[1],"led5")==0) || 		
		(strcmp(argv[1],"leds")==0))
	{
		ret=LEDSwitch(argv);
	} 
	else if ((strcmp(argv[1],"blinkled0")==0) || (strcmp(argv[1],"blinkled1")==0) || 
		(strcmp(argv[1],"blinkled2")==0) || (strcmp(argv[1],"blinkled3")==0) || 
		(strcmp(argv[1],"blinkled4")==0) || (strcmp(argv[1],"blinkled5")==0) || 		
		(strcmp(argv[1],"blinkleds")==0))
	{
		ret=LEDBlinking(argv);
	}
	else if ((strcmp(argv[1],"delayled0")==0) || (strcmp(argv[1],"delayled1")==0) || 
		(strcmp(argv[1],"delayled2")==0) || (strcmp(argv[1],"delayled3")==0) || 
		(strcmp(argv[1],"delayled4")==0) || (strcmp(argv[1],"delayled5")==0) || 		
		(strcmp(argv[1],"delayleds")==0))
	{
		if (argc<4)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=setLEDBlinkingdelay(argv);
		}
	}
	else if ((strcmp(argv[1],"wdt")==0))
	{
		ret = WatchdogTrigger();
	}
	else if ((strcmp(argv[1],"ipaddr")==0))
	{
		if (argc<4)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=setIPAddr(argv);
		}		
	}
	else if ((strcmp(argv[1],"hwaddr")==0))
	{
		if (argc<4)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=setHwAddr(argv);
		}		
	}
	else if ((strcmp(argv[1],"netmask")==0))
	{
		if (argc<4)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=setNetmask(argv);
		}		
	}
	else if ((strcmp(argv[1],"broadcast")==0))
	{
		if (argc<4)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=setBroadcast(argv);
		}		
	}
	else if ((strcmp(argv[1],"gw")==0))
	{
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=setGateway(argv);
		}		
	}
	else if ((strcmp(argv[1],"ip")==0))
	{
		if (argc<4)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=setIpUpDown(argv);
		}
	}
	else if ((strcmp(argv[1],"link")==0))
	{
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=getLinkState(argv);
		}		
	}
	else if ((strcmp(argv[1],"ipstatus")==0))
	{
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=getIPStatus(argv);
		}		
	}
	else if ((strcmp(argv[1],"resetsw")==0))
	{
		ret = hal::ResetSwitch();
	}
	else if ((strcmp(argv[1],"configsw")==0))
	{
		ret = hal::ConfigureSwitch();
	}
	else if ((strcmp(argv[1],"powgsm")==0))
	{
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret = powergsm(argv);
		}		
	}
	else if ((strcmp(argv[1],"powwmbus")==0))
	{
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret = powerwmbus(argv);
		}		
	}
	else if ((strcmp(argv[1],"pow12v")==0))
	{
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret = power12v(argv);
		}
	} 
	else if ((strcmp(argv[1],"iptblockin")==0))
	{
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret = ipt_blockinputdevice(argv);
		}
	}
	else if ((strcmp(argv[1],"iptopenport")==0))
	{
		if (argc<5)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret = ipt_openport(argv);
		}
	}
	else if ((strcmp(argv[1],"iptcloseport")==0))
	{
		if (argc<5)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret = ipt_closeport(argv);
		}
	}
	else if ((strcmp(argv[1],"iptflush")==0))
	{
		ret = ipt_flush();		
	}
	else if ((strcmp(argv[1],"eeprom")==0))
	{		 
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		}
		else
		{			
			if ((strcmp(argv[2],"bootread")==0) || (strcmp(argv[2],"bootwrite")==0))
			{
				ret = eeprom_boot(argc, argv);		// EEPROM Bootbyte Handling, RC ist im Gutfall der Wert des Bytes (dez)
			}
			else
			{
				ret = eeprom(argc, argv);			// allg. Lesen/Schreiben der Gatewaywerte
			}								
		}		
		// Muster zum Schreiben:
		// ./halcl eeprom write -ver 1 -hwid 00-000 -swid 00-000 -hwver 12345 -osc 0 -bc 0 -mac0 02:08:02:C0:FF:EE -mac1 00:08:02:C0:FF:ED -sn 0123456789ABCDEF -pdate 20131014 -key 00000000 -oem "Dr. Neuhaus" -wmbus 0123456789ABCDEF
	}
	else if ((strcmp(argv[1],"ver")==0))
	{
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		}
		else
		{			
			ret = VersionRequest(argc, argv);
		}		
		// Muster zum Lesen:
		// ./halcl eeprom write -ver 1 -hwid 00-000 -swid 00-000 -hwver 12345 -osc 0 -bc 0 -mac0 02:08:02:C0:FF:EE -mac1 00:08:02:C0:FF:ED -sn 0123456789ABCDEF -pdate 20131014 -key 00000000 -oem "Dr. Neuhaus" -wmbus 0123456789ABCDEF
	}
	else if ((strcmp(argv[1],"diskfree")==0))
	{
		ret = diskfree(argc, argv);		
	}
	else if ((strcmp(argv[1],"disk")==0))
	{
		ret = disk(argc, argv);		
	}
	else if ((strcmp(argv[1],"diskuse")==0))
	{
		ret = usedDisk(argc, argv);		
	}
	else if ((strcmp(argv[1],"tamper")==0))
	{
		ret = tamper();		
	}
	else if ((strcmp(argv[1],"tamperreboot")==0))
	{
		ret = tamperreboot();		
	}
	else if ((strcmp(argv[1],"setdtr")==0))
	{
		ret = setdtr();		
	}
	else if ((strcmp(argv[1],"pppdpid")==0))
	{
		ret = get_pppd_pid();
	}
	else if ((strcmp(argv[1],"resetwmbus")==0))
	{
		ret = resetwmbu();
	}
	else if ((strcmp(argv[1],"resetgsm")==0))
	{
		ret = resetgsm();
	}
	else if ((strcmp(argv[1],"rmgw")==0))
	{
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret = remove_gw(argv);
		}			
	}
	else if ((strcmp(argv[1],"powport1")==0) || (strcmp(argv[1],"powport2")==0))
	{
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret = power_port(argc, argv);
		}
	}
	else if ((strcmp(argv[1],"getdsr")==0))
	{
		ret = get_DSR();
	}
	else if ((strcmp(argv[1],"getbatt")==0))
	{
		ret = get_BattVoltage();
	}
	else if ((strcmp(argv[1],"ntpsync")==0))
	{
		ret = get_ntpstat(argc, argv);
	}
	else if ((strcmp(argv[1],"bootconf")==0))
	{
		ret = get_bootconf();
	}
	else if ((strcmp(argv[1],"bootbit")==0))
	{
		if (argc<4)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=BootBitManipulate(argv);
		}		
	}
	else if (strcmp(argv[1],"setcpuboot") == 0)
	{
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=setcpuboot(argv);
		}	
	}
	else if (strcmp(argv[1],"tz") == 0)
	{
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=timezoneinfo(argc, argv);
		}	
	}
	else if ((strcmp(argv[1],"getcurr12v")==0))
	{
		ret = get_current_12v();
	}
	printf("ret=%d\n",ret);	// returncode der Ausfhrung
			

	// Deinitialisierung der libhal
	halDeinit();

#ifdef WIN32
	// Disable the privilege
	tkp.Privileges[0].Attributes = 0; 
	AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, (PTOKEN_PRIVILEGES) NULL, 0); 
#endif

	return ret;
}

