/*****************************************************************
*
* HAL - Hardware Abstraction Layer implementation
*
* Copyright (c) 2013 Dr. Neuhaus Telekommunikation GmbH
*
*****************************************************************/

#include "stdafx.h"
#include "hal_sdk.h"

#ifdef LINUX
	#include <sys/sysinfo.h>
	//#include <sys/statvfs.h>
	#include <sys/vfs.h>
	#include <arpa/inet.h>
	#include <net/if.h>
	#include <net/if_arp.h>
	#include <sys/ioctl.h>
	#include <sys/stat.h>
	#include <fcntl.h>
	#include <signal.h>
	#include <termios.h>
	#include <errno.h>
	#include <sys/types.h>
	#include <net/route.h>
	//#include "ksz8863.h"

	//#include <linux/crc16.h>
	
	#include <sys/time.h>
	#include <error.h>	

	#include <linux/netlink.h>
	#include <linux/rtnetlink.h>
	#include <linux/types.h>
	#include <linux/sockios.h>
	#include <net/ethernet.h> 
	#include <netinet/if_ether.h>
	#include <sys/socket.h>
	#include <string.h>
	#include <time.h>
	#include <stdlib.h>
	#include <unistd.h>
	#include <asm/types.h>

	#include <linux/if_vlan.h>

	#include "setif.h"
	#include "mdio.h"

#endif

#ifdef WIN32
 #include <windows.h>
 #include <Psapi.h>
 //warning C4996:: Funktion wurde als veraltet deklariert
 #pragma warning (disable: 4996)
#endif 

using namespace std;

namespace hal
{
#define IN_PORT				"/dev/in_port"
#define OUT_PORT			"/dev/out_port"
#define SEL_SIM				"/dev/sel_sim"
#define POWER_IND			"/dev/power_indication"
#define SWITCH_INTERRUPT	"/dev/switch_interrupt"
#define RESET_SWITCH		"/dev/reset_switch"
#define SWITCH_ON			"/dev/switch_on"
#define SIM_IND				"/dev/sim_ind"
#define EMERGENCY			"/dev/emergency"
#define SERVICE				"/dev/service"
#define GSM_IGT				"/dev/gsm_igt"
#define	GSM_ON				"/dev/gsm_on"
#define DSR_ON				"/dev/dsr_on"
#define RING_GSM			"/dev/ring_gsm"
#define DCD_ON				"/dev/dcd_on"
#define DTR_ON				"/dev/dtr_on"

#define LED_POWER			"/dev/power_led"
#define LED_VPN				"/dev/vpn_led"
#define LED_IN				"/dev/in_led"
#define LED_OUT				"/dev/out_led"
#define LED_4				"/dev/led4"
#define LED_CONNECTION		"/dev/connection_led"
#define LED_QUALITY			"/dev/quality_led"
#define LED_SIGNAL			"/dev/signal_led"
#define LED_SIM_IND			"/dev/sim_ind"

#define NET_FORWARD         "/proc/sys/net/ipv4/ip_forward"
#define NHRP_FILE			"/etc/opennhrp.conf"
#define IPSEC_POLICY_FILE	"/etc/setkey.conf"
#define IPSEC_CONFIG_FILE	"/etc/racoon.conf"
#define IPSEC_PSK_FILE		"/etc/psk.txt"

	//-----------------------------------------------------------------------------------------------------------

	bool halInit()
	{
		return true;
	}

	void halDeinit()
	{
	}

	//-----------------------------------------------------------------------------------------------------------

	vector<string> GetStringTokens(const string& s, const string& sep, const unsigned int count = 0)
	{
		unsigned int cnt = 0;
		vector<string> toks;

		if(!s.empty() && !sep.empty()) {

			int pos1 = 0; int pos2 = 0;
			while( (pos2 = s.find(sep, pos1)) != -1) 
			{
				cnt++;
				if (count != 0 && cnt == count)
				{
					break;
				}

				toks.push_back(s.substr(pos1, pos2-pos1));
				pos1 = pos2 + sep.length();
			}

			toks.push_back(s.substr(pos1));
		}

		return toks;
	}

	//-----------------------------------------------------------------------------------------------------------

	static const unsigned short _fcs16tab[256] = {
		0x0000, 0x1189, 0x2312, 0x329b, 0x4624, 0x57ad, 0x6536, 0x74bf,
		0x8c48, 0x9dc1, 0xaf5a, 0xbed3, 0xca6c, 0xdbe5, 0xe97e, 0xf8f7,
		0x1081, 0x0108, 0x3393, 0x221a, 0x56a5, 0x472c, 0x75b7, 0x643e,
		0x9cc9, 0x8d40, 0xbfdb, 0xae52, 0xdaed, 0xcb64, 0xf9ff, 0xe876,
		0x2102, 0x308b, 0x0210, 0x1399, 0x6726, 0x76af, 0x4434, 0x55bd,
		0xad4a, 0xbcc3, 0x8e58, 0x9fd1, 0xeb6e, 0xfae7, 0xc87c, 0xd9f5,
		0x3183, 0x200a, 0x1291, 0x0318, 0x77a7, 0x662e, 0x54b5, 0x453c,
		0xbdcb, 0xac42, 0x9ed9, 0x8f50, 0xfbef, 0xea66, 0xd8fd, 0xc974,
		0x4204, 0x538d, 0x6116, 0x709f, 0x0420, 0x15a9, 0x2732, 0x36bb,
		0xce4c, 0xdfc5, 0xed5e, 0xfcd7, 0x8868, 0x99e1, 0xab7a, 0xbaf3,
		0x5285, 0x430c, 0x7197, 0x601e, 0x14a1, 0x0528, 0x37b3, 0x263a,
		0xdecd, 0xcf44, 0xfddf, 0xec56, 0x98e9, 0x8960, 0xbbfb, 0xaa72,
		0x6306, 0x728f, 0x4014, 0x519d, 0x2522, 0x34ab, 0x0630, 0x17b9,
		0xef4e, 0xfec7, 0xcc5c, 0xddd5, 0xa96a, 0xb8e3, 0x8a78, 0x9bf1,
		0x7387, 0x620e, 0x5095, 0x411c, 0x35a3, 0x242a, 0x16b1, 0x0738,
		0xffcf, 0xee46, 0xdcdd, 0xcd54, 0xb9eb, 0xa862, 0x9af9, 0x8b70,
		0x8408, 0x9581, 0xa71a, 0xb693, 0xc22c, 0xd3a5, 0xe13e, 0xf0b7,
		0x0840, 0x19c9, 0x2b52, 0x3adb, 0x4e64, 0x5fed, 0x6d76, 0x7cff,
		0x9489, 0x8500, 0xb79b, 0xa612, 0xd2ad, 0xc324, 0xf1bf, 0xe036,
		0x18c1, 0x0948, 0x3bd3, 0x2a5a, 0x5ee5, 0x4f6c, 0x7df7, 0x6c7e,
		0xa50a, 0xb483, 0x8618, 0x9791, 0xe32e, 0xf2a7, 0xc03c, 0xd1b5,
		0x2942, 0x38cb, 0x0a50, 0x1bd9, 0x6f66, 0x7eef, 0x4c74, 0x5dfd,
		0xb58b, 0xa402, 0x9699, 0x8710, 0xf3af, 0xe226, 0xd0bd, 0xc134,
		0x39c3, 0x284a, 0x1ad1, 0x0b58, 0x7fe7, 0x6e6e, 0x5cf5, 0x4d7c,
		0xc60c, 0xd785, 0xe51e, 0xf497, 0x8028, 0x91a1, 0xa33a, 0xb2b3,
		0x4a44, 0x5bcd, 0x6956, 0x78df, 0x0c60, 0x1de9, 0x2f72, 0x3efb,
		0xd68d, 0xc704, 0xf59f, 0xe416, 0x90a9, 0x8120, 0xb3bb, 0xa232,
		0x5ac5, 0x4b4c, 0x79d7, 0x685e, 0x1ce1, 0x0d68, 0x3ff3, 0x2e7a,
		0xe70e, 0xf687, 0xc41c, 0xd595, 0xa12a, 0xb0a3, 0x8238, 0x93b1,
		0x6b46, 0x7acf, 0x4854, 0x59dd, 0x2d62, 0x3ceb, 0x0e70, 0x1ff9,
		0xf78f, 0xe606, 0xd49d, 0xc514, 0xb1ab, 0xa022, 0x92b9, 0x8330,
		0x7bc7, 0x6a4e, 0x58d5, 0x495c, 0x3de3, 0x2c6a, 0x1ef1, 0x0f78
	};

	//-----------------------------------------------------------------------------------------------------------
#if 0
	static const char *_LedPathFilenamesBrigthness[] = {
		"/sys/class/leds/led0/brightness",
		"/sys/class/leds/led1/brightness",
		"/sys/class/leds/led2/brightness",
		"/sys/class/leds/led3/brightness",
		"/sys/class/leds/led4/brightness",
		"/sys/class/leds/led5/brightness",
	};
#endif
	//-----------------------------------------------------------------------------------------------------------

	static const char *_LedPathFilenamesTrigger[] = {
		"/sys/class/leds/led0/trigger",
		"/sys/class/leds/led1/trigger",
		"/sys/class/leds/led2/trigger",
		"/sys/class/leds/led3/trigger",
		"/sys/class/leds/led4/trigger",
		"/sys/class/leds/led5/trigger",
	};

	//-----------------------------------------------------------------------------------------------------------

	static const char *_LedPathFilenamesDelayOn[] = {
		"/sys/class/leds/led0/delay_on",
		"/sys/class/leds/led1/delay_on",
		"/sys/class/leds/led2/delay_on",
		"/sys/class/leds/led3/delay_on",
		"/sys/class/leds/led4/delay_on",
		"/sys/class/leds/led5/delay_on",
	};

	//-----------------------------------------------------------------------------------------------------------

	static const char *_LedPathFilenamesDelayOff[] = {
		"/sys/class/leds/led0/delay_off",
		"/sys/class/leds/led1/delay_off",
		"/sys/class/leds/led2/delay_off",
		"/sys/class/leds/led3/delay_off",
		"/sys/class/leds/led4/delay_off",
		"/sys/class/leds/led5/delay_off",
	};

	//-----------------------------------------------------------------------------------------------------------

	unsigned short _createCRC16(const char* data, register int len, unsigned short fcs)
	{			
		while(len--) {		
			fcs = (fcs >> 8) ^ _fcs16tab[(fcs ^ *data++) & 0x0ff];
		}
		return fcs ^ 0x0ffff;
	}

	//-----------------------------------------------------------------------------------------------------------
		
	int _write_switch_register(const int fd,const unsigned char address, const unsigned char value)
	{
		int ret = 0;

#ifdef LINUX
#if 0
		unsigned char buf[2];

		buf[0]=address;
		buf[1]=value;
		ret = ioctl(fd, IOCTL_KSZ8863_WRITE, buf);
#endif

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	unsigned char _read_switch_register(const int fd,const unsigned char address)
	{
		unsigned char ret = 0;

#ifdef LINUX
		unsigned char buf[2];

		buf[0]=address;
		buf[1]=0;
#if 0
		ioctl(fd, IOCTL_KSZ8863_READ, buf);
#endif

		ret = buf[1];

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _get_eeprom_size()
	{		
		int eepromSize = 1;

#ifdef LINUX
		// Gre des EEPROM bestimmen ber dessen Namen

		char eepromName[5];
		memset(eepromName,0,5);
		FILE* fp=fopen("/sys/bus/i2c/devices/0-0050/name","r");
		if(fp != NULL)
		{			
			fscanf(fp,"%s",eepromName);
			fclose(fp);			
			eepromSize = (strcmp(eepromName,"24c32") == 0) ? 4096 : -1;
		}
#else
		
#endif
		return eepromSize;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool _writeEepromEntry(const unsigned int _addr, const unsigned int _valuelength, const std::string& _value)
	{
		bool ret = false;
		
#ifdef LINUX
	
		const unsigned int blocksize = _valuelength + LENGTH_BYTE + FCS16_LENGTH;
		int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDWR);

		if ( fd < 0 )
		{		
			return false;
		}
		else
		{
			char wbuf[blocksize];
			char rbuf[blocksize];
			::memset(wbuf,0,blocksize);
			::memset(rbuf,0,blocksize);

			if (_value.length() > _valuelength)
			{			
				close(fd);
				return false;
			}

			wbuf[0] = _value.length();			// Lngenbyte setzen
			memcpy(wbuf+1, _value.c_str(),_valuelength);		// _value in den Schreibpuffer ablegen		

			unsigned short fcs16_calc = _createCRC16(wbuf, blocksize, 0xffff);	// Berechnung der CRC16 ber den GESAMTEN Schreibpuffer	

			wbuf[blocksize-2] = (fcs16_calc >> 8) & 0x00FF;		// Eintragen der CRC16 in den Schreibpuffer
			wbuf[blocksize-1] = (fcs16_calc) & 0x00FF;

			// 3x retry Buffer schreiben und anschliessend verifizieren		
			for (int i=0; i<3; i++)
			{
				::lseek( fd, _addr, SEEK_SET);				// Deskriptor auf Position _addr setzen			

				// Schreiben des gesamten Buffers schlgt in manchen Fllen fehl, deshalb byteweise schreiben			
				for (unsigned int j = 0 ; j < blocksize ; j++) ::write(fd, wbuf+j, 1);						
				//::write( fd, wbuf, blocksize);				// Block schreiben
				::lseek( fd, _addr, SEEK_SET);				// Deskriptor zurcksetzen
				::read( fd, rbuf, blocksize);				// Block lesen			
				if (memcmp(wbuf,rbuf,blocksize) == 0)
				{
					ret = true;
					break;
				}			

			}		
			close (fd);
		}
		
#endif		

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool _readEepromEntry( const unsigned int _addr, const unsigned int _valuelength, std::string& _value )
	{
		bool ret = false;
		
#ifdef LINUX
		
		const unsigned int blocksize = _valuelength + LENGTH_BYTE + FCS16_LENGTH;

		int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDONLY);

		if ( fd < 0 )
		{			
			return false;
		}
		else
		{
			char rbuf[blocksize];				
			::memset(rbuf,0,blocksize);

			for (int i=0; i < 3; i++)
			{				
				::lseek(fd, _addr, SEEK_SET);
				::read(fd, rbuf,blocksize);

				unsigned short fcs16_readed = (rbuf[blocksize - 2] << 8) | rbuf[blocksize - 1];	
				// CRC (die letzten beiden bytes) aus dem readbuffer lschen
				rbuf[blocksize - 2] = 0;
				rbuf[blocksize - 1] = 0;
				unsigned short fcs16_calc = _createCRC16(rbuf, blocksize, 0xffff);
				//unsigned short fcs16_calc = _createCRC16(rbuf, blocksize, 0xffff);
				//printf("fcs16_readed: %x	fcs16_calc: %x \n",fcs16_readed, fcs16_calc);		

				if(fcs16_calc == fcs16_readed)
				{
					_value.assign(rbuf+1,_valuelength);
					ret=true;
					break;
				}
				else 
				{
					_value="n/a";
				}
			}

			close(fd);
		}
		
		
		return ret;
#else
		return true;
#endif
	}

	//-----------------------------------------------------------------------------------------------------------
	//	NTP Statusabfrage:	- erfolgt ber NTP Control Message Protokoll (mode 6)
	//						- Mechanismus ist von ntpstat bernommen worden
	

#define RMASK 0x80  /* bit mask for the response bit in Status Byte2 */
#define EMASK 0x40  /* bit mask for the error bit in Status Byte2 */
#define MMASK 0x20  /* bit mask for the more bit in Status Byte2 */
#define PAYLOADSIZE 468 /* size in bytes of the message payload string */
//#define MAX_OFFSET	2000	// maximal zulssiger Offset in ms

	struct  NtpControlMessageHeader					// RFC-1305 NTP control message format Appendix B
	{
		// Bit 0: MSB des jeweiligen Registers
		unsigned char byte1;	// Bit 0,1:		00
								// Bit 2-4:		NTP-Version, Version: 2 (010)
								// Bit 5-7:		Mode, 6 - NTP control message
		unsigned char byte2;	// Bit 0:		Response Bit
								// Bit 1:		Error Bit
								// Bit 2:		More Bit
								// Bit 3-7:		Operation Code:
								//					1 - read status
								//					2 - read variables
								//					4 - read clock	
		unsigned short sequence;
		unsigned char  status1;		// Bit 0-1: Leap indicator: 
									//				00 - No warning, 
									//				01 - last minute has 61 seconds
									//				10 - last minute has 59 seconds
									//				11 - alarm condition (clock not synchronized)
									// Bit 2-7:	Clock Source
		unsigned char  status2;		// Bit 0-3: System Event Counter
									// Bit 4-7: System Event Code
		unsigned short AssocID;	
		unsigned short Offset;
		unsigned short Count;
		char payload[PAYLOADSIZE];
		char authenticator[96];
	};

	// Senden/Empfangen des UDP-Paketes zum/vom ntpd
	bool _sendReceive_NTPControlMessage(NtpControlMessageHeader &_ntpmsg, const sockaddr_in &_sock, int sd, fd_set &fds, string &_errortext)
	{
		bool ret = true;
		
#ifdef LINUX
		struct timeval	tv;

		// Timeout = 1s
		tv.tv_sec = 1;
		tv.tv_usec = 0;

		// Sendet die Command-Message
		if ((sd = socket (PF_INET, SOCK_DGRAM, 0)) < 0)
		{
			_errortext = "unable to open socket";
			return false;
		}    

		if (connect(sd, (struct sockaddr *)&_sock, sizeof(_sock)) < 0) {
			perror ("On connect");
			_errortext = "unable to connect to socket";
			return false;	
		}

		FD_SET(sd, &fds);

		if (send(sd, &_ntpmsg, sizeof(_ntpmsg), 0) < 0) {
			perror ("On send");
			_errortext = "unable to send command to NTP port";
			return false;
		}

		int n = select (sd+1, &fds, (fd_set *)0, (fd_set *)0 , &tv);

		if (n == 0)
		{
			_errortext = "timeout";
			return false;
		}   

		if (n == -1)
		{
			_errortext = "error on select";
			return false;
		}

		if ((n = recv (sd, &_ntpmsg, sizeof(_ntpmsg), 0)) < 0)
		{
			_errortext = "Unable to talk to NTP daemon. Is it running?";
			return false;
		}
#endif
		return ret;
	}

	// berprft, ob das Response-Paket valid ist
	bool _checkResponseMsg( const NtpControlMessageHeader &_ntpmsg, unsigned char _opcode, string &_errortext )
	{
		bool ret = true;

		unsigned char byte1ok = ((_ntpmsg.byte1&0x3F) == 0x16);
		unsigned char byte2ok = ((_ntpmsg.byte2 & ~MMASK) == (_opcode|RMASK));
  
		if (!(byte1ok && byte2ok)) {
			fprintf (stderr,"status word is 0x%02x%02x\n", _ntpmsg.byte1,_ntpmsg.byte2 );
			printf("return data appears to be invalid based on status word\n");
			return false;	
		}

		if (!(_ntpmsg.byte2 | EMASK)) {
			fprintf (stderr,"status byte2 is %02x\n", _ntpmsg.byte2 );
			printf("error bit is set in reply\n");
			return false;	    
		}

		if (!(_ntpmsg.byte2 | MMASK)) {
			fprintf (stderr,"status byte2 is %02x\n", _ntpmsg.byte2 );
			fprintf (stderr,"More bit unexpected in reply");
		}

		return ret;
	}

	bool _isTimeSynchronized( std::map<std::string, std::string> &_ntpParams, unsigned int _maxOffset, string &_errortext )
	{
		bool ret = true;
		//int reach;		

		std::map<std::string, std::string>::iterator it;		
		
		// Der leap indicator im status1 byte wird u.U. erst Stunden spter gesetzt
		///* if the leap indicator (LI), which is the two most significant bits
		//in status byte1, are both one, then the clock is not synchronised. */
		//if ((_ntpmsg.status1 >> 6) == 3) 
		//{
		//	ret = false;			
		//	/* look at the system event code and see if indicates system restart */
		//	if ((_ntpmsg.status2 & 0x0F) == 1)
		//	{
		//		_errortext = "time server re-starting";
		//	}
		//	else
		//	{
		//		_errortext = "unsynchronized";
		//	}				
		//}

		// Kriterien fr eine bestehende Synchronisation der Zeit:
		//	1. Leap indicator != 3 (unsynchronized), 1,2 - warning, 0 - synchronized
		//	2. reach > 0, sonst besteht keine Konnektivitt zwischen ntpd und einem Zeitserver
		//	3. abs(offset) < 2s

		it = _ntpParams.find("leap");
		if ((it != _ntpParams.end()) && (it->second == "3"))
		{
			_errortext = "unsynchronized";
			return false;
		}

		it = _ntpParams.find("reach");
		if ((it != _ntpParams.end()) && (it->second == "0x0"))
		{
			_errortext = "reach level zero or not found";
			return false;
		}

		it = _ntpParams.find("offset");		
		if (it != _ntpParams.end())
		{
			unsigned int offset;
			if(sscanf(it->second.c_str(), "%u", &offset) == 1)
			{
				if (offset > _maxOffset)
				{
					_errortext = "time offset exceeds maximum value";
					return false;
				}
				else 
				{			
					_errortext = "offset value unknown";
					return false;
				}
			}
			//printf("offset: %d\n", offset);			
		}
		else 
		{			
			_errortext = "offset value unknown";
			return false;		
		}

		return ret;
	}	

	// Hilfsfunktion zum Entfernen von strenden Zeichen
	void StringTrimLeft(std::string& str, const string& chars2remove)
	{
		if (!str.empty()) {
			std::string::size_type pos = str.find_first_not_of(chars2remove);
			if (pos != std::string::npos)
				str.erase(0,pos);
			else
				str.erase( str.begin() , str.end() ); // make empty
		}
	}

	// Parsen des Response und Ablegen der Werte in einer cMap
	void _readNtpParams( const NtpControlMessageHeader &_ntpmsg , std::map<std::string, std::string> &_ntpParams )
	{
		string msgbuf = _ntpmsg.payload;		
		
		//printf("Payload: \n%s\n", msgbuf.c_str());
		
		// clocksource bestimmen
		int clksrc = (_ntpmsg.status1 & 0x3F);
		
		switch (clksrc)
		{
		case 0:		_ntpParams["Clocksource"] = "unspecified";
			break;
		case 1:		_ntpParams["Clocksource"] = "atomic clock";
			break;
		case 2:		_ntpParams["Clocksource"] = "VLF radio";
			break;
		case 3:		_ntpParams["Clocksource"] = "HF radio";
			break;
		case 4:		_ntpParams["Clocksource"] = "UHF radio";
			break;
		case 5:		_ntpParams["Clocksource"] = "local net";
			break;
		case 6:		_ntpParams["Clocksource"] = "NTP server";
			break;
		case 7:		_ntpParams["Clocksource"] = "UDP/TIME";
			break;
		case 8:		_ntpParams["Clocksource"] = "wristwatch";
			break;
		case 9:		_ntpParams["Clocksource"] = "modem";
			break;
		default:	_ntpParams["Clocksource"] = "unknown source";
			break;
		}
				
		// Zerlegen des Payload in Key=value Paare
		vector<string> params = GetStringTokens(msgbuf, ",", 0 );

		vector<string> keyvalue;
		string key, value;
				
		// Auftrennen von key/value und Eintragen in die Map
		for(vector<string>::iterator it=params.begin(); it!=params.end(); it++)
		{				
			keyvalue = GetStringTokens(it->c_str(), "=", 0);	// Trennung bei "="			
			StringTrimLeft(keyvalue[0], " ");					// fhrende Leerzeichen entfernen
			StringTrimLeft(keyvalue[0], "\r\n");				// Zeilenumbrche entfernen
			_ntpParams[keyvalue[0]] = keyvalue[1];				// Eintragen in die Map
		}
	}

	// Ausgabe von Parametern, nur zum Debuggen
	void _printNtpParams( std::map<std::string, std::string> &_ntpParams )
	{
		std::map<std::string, std::string>::iterator it;

		/*for(it=_ntpParams.begin(); it!=_ntpParams.end(); it++)
		{			
			printf("%s\t%s\n",it->first.c_str(),it->second.c_str());
		}*/

		it = _ntpParams.find("leap");
		if (it != _ntpParams.end())
		{			
			printf("%s\t%s\n",it->first.c_str(),it->second.c_str());
		}
		it = _ntpParams.find("reach");
		if (it != _ntpParams.end())
		{			
			printf("%s\t%s\n",it->first.c_str(),it->second.c_str());
		}
		it = _ntpParams.find("offset");
		if (it != _ntpParams.end())
		{			
			printf("%s\t%s\n",it->first.c_str(),it->second.c_str());
		}
	}


	int _ntpstat (unsigned short _ntp_port, unsigned int _maxOffset) 
	{
		int								rc = 0;        //  return code
	
#ifdef LINUX
		struct sockaddr_in				sock;
		struct in_addr					address;
		int								sd = 0;
		fd_set							fds;     
		struct NtpControlMessageHeader	ntpmsg;
		string							errortext;
		string							payload_string;

		std::map<std::string, std::string> NtpParams;  

		// File descriptor set
		FD_ZERO(&fds);

		inet_aton("127.0.0.1", &address);
		sock.sin_family = AF_INET;
		sock.sin_addr = address;
		sock.sin_port = htons(_ntp_port);

		//---------------------------------------------------------------------------------------------------
		// Aufbau der ersten ntpd Abfrage (nur Status, Association ID wird bentigt)

		memset ( &ntpmsg, 0, sizeof(ntpmsg));
		// 0x1601  
		ntpmsg.byte1 = 0x16;
		ntpmsg.byte2 = 0x01;    
		ntpmsg.sequence=htons(1);  

		if (!_sendReceive_NTPControlMessage(ntpmsg, sock, sd, fds, errortext))
		{
			// Fehler beim Senden
			printf("%s\n", errortext.c_str());
			return 2;
		}

		if (!_checkResponseMsg(ntpmsg, 0x01, errortext))
		{
			// Fehlerhafte Antwort erhalten
			printf("%s\n", errortext.c_str());
			return 2;
		}
	
		// Auslesen der Association ID aus der ersten Statusantwort
		char assoID1 = ntpmsg.payload[0];
		char assoID2 = ntpmsg.payload[1];

		//---------------------------------------------------------------------------------------------------
		// Aufbau der zweiten ntpd Abfrage (Abfrage der Variablen)

		memset ( &ntpmsg, 0, sizeof(ntpmsg));
		ntpmsg.byte1 = 0x16;
		ntpmsg.byte2 = 0x02;  
		ntpmsg.sequence=htons(2);
		// Setzen der Association ID, wird fr den erweiterten Report bentigt
		ntpmsg.AssocID = (assoID2 << 8);
		ntpmsg.AssocID |= assoID1;
 
		if (!_sendReceive_NTPControlMessage(ntpmsg, sock, sd, fds, errortext))
		{
			// Fehler beim Senden
			printf("%s\n", errortext.c_str());
			return 2;
		}  

		if (!_checkResponseMsg(ntpmsg, 0x02, errortext))
		{
			// Fehlerhafte Antwort erhalten
			printf("%s\n", errortext.c_str());
			return 2;
		}

		// Payload der zweiten ntp Statusabfrage Bsp:
		/* srcadr=172.24.1.0, srcport=123, dstadr=172.24.100.24, dstport=123,
			leap=0, stratum=4, precision=-6, rootdelay=2839.066, rootdisp=189.651,
			refid=172.23.1.151, reftime=0xd6bee212.ff9d4017,
			rec=0xd6beec04.43396326, reach=0xff, unreach=0, hmode=3, pmode=4,
			hpoll=3, ppoll=3, headway=6, flash=0x400, keyid=0, offset=846.895,
			delay=3.991, dispersion=15.684, jitter=5.191, xleave=3.315,
			filtdelay= 3.99 4.18 4.22 4.05 2.81 1.52 4.10 4.31,*/
  
		_readNtpParams( ntpmsg , NtpParams );
		_printNtpParams( NtpParams );

		if (_isTimeSynchronized(NtpParams, _maxOffset, errortext))
		{
			rc = 0;
		}
		else
		{
			rc = 1;
			printf("%s\n", errortext.c_str());
		} 
#endif

		return rc;
	}

	//-----------------------------------------------------------------------------------------------------------
	
	int ResetSwitch()
	{
		int ret = 0;
#ifdef LINUX
#if 0
		int fd = open("/dev/ksz8863", O_RDWR);
		if (fd < 0)
		{
				printf("%s: can't open /dev/ksz8863\n", __FUNCTION__);
				ret = fd;				
		}
		else
		{
			// addr: 0x43, set 0x11 total reset to the switch	
			ret = _write_switch_register(fd,0x43,0x11);		
			close(fd);
		}
#else
		FILE* pFile = fopen(RESET_SWITCH, "w");
		if (pFile != NULL)
		{
			ret = (fprintf(pFile, SIG_ON)>0);
			usleep(500000);
			ret = (fprintf(pFile, SIG_OFF)>0);
			fclose(pFile);
		}
		else
		{
			ret = -1;
		}
#endif
		
#endif
		return ret;
	}


	//-----------------------------------------------------------------------------------------------------------
	int ConfigureSwitch()
	{	
		int ret = 0;

#ifdef LINUX
		int fd = open("/dev/ksz8863", O_RDWR);
		if (fd < 0)
		{
			printf("%s: can't open /dev/ksz8863\n", __FUNCTION__);
			ret = fd;
			return ret;
		}
		else
		{
			// ###########  Schreiben der Register des Switches Micrel KSZ8863

			// cpu interface clock selection
			// supported spi clock is now below 6 MHz (power saving)	
			ret = _write_switch_register(fd,0x0B,0x08);

			// assign port 1 & port 3 to vlan 1	
			ret = _write_switch_register(fd,0x81,0x0D);		// mbr
			ret = _write_switch_register(fd,0x82,0x10);		// fid
			ret = _write_switch_register(fd,0x83,0x01);		// vid
			ret = _write_switch_register(fd,0x79,0x04);		// write vlan table selected
			ret = _write_switch_register(fd,0x7A,0x01);		// trigger write operation

			// assign port 2 & port 3 to vlan 2
			ret = _write_switch_register(fd,0x81,0x0E);		// mbr
			ret = _write_switch_register(fd,0x82,0x20);		// fid
			ret = _write_switch_register(fd,0x83,0x02);		// vid
			ret = _write_switch_register(fd,0x79,0x04);		// write vlan table selected
			ret = _write_switch_register(fd,0x7A,0x02);		// trigger write operation

			// configure default vlan id
			ret = _write_switch_register(fd,0x14,0x01);		// port 1, vlan_id=1
			ret = _write_switch_register(fd,0x24,0x02);		// port 2, vlan_id=2

			// discard non pvid packets
			ret = _write_switch_register(fd,0x12,0x46);
			ret = _write_switch_register(fd,0x22,0x46);

			// enable tag insertion on port 3
			// remove vlan-tags on port 1 & 2
			ret = _write_switch_register(fd,0x30,0x04);
			ret = _write_switch_register(fd,0x10,0x02);
			ret = _write_switch_register(fd,0x20,0x02);

			// insert src pvid
			ret = _write_switch_register(fd,0xC2,0x14);
			//ret = _write_switch_register(fd,0xC2,0x3F);

			// enable vlan
			ret = _write_switch_register(fd,0x05,0x80);

			// ###################################################################		
			close(fd);
		}
		
		
#endif

		return ret;
	}

	int _GetSMIPhyAddr(uint32_t _port, uint32_t& _phyAddr)
	{
		switch (_port)
		{
			case LAN_PORT_1:
				{
					_phyAddr = LAN_1_SMI_PHY_ADDR;
				}
				break;
			case LAN_PORT_2:
				{
					_phyAddr = LAN_2_SMI_PHY_ADDR;
				}
				break;
			case LAN_PORT_3:
				{
					_phyAddr = LAN_3_SMI_PHY_ADDR;
				}
				break;
			case LAN_PORT_4:
				{
					_phyAddr = LAN_4_SMI_PHY_ADDR;
				}
				break;
			case LAN_PORT_5:
				{
					_phyAddr = LAN_5_SMI_PHY_ADDR;
				}
				break;
			default:
				{
					// Illegal LAN Port
					return -1;
				}
				break;
		}
		return 0;
	}

	int _GetSwitchPortPhyAddr(uint32_t _port, uint32_t& _phyAddr)
	{
		switch (_port)
		{
			case LAN_PORT_1:
				{
					_phyAddr = LAN_1_SWITCH_PORT_PHYID;
				}
				break;
			case LAN_PORT_2:
				{
					_phyAddr = LAN_2_SWITCH_PORT_PHYID;
				}
				break;
			case LAN_PORT_3:
				{
					_phyAddr = LAN_3_SWITCH_PORT_PHYID;
				}
				break;
			case LAN_PORT_4:
				{
					_phyAddr = LAN_4_SWITCH_PORT_PHYID;
				}
				break;
			case LAN_PORT_5:
				{
					_phyAddr = LAN_5_SWITCH_PORT_PHYID;
				}
				break;
			default:
				{
					// Illegal LAN Port
					return -1;
				}
				break;
		}
		return 0;
	}

	//-----------------------------------------------------------------------------------------------------------
	int GetSwitchReg(uint32_t _port, uint32_t _reg, uint32_t& _result)
	{
		int ret = 0;
#ifdef LINUX
		// Device is always "eth1"
		string strDevice = "eth1";
		// Phy Index is defined by the port
		uint32_t uiPhyIndex = 0;
		if (_GetSwitchPortPhyAddr(_port, uiPhyIndex) != 0)
		{
			// Illegal Port
			return -240;
		}

		cMDIO mdio(uiPhyIndex, strDevice.c_str());
		int iTemp = mdio.read(uiPhyIndex, _reg);
		if (iTemp < 0)
		{
			ret = -241;
		}
		else
		{
			_result = iTemp;
		}
		
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	int ModSwitchReg(uint32_t _port, uint32_t _reg, uint16_t _setMask, uint16_t _clearMask)
	{
		int ret = 0;
#ifdef LINUX
		// Device is always "eth1"
		string strDevice = "eth1";
		// Phy Index is defined by the port
		uint32_t uiPhyIndex = 0;
		if (_GetSwitchPortPhyAddr(_port, uiPhyIndex) != 0)
		{
			// Illegal Port
			return -240;
		}

		cMDIO mdio(uiPhyIndex, strDevice.c_str());
		uint32_t uiRegValue=0;
		int iTemp = mdio.read(uiPhyIndex, uiRegValue);
		if (iTemp < 0)
		{
			ret = -241;
		}
		else
		{
			uiRegValue &= ~_clearMask;
			uiRegValue |= _setMask;
			iTemp = mdio.write(uiPhyIndex, _reg, uiRegValue);
			if (iTemp < 0)
			{
				ret = -242;
			}
		}

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int SetSwitchReg(uint32_t _port, uint32_t _reg, uint32_t _val)
	{
		int ret = 0;
#ifdef LINUX
		// Device is always "eth1"
		string strDevice = "eth1";
		// Phy Index is defined by the port
		uint32_t uiPhyIndex = 0;
		if (_GetSwitchPortPhyAddr(_port, uiPhyIndex)!=0)
		{
			// Illegal Port
			return -240;
		}

		cMDIO mdio(uiPhyIndex, strDevice.c_str());
		ret = mdio.write(uiPhyIndex, _reg, _val);
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int SetSwitchPortEnabled(uint32_t _port, bool _enabled)
	{
		int ret = 0;
#ifdef LINUX
		uint32_t uiPhyAddr = 0; // PHY Addr for "Real Phy" used via SMI Cmd by the Switch (Already Aligned to SMI CMD Reg 0x18's Layout)
		if (_GetSMIPhyAddr(_port, uiPhyAddr) != 0)
		{
			return -2;
		}

		string strDevice = "eth1";
		cMDIO mdio(SWITCH_GLOBAL_REG_2_PHYID, strDevice.c_str()); // MDIO mapped to Switch Global 2 Reg
		uint32_t uiSMIFrame = 0;
		if (_enabled)
		{
			uiSMIFrame = 0x3300; // Power Up Cmd, 100MBits, Auto Neg., Normal Operation
		}
		else
		{
			uiSMIFrame = 0x0800; // Power Down Cmd
		}

		
		uint32_t uiSMIRegister = 0x19;
		uint32_t uiSMICmdReg = 0x18;
		uint32_t uiSMICmd = 0x8000 | 0x1000 | 0x0400 | uiPhyAddr | 0x0000; // Start Op | Clause22 SMI Frames | Write Data Register Cmd | Phy Dev Addr | Phy Reg Addr

		mdio.write(SWITCH_GLOBAL_REG_2_PHYID, uiSMIRegister, uiSMIFrame); // Write SMI Frame to Reg
		mdio.write(SWITCH_GLOBAL_REG_2_PHYID, uiSMICmdReg, uiSMICmd); // Initiate SMI Frame Transmission/CMD Execution

		uint16_t uiSetMask = 0x00;
		uint16_t uiClearMask = 0x00;
		if (_enabled)
		{
			uiClearMask = 0x00;
			uiSetMask = 0x03;
		}
		else
		{
			uiClearMask = 0x03;
			uiSetMask = 0x00;
		}
		ret = ModSwitchReg(_port, 0x04, uiSetMask, uiClearMask); // Switch On/Off Forwarding within Switch
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	int GetSwitchPortMode(uint32_t _port, uint32_t& _modeResult)
	{
		int ret = 0;
#ifdef LINUX
		uint32_t uiPortStatus = 0;
		ret = GetSwitchReg(_port, SWITCH_PORT_REG_STATUS, uiPortStatus);
		if (ret == 0)
		{
			if ((uiPortStatus & 0x0B00) == 0x0B00) // 100-FDX
			{
				_modeResult = 2;
			}
			else if ((uiPortStatus & 0x0A00) == 0x0A00) // 10-FDX
			{
				_modeResult = 4;
			}
			else if ((uiPortStatus & 0x0900) == 0x0900) // 100-HDX
			{
				_modeResult = 3;
			}
			else if ((uiPortStatus & 0x0800) == 0x0800) // 10-HDX
			{
				_modeResult = 5;
			}
			else // Reserved
			{
				_modeResult = 0;
			}
		}
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------
	int GetSwitchPortLinkState(uint32_t _port, uint32_t& _linkState)
	{
		int ret = 0;
#ifdef LINUX
		uint32_t uiPortStatus = 0;
		ret = GetSwitchReg(_port, SWITCH_PORT_REG_STATUS, uiPortStatus);
		if (ret == 0)
		{
			if ((uiPortStatus & 0x1000) == 0x1000) // Link is UP
			{
				if ((uiPortStatus & 0x2000) == 0x2000) // Is Link Resolved?
				{
					_linkState = LAN_LINKSTATE_UP_RESOLVED;
				}
				else
				{
					_linkState = LAN_LINKSTATE_UP_UNRESOLVED;
				}
			}
			else // Link is down otherwise ...
			{
				_linkState = LAN_LINKSTATE_DOWN;
			}
		}
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------
	int SetSwitchVLANTagStripping(uint32_t _port, bool _strip)
	{
		int ret = 0;
#ifdef LINUX
		uint16_t uiSetMask = 0x00;
		uint16_t uiClearMask = 0x00;
		if (_strip)
		{
			uiSetMask = 0x1000;
			uiClearMask = 0x0000;
		}
		else
		{
			uiSetMask = 0x0000;
			uiClearMask = 0x3000;
		}
		ret = ModSwitchReg(_port, SWITCH_PORT_REG_PORT_CTRL, uiSetMask, uiClearMask);
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------
	int SetSwitchVLANTagEnforcing(uint32_t _port, bool _enforce)
	{
		int ret = 0;
#ifdef LINUX
		uint16_t uiSetMask = 0x00;
		uint16_t uiClearMask = 0x00;
		if (_enforce)
		{
			uiSetMask = 0x0B00;
			uiClearMask = 0x0000;
		}
		else
		{
			uiSetMask = 0x0000;
			uiClearMask = 0x0B00;
		}
		ret = ModSwitchReg(_port, SWITCH_PORT_REG_PORT_CTRL_2, uiSetMask, uiClearMask);
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	int SetSwitchPortMode(uint32_t _port, uint32_t _mode)
	{
		int ret = 0;
#ifdef LINUX
		uint16_t uiSetMask = 0x00;

		switch(_mode)
		{
			case 1: // AUTO
				{
					uiSetMask = 0x03;
				}
				break;
			case 2: // 100-FDX
				{
					uiSetMask = 0xC1;
				}
				break;
			case 3:// 100-HDX
				{
					uiSetMask = 0x41;
				}
				break;
			case 4://10-FDX
				{
					uiSetMask = 0xC0;
				}
				break;
			case 5://10-HDX
				{
					uiSetMask = 0x40;
				}
				break;
			default:
				{
					return -1; // Illegal Mode
				}
				break;
		}

		ret = SetSwitchReg(_port, SWITCH_PORT_REG_MAC_CTRL, uiSetMask);
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	void _parse_mac(const char* mac_char, unsigned char* mac_byte)
	{
		for(int i=0;i<16;i+=3)
		{
			if(sscanf(mac_char + i,"%c",mac_byte++) != 1) return;
		}
	}

	//-----------------------------------------------------------------------------------------------------------

#ifdef LINUX
	static void _SetDirIN(int dev)
	{
		::lseek(dev, 0, SEEK_SET); 
		::write(dev, "in", 2); 		
	}

	static void _SetDirOUT(int dev)
	{
		::lseek(dev, 0, SEEK_SET); 
		::write(dev, "out", 3); 		
	}

	void _writeGPIO(bool logic, int fd)
	{
		::lseek(fd, 0, SEEK_SET);
		::write(fd, logic ? "1" : "0", 1);
	}

	char _readGPIO(int fd)
	{
		char buf;

		::lseek(fd, 0, SEEK_SET); 
		::read(fd, &buf, 1);
		return buf;		
	}

	//-----------------------------------------------------------------------------------------------------------

	int _TamperRequest(const char* request, char* response)
	{
		int ret = 0;
		int i;

		int dev_CLK_IO   = ::open("/sys/class/gpio/gpio42/value", O_RDWR);
		int dev_CLK_DIR  = ::open("/sys/class/gpio/gpio42/direction", O_RDWR);
		int dev_DATA_IO  = ::open("/sys/class/gpio/gpio44/value", O_RDWR);
		int dev_DATA_DIR = ::open("/sys/class/gpio/gpio44/direction", O_RDWR);

		if(dev_CLK_IO == -1 || dev_CLK_DIR == -1 || dev_DATA_IO == -1 || dev_DATA_DIR == -1)
		{
			printf("sendTamperRequest open device error \n");	
			::close(dev_CLK_IO); ::close(dev_CLK_DIR); ::close(dev_DATA_IO); ::close(dev_DATA_DIR);
			return -1;
		}
		else
		{
			_SetDirOUT(dev_CLK_DIR);			// auf request vorbereiten
			_SetDirOUT(dev_DATA_DIR);		
			usleep(UTIME);

			for (i=0; i<8; i++)
			{
				if (*request=='0')
				{
					_writeGPIO(LOW, dev_DATA_IO);
				}
				else
				{
					_writeGPIO(HIGH, dev_DATA_IO);
				}			
				_writeGPIO(HIGH, dev_CLK_IO);
				usleep(UTIME);
				_writeGPIO(LOW, dev_CLK_IO);
				usleep(UTIME);
				request++;
			}

			usleep(UTIME);
			_SetDirIN(dev_DATA_DIR);
			usleep(UTIME);			

			for (i=0; i<8; i++)
			{
				_writeGPIO(HIGH, dev_CLK_IO);
				usleep(UTIME);			
				*response = _readGPIO(dev_DATA_IO);
				response++;				
				_writeGPIO(LOW, dev_CLK_IO);
				usleep(UTIME);
			}

			::close(dev_CLK_IO); ::close(dev_CLK_DIR); ::close(dev_DATA_IO); ::close(dev_DATA_DIR);

		}

		
		return ret;
	}

#endif
		
	//-----------------------------------------------------------------------------------------------------------

	bool SwitchLed(int _led, bool _on)
	{						
		//int wlen = 0;

#ifdef LINUX
#if 0
		int device = ::open(_LedPathFilenamesBrigthness[_led] , O_WRONLY);

		if (device < 0) 
		{
				return false;
		}
		else
		{            			
			if (_on==false) 
			{	
				// berprfung, ob entsprechende LED im Blink-Modus ist
				// Denn dann sind die Dateien delay_on, delay_off vorhanden
				// Um eine blinkende LED auszuschalten muss sie zuerst eingeschaltet werden
				// dank der Low-Active Logik bei DNT
				int delay=::open(_LedPathFilenamesDelayOn[_led], O_WRONLY);

				if (delay > 0)
				{
					// LED einschalten
					::write(device, "0", 1);	
					// Datei "delay_on" schlieen
					::close(delay);
					// reopen the device file
					if(::close(device) == 0)			
					{
						device = ::open(_LedPathFilenamesBrigthness[_led] , O_WRONLY);
						
						if(device)
						{
							// LED ausschalten
							wlen = ::write(device, "255", 3);		
						}
					}					
				}
				else
				{
					// LED ausschalten
					wlen = ::write(device, "255", 3);		
				}
			} 
			else 
			{
				// LED einschalten
				wlen = ::write(device, "0", 1);			
			}		

			::close(device);
			return (wlen > 0);
		}
#else
		FILE* pFile = NULL;
		switch(_led)
		{
			case HAL_LED_ID_POWER:
				{
					pFile = fopen(LED_POWER, "w");
					if (pFile!=NULL)
					{
						fprintf(pFile, _on?LED_ON:LED_OFF);
						fclose(pFile);
						return true;
					}
					else
					{
						return false;
					}
				}
				break;
			case HAL_LED_ID_VPN:
				{
					pFile = fopen(LED_VPN, "w");
					if (pFile!=NULL)
					{
						fprintf(pFile, _on?LED_ON:LED_OFF);
						fclose(pFile);
						return true;
					}
					else
					{
						return false;
					}
				}
				break;
			case HAL_LED_ID_IN:
				{
					pFile = fopen(LED_IN, "w");
					if (pFile!=NULL)
					{
						fprintf(pFile, _on?LED_ON:LED_OFF);
						fclose(pFile);
						return true;
					}
					else
					{
						return false;
					}
				}
				break;
			case HAL_LED_ID_OUT:
				{
					pFile = fopen(LED_OUT, "w");
					if (pFile!=NULL)
					{
						fprintf(pFile, _on?LED_ON:LED_OFF);
						fclose(pFile);
						return true;
					}
					else
					{
						return false;
					}
				}
				break;
			case HAL_LED_ID_OTHER:
				{
					pFile = fopen(LED_4, "w");
					if (pFile!=NULL)
					{
						fprintf(pFile, _on?LED_ON:LED_OFF);
						fclose(pFile);
						return true;
					}
					else
					{
						return false;
					}
				}
				break;
			case HAL_LED_ID_CONNECTION:
				{
					pFile = fopen(LED_CONNECTION, "w");
					if (pFile!=NULL)
					{
						fprintf(pFile, _on?LED_ON:LED_OFF);
						fclose(pFile);
						return true;
					}
					else
					{
						return false;
					}
				}
				break;
			case HAL_LED_ID_QUALITY:
				{
					pFile = fopen(LED_QUALITY, "w");
					if (pFile!=NULL)
					{
						fprintf(pFile, _on?LED_ON:LED_OFF);
						fclose(pFile);
						return true;
					}
					else
					{
						return false;
					}
				}
				break;
			case HAL_LED_ID_SIGNAL:
				{
					pFile = fopen(LED_SIGNAL, "w");
					if (pFile!=NULL)
					{
						fprintf(pFile, _on?LED_ON:LED_OFF);
						fclose(pFile);
						return true;
					}
					else
					{
						return false;
					}
				}
				break;
			case HAL_LED_ID_SIM_IND:
				{
					pFile = fopen(LED_SIM_IND, "w");
					if (pFile!=NULL)
					{
						fprintf(pFile, _on?LED_ON:LED_OFF);
						fclose(pFile);
						return true;
					}
					else
					{
						return false;
					}
				}
				break;
			default:
				{
					return false;
				}
				break;
		}
#endif
		
		
#else
		return true; //dummy
#endif
	}

	//-----------------------------------------------------------------------------------------------------------

	bool BlinkLed(int _led)
	{
		bool ret = true;		
		int wlen = 0;
				
#ifdef LINUX
		int fd = ::open(_LedPathFilenamesTrigger[_led], O_WRONLY);
		if (fd > 0)
		{
			wlen = ::write(fd, "timer", 5);		// startet den Blinkmodus, Intervall On zu Off=500 ms
			
			::close(fd);
		} 

		ret = wlen == 5;
#endif

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool setLedDelay(int _led, const std::string& _delayofftime_ms, const std::string& _delayontime_ms)
	{
		bool ret = true;

#ifdef LINUX		
		int fd_delay_on = ::open(_LedPathFilenamesDelayOn[_led], O_WRONLY);
		int fd_delay_off = ::open(_LedPathFilenamesDelayOff[_led], O_WRONLY);

		if ((fd_delay_on * fd_delay_off) > 0)	// hat das ffnen beider Dateien funktioniert ?
		{			
			::write(fd_delay_off, _delayofftime_ms.c_str(), _delayofftime_ms.length());
			::write(fd_delay_on, _delayontime_ms.c_str(), _delayontime_ms.length());

			::close(fd_delay_off);
			::close(fd_delay_on);
		} 
		else 
		{
			ret = false;
		}
#endif		
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int nl_RemoveIpAddr(const string& _ethname, const string& _ipv4addr, const uint32_t _prefixlen)
	{
		int ret = 0;
#ifdef LINUX
		struct in_addr _addr;
		inet_pton(AF_INET, _ipv4addr.c_str(), &_addr);

		ret = del_address(_ethname.c_str(), AF_INET, &_addr, _prefixlen);
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int nl_ConfigureIpAddr(const string& _ethname, const string& _ipv4addr, const uint32_t _prefixlen, const string& _bcastAddr)
	{
		int ret = 0;
#ifdef LINUX
		struct in_addr _addr;
		inet_pton(AF_INET, _ipv4addr.c_str(), &_addr);

		struct in_addr _bcast;
		inet_pton(AF_INET, _bcastAddr.c_str(), &_bcast);

		ret = add_address(_ethname.c_str(), AF_INET, &_addr, _prefixlen, &_bcast);
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	int AddVLAN(const std::string& _physInterface, const int& _vlanID)
	{
		int ret = 0;
#ifdef LINUX
		struct vlan_ioctl_args req;

		int fd = socket(AF_INET, SOCK_STREAM, 0);
		if (fd > 0)
		{
			strncpy(req.device1, _physInterface.c_str(), IFNAMSIZ);
			req.cmd = ADD_VLAN_CMD;
			req.u.VID = _vlanID;

			ret = ioctl(fd, SIOCSIFVLAN, &req);
			printf("ioctl: %d/%d/%s\n", ret, errno, strerror(errno));
			close(fd);
		}

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	int DelVLAN(const std::string& _vlanName)
	{
		int ret = 0;
#ifdef LINUX
		struct vlan_ioctl_args req;

		int fd = socket(AF_INET, SOCK_STREAM, 0);
		if (fd > 0)
		{
			strncpy(req.device1, _vlanName.c_str(), IFNAMSIZ);
			req.cmd = DEL_VLAN_CMD;

			ret = ioctl(fd, SIOCSIFVLAN, &req);
			close(fd);
		}

#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------

	int ConfigureIpAddr(const string& _ethname, const string& _ipv4addr)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;
		
		int fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
		if(fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ifr.ifr_addr.sa_family=AF_INET;
			struct sockaddr_in* addr = (struct sockaddr_in*)&ifr.ifr_addr;
			//inet_pton(AF_INET, ipv4_addr, ifr.ifr_addr.sa_data+2);
			inet_pton(AF_INET, _ipv4addr.c_str(), &addr->sin_addr);
			ret = ioctl(fd,SIOCSIFADDR,&ifr);
			close(fd);
		}		
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int ConfigureIpHw(const std::string& _ethname, const std::string& _macaddr)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;
		unsigned char mac_byte[6];	

		_parse_mac(_macaddr.c_str(), mac_byte);
		
		int fd=socket(AF_INET, SOCK_DGRAM, 0);
		if(fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ifr.ifr_hwaddr.sa_family=ARPHRD_ETHER;
			memcpy(ifr.ifr_hwaddr.sa_data,mac_byte,6);
			ret = ioctl(fd,SIOCSIFHWADDR,&ifr);
			close(fd);		
		}
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	
	int ConfigureIpNetmask(const std::string& _ethname, const std::string& _netmask)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;

		int fd=socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
		if(fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ifr.ifr_addr.sa_family=AF_INET;
			inet_pton(AF_INET, _netmask.c_str(), ifr.ifr_addr.sa_data+2);	
			ret = ioctl(fd,SIOCSIFNETMASK, &ifr);
			close(fd);
		}
		
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int ConfigureIpBroadcast(const std::string& _ethname, const std::string& _broadcast)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;

		int fd=socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
		if (fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ifr.ifr_addr.sa_family=AF_INET;
			inet_pton(AF_INET, _broadcast.c_str(), ifr.ifr_addr.sa_data+2);	
			ret = ioctl(fd,SIOCSIFBRDADDR,&ifr);
			close(fd);
		}

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	int ConfigureIPGateway(const std::string& _network, const std::string& _netmask, const std::string& _gateway, const std::string& _interface, const uint32_t& _metric, const bool& _valid)
	{
		int ret = 0;
#ifdef LINUX
		struct rtentry route;
		struct sockaddr_in *addr;

		int fd = socket(AF_INET, SOCK_DGRAM, 0);

		if (fd > 0)
		{
			memset(&route, 0, sizeof(route));

			if (_gateway.empty() || _gateway.compare("0.0.0.0")==0)
			{
				if (_interface.empty())
				{
					close(fd);
					return -200;
				}
				else
				{
					route.rt_dev = (char*)_interface.c_str();
				}
				
			}
			else
			{
				addr = (struct sockaddr_in*)&route.rt_gateway;
				addr->sin_family = AF_INET;
				addr->sin_addr.s_addr = inet_addr(_gateway.c_str());
			}

			addr = (struct sockaddr_in*)&route.rt_dst;
			addr->sin_family = AF_INET;
			addr->sin_addr.s_addr = inet_addr(_network.c_str());

			addr = (struct sockaddr_in*)&route.rt_genmask;
			addr->sin_family = AF_INET;
			addr->sin_addr.s_addr = inet_addr(_netmask.c_str());
			if (_valid)
			{
				route.rt_flags = RTF_UP | RTF_GATEWAY;
			}
			else
			{
				route.rt_flags = RTF_REJECT;
			}
			route.rt_metric = _metric;
			ret = ioctl(fd, SIOCADDRT, &route);
			//perror("ConfigureIpDefaultGateway");
			close(fd);
		}
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int ConfigureIpDefaultGateway(const std::string& _gateway, uint32_t _metric)
	{
		int ret = 0;
#ifdef LINUX
		struct rtentry route;
		struct sockaddr_in *addr;

		int fd = socket(AF_INET, SOCK_DGRAM,0);

		if (fd > 0)
		{
			memset(&route, 0, sizeof(route));
			addr = (struct sockaddr_in*)&route.rt_gateway;
			addr->sin_family=AF_INET;
			addr->sin_addr.s_addr = inet_addr(_gateway.c_str());
			addr = (struct sockaddr_in*)&route.rt_dst;
			addr->sin_family = AF_INET;
			addr->sin_addr.s_addr = inet_addr("0.0.0.0");
			addr = (struct sockaddr_in*)&route.rt_genmask;
			addr->sin_family = AF_INET;
			addr->sin_addr.s_addr = inet_addr("0.0.0.0");
			route.rt_flags = RTF_UP | RTF_GATEWAY;
			route.rt_metric = _metric;
			ret = ioctl(fd, SIOCADDRT, &route);
			//perror("ConfigureIpDefaultGateway");
			close(fd);
		}

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int ConfigureMTU(const std::string& _ethname, const int _mtu)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;

		int fd=socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
		if (fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ifr.ifr_addr.sa_family=AF_INET;
			ifr.ifr_mtu = _mtu;
			ret = ioctl(fd,SIOCSIFMTU,&ifr);
			close(fd);
		}

#endif
		return ret;
	}

	//-------------------------------
	int RemoveIPGateway(const std::string& _network, const std::string& _netmask, const std::string& _gateway, const bool& _valid)
	{
		int ret = 0;
#ifdef LINUX
		struct rtentry route;
		struct sockaddr_in *addr;

		int fd = socket(AF_INET, SOCK_DGRAM,0);

		if (fd > 0)
		{
			memset(&route, 0, sizeof(route));
			addr = (struct sockaddr_in*)&route.rt_gateway;
			addr->sin_family=AF_INET;
			addr->sin_addr.s_addr = inet_addr(_gateway.c_str());
			addr = (struct sockaddr_in*)&route.rt_dst;
			addr->sin_family = AF_INET;
			addr->sin_addr.s_addr = inet_addr(_network.c_str());
			addr = (struct sockaddr_in*)&route.rt_genmask;
			addr->sin_family = AF_INET;
			addr->sin_addr.s_addr = inet_addr(_netmask.c_str());
			if (_valid)
			{
				route.rt_flags = RTF_UP | RTF_GATEWAY;
			}
			else
			{
				route.rt_flags = RTF_REJECT;
			}
			route.rt_metric = 0;
			ret = ioctl(fd, SIOCDELRT, &route);
			//perror("ConfigureIpDefaultGateway");
			close(fd);
		}

#endif
		return ret;
	}


	//-------------------------------
	int RemoveIPDefaultGateway(const std::string& _gateway)
	{
		int ret = 0;
#ifdef LINUX
		struct rtentry route;
		struct sockaddr_in *addr;

		int fd = socket(AF_INET, SOCK_DGRAM,0);

		if (fd > 0)
		{
			memset(&route, 0, sizeof(route));
			addr = (struct sockaddr_in*)&route.rt_gateway;
			addr->sin_family=AF_INET;
			addr->sin_addr.s_addr = inet_addr(_gateway.c_str());
			addr = (struct sockaddr_in*)&route.rt_dst;
			addr->sin_family = AF_INET;
			addr->sin_addr.s_addr = inet_addr("0.0.0.0");
			addr = (struct sockaddr_in*)&route.rt_genmask;
			addr->sin_family = AF_INET;
			addr->sin_addr.s_addr = inet_addr("0.0.0.0");
			route.rt_flags = RTF_GATEWAY;
			route.rt_metric = 0;
			ret = ioctl(fd, SIOCDELRT, &route);
			//perror("ConfigureIpDefaultGateway");
			close(fd);
		}

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	
	int ConfigureIpUpDown(const std::string& _ethname, bool _ipupdown)
	{
		int ret = 0;
#ifdef LINUX
		if (_ipupdown == false)
		{
			std::string sCommand = "ifconfig " + _ethname + " down";
			system(sCommand.c_str());
			return 0;
		} 

		int sockfd;
		struct ifreq ifr;

		sockfd = socket(AF_INET, SOCK_DGRAM, 0);

		if (sockfd < 0)
			return -1;

		memset(&ifr, 0, sizeof ifr);

		strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);

		ifr.ifr_flags |= IFF_UP;
		ioctl(sockfd, SIOCSIFFLAGS, &ifr);

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int ConfigureIpMulticast(const std::string& _ethname, bool _multicast)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;

		int fd = socket(AF_INET, SOCK_DGRAM, 0);

		if (fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ret = ioctl(fd, SIOCGIFFLAGS, &ifr);

			if (_multicast == false)
			{
				
				ifr.ifr_flags &= ~(IFF_MULTICAST);
			}
			else
			{
				ifr.ifr_flags |= (IFF_MULTICAST);
			}
			ret = ioctl(fd, SIOCSIFFLAGS, &ifr);
			close(fd);
		}

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool GetLinkState(const std::string& _ethname)
	{
		bool ret = false;

		if (_ethname == "eth0")
		{
#ifdef LINUX
			FILE* fp = ::fopen("/sys/class/net/eth0/operstate", "r");
			if(fp !=NULL)
			{
				char read_state[4];
				memset(read_state,0,4);
				if(fscanf(fp,"%s",read_state) == 1)
				
				ret = strcmp(read_state,"up") == 0;

				fclose(fp);
			}			
#endif
		}
		else if ((_ethname == "eth1.1") || (_ethname == "eth1.2"))
		{
#ifdef LINUX
			int fd = open("/dev/ksz8863", O_RDWR);
			if (fd < 0)
			{
					printf("%s: can't open /dev/ksz8863\n", __FUNCTION__);
					ret = false;					
			} 
			else 
			{				
				unsigned char switch_reg_addr = (_ethname == "eth1.1") ? 0x1E : 0x2E;
				unsigned char switch_reg_value = _read_switch_register(fd,switch_reg_addr);			
				
				// Bit 5 = LINK GOOD				
				ret = (switch_reg_value & 0x20) ? UP : DOWN;
				
				close(fd);				
			}			
#endif
		}
		else // unbekanntes Device
		{
			printf("unknown eth-device !\n");
		}

#ifdef WIN32
		ret = UP;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool GetIPDeviceState(const std::string& _ethname)
	{
		bool ret = DOWN;
#ifdef LINUX
		struct ifreq ifr;

		int fd=socket(AF_INET, SOCK_DGRAM, 0);
		if (fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ioctl(fd, SIOCGIFFLAGS, &ifr);
	
			ret = (ifr.ifr_flags & IFF_UP);

			close(fd);
		}

#else
		ret = UP;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	cIFStatistics GetIFStatistics(const std::string& _ethname)
	{
		cIFStatistics result;
#ifdef LINUX
		string strRxBytes;
		string strTxBytes;
		string strRxPackets;
		string strTxPackets;
		string strRxErrors;
		string strTxErrors;
		char acFilename[1024];

		snprintf(acFilename, sizeof(acFilename), "/sys/class/net/%s/statistics/rx_bytes", _ethname.c_str());
		string strFilename = acFilename;
		ReadFromFile(strFilename, strRxBytes);

		snprintf(acFilename, sizeof(acFilename), "/sys/class/net/%s/statistics/tx_bytes", _ethname.c_str());
		strFilename = acFilename;
		ReadFromFile(strFilename, strTxBytes);

		snprintf(acFilename, sizeof(acFilename), "/sys/class/net/%s/statistics/rx_packets", _ethname.c_str());
		strFilename = acFilename;
		ReadFromFile(strFilename, strRxPackets);

		snprintf(acFilename, sizeof(acFilename), "/sys/class/net/%s/statistics/tx_packets", _ethname.c_str());
		strFilename = acFilename;
		ReadFromFile(strFilename, strTxPackets);

		snprintf(acFilename, sizeof(acFilename), "/sys/class/net/%s/statistics/rx_errors", _ethname.c_str());
		strFilename = acFilename;
		ReadFromFile(strFilename, strRxErrors);

		snprintf(acFilename, sizeof(acFilename), "/sys/class/net/%s/statistics/tx_errors", _ethname.c_str());
		strFilename = acFilename;
		ReadFromFile(strFilename, strTxErrors);

		result.m_rxBytes = strtoul(strRxBytes.c_str(), NULL, 10);
		result.m_txBytes = strtoul(strTxBytes.c_str(), NULL, 10);
		result.m_rxPackets = strtoul(strRxPackets.c_str(), NULL, 10);
		result.m_txPackets = strtoul(strTxPackets.c_str(), NULL, 10);
		result.m_rxErrors = strtoul(strRxErrors.c_str(), NULL, 10);
		result.m_txErrors = strtoul(strTxErrors.c_str(), NULL, 10);
#endif
		return result;
	}

	//-----------------------------------------------------------------------------------------------------------

	int GetIpv4Addr(const std::string& _ethname, std::string& _ipv4addr)
	{
		int ret = 0;

#ifdef LINUX
		struct ifreq ifr;

		int fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);

		if (fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ret = ioctl(fd,SIOCGIFADDR,&ifr);
			struct sockaddr_in* addr = (struct sockaddr_in*)&ifr.ifr_addr;
			_ipv4addr = inet_ntoa(addr->sin_addr);	
			close(fd);
		}

#else
		_ipv4addr = "192.168.168.168";
#endif

		return ret;
	}
	
	//-----------------------------------------------------------------------------------------------------------

	int GetNetmask(const std::string& _ethname, std::string& _netmask)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;

		int fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
		if (fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ret = ioctl(fd,SIOCGIFNETMASK,&ifr);
			struct sockaddr_in* addr = (struct sockaddr_in*)&ifr.ifr_addr;
			_netmask = inet_ntoa(addr->sin_addr);
			close(fd);
		}

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int GetBroadcast(const std::string& _ethname, std::string& _broadcast)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;

		int fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
		if (fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ret = ioctl(fd,SIOCGIFBRDADDR,&ifr);
			struct sockaddr_in* addr = (struct sockaddr_in*)&ifr.ifr_addr;
			_broadcast = inet_ntoa(addr->sin_addr);
			close(fd);
		}

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int GetRoute(const std::string& _dst, std::string& _gateway, std::string& _ethname, std::string& _src)
	{
		int ret = 0;
#ifdef LINUX

		_gateway = "";
		_ethname = "";
		_src = "";

		int                     fd = 0;
		struct sockaddr_nl      local;
		uint32_t                seq;
		socklen_t               addr_len;
		struct rtattr*          rta;
		int                     status;
		struct nlmsghdr*        h;
		struct sockaddr_nl      nladdr;

		struct {
			struct nlmsghdr n;
			struct rtmsg    r;
			char            buf[1024];
		} req;

		memset(&req, 0, sizeof(req));
		req.n.nlmsg_len = NLMSG_LENGTH(sizeof(struct rtmsg));
		req.n.nlmsg_flags = NLM_F_REQUEST;
		req.n.nlmsg_type = RTM_GETROUTE;
		req.r.rtm_family = AF_INET;
		req.r.rtm_dst_len = 32;

		in_addr_t ip = inet_addr(_dst.c_str());

		rta = (struct rtattr*)(((char*)&req.n) + NLMSG_ALIGN(req.n.nlmsg_len));
		rta->rta_type = RTA_DST;
		rta->rta_len = RTA_LENGTH(4);;
		memcpy(RTA_DATA(rta), (char*) &ip, 4);
		req.n.nlmsg_len = NLMSG_ALIGN(req.n.nlmsg_len) + RTA_LENGTH(4);

		fd = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
		local.nl_family = AF_NETLINK;
		bind(fd, (struct sockaddr*)&local, sizeof(local));
		addr_len = sizeof(local);
		getsockname(fd, (struct sockaddr*)&local, &addr_len);

		seq = time(NULL);

		struct iovec iov = { &req.n, req.n.nlmsg_len };
		char   *buf = (char*)malloc(8 * 1024);
		struct msghdr msg = {
			(void*)&nladdr, sizeof(nladdr),
			&iov, 1,
			NULL, 0,
			0
		};

		memset(&nladdr, 0, sizeof(nladdr));
		nladdr.nl_family = AF_NETLINK;

		req.n.nlmsg_seq = ++seq;
		status = sendmsg(fd, &msg, 0);

		if (status < 0) {
			free(buf);
			return -1;
		}

		iov.iov_base = buf;

		int done = 0;
		while (!done) {
			iov.iov_len = 8 * 1024;
			status = recvmsg(fd, &msg, 0);

			if (status < 0) {
				continue;
			}
			if (status == 0) {
				free(buf);
				return -1;
			}
			if (msg.msg_namelen != sizeof(nladdr)) {
				free(buf);
				return -1;
			}
			for (h = (struct nlmsghdr*)buf; status >= (int)sizeof(*h) && !done;) {
				int len = h->nlmsg_len;
				int l = len - sizeof(*h);

				if (l < 0 || len > status) {
					free(buf);
					return -1;
				}

				if (h->nlmsg_type == NLMSG_ERROR) {
					struct nlmsgerr *err = (struct nlmsgerr*)NLMSG_DATA(h);
					if (l >= (int)sizeof(struct nlmsgerr)) {
						int errn = -err->error;
						if (errn == 0) {
							if (&req.n) {
								memcpy(&req.n, h, h->nlmsg_len);
							}
							done = 1;
							free(buf);
						}
					}
					free(buf);
					return -1;
				}
				else
				{
					if (&req.n) {
						memcpy(&req.n, h, h->nlmsg_len);
						done = 1;
						free(buf);
					}
					else
					{
						status -= NLMSG_ALIGN(len);
						h = (struct nlmsghdr*)((char*)h + NLMSG_ALIGN(len));
					}
				}
			}
			if (msg.msg_flags & MSG_TRUNC) {
				continue;
			}
			if (status && !done) {
				free(buf);
				return -1;
			}
		}

		close(fd);

		struct rtmsg *r = (struct rtmsg *) NLMSG_DATA(&req.n);
		struct rtattr *tb[RTA_MAX + 1];
		char abuf[256];
		memset(tb, 0, sizeof(tb));

		int rt_len = req.n.nlmsg_len;
		rt_len -= NLMSG_LENGTH(sizeof(*r));
		rta = RTM_RTA(r);
		while (RTA_OK(rta, rt_len)) {
			if (rta->rta_type <= RTA_MAX) {
				tb[rta->rta_type] = rta;
			}
			rta = RTA_NEXT(rta, rt_len);
		}

		if (tb[RTA_GATEWAY]) {
			_gateway = inet_ntop(r->rtm_family,	RTA_DATA(tb[RTA_GATEWAY]), abuf, sizeof(abuf));
		}

		if (tb[RTA_OIF]) {
			struct ifreq ifr;
			memset(&ifr, 0, sizeof(ifr));
			ifr.ifr_ifindex = *(int*)RTA_DATA(tb[RTA_OIF]);

			// create socket
			fd = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
			if (fd == -1) {
				return fd;
			}

			// get index 
			if (ioctl(fd, SIOCGIFNAME, &ifr) != -1)
			{
				_ethname = ifr.ifr_name;
			}

			close(fd);
		}

		if (tb[RTA_PREFSRC]) {
			_src = inet_ntop(r->rtm_family,	RTA_DATA(tb[RTA_PREFSRC]), abuf, sizeof(abuf));
		}

		return ret;

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int SetDNS(const std::list<std::string>& _dns, uint32_t _SetDnsMode)
	{

		FILE* pFile = fopen(RESOLV_FILE, _SetDnsMode == SET_DNS_MODE_OVERWRITE ? "w" : "a");
		if (pFile)
		{
			std::list<std::string>::const_iterator it;
			for (it = _dns.begin(); it != _dns.end(); it++)
			{
				fprintf(pFile, "nameserver %s\n", (*it).c_str());
			}
			fclose(pFile);
			return 0;
		}
		return -1;
	}

	//-----------------------------------------------------------------------------------------------------------

	int GetIpGateway(const std::string& _ethname, std::string& _gateway)
	{
		int ret = 0;

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int TriggerSysWD()
	{
		int ret = 0;
#ifdef LINUX
		int fd = ::open("/dev/watchdog", O_RDWR);
		if(fd > 0) 
		{
			::write(fd, "1", 2);	// Watchdog triggern
			::close(fd);
		} 
		else 
		{
			ret = -1;
		}
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	bool GetPowerInd(void)
	{
		bool ret = false;
#ifdef LINUX
		char value;
		FILE *fp=::fopen(POWER_IND, "r");
		if(fp != NULL)
		{
			fscanf(fp,"%c",&value);
			ret = (value == 0x31);
			fclose(fp);
		}
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------
	bool GetGSMRing(void)
	{
		bool ret = false;
#ifdef LINUX
		char value;
		FILE *fp=::fopen(RING_GSM, "r");
		if(fp != NULL)
		{
			fscanf(fp,"%c",&value);
			ret = value == 1;
			fclose(fp);
		}
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------
	bool GetInputPort(void)
	{
		bool ret = false;
#ifdef LINUX
		char value;
		FILE *fp=::fopen(IN_PORT, "r");
		if(fp != NULL)
		{
			fscanf(fp,"%c",&value);
			ret = value == 1;
			fclose(fp);
		}
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------
	bool GetServicePort(void)
	{
		bool ret = false;
#ifdef LINUX
		char value;
		FILE *fp=::fopen(SERVICE, "r");
		if(fp != NULL)
		{
			fscanf(fp,"%c",&value);
			ret = value == 1;
			fclose(fp);
		}
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------
	bool SetOutputPort(bool onoff)
	{
#ifdef LINUX
		bool bRet = false;
		FILE* pFile = fopen(OUT_PORT, "w");
		if (pFile != NULL)
		{
			bRet = fprintf(pFile, onoff?"SIG_ON":"SIG_OFF")>0;
			fclose(pFile);
		}
		else
		{
			bRet = false;
		}
		return bRet;
#else
		return true;
#endif
	}
	//-----------------------------------------------------------------------------------------------------------
	bool PowerSwitch(bool onoff)
	{
#ifdef LINUX
		bool bRet = false;
		FILE* pFile = fopen(SWITCH_ON, "w");
		if (pFile != NULL)
		{
			bRet = fprintf(pFile, onoff?"SIG_ON":"SIG_OFF")>0;
			fclose(pFile);
		}
		else
		{
			bRet = false;
		}
		return bRet;
#else
		return true;
#endif
	}

	//-----------------------------------------------------------------------------------------------------------
	bool GSMIgnition(bool bIgnition)
	{
		bool ret = false;
#ifdef LINUX
		int fd_gsmIgn = ::open(GSM_IGT, O_RDWR);
		if (fd_gsmIgn < 0)
		{
			ret = false;			
		} 
		else 
		{
			if (bIgnition == ON)
			{
				::write(fd_gsmIgn, "1", 1);	// Ignition = high 
			} 
			else 
			{
				::write(fd_gsmIgn, "0", 1);	// Ignition = low -> Module boots
			}
		}
		close(fd_gsmIgn);
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------
	bool SelSIM(bool bSlot)
	{
		bool ret = false;
#ifdef LINUX
		int fd_selsim = ::open(SEL_SIM, O_RDWR);
		if (fd_selsim < 0)
		{
			ret = false;			
		} 
		else 
		{
			if (bSlot == 1)
			{
				::write(fd_selsim, "1", 1);	// SIM1
			} 
			else 
			{
				::write(fd_selsim, "0", 1);	// SIM2
			}
		}
		close(fd_selsim);
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------
	bool PowerGSM(bool onoff)
	{
		bool ret = true;
#ifdef LINUX
		int fd_gsmOn = ::open(GSM_ON, O_RDWR);
		//int fd_PokIn = ::open("/sys/class/gpio/gpio125/value", O_RDWR);
		//if ((fd_gsmOn<0) || (fd_PokIn<0))
		if (fd_gsmOn<0)
		{
			ret = false;			
		} 
		else 
		{
			if (onoff == ON)
			{
				::write(fd_gsmOn, "1", 1);	// VBATT am GSM Modul zuschalten
				//::write(fd_PokIn, "0", 1);	// GSM Modul starten
			} 
			else 
			{
				::write(fd_gsmOn, "0", 1);	// VBATT am GSM Modul ausschalten
			}
		}
		close(fd_gsmOn);
		//close(fd_PokIn);
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------
	bool GSMEmergency(bool onoff)
	{
#ifdef LINUX
		bool bRet = false;
		FILE* pFile = fopen(EMERGENCY, "w");
		if (pFile != NULL)
		{
			bRet = fprintf(pFile, onoff?"SIG_ON":"SIG_OFF")>0;
			fclose(pFile);
		}
		else
		{
			bRet = false;
		}
		return bRet;
#else
		return true;
#endif
	}

	//-----------------------------------------------------------------------------------------------------------

	bool PowerWMBUS(bool onoff)
	{
		bool ret = true;
#ifdef LINUX
		// Deprecated
		//int fd = ::open("/sys/class/gpio/gpio111/value", O_RDWR);
		//if (fd<0)
		//{
		//	ret = false;
		//} 
		//else 
		//{
		//	if (onoff == ON)
		//	{
		//		::write(fd, "0", 1);	// Versorgungsspannung am WMBUS-Modul zuschalten (default=ON)
		//	} 
		//	else 
		//	{
		//		::write(fd, "1", 1);	// Versorgungsspannung am WMBUS-Modul abschalten
		//	}
		//	::close(fd);
		//}
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool ResetWMBUS()
	{
		bool ret = false;
#ifdef LINUX		 
		int fd = ::open("/sys/class/gpio/gpio126/value", O_RDWR);
		if (fd>0)
		{
			::write(fd, "1", 1);	// Reset auslsen
			usleep(500000);
			::write(fd, "0", 1);	// Reset aufheben
			::close(fd);
			ret = true;
		}	
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool ResetGSM()
	{
		bool ret = false;
		
#ifdef LINUX		 
		int fd = ::open("/sys/class/gpio/gpio76/value", O_RDWR);
		if (fd>0)
		{
			::write(fd, "1", 1);	// Reset auslsen
			usleep(300000);
			::write(fd, "0", 1);	// Reset aufheben
			::close(fd);
			ret = true;
		}	
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool PowerMBUS12V(bool onoff)
	{
		bool ret = false;
#ifdef LINUX
		int fd = ::open("/sys/class/gpio/gpio67/value", O_RDWR);
		if (fd<0)
		{
			ret = false;
		} 
		else 
		{
			// +12V am Meter-Bus zuschalten
			// +12V am Meter-Bus abschalten
			::write(fd, (onoff ? "1" : "0"), 1);
			
			::close(fd);
			ret = true;
		}
#endif
		return ret;
	}	

	//-----------------------------------------------------------------------------------------------------------

	bool PowerSwitchPort(unsigned int portnr, bool onoff)
	{
		bool ret = false;
#ifdef LINUX
		unsigned char addr;
		unsigned char value;
		int fd;

		fd = open("/dev/ksz8863", O_RDWR);
		
		if (fd < 0)
		{		
			return ret;			
		}
		else
		{
			switch (portnr)
			{
			case 1:	addr = 0x1D;			
				break;
			case 2: addr = 0x2D;			
				break;
			default: return ret;			
				break;
			}

			value = _read_switch_register(fd, addr);

			if (onoff == OFF)
			{
				value |= 0x08;			// Das Setzen von Bit 3 schaltet den Port ab 
			}
			else
			{
				value &= ~(0x08);		// Das Lschen von Bit 3 schaltet den Port an
			}
		
			if (_write_switch_register(fd, addr, value) >= 0)
			{
				ret = true;			
			}
			close (fd);
		}
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool SetDTR()
	{
		bool ret = true;
#ifdef LINUX
		int fd = ::open(DTR_ON, O_RDWR);
		if (fd<0)
		{
			ret = false;
		} 
		else 
		{
			::write(fd, "1", 1);			
			::close(fd);
			ret = true;
		}
#endif
		return ret;
	}	

	bool ResetDTR()
	{
		bool ret = true;
#ifdef LINUX
		int fd = ::open(DTR_ON, O_RDWR);
		if (fd<0)
		{
			ret = false;
		} 
		else 
		{
			::write(fd, "0", 1);			
			::close(fd);
			ret = true;
		}
#endif
		return ret;
	}	

	//-----------------------------------------------------------------------------------------------------------

	bool GetSysMemory(uint64_t& _total_KB, uint64_t& _avail_KB, uint64_t& _process_KB)
	{	
		bool ret = false;

#ifdef iMX28
		FILE* fp = ::fopen( "/proc/self/statm", "r" );
		if(fp != NULL)
		{
			// 481 209 158 15 0 90 0
			// parsen der ersten 2 Zahlen 481 209						
			// erste Zahl - Gesamtgre des Programmes
			// zweite Zahl - im RAM befindlicher Teil des Programmes - RSS - Resident Set Size
			// die Angabe der Zahlen ist in Pages, so dass diese mit der Pagesize multipliziert wird
			// das Ergebnis ist ein Wert in Bytes
			// die Schnittstelle ist in KBytes, so dass durch 1024 dividiert == >> 10 wird
			long int tmp_process_KB[2]; 					
			if(fscanf(fp,"%ld %ld",tmp_process_KB,tmp_process_KB+1) == 2)
			{
				_process_KB = tmp_process_KB[1] * ::sysconf(_SC_PAGESIZE) >> 10;
				
				struct sysinfo si; 
				ret=::sysinfo(&si) == 0;

				if(ret)
				{
					_total_KB = si.totalram >> 10;
					_avail_KB = si.freeram >> 10;
				}
			}
			/*
			char line[128];
			if(::fgets(line, sizeof(line)-1, fp) != NULL)
			{
				string sline(line);
				vector<string> tok = GetStringTokens(sline, " ");
				if(tok.size() > 1)
				{
					_process_KB = (size_t)atol(tok[1].c_str()) * (size_t)::sysconf(_SC_PAGESIZE) / 1024;
					ret = true;
				}
			}
			*/
			::fclose(fp);
		}



		
#endif

#ifdef WIN32
		MEMORYSTATUSEX statex;
		statex.dwLength = sizeof (statex);

		ret = ::GlobalMemoryStatusEx(&statex)==TRUE;
		if(ret)
		{
			_total_KB = statex.ullTotalPhys >> 10;
			_avail_KB = statex.ullAvailPhys >> 10;
		}

		PROCESS_MEMORY_COUNTERS info;
		::GetProcessMemoryInfo(GetCurrentProcess(), &info, sizeof(info));
		_process_KB = info.WorkingSetSize >> 10;
#endif		

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int IPTablesBlockInputDevice(const std::string& _ethname)
	{			
		int ret = 0;
		const char* const arguments[]= { "iptables","-I","INPUT","1","-i",_ethname.c_str(),"-j","DROP",NULL };		
				
#ifdef LINUX
		pid_t child_pid;
		child_pid=fork();

		if (child_pid==0)
		{			
			execv ("/usr/sbin/iptables", const_cast<char* const*>(arguments));
			// sollte execv hierher zurckkehren, ist ein Fehler aufgetreten
			ret = -1;
		}
#endif	
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int IPTablesOpenPort(const std::string& _device, const std::string& _port, const std::string& _protokoll)
	{
		int ret = 0;		

		//string iptCommand = "";
		string iptCommand = "/usr/sbin/iptables -I INPUT 1 -p " + _protokoll + " --dport " + _port + " -j ACCEPT";

		if(_device != "any") iptCommand +=  " -i " + _device;
		/*
		if (_device=="any")
		{			
			//iptables -I INPUT 1 -i %_device% -p %_protokoll% --dport %_port% -j ACCEPT
			//iptCommand = "/usr/sbin/iptables -I INPUT 1 -p " + _protokoll + " --dport " + _port + " -j ACCEPT";
		}
		else
		{
			//iptables -I INPUT 1 -i %_device% -p %_protokoll% --dport %_port% -j ACCEPT
			//iptCommand = "/usr/sbin/iptables -I INPUT 1 -i " + _device + " -p " + _protokoll + " --dport " + _port + " -j ACCEPT";
		}
		*/
			
#ifdef LINUX		
		ret = system(iptCommand.c_str());
#endif		
		
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int IPTablesClosePort(const std::string& _device, const std::string& _port, const std::string& _protokoll)
	{
		int ret = 0;	

		//string iptCommand = "";

		string iptCommand = "/usr/sbin/iptables -D INPUT -p " + _protokoll + " --dport " + _port + " -j ACCEPT";

		if(_device != "any") iptCommand +=  " -i " + _device;

		/*
		if (_device == "any")
		{
			// iptables -D INPUT -p tcp --dport %_port% -j ACCEPT
			iptCommand = "/usr/sbin/iptables -D INPUT -p " + _protokoll + " --dport " + _port + " -j ACCEPT";
		}
		else
		{
			// iptables -D INPUT -i eth0 -p tcp --dport %_port% -j ACCEPT
			iptCommand = "/usr/sbin/iptables -D INPUT -i " + _device + " -p " + _protokoll + " --dport " + _port + " -j ACCEPT";
		}
		*/
				
#ifdef LINUX		
		ret = system(iptCommand.c_str());
#endif
		
		return ret;
	}	

	//-----------------------------------------------------------------------------------------------------------

//	int IPTablesOpenPort(const std::string& _ipv4addr, unsigned short _port)
//	{
//		int ret = 0;
//		char _port_cstring[6];
//		
//		sprintf(_port_cstring,"%d",_port);
//		
//		char* arguments[]= { (char*)"iptables",(char*)"-I",(char*)"INPUT",(char*)"1",
//			(char*)"-d",(char*)_ipv4addr.c_str(),
//			(char*)"-p", (char*)"tcp",
//			(char*)"--dport",(char*)_port_cstring,(char*)"-j",(char*)"ACCEPT",NULL };
//	
//#ifdef LINUX
//		pid_t child_pid;
//		child_pid=fork();
//
//		if (child_pid==0)
//		{		
//			execv ("/usr/sbin/iptables", arguments);
//			// sollte execv hierher zurckkehren, ist ein Fehler aufgetreten
//			ret = -1;
//		}
//				
//		arguments[7] = (char*)"udp";
//
//		child_pid=fork();
//
//		if (child_pid==0)
//		{		
//			execv ("/usr/sbin/iptables", arguments);
//			// sollte execv hierher zurckkehren, ist ein Fehler aufgetreten
//			ret = -1;
//		}
//#endif		
//
//		return ret;
//	}

	//-----------------------------------------------------------------------------------------------------------

	

//	int IPTablesClosePort(const std::string& _ipv4addr, unsigned short _port)
//	{
//		int ret = 0;
//		char _port_cstring[6];
//		
//		sprintf(_port_cstring,"%d",_port);
//		
//		char* arguments[]= { (char*)"iptables",(char*)"-D",(char*)"INPUT",
//			(char*)"-d",(char*)_ipv4addr.c_str(),(char*)"-p",(char*)"tcp",
//			(char*)"--dport", (char*)_port_cstring,(char*)"-j",(char*)"ACCEPT",NULL };
//		
//#ifdef LINUX
//		pid_t child_pid;
//		child_pid=fork();
//
//		if (child_pid==0)
//		{
//			execv ("/usr/sbin/iptables", arguments);
//			// sollte execv hierher zurckkehren, ist ein Fehler aufgetreten
//			ret = -1;
//		}
//
//		arguments[6] = (char*)"udp";
//
//		child_pid=fork();
//
//		if (child_pid==0)
//		{		
//			execv ("/usr/sbin/iptables", arguments);
//			// sollte execv hierher zurckkehren, ist ein Fehler aufgetreten
//			ret = -1;
//		}
//#endif
//
//		return ret;
//	}

	//-----------------------------------------------------------------------------------------------------------

	int IPTablesFlush()
	{			
		int ret = 0;
		const char* const arguments[]= { "iptables", "-F", "INPUT", NULL };
				
#ifdef LINUX
		pid_t child_pid;
		child_pid=fork();

		if (child_pid==0)
		{
			execv ("/usr/sbin/iptables", const_cast<char* const*>(arguments));

			// sollte execv hierher zurckkehren, ist ein Fehler aufgetreten
			ret = -1;
		}		
#endif
		return ret;
	}


	//-----------------------------------------------------------------------------------------------------------
	//EEPROM:GetBootConf
	//-----------------------------------------------------------------------------------------------------------
	bool GetBootConf(uint8_t& _bootByteEeprom, uint8_t& _bootByteCPU)
	{
		//  Legende der persistenten Bits (Bootkonfiguration im EEPROM, BIT0 = LSB)
		//		Bit0 0 = rootfs0, 1 = rootfs1
		//		Bit1 Updated rootfs
		//		Bit2 Updated BootStream
		//		Bit3 Update Failed
		//		Bit4 previous rootsfs updated
		//		Bit5 previous BootStream updated
		//		Bit6 BootSelector Soll		
#ifdef LINUX	
		bool bRet_EepromReadBootConf = EepromReadBootConf(_bootByteEeprom);
		if(bRet_EepromReadBootConf == false)
		{
#ifdef EEPROM_DEBUG_OUT
			printf("GetBootConf:EepromReadBootConf:Error\n");
#endif	
			return false;
		}
		else
		{
#ifdef EEPROM_DEBUG_OUT
			printf("GetBootConf:EepromReadBootConf:Success\n");
#endif	
			return true;
		}
		/*
		bool bRet_BootConfFromCPU = _readBootConfFromCPU(_bootByteCPU);
		if (bRet_BootConfFromCPU == false)
		{
#ifdef EEPROM_DEBUG_OUT
			printf("GetBootConf:BootConfFromCPU:Error\n");
#endif	
		}
		else
		{
#ifdef EEPROM_DEBUG_OUT
			printf("GetBootConf:BootConfFromCPU:Success\n");
#endif	
		}
		if ((bRet_EepromReadBootConf == false) || (!bRet_BootConfFromCPU == false))
		return false;
		else
		return true;

		*/

#endif
		return true;
	}

	//-----------------------------------------------------------------------------------------------------------
	bool _readBootConfBitFromCPU(int32_t bitnr)
	{
#ifdef LINUX
		int32_t fd;
		char buf[1];

		switch (bitnr)
		{
		case 0:	fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_0", O_RDONLY);
			break;
		case 1: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_1", O_RDONLY);
			break;
		case 2: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_2", O_RDONLY);
			break;
		case 3: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_3", O_RDONLY);
			break;
		case 4: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_4", O_RDONLY);
			break;
		case 5: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_5", O_RDONLY);
			break;
		case 6: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_6", O_RDONLY);
			break;
		case 7: fd = ::open("/sys/devices/platform/imx28_regs.0/rom_secondary_boot", O_RDONLY);
			break;
		default: return false;
			break;
		}

		::read(fd, buf, 1);
		::close(fd);

		return (buf[0] == '1' ? true : false);
#else
		return false;
#endif
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	bool _setBootConfBitToCPU(int32_t bitnr, bool _value)
	{
#ifdef LINUX
		int32_t fd;
		//char buf[1];

		switch (bitnr)
		{
		case 0:	fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_0", O_RDWR);
			break;
		case 1: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_1", O_RDWR);
			break;
		case 2: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_2", O_RDWR);
			break;
		case 3: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_3", O_RDWR);
			break;
		case 4: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_4", O_RDWR);
			break;
		case 5: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_5", O_RDWR);
			break;
		case 6: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_6", O_RDWR);
			break;
		case 7: fd = ::open("/sys/devices/platform/imx28_regs.0/rom_secondary_boot", O_RDWR);
			break;
		default: return false;
			break;
		}

		if (fd <= 0) return false;

		if (_value)
		{
			::write(fd, "1", 1);
		}
		else
		{
			::write(fd, "0", 1);
		}
		::close(fd);
#endif
		return true;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	bool _readBootConfFromCPU(uint8_t& boot_byte)
	{
		int32_t i;
		// Zugriff auf die CPU-Register mglich ? 
#ifdef LINUX
		int32_t fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_0", O_RDONLY);
		if (fd <= 0) return false;
		close(fd);
#endif
		boot_byte = 0;

		for (i = 0; i < 8; i++)
		{
			boot_byte |= (_readBootConfBitFromCPU(i) ? (0x01 << i) : 0x00);
		}

		return true;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	bool WriteBootConfToCPU(uint8_t& boot_byte)
	{
#ifdef LINUX
		int32_t i;
		// Zugriff auf die CPU-Register mglich ? 
		int32_t fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_0", O_RDWR);
		if (fd <= 0) return false;
		close(fd);

		for (i = 0; i < 8; i++)
		{
			if ((boot_byte & (0x01 << i)) > 0)
			{
				_setBootConfBitToCPU(i, HIGH);
			}
			else
			{
				_setBootConfBitToCPU(i, LOW);
			}
		}
#endif

		return true;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	//EEPROM:_eepromBootBitManipulate
	//-----------------------------------------------------------------------------------------------------------
	bool _eepromBootBitManipulate(bool setter, uint8_t bitnr)
	{
		uint8_t boot_byte_eeprom = 0;
#ifdef EEPROM_DEBUG_OUT
		printf("_eepromBootBitManipulate:Bit=%d\n", bitnr);
#endif	

		if ((!EepromReadBootConf(boot_byte_eeprom)) || (bitnr > 7))
		{
#ifdef EEPROM_DEBUG_OUT
			printf("_eepromBootBitManipulate:return false\n");
#endif	
			return false;
		}
		
		if (setter)
		{
#ifdef EEPROM_DEBUG_OUT
			printf("_eepromBootBitManipulate:setter true (Set)\n");
#endif	

			boot_byte_eeprom |= (0x01 << bitnr);
		}
		else
		{
#ifdef EEPROM_DEBUG_OUT
			printf("_eepromBootBitManipulate:setter false (Clear)\n");
#endif	

			boot_byte_eeprom &= ~(0x01 << bitnr);
		}

		bool bRet = EepromWriteBootConf(boot_byte_eeprom);
#ifdef EEPROM_DEBUG_OUT
		printf("_eepromBootBitManipulate:EepromWriteBootConf ret %d\n", (int)bRet);
#endif	

		return bRet;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	//EEPROM:EepromBootConfSetBit
	//-----------------------------------------------------------------------------------------------------------
	bool EepromBootConfSetBit(uint8_t bitnr)
	{
		return _eepromBootBitManipulate(true, bitnr);
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	//EEPROM:EepromBootConfClrBit
	//-----------------------------------------------------------------------------------------------------------
	bool EepromBootConfClrBit(uint8_t bitnr)
	{
		return _eepromBootBitManipulate(false, bitnr);
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	bool EepromReadBootConf(uint8_t& byte_bootregister)
	{
		bool ret = false;

#ifdef LINUX
		const unsigned int blocksize = 3;		// ein byte Bindaten + 2 byte CRC16

		int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDONLY);

		if (fd < 0)
		{
			return ret;
		}
		else
		{
			char rbuf[blocksize];
			::memset(rbuf, 0, blocksize);

			for (int i = 0; i < 3; i++)
			{
				::lseek(fd, BOOTCONF_ADDR, SEEK_SET);
				::read(fd, rbuf, blocksize);

				unsigned short fcs16_readed = (rbuf[blocksize - 2] << 8) | rbuf[blocksize - 1];
				unsigned short fcs16_calc = _createCRC16(rbuf, 1, 0xffff);		// Berechnung der CRC16 ber das gelesene Binrbyte
				//unsigned short fcs16_calc = _createCRC16(rbuf, blocksize, 0xffff);
				//printf("fcs16_readed: %x	fcs16_calc: %x \n",fcs16_readed, fcs16_calc);		

				if (fcs16_calc == fcs16_readed)
				{
					byte_bootregister = rbuf[0];
					ret = true;
					break;
				}
			}

			close(fd);
		}
#else
		byte_bootregister = 0x00;
		return true;
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	bool EepromWriteBootConf(const uint8_t byte_bootregister)
	{
		bool ret = false;
#ifdef LINUX
		const unsigned int blocksize = 3;	// ein byte Bindaten + 2 byte CRC16
		int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDWR);

		if (fd < 0)
		{
			return ret;
		}
		else
		{
			char wbuf[blocksize];
			char rbuf[blocksize];
			::memset(wbuf, 0, blocksize);
			::memset(rbuf, 0, blocksize);

			wbuf[0] = byte_bootregister;
			unsigned short fcs16_calc = _createCRC16(wbuf, 1, 0xffff);	// Berechnung der CRC16 ber das Binrbyte	

			wbuf[blocksize - 2] = (fcs16_calc >> 8) & 0x00FF;		// Eintragen der CRC16 in den Schreibpuffer
			wbuf[blocksize - 1] = (fcs16_calc)& 0x00FF;

			// 3x retry Buffer schreiben und anschliessend verifizieren		
			for (int i = 0; i<3; i++)
			{
				::lseek(fd, BOOTCONF_ADDR, SEEK_SET);				// Deskriptor auf Position _addr setzen			

				// Schreiben des gesamten Buffers schlgt in manchen Fllen fehl, deshalb byteweise schreiben			
				for (unsigned int j = 0; j < blocksize; j++) ::write(fd, wbuf + j, 1);
				::lseek(fd, BOOTCONF_ADDR, SEEK_SET);				// Deskriptor zurcksetzen
				::read(fd, rbuf, blocksize);				// Block lesen			
				if (memcmp(wbuf, rbuf, blocksize) == 0)
				{
					ret = true;
					break;
				}
				/*else
				{
				printf("Writeerror !\n");
				}*/
			}
			close(fd);
		}
#else
		return true;
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------


	//-----------------------------------------------------------------------------------------------------------
	//EEPROM Functions
	//Use EEPROM_DEBUG_OUT for Debug
	//-----------------------------------------------------------------------------------------------------------
	bool EepromCreateSignature()
	{
		bool ret = false;
#ifdef LINUX
		std::map<std::string, std::string> data;
		std::map<std::string, std::string>::iterator it;

		ret = hal::EepromRead(data);

		if (data.size() <= 0)
			return false;

		string sName = "";
		string sValue = "";
		string sEEPROM_Value_String = "";
		for(it = data.begin(); it != data.end(); it++)
		{
			sName = (*it).first;
			sValue = (*it).second;
#ifdef EEPROM_DEBUG_OUT
			printf("EepromCreateSignature:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	
			if (sName.compare("ver") == 0)
			{
				sEEPROM_Value_String.append(sValue);
#ifdef EEPROM_DEBUG_OUT
				printf("Append ver:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	

			}
			if (sName.compare("hwid") == 0)
			{
				sEEPROM_Value_String.append(sValue);
#ifdef EEPROM_DEBUG_OUT
				printf("Append hwid:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	

			}
			if (sName.compare("hwver") == 0)
			{
				sEEPROM_Value_String.append(sValue);
#ifdef EEPROM_DEBUG_OUT
				printf("Append hwver:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	

			}
			if (sName.compare("mac0") == 0)
			{
				sEEPROM_Value_String.append(sValue);
#ifdef EEPROM_DEBUG_OUT
				printf("Append mac0:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	

			}
			if (sName.compare("mac1") == 0)
			{
				sEEPROM_Value_String.append(sValue);
#ifdef EEPROM_DEBUG_OUT
				printf("Append mac1:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	

			}
			if (sName.compare("mac2") == 0)
			{
				sEEPROM_Value_String.append(sValue);
#ifdef EEPROM_DEBUG_OUT
				printf("Append mac2:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	

			}
			if (sName.compare("mac3") == 0)
			{
				sEEPROM_Value_String.append(sValue);
#ifdef EEPROM_DEBUG_OUT
				printf("Append mac3:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	

			}
			if (sName.compare("mac4") == 0)
			{
				sEEPROM_Value_String.append(sValue);
#ifdef EEPROM_DEBUG_OUT
				printf("Append mac4:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	

			}
			if (sName.compare("mac5") == 0)
			{
				sEEPROM_Value_String.append(sValue);
#ifdef EEPROM_DEBUG_OUT
				printf("Append mac5:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	

			}
			if (sName.compare("sn") == 0)
			{
				sEEPROM_Value_String.append(sValue);
#ifdef EEPROM_DEBUG_OUT
				printf("Append sn:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	

			}
			if (sName.compare("devid") == 0)
			{
				sEEPROM_Value_String.append(sValue);
#ifdef EEPROM_DEBUG_OUT
				printf("Append devid:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	

			}
			if (sName.compare("pdate") == 0)
			{
				sEEPROM_Value_String.append(sValue);
#ifdef EEPROM_DEBUG_OUT
				printf("Append pdate:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
				printf("eeprom.string:%s\n", (char*)sEEPROM_Value_String.c_str() );
#endif	
			}
			if (sName.compare("oem") == 0)
			{
				sEEPROM_Value_String.append(sValue);
#ifdef EEPROM_DEBUG_OUT
				printf("Append oem:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	

			}
		}
		
#ifdef EEPROM_DEBUG_OUT
		printf("EepromCreateSignature:eeprom.string:%s\n", (char*)sEEPROM_Value_String.c_str() );
#endif	

		string sSystemCommand = "";
		string sPathAndNameEEPRON(EEPROM_WORK_PATH);
		sPathAndNameEEPRON.append("*");
		sSystemCommand.append("chmod -R 777 ");
		sSystemCommand.append(sPathAndNameEEPRON);

#ifdef EEPROM_DEBUG_OUT
		printf("EepromCreateSignature:sSystemCommand:%s\n", (char*)sSystemCommand.c_str());
#endif	
		system((char*)sSystemCommand.c_str());

		sSystemCommand = "";
		string sPathAndNameEEPRONString(EEPROM_WORK_PATH);
		sPathAndNameEEPRONString.append("eeprom.string");
		//Remove eeprom.string if exist
		sSystemCommand.append("rm ");
		sSystemCommand.append(sPathAndNameEEPRONString);
#ifdef EEPROM_DEBUG_OUT
		printf("EepromCreateSignature:sSystemCommand:%s\n", (char*)sSystemCommand.c_str());
#endif	
		system((char*)sSystemCommand.c_str());

		sSystemCommand = "";
		sPathAndNameEEPRON = EEPROM_WORK_PATH;
		sPathAndNameEEPRON.append("eeprom.sig");
		sSystemCommand.append("rm ");
		sSystemCommand.append(sPathAndNameEEPRON);
#ifdef EEPROM_DEBUG_OUT
		printf("EepromCreateSignature:sSystemCommand:%s\n", (char*)sSystemCommand.c_str());
#endif	
		system((char*)sSystemCommand.c_str());

		sSystemCommand = "";
		sPathAndNameEEPRON = EEPROM_WORK_PATH;
		sPathAndNameEEPRON.append("eeprom.sig.64");
		sSystemCommand.append("rm ");
		sSystemCommand.append(sPathAndNameEEPRON);
#ifdef EEPROM_DEBUG_OUT
		printf("EepromCreateSignature:sSystemCommand:%s\n", (char*)sSystemCommand.c_str());
#endif	
		system((char*)sSystemCommand.c_str());

		sSystemCommand = "";
		sPathAndNameEEPRON = EEPROM_WORK_PATH;
		sPathAndNameEEPRON.append("verifyout.txt");
		sSystemCommand.append("rm ");
		sSystemCommand.append(sPathAndNameEEPRON);
#ifdef EEPROM_DEBUG_OUT
		printf("EepromCreateSignature:sSystemCommand:%s\n", (char*)sSystemCommand.c_str());
#endif	
		system((char*)sSystemCommand.c_str());

		//Write EEPROM Values to eeprom.string File
		int iRet = WriteToFile(sPathAndNameEEPRONString, sEEPROM_Value_String, 1);
		if (iRet != 0)
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromCreateSignature:Write File: Error\n");
#endif	
			return false;
		}
		else
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromCreateSignature:Write File: Success\n");
#endif	
		}

		//Create the Signature, using the private RSA key, using openssl command
		//Do this:
		//openssl dgst -sha1 -sign /etc/eep_key/rsa_eep_private.pem -out /etc/eep_key/eeprom.sig /etc/eep_key/eeprom.string 
		string sPathAndNamePrivateKey(EEPROM_WORK_PATH);
		sPathAndNamePrivateKey.append("rsa_eep_private.pem");

		string sOpensslCommand = "openssl dgst -sha1 -sign ";
		sOpensslCommand.append(sPathAndNamePrivateKey);
		sOpensslCommand.append(" -out ");

		string sPathAndNameOutSig(EEPROM_WORK_PATH);
		sPathAndNameOutSig.append("eeprom.sig ");

		sOpensslCommand.append(sPathAndNameOutSig);
		sOpensslCommand.append(" ");
		sOpensslCommand.append(sPathAndNameEEPRONString);

#ifdef EEPROM_DEBUG_OUT
		printf("EepromCreateSignature:sOpensslCommand:%s\n", (char*)sOpensslCommand.c_str());
#endif	
		ret = system((char*)sOpensslCommand.c_str());
		if (ret == 0)
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromCreateSignature:sOpensslCommand: Success\n");
#endif	
		}
		else
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromCreateSignature:sOpensslCommand: Error\n");
#endif	
			return false;
		}

		//Create base64 File
		//base64 --wrap=0 /etc/eep_key/eeprom.sig > /etc/eep_key/eeprom.sig.64
		sOpensslCommand = "base64 --wrap=0 ";
		sOpensslCommand.append(sPathAndNameOutSig);
		sOpensslCommand.append(" > ");
		sOpensslCommand.append(EEPROM_WORK_PATH);
		sOpensslCommand.append("eeprom.sig.64");

#ifdef EEPROM_DEBUG_OUT
		printf("EepromCreateSignature:Command base64:%s\n", (char*)sOpensslCommand.c_str());
#endif	
		ret = system((char*)sOpensslCommand.c_str());
		if (ret == 0)
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromCreateSignature:Command base64: Success\n");
#endif	
		}
		else
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromCreateSignature:Command base64: Error\n");
#endif	
			return false;
		}

		//Read Content back from sig base64 file
		sOpensslCommand = EEPROM_WORK_PATH;
		sOpensslCommand.append("eeprom.sig.64");
		string sFileContentBase64 = "";

		int iRetFromFile = ReadFromFile(sOpensslCommand, sFileContentBase64, 1024);
		if (iRetFromFile == 0)
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromCreateSignature:ReadFromFile: Success\n");
#endif	
			if (sFileContentBase64.size() <= 0)
			{
#ifdef EEPROM_DEBUG_OUT
				printf("EepromCreateSignature:ReadFromFile base64: Error, length=0\n");
#endif	
				return false;
			}
			else
			{
#ifdef EEPROM_DEBUG_OUT
				printf("EepromCreateSignature:ReadFromFile base64: Success, %s\n", (char*)sFileContentBase64.c_str());
#endif	
				std::map<std::string, std::string> data;
				data["sig"] = sFileContentBase64;
				bool bRet = hal::EepromWrite(data);
				if (bRet == false)
				{		
#ifdef EEPROM_DEBUG_OUT
					printf("EepromCreateSignature:Error, unable to write EEPROM Signature\n");
#endif	
					return false;
				}
				else
				{		
#ifdef EEPROM_DEBUG_OUT
					printf("EepromCreateSignature: Signature Success\n");
#endif	
					return true;
				}
			}
		}
		else
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromCreateSignature:ReadFromFile: Error\n");
#endif	
			return false;
		}
#endif
		return ret;//Windows
	}
	//------------------------------------------------------------------------------------------------------------------------
	bool EepromVerifySignature()
	{
		bool ret = false;
#ifdef LINUX
		string sSystemCommand = "";
		string sPathAndName(EEPROM_WORK_PATH);
		sPathAndName.append("*");
		sSystemCommand.append("chmod -R 777 ");
		sSystemCommand.append(sPathAndName);
		system((char*)sSystemCommand.c_str());

		sSystemCommand = "";
		sPathAndName = EEPROM_WORK_PATH;
		sPathAndName.append("eepromver.string");
		sSystemCommand.append("rm ");
		sSystemCommand.append(sPathAndName);
		system((char*)sSystemCommand.c_str());

		sSystemCommand = "";
		sPathAndName = EEPROM_WORK_PATH;
		sPathAndName.append("eepromver.sig");
		sSystemCommand.append("rm ");
		sSystemCommand.append(sPathAndName);
		system((char*)sSystemCommand.c_str());

		sSystemCommand = "";
		sPathAndName = EEPROM_WORK_PATH;
		sPathAndName.append("verifyout.txt");
		sSystemCommand.append("rm ");
		sSystemCommand.append(sPathAndName);
		system((char*)sSystemCommand.c_str());

		std::map<std::string, std::string> data;
		std::map<std::string, std::string>::iterator it;

		ret = hal::EepromRead(data);

		if (data.size() <= 0)
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromVerifySignature: Error EEPROM empty no Data\n");
#endif	
			return false;
		}

		string sName = "";
		string sValue = "";
		sValue = data["sig"];
#ifdef EEPROM_DEBUG_OUT
		printf("EepromVerifySignature: Value:%s\n", (char*)sValue.c_str());
#endif	
		sSystemCommand = "";
		sPathAndName = EEPROM_WORK_PATH;
		sPathAndName.append("eepromver.sig");

		//Write Signature File
		int iRet = WriteToFile(sPathAndName, sValue, 1);
		if (iRet != 0)
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromVerifySignature:Write File: Error\n");
#endif	
			return false;
		}
		else
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromVerifySignature:Write File: Success\n");
#endif	
		}

		string sEEPROM_Value_String = "";
		//Create EEPROM Value File
		for(it = data.begin(); it != data.end(); it++)
		{
			sName = (*it).first;
			sValue = (*it).second;
#ifdef EEPROM_DEBUG_OUT
			printf("EepromVerifySignature:Name:%s, Value:%s\n", (char*)sName.c_str(), (char*)sValue.c_str());
#endif	
			if (sName.compare("ver") == 0)
			{
				sEEPROM_Value_String.append(sValue);
			}
			if (sName.compare("hwid") == 0)
			{
				sEEPROM_Value_String.append(sValue);
			}
			if (sName.compare("hwver") == 0)
			{
				sEEPROM_Value_String.append(sValue);
			}
			if (sName.compare("mac0") == 0)
			{
				sEEPROM_Value_String.append(sValue);
			}
			if (sName.compare("mac1") == 0)
			{
				sEEPROM_Value_String.append(sValue);
			}
			if (sName.compare("mac2") == 0)
			{
				sEEPROM_Value_String.append(sValue);
			}
			if (sName.compare("mac3") == 0)
			{
				sEEPROM_Value_String.append(sValue);
			}
			if (sName.compare("mac4") == 0)
			{
				sEEPROM_Value_String.append(sValue);
			}
			if (sName.compare("mac5") == 0)
			{
				sEEPROM_Value_String.append(sValue);
			}
			if (sName.compare("sn") == 0)
			{
				sEEPROM_Value_String.append(sValue);
			}
			if (sName.compare("devid") == 0)
			{
				sEEPROM_Value_String.append(sValue);
			}
			if (sName.compare("pdate") == 0)
			{
				sEEPROM_Value_String.append(sValue);
			}
			if (sName.compare("oem") == 0)
			{
				sEEPROM_Value_String.append(sValue);
			}
		}

		sPathAndName = EEPROM_WORK_PATH;
		sPathAndName.append("eepromver.string");
		//Write EEPROM Values to eeprom.string File
#ifdef EEPROM_DEBUG_OUT
		printf("EepromVerifySignature:eepromver.string:%s\n", (char*)sEEPROM_Value_String.c_str() );
#endif	
		iRet = WriteToFile(sPathAndName, sEEPROM_Value_String, 1);
		if (iRet != 0)
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromVerifySignature:Write File: Error\n");
#endif	
			return false;
		}
		else
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromVerifySignature:Write File: Success\n");
#endif	
		}


		//Create Openssl Verify Command
		//1. openssl enc -d -A -base64 -in /etc/eep_key/eeprom.sig -out /etc/eep_key/eeprom_signature.sha1
		//2. openssl dgst -sha1 -verify /etc/eep_key/rsa_eep_public.pem -signature /etc/eep_key/eeprom_signature.sha1 /etc/eep_key/eeprom.string > /etc/eep_key/verifyout.txt 

		string sPathAndNameSignature(EEPROM_WORK_PATH);
		sPathAndNameSignature.append("eepromver.sig");

		string sPathAndNameSignatureSha1(EEPROM_WORK_PATH);
		sPathAndNameSignatureSha1.append("eeprom_signature.sha1");

		string sOpensslCommand = "openssl enc -d -A -base64 -in ";
		sOpensslCommand.append(sPathAndNameSignature);
		sOpensslCommand.append(" -out ");
		sOpensslCommand.append(sPathAndNameSignatureSha1);

#ifdef EEPROM_DEBUG_OUT
		printf("EepromVerifySignature:sOpensslCommand:%s\n", (char*)sOpensslCommand.c_str());
#endif	
		ret = system((char*)sOpensslCommand.c_str());
		if (ret == 0)
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromVerifySignature:sOpensslCommand: Success\n");
#endif	
		}
		else
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromVerifySignature:sOpensslCommand: Error\n");
#endif	
			return false;
		}

		string sPathAndNamePublicKey(EEPROM_WORK_PATH);
		sPathAndNamePublicKey.append("rsa_eep_public.pem");

		string sPathAndNameEEPROMDataString(EEPROM_WORK_PATH);
		sPathAndNameEEPROMDataString.append("eepromver.string");

		string sPathAndNameVerifyOutput(EEPROM_WORK_PATH);
		sPathAndNameVerifyOutput.append("verifyout.txt");

		sOpensslCommand = "openssl dgst -sha1 -verify ";
		sOpensslCommand.append(sPathAndNamePublicKey);
		sOpensslCommand.append(" -signature ");
		sOpensslCommand.append(sPathAndNameSignatureSha1);
		sOpensslCommand.append(" ");
		sOpensslCommand.append(sPathAndNameEEPROMDataString);
		sOpensslCommand.append(" > ");
		sOpensslCommand.append(sPathAndNameVerifyOutput);
#ifdef EEPROM_DEBUG_OUT
		printf("EepromVerifySignature:sOpensslCommand:%s\n", (char*)sOpensslCommand.c_str());
#endif	
		ret = system((char*)sOpensslCommand.c_str());
		if (ret == 0)
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromVerifySignature:sOpensslCommand: Signature Success\n");
#endif	
			return true;
		}
		else
		{
#ifdef EEPROM_DEBUG_OUT
			printf("EepromVerifySignature:sOpensslCommand: Signature Error\n");
#endif	
			return false;
		}

#endif
		return ret;//Windows
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	bool EepromErase()
	{
		bool ret = false;
		int eepromSize =  _get_eeprom_size();
		if (eepromSize < 0)
		{			
			return false;
		}

#ifdef LINUX
							
		int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_WRONLY);

		if ( fd < 0 )
		{			
			return false;
		}
		else
		{
			char wbuf[eepromSize];
			memset(wbuf,0,eepromSize);

			for (int i = 0; i < 3 && !ret; i++)
			{		
				int wlen=::write(fd,wbuf,eepromSize);
				ret = wlen == eepromSize;
			}

			close(fd);
		}
		
#endif

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool EepromRead( std::map<std::string, std::string>& data )
	{
		bool ret=true;

	#ifdef LINUX
		std::string eepromEntry = "";
		if (_readEepromEntry(VERSION_ADDR, VERSION_LENGTH, eepromEntry))
		{
			data["ver"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(HWID_ADDR, HWID_LENGTH, eepromEntry))
		{
			data["hwid"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(HWVER_ADDR, HWVER_LENGTH, eepromEntry))
		{
			data["hwver"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(OSC_ADDR, OSC_LENGTH, eepromEntry))
		{
			data["osc"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(BC_ADDR, BC_LENGTH, eepromEntry))
		{
			data["bc"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(MAC0_ADDR, MAC_LENGTH, eepromEntry))
		{
			data["mac0"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(MAC1_ADDR, MAC_LENGTH, eepromEntry))
		{
			data["mac1"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(MAC2_ADDR, MAC_LENGTH, eepromEntry))
		{
			data["mac2"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(MAC3_ADDR, MAC_LENGTH, eepromEntry))
		{
			data["mac3"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(MAC4_ADDR, MAC_LENGTH, eepromEntry))
		{
			data["mac4"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(MAC5_ADDR, MAC_LENGTH, eepromEntry))
		{
			data["mac5"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(SN_ADDR, SN_LENGTH, eepromEntry))
		{
			data["sn"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(DEVID_ADDR, DEVID_LENGTH, eepromEntry))
		{
			data["devid"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(PDATE_ADDR, PDATE_LENGTH, eepromEntry))
		{
			data["pdate"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(OEM_ADDR, OEM_LENGTH, eepromEntry))
		{
			data["oem"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(MOUNTERRORINIT_ADDR, MOUNTERRORINIT_LENGTH, eepromEntry))
		{
			data["mnterrinit"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(MOUNTERRORRUN_ADDR, MOUNTERRORRUN_LENGTH, eepromEntry))
		{
			data["mnterrrun"] = eepromEntry;
		}
		else
		{
			ret = false;
		}
		if (_readEepromEntry(SIG_ADDR, SIG_LENGTH, eepromEntry))
		{
			data["sig"] = eepromEntry;
		}
		else
		{
			ret = false;
		}

	#else

		data["ver"] = "1";
		data["hwid"] = "00-000";
		data["hwver"] = "12345";
		data["osc"] = "0";
		data["bc"] = "0";
		data["mac0"] = "02:08:02:C0:FF:EA";
		data["mac1"] = "00:08:02:C0:FF:EB";
		data["mac2"] = "00:08:02:C0:FF:EC";
		data["mac3"] = "00:08:02:C0:FF:ED";
		data["mac4"] = "00:08:02:C0:FF:EE";
		data["mac5"] = "00:08:02:C0:FF:EF";
		data["sn"] = "0123456789";
		data["devid"] = "FFAA4E540101020304";
		data["pdate"] = "20141119";
		data["oem"] = "Dr. Neuhaus";
		data["sig"] = "hzdfu36423846wfbdwfhbsdu";

	#endif	

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool EepromWrite( std::map<std::string, std::string>& data )
	{
		bool ret = true;

		std::map<std::string, std::string>::iterator it;
		
		it=data.find("ver");		
		if ( it != data.end() && !_writeEepromEntry(VERSION_ADDR, VERSION_LENGTH, it->second)) ret=false;

		it=data.find("hwid");		
		if ( it != data.end() && !_writeEepromEntry(HWID_ADDR, HWID_LENGTH, it->second)) ret=false;

		it=data.find("hwver");		
		if ( it != data.end() && !_writeEepromEntry(HWVER_ADDR, HWVER_LENGTH, it->second)) ret=false;

		it=data.find("osc");		
		if ( it != data.end() && !_writeEepromEntry(OSC_ADDR, OSC_LENGTH, it->second)) ret=false;

		it=data.find("bc");		
		if ( it != data.end() && !_writeEepromEntry(BC_ADDR, BC_LENGTH, it->second)) ret=false;

		it=data.find("mac0");		
		if ( it != data.end() && !_writeEepromEntry(MAC0_ADDR, MAC_LENGTH, it->second)) ret=false;

		it=data.find("mac1");		
		if ( it != data.end() && !_writeEepromEntry(MAC1_ADDR, MAC_LENGTH, it->second)) ret=false;

		it=data.find("mac2");		
		if ( it != data.end() && !_writeEepromEntry(MAC2_ADDR, MAC_LENGTH, it->second)) ret=false;

		it = data.find("mac3");
		if (it != data.end() && !_writeEepromEntry(MAC3_ADDR, MAC_LENGTH, it->second)) ret = false;

		it = data.find("mac4");
		if (it != data.end() && !_writeEepromEntry(MAC4_ADDR, MAC_LENGTH, it->second)) ret = false;

		it = data.find("mac5");
		if (it != data.end() && !_writeEepromEntry(MAC5_ADDR, MAC_LENGTH, it->second)) ret = false;

		it=data.find("sn");		
		if ( it != data.end() && !_writeEepromEntry(SN_ADDR, SN_LENGTH, it->second)) ret=false;

		it=data.find("devid");		
		if ( it != data.end() && !_writeEepromEntry(DEVID_ADDR, DEVID_LENGTH, it->second)) ret=false;
		
		it=data.find("pdate");		
		if ( it != data.end() && !_writeEepromEntry(PDATE_ADDR, PDATE_LENGTH, it->second)) ret=false;

		it=data.find("oem");		
		if ( it != data.end() && !_writeEepromEntry(OEM_ADDR, OEM_LENGTH, it->second)) ret=false;
		
		it = data.find("mnterrinit");
		if (it != data.end() && !_writeEepromEntry(MOUNTERRORINIT_ADDR, MOUNTERRORINIT_LENGTH, it->second)) ret = false;

		it = data.find("mnterrrun");
		if (it != data.end() && !_writeEepromEntry(MOUNTERRORRUN_ADDR, MOUNTERRORRUN_LENGTH, it->second)) ret = false;

		it=data.find("sig");		
		if ( it != data.end() && !_writeEepromEntry(SIG_ADDR, SIG_LENGTH, it->second)) ret=false;
		
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	int WriteToFile(const std::string _fileName, const std::string _content, bool _append)
	{
		int iRet = -1;
#ifdef LINUX
		FILE* pFile = fopen(_fileName.c_str(), _append?"a":"w");
		if (pFile)
		{
			fprintf(pFile, "%s", _content.c_str());
			fclose(pFile);
			iRet = 0;
		}
#endif
		return iRet;
	}

	//-----------------------------------------------------------------------------------------------------------
	int ReadFromFile(const std::string _fileName, std::string& _content, uint32_t _maxLen)
	{
		int iRet = -1;
#ifdef LINUX
		FILE* pFile = fopen(_fileName.c_str(), "r");
		if (pFile)
		{
			// Read File to String ...
			int iByte = 0;
			char c;
			uint32_t iCount = 0;
			while(!feof(pFile))
			{
				iByte = fgetc(pFile);
				iCount += 1;
				if (iCount > _maxLen)
				{
					break;
				}

				if (iByte != EOF)
				{
					c = (char)iByte;
					_content.append(&c, 1);
				}
			}
			fclose(pFile);
			if (iCount > _maxLen)
			{
				iRet = -2;
			}
			else
			{
				iRet = 0;
			}
		}
#endif
		return iRet;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _getOsVersion(std::string& _value)
	{
		int ret = 0;		
		string bspname = "";
		string bspversion = "";

#ifdef LINUX
		FILE *osreleasefile;
		char buffer[255];		
		char* start;
		char* stop;

		if ((osreleasefile=fopen("/etc/os-release","r")) == NULL)
		{
			printf("ERROR open os-release file !\n");
			return -1;
		}
		else
		{
			while (fgets(buffer,255,osreleasefile))
			{
				if (strstr(buffer, "PTXDIST_BSP_NAME"))
				{
					start= strstr(buffer,"\"") + 1;
					stop = strstr(start,"\"");
					bspname.assign(start,stop-start);
				}
				else if (strstr(buffer, "PTXDIST_BSP_VERSION"))
				{
					start= strstr(buffer,"\"") + 1;
					stop = strstr(start,"\"");
					bspversion.assign(start,stop-start);
				}
			}
			fclose(osreleasefile);
		}
		
		

#else
		bspname = "DNT8209";
		bspversion = "-88";		
		
#endif		
		_value = bspname + bspversion;
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _getKernelVersion(std::string& _value)
	{
		int ret = 0;
		std::string kversion = "";
		std::string kbuild = "";

#ifdef LINUX
		FILE *kversionfile=fopen("/proc/version","r");
		char buffer[255];
		char* start;
		char* stop;

		if (kversionfile == NULL)
		{
			printf("ERROR open kernel version file !\n");
			return -1;
		}
		else
		{
			if (fgets(buffer, 255, kversionfile))
			{
				start = &buffer[0];
				stop = strchr(buffer, '(');
				kversion.assign(start,stop-start);

				start = strchr(buffer, '#');
				stop = strstr(buffer, "PREEMPT");
				kbuild.assign(start,stop-start);
			}
			else
			{
				printf("ERROR reading kernel version file !\n");
				ret = -1;
			}

			fclose(kversionfile);
		}

		

#else
		kversion = "Windows XXX";
		kbuild = "x.y";
#endif		
		_value = kversion + kbuild;

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _getAppVersion(std::string& _value)
	{
		int ret = 0;

		_value = "0.1-546";

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _getPacketVersion(std::string& _packetname, std::string& _value)
	{
		int ret = 0;
		_value = "";

#ifdef LINUX
		FILE *opkfile=fopen("/usr/lib/opkg/status","r");
		char buffer[255];
		char* start;
		char* stop;
		char* checkpoint;
		bool isComplete=false;

		if (opkfile == NULL)
		{
			printf("ERROR open opkg-status file !\n");
			return -1;
		}
		else
		{
			while ((fgets(buffer,255,opkfile)) && (!isComplete))
			{			
				start = strstr(buffer, _packetname.c_str());
				if (start)										// Package-Name wurde gefunden
				{
					if (strstr(buffer, "Package"))				// tritt der Name als Paketname auf oder als dependency
					{
						checkpoint = strrchr(buffer, ':');
						if ((checkpoint+2)==start)				// steht der Paketname allein dort
						{
							stop = strrchr(buffer,'\n');

							//printf("stop-start: %d	_packetname.length: %d\n", stop-start, _packetname.length());
							if ((unsigned int)(stop-start)==_packetname.length())
							{
								fgets(buffer,255,opkfile);
								start = strrchr(buffer,':')+2;
								stop = strrchr(buffer,'\n');
								_value.assign(start,stop-start);
								isComplete = true;
							}
						}
					}				
				}
			}
			fclose(opkfile);
		}

		
#else		
		_value = "1.0";
#endif

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _getAllPacketVersions(std::map<std::string, std::string>& data)
	{
		int ret = 0;


#ifdef LINUX
		FILE *opkfile=fopen("/usr/lib/opkg/status","r");
		char buffer[255];
		char* start;
		char* stop;

		std::string pname = "";
		std::string pversion = "";

		if (opkfile == NULL)
		{
			printf("ERROR open opkg-status file !\n");
			return -1;
		}
		else
		{
			while (fgets(buffer,255,opkfile))
			{
				if(strstr(buffer,"Package"))
				{
					start = strrchr(buffer,':')+2;
					stop = strrchr(buffer,'\n');
					pname.assign(start,stop-start);

					fgets(buffer,255,opkfile);
					start = strrchr(buffer,':')+2;
					stop = strrchr(buffer,'\n');
					pversion.assign(start,stop-start);

					//printf("Paket: %s	Version: %s\n", pname.c_str(), pversion.c_str());
					data[pname] = pversion;
				}
			}

			fclose(opkfile);
		}

		
#else
		data["glib"] = "2.3.4";
		data["zlib"] = "1.2";
		data["openssl"] = "1.0.1e";
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _getPatches(std::map<std::string, std::string>& data)
	{
		int ret = 0;		

#ifdef LINUX

		FILE *opkfile=fopen("/usr/lib/opkg/status","r");
		char buffer[255];
		char* start;
		char* stop;

		std::string pname = "";
		std::string pversion = "";

		if (opkfile == NULL)
		{
			printf("ERROR open opkg-status file !\n");
			return -1;
		}
		else
		{
			while (fgets(buffer,255,opkfile))
			{
				if(strstr(buffer,"Package"))
				{
					if (strstr(buffer, "patch"))
					{
						start = strrchr(buffer,':')+2;
						stop = strrchr(buffer,'\n');
						pname.assign(start,stop-start);

						fgets(buffer,255,opkfile);
						start = strrchr(buffer,':')+2;
						stop = strrchr(buffer,'\n');
						pversion.assign(start,stop-start);

						//printf("Paket: %s	Version: %s\n", pname.c_str(), pversion.c_str());
						data[pname] = pversion;
					}								
				}
			}

			fclose(opkfile);
		}

		
#else
		data["patch1"] = "1.0";
		data["patch2"] = "1.0";
		data["patch4"] = "1.0";
#endif

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	
	bool QuerySystemVersion(const std::string& _mode, std::map<std::string, std::string>& data, string& _error)
	{
		bool ret = true;
		int retint = 0;
		int retprev = 0;
		string key;

		//printf("mode: %s	\n", _mode.c_str());
		
		std::map<std::string, std::string>::iterator i;

		if (_mode == "single")
		{							
			for (i = data.begin(); i!=data.end(); i++)
			{
				key = i->first;			
				retprev = retint;							// Error detector

				if (key == "Os")
				{
					if (_getOsVersion(data["Os"]) != 0)
					{				
						retint--;
					}
				}
				else if (key == "Kernel")
				{				
					if (_getKernelVersion(data["Kernel"]) != 0)
					{				
						retint--;
					}
				}
				else
				{				
					if (_getPacketVersion(key, i->second) != 0)
					{
						retint--;
					}
				}

				if (retint != retprev)				// es ist ein Fehler aufgetreten
				{
					_error = "QuerySystemVersion failed: reading Version for '" + key + "=" + i->second + "' ";
					ret = false;
				}
			}			
		} 
		else if (_mode == "allpackages")
		{
			_getAllPacketVersions(data);
		}
		else if (_mode == "patches")
		{
			_getPatches(data);
		}
		else
		{
			_error = "QuerySystemVersion: invalid mode '" + _mode + "'";
			ret = false;
		}

		if (retint < 0)
		{
			ret = false;
		}

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool SystemUpdateRequest(const std::string& _pathfilename)
	{
		bool ret = true;

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	long long int _statFS(const std::string& _typ, const std::string& _pathfilename)
	{
		long long int ret = 0;
#ifdef LINUX

		struct statfs buf;

		if (!statfs(_pathfilename.c_str(),&buf))
		{
			long long int blksize, blocks, disk_size, used, free, bavail;
			blksize = buf.f_bsize;
			blocks = buf.f_blocks;
			//freeblks = buf.f_bfree;
			bavail = buf.f_bavail;

			disk_size = blocks*blksize;
			//free = freeblks*blksize;
			free = bavail*blksize;
			used = disk_size - free;

			//printf("blksize: %lli	blocks: %lli	freeblks: %lli disk_size: %lli free: %lli used: %lli\n", blksize,blocks,freeblks,disk_size,free,used);
			
			if (_typ=="free")
			{
				ret = free;
			}
			else if (_typ=="disksize")
			{
				ret = disk_size;
			}
			else if (_typ=="used")
			{
				ret = used;
			}
		}
		else
		{
			ret = -1;
		}
		

#else
		ret = 123456;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	long long int GetFreeDiskSpace(const std::string& _pathfilename)
	{
		long long int ret = 0;

		ret = _statFS("free",_pathfilename);

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	long long int GetDiskSize(const std::string& _pathfilename)
	{
		long long int ret = 0;

		ret = _statFS("disksize",_pathfilename);

		return ret;
	}
	
	//-----------------------------------------------------------------------------------------------------------

	long long int GetUsedDiskSpace(const std::string& _pathfilename)
	{
		long long int ret = 0;

		ret = _statFS("used",_pathfilename);

		return ret;
	}	

	//-----------------------------------------------------------------------------------------------------------

	int ReadTamperStatus()
	{
		int ret = 0;		

#ifdef LINUX
				
		char tamperResponse[8];	
		int retry;
		
		for (retry=0; retry<3; retry++)
		{			
			_TamperRequest(TAMPER_CMD_READ,tamperResponse);

			if (strcmp(tamperResponse, TAMPER_READ_ACK_OK)==0)
			{
				ret = 0x01;
				break;
			}
			else if (strcmp(tamperResponse, TAMPER_READ_ACK_SWITCH)==0)
			{
				ret = 0x02;
				break;
			}
			else if (strcmp(tamperResponse, TAMPER_READ_ACK_EEPROM)==0)
			{
				ret = 0x03;
				break;
			}
			else
			{
				ret = -1;			
			}
		}		
#else
		ret = 0x01;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int TamperReboot()
	{
		int ret = 0;
		

#ifdef LINUX

		int retry;
		
		char tamperResponse[8];

		for (retry=0; retry<3; retry++)
		{
			_TamperRequest(TAMPER_CMD_REBOOT_REQ, tamperResponse);			

			printf("Tamper response: %s\n", tamperResponse);

			if (strcmp(tamperResponse, TAMPER_CMD_REBOOT_ACK)==0)
			{
				ret = 0;
				break;
			}			
			else
			{
				ret = -1;			
			}
		}		
#else
		ret = 0x01;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	unsigned int getPppdPid()
	{
		unsigned int ret = 0;

#ifdef LINUX
		FILE *ppp0_pid_file=fopen("/var/run/ppp0.pid","r");
		//char buffer[255];
		
		if (ppp0_pid_file == NULL)
		{		
			return 0;
		}
		else
		{
			if(fscanf(ppp0_pid_file,"%d",&ret) != 1) ret=0;

			fclose(ppp0_pid_file);
		}		

#else
		ret = 100;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool GetDSRPegel()
	{
		bool ret = false;
#ifdef LINUX

		char value;
		FILE *fp=::fopen(DSR_ON, "r");
		if(fp != NULL)
		{
			fscanf(fp,"%c",&value);
			ret = value == 1;

			fclose(fp);
		}
		
#endif
		
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool GetDCDPegel()
	{
		bool ret = false;
#ifdef LINUX

		char value;
		FILE *fp=::fopen(DCD_ON, "r");
		if(fp != NULL)
		{
			fscanf(fp,"%c",&value);
			ret = value == 1;

			fclose(fp);
		}
		
#endif
		
		return ret;
	}

//-----------------------------------------------------------------------------------------------------------

	bool GetSwitchInterrupt()
	{
		bool ret = false;
#ifdef LINUX

		char value;
		FILE *fp=::fopen(SWITCH_INTERRUPT, "r");
		if(fp != NULL)
		{
			fscanf(fp,"%c",&value);
			ret = value == 1;

			fclose(fp);
		}
		
#endif
		
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool EthExists(std::string &_networkDevice)
	{		
		return ((_networkDevice == "eth0") || (_networkDevice == "eth1.1") || (_networkDevice == "eth1.2") || (_networkDevice == "ppp0"));
	}

	int _getRawBattVoltage()
	{
		int ret = 0;
#ifdef LINUX

		unsigned short value;
		FILE *fp=::fopen("/sys/bus/iio/devices/iio:device0/in_voltage7_raw", "r");
		if(fp != NULL)
		{
			if(fscanf(fp,"%hu",&value) != 1) ret=-1;
			else ret = value > 1160 ? 1160 : value;	// Begrenzung

			fclose(fp);
		}
		else
		{
			ret=-1;
		}
		
#else
		ret = 1160;		// fr Windows: stets voller Ladezustand
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	unsigned int GetBattVoltage_mv()
	{
		unsigned int ret = 1;
		int rawBattVoltage = _getRawBattVoltage();

		if (rawBattVoltage > 0)
		{
			ret = ( rawBattVoltage * 4240 ) / 1160;
		}

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	unsigned int GetBattVoltage_percent()
	{
		unsigned int ret = 0;

		int rawBattVoltage = _getRawBattVoltage();

		if (rawBattVoltage >= 0)
		{
			ret = ( rawBattVoltage * 100 / 1160 );
		}

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool IsNTPsynchronized(uint16_t _port, uint32_t _maxOffset_ms)
	{
#ifdef WIN32
		return true;
#endif

		// rc von ntpstat:
		//		0:   Uhr ist synchronisiert
		//		1:   Uhr ist nicht synchronisiert, Eines der Kriterien wurde nicht erfllt
		//		2:   Status der Synchronisation konnte nicht ermittelt werden (ntpd nicht erreichbar)

		return (_ntpstat(_port, _maxOffset_ms) == 0 ? true : false);		
	}


	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int AddGRETunnel(const std::string& _ethname, const std::string& _key, const std::string& _ttl, const std::string& _localAddr, const std::string& _remoteAddr)
	{
		int ret = 0;
#ifdef LINUX
		string command = "ip tunnel add " + _ethname + " mode gre";
		if(!_key.empty() && _key.compare("0") != 0)
		{
			command += " key " + _key;
		}
		if (!_ttl.empty())
		{
			command += " ttl " + _ttl;
		}
		if (!_localAddr.empty())
		{
			command += " local " + _localAddr;
		}
		if (!_remoteAddr.empty())
		{
			command += " remote " + _remoteAddr;
		}

		ret = system(command.c_str());
#else
		ret = 1;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int RemoveGRETunnel(const std::string& _ethname)
	{
		int ret = 0;
#ifdef LINUX
		string command = "ip tunnel del " + _ethname;

		ret = system(command.c_str());
#else
		ret = 1;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int FLushNHRPConfig()
	{
		int ret = 0;
#ifdef LINUX
		FILE* pFile = NULL;
		pFile = fopen(NHRP_FILE, "w");
		if (pFile!=NULL)
		{
			fclose(pFile);
			ret = 1;
		}
#else
		ret = 1;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int AddNHRPInterface(const std::string& _ethname,
		const std::string& _network,
		const std::string& _nhs,
		const std::string& _holdingtime, 
		bool _multicast,
		bool _hubmode,
		bool _ciscoauth,
		const std::string& _ciscokey
		)
	{
		int ret = 0;
#ifdef LINUX
		FILE* pFile = NULL;
		pFile = fopen(NHRP_FILE, "a");
		if (pFile!=NULL)
		{
			fprintf(pFile, "interface %s\n",_ethname.c_str());
			if(!_hubmode)
			{
				fprintf(pFile, "  dynamic-map %s %s cisco\n",_network.c_str(), _nhs.c_str());
			}
			if (_multicast)
			{
				fprintf(pFile, "  multicast dynamic\n");
			}
			if(!_hubmode)
			{
				fprintf(pFile, "  holding-time %s\n", _holdingtime.c_str());
				fprintf(pFile, "  shortcut\n");
			}
			else
			{
				fprintf(pFile, "  redirect\n");
			}
			if (_ciscoauth)
			{
				fprintf(pFile, "  cisco-authentication \"%s\"\n", _ciscokey.c_str());
			}
			fprintf(pFile, "\n");
			
			fclose(pFile);
			ret = 1;
		}
#else
		ret = 1;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	int FlushIPSecPolicy()
	{
		int ret = 0;
#ifdef LINUX
		FILE* pFile = NULL;
		pFile = fopen(IPSEC_POLICY_FILE, "w");
		if (pFile != NULL)
		{
			fprintf(pFile, "flush;\n");
			fprintf(pFile, "spdflush;\n");
			fclose(pFile);

			string command = "setkey -f ";
			command += IPSEC_POLICY_FILE;
			ret = system(command.c_str());
		}
#else
		ret = 1;
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int PrepareIPSecPolicy(
		const std::string& _sourcenet,
		const std::string& _destnet,
		const std::string& _prot,
		const std::string& _direction,
		const std::string& _mode,
		const std::string& _endpoints,
		const std::string& _policy
		)
	{
		int ret = 0;
#ifdef LINUX
		FILE* pFile = NULL;
		pFile = fopen(IPSEC_POLICY_FILE, "a");
		if (pFile != NULL)
		{
			fprintf(pFile, "spdadd %s %s %s -P %s ipsec esp/%s/%s/%s;\n",_sourcenet.c_str(), _destnet.c_str(), _prot.c_str(), _direction.c_str(), _mode.c_str(), _endpoints.c_str(), _policy.c_str());
			fclose(pFile);
			ret = 1;
		}
#else
		ret = 1;
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int SetIPSecPolicy()
	{
		int ret = 0;
#ifdef LINUX
		string command = "setkey -f ";
		command += IPSEC_POLICY_FILE;
		ret = system(command.c_str());
#else
		ret = 1;
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int InitIPSecConfig(
		unsigned int _retrycounter,
		unsigned int _resendinterval,
		unsigned int _ph1timeout,
		unsigned int _ph2timeout,
		unsigned int _nattkeepalive
		)
	{
		int ret = 0;
#ifdef LINUX
		FILE* pFile = NULL;
		pFile = fopen(IPSEC_CONFIG_FILE, "w");
		if (pFile != NULL)
		{
			fprintf(pFile, "path pre_shared_key \"%s\";\n", IPSEC_PSK_FILE);
			fprintf(pFile, "path script \"/dnt/tainy/scripts/\";\n");
			fprintf(pFile, "timer {\n");
			fprintf(pFile, "\tcounter %u;\n", _retrycounter);
			fprintf(pFile, "\tinterval %u sec;\n", _resendinterval);
			fprintf(pFile, "\tphase1 %u sec;\n", _ph1timeout);
			fprintf(pFile, "\tphase2 %u sec;\n", _ph2timeout);
			fprintf(pFile, "\tnatt_keepalive %u sec;\n", _nattkeepalive);
			fprintf(pFile, "}\n");
			fclose(pFile);
			ret = 1;
		}
#else
		ret = 1;
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int AddAnonymousIPSecGateway(
		const std::string& _mode,
		unsigned int _lifetime,
		const std::string& _identifier,
		const std::string& _natt,
		const std::string& _encryption,
		const std::string& _hash,
		unsigned int _dhgroup,
		bool _dpd,
		unsigned int _dpddelay,
		unsigned int _dpdretry,
		unsigned int _dpdmaxfail
		)
	{
		int ret = 0;
#ifdef LINUX
		FILE* pFile = NULL;
		pFile = fopen(IPSEC_CONFIG_FILE, "a");
		if (pFile != NULL)
		{
			fprintf(pFile, "remote anonymous {\n");
			fprintf(pFile, "\texchange_mode %s;\n", _mode.c_str());
			fprintf(pFile, "\tlifetime time %u sec;\n", _lifetime);
			if(!_identifier.empty())
			{
				if (_identifier.find("@") != std::string::npos)
				{
					fprintf(pFile, "\tmy_identifier user_fqdn \"%s\";\n", _identifier.c_str());
				}
				else
				{
					fprintf(pFile, "\tmy_identifier fqdn \"%s\";\n", _identifier.c_str());
				}
			}
			fprintf(pFile, "\tnat_traversal %s;\n", _natt.c_str());
			if(_dpd)
			{
				fprintf(pFile, "\tdpd_delay %u;\n", _dpddelay);
				fprintf(pFile, "\tdpd_retry %u;\n", _dpdretry);
				fprintf(pFile, "\tdpd_maxfail %u;\n", _dpdmaxfail);
			}
			fprintf(pFile, "\tscript \"racoon-script\" phase1_up; \n");
			fprintf(pFile, "\tscript \"racoon-script\" phase1_down; \n");
			fprintf(pFile, "\tscript \"racoon-script\" phase1_dead; \n");
			fprintf(pFile, "\tproposal {\n");
			fprintf(pFile, "\t\tencryption_algorithm %s;\n", _encryption.c_str());
			fprintf(pFile, "\t\thash_algorithm %s;\n", _hash.c_str());
			fprintf(pFile, "\t\tauthentication_method pre_shared_key;\n");
			fprintf(pFile, "\t\tdh_group %u;\n", _dhgroup);
			fprintf(pFile, "\t}\n");
			fprintf(pFile, "}\n");

			fclose(pFile);
			ret = 1;
		}
#else
		ret = 1;
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int AddAnonymousIPSecSAInfo(
		unsigned int _pfsgroup,
		unsigned int _lifetime,
		const std::string& _encryption,
		const std::string& _hash
		)
	{
		int ret = 0;
#ifdef LINUX
		FILE* pFile = NULL;
		pFile = fopen(IPSEC_CONFIG_FILE, "a");
		if (pFile != NULL)
		{
			fprintf(pFile, "sainfo anonymous {\n");
			fprintf(pFile, "\tpfs_group %u;\n", _pfsgroup);
			fprintf(pFile, "\tlifetime time %u sec;\n", _lifetime);
			fprintf(pFile, "\tencryption_algorithm %s;\n", _encryption.c_str());
			fprintf(pFile, "\tauthentication_algorithm %s;\n", _hash.c_str());
			fprintf(pFile, "\tcompression_algorithm deflate;\n");
			fprintf(pFile, "}\n");

			fclose(pFile);
			ret = 1;
		}
#else
		ret = 1;
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int EstablishIPSecSA(
		const std::string& _satype,
		const std::string& _sourceaddr,
		const std::string& _destaddr,
		const std::string& _prot
		)
	{
		int ret = 0;
#ifdef LINUX
		string command = "racoonctl establish-sa -w ";
		command += _satype;
		command += " inet ";
		command += _sourceaddr;
		command += " ";
		command += _destaddr;
		if (!_prot.empty())
		{
			command += " ";
			command += _prot;
		}
		command += "&";
		ret = system(command.c_str());
#else
		ret = 1;
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int DeleteIPSecSA(
		const std::string& _satype,
		const std::string& _sourceaddr,
		const std::string& _destaddr
		)
	{
		int ret = 0;
#ifdef LINUX
		string command = "racoonctl delete-sa ";
		command += _satype;
		command += " inet ";
		command += _sourceaddr;
		command += " ";
		command += _destaddr;
		command += "&";
		ret = system(command.c_str());
#else
		ret = 1;
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int FlushIPSecPSKs()
	{
		int ret = 0;
#ifdef LINUX
		FILE* pFile = NULL;
		pFile = fopen(IPSEC_PSK_FILE, "w");
		if (pFile!=NULL)
		{
			fclose(pFile);
			ret = 1;
		}

		chmod(IPSEC_PSK_FILE, 0600);
		chown(IPSEC_PSK_FILE, 0, 0);
#else
		ret = 1;
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int AddIPSecPSK(const std::string& _identity, const std::string& _psk)
	{
		int ret = 0;
#ifdef LINUX
		FILE* pFile = NULL;
		pFile = fopen(IPSEC_PSK_FILE, "a");
		if (pFile!=NULL)
		{
			fprintf(pFile, "%s\t\t%s\n", _identity.c_str(), _psk.c_str());
			fclose(pFile);
			ret = 1;
		}

		chmod(IPSEC_PSK_FILE, 0600);
		chown(IPSEC_PSK_FILE, 0, 0);
#else
		ret = 1;
#endif
		return ret;
	}
	//-----------------------------------------------------------------------------------------------------------

	//-----------------------------------------------------------------------------------------------------------
	int EnableForwarding(bool _enabled)
	{
		int ret = 0;
#ifdef LINUX
		FILE* pFile = NULL;
		pFile = fopen(NET_FORWARD, "w");
		if (pFile!=NULL)
		{
			fprintf(pFile, _enabled?"1":"0");
			fclose(pFile);
			ret = 1;
		}
#else
		ret = 1;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

}	//namespace hal

