/*****************************************************************
*
* hal header
*
* Copyright (c) 2013 Dr. Neuhaus Telekommunikation GmbH
*
*
*****************************************************************/

#ifndef __HAL__
#define __HAL__

#include <stdio.h>
#include <assert.h>
#include <stdint.h>

#include <string>
#include <map>
#include <list>

#define ON		true
#define OFF		false
#define UP		true
#define DOWN	false

#define SIG_ON			"1"
#define SIG_OFF			"0"

#define LED_ON			"0"
#define LED_OFF			"1"

#define HAL_LED_ID_POWER		1
#define HAL_LED_ID_VPN			2
#define HAL_LED_ID_IN			3
#define HAL_LED_ID_OUT			4
#define HAL_LED_ID_OTHER		5
#define HAL_LED_ID_CONNECTION	6
#define HAL_LED_ID_QUALITY		7
#define HAL_LED_ID_SIGNAL		8
#define HAL_LED_ID_SIM_IND		9


#define SET_DNS_MODE_OVERWRITE	1
#define SET_DNS_MODE_APPEND		2

#define LAN_PORT_1				1
#define LAN_PORT_2				2
#define LAN_PORT_3				3
#define LAN_PORT_4				4
#define LAN_PORT_5				5

#define LAN_PORT_IMX_SWITCH		99

#define GLOBAL_1_REG			100
#define GLOBAL_2_REG			101

#define LAN_LINKSTATE_DOWN				0
#define LAN_LINKSTATE_UP_RESOLVED		1
#define LAN_LINKSTATE_UP_UNRESOLVED		2

#define LAN_1_SWITCH_PORT_PHYID 0x18
#define LAN_2_SWITCH_PORT_PHYID 0x19
#define LAN_3_SWITCH_PORT_PHYID 0x1A
#define LAN_4_SWITCH_PORT_PHYID 0x1B
#define LAN_5_SWITCH_PORT_PHYID 0x1C

#define LAN_PORT_IMX_SWITCH_PHYID	0x1E

#define GLOBAL_1_REG_PHYID		0x1F
#define GLOBAL_2_REG_PHYID		0x17

#define LAN_1_SMI_PHY_ADDR		0x0200
#define LAN_2_SMI_PHY_ADDR		0x0220
#define LAN_3_SMI_PHY_ADDR		0x0240
#define LAN_4_SMI_PHY_ADDR		0x0260
#define LAN_5_SMI_PHY_ADDR		0x0280

#define SWITCH_GLOBAL_REG_1_PHYID		0x1F
#define SWITCH_GLOBAL_REG_2_PHYID		0x17

#define SWITCH_PORT_REG_STATUS				0x00
#define SWITCH_PORT_REG_MAC_CTRL			0x01
#define SWITCH_PORT_REG_JAM_CTRL			0x02
#define SWITCH_PORT_REG_IDENT				0x03
#define SWITCH_PORT_REG_PORT_CTRL			0x04
#define SWITCH_PORT_REG_PORT_CTRL_1			0x05
#define SWITCH_PORT_REG_VLAN_HDR			0x06
#define SWITCH_PORT_REG_DFLT_VLAN			0x07
#define SWITCH_PORT_REG_PORT_CTRL_2			0x08
#define SWITCH_PORT_REG_EGRESS_RATE_CTRL	0x09
#define SWITCH_PORT_REG_EGRESS_RATE_CTRL_2	0x0A
#define SWITCH_PORT_REG_PORT_ASSOC_VEC		0x0B
// Reserved 0x0C
#define SWITCH_PORT_REG_PRIO_OVERRIDE		0x0D
#define SWITCH_PORT_REG_POLICY_CTRL			0x0E
#define SWITCH_PORT_REG_ETYPE				0x0F
#define SWITCH_PORT_REG_RX_FRAMES			0x10
#define SWITCH_PORT_REG_TX_FRAMES			0x11
#define SWITCH_PORT_REG_IN_DISCRARDS		0x12
#define SWITCH_PORT_REG_IN_FILTERED			0x13
// Reserved 0x14, 0x15
#define SWITCH_PORT_REG_LED_CTRL			0x16
// Reserved 0x17
#define SWITCH_PORT_REG_TAG_REMAP_1			0x18
#define SWITCH_PORT_REG_TAG_REMAP_2			0x19
// Reserved 0x1A
#define SWITCH_PORT_REG_QUEUE_COUNTERS		0x1B
// Reserved 0x1C, 0x1D, 0x1E, 0x1F

#define VLAN_MODE_MEMBER_EGR_UNMOD			0x00
#define VLAN_MODE_MEMBER_EGR_UNTAGGED		0x01
#define VLAN_MODE_MEMBER_EGR_TAGGED			0x02
#define VLAN_MODE_NOT_MEMBER				0x03

#define RESOLV_FILE				"/etc/resolv.conf"

namespace hal
{
	bool halInit();
	void halDeinit();

	/// <summary>
	/// Simple Data Class to abstract interface rx/tx/error statistics
	/// </summary>
	class cIFStatistics
	{
	public:
		cIFStatistics() : m_rxBytes(0), m_txBytes(0), m_rxPackets(0), m_txPackets(0), m_rxErrors(0), m_txErrors(0) {}
		~cIFStatistics() {}

		uint32_t m_rxBytes;
		uint32_t m_txBytes;

		uint32_t m_rxPackets;
		uint32_t m_txPackets;

		uint32_t m_rxErrors;
		uint32_t m_txErrors;
	};

	/// <summary>
	/// LED's schalten
	/// </summary>
	bool SwitchLed(int _led, bool _on);

	/// <summary>
	/// Watchdog des Systems (Hardware) triggern
	/// </summary>
	int TriggerSysWD();

	/*enum eTamperState
	{
		TAMPER_OK = 0,
		TAMPER_FAIL_PRESS = 1,
		TAMPER_FAIL_PRESSED = 2,
		TAMPER_FAIL_READ = 3
	};

	 <summary>
	 Abfrage des Tamper Kontaktes
	 </summary>
	int GetTamperState();*/

	/// <summary>
	/// Abfrage des Systemspeichers
	/// </summary>
	bool GetSysMemory(uint64_t& _total_KB, uint64_t& _avail_KB, uint64_t& _process_KB);

	/// <summary>
	/// LED blinken lassen
	/// </summary>
	bool BlinkLed(int _led);

	/// <summary>
	/// Setzen des Blinkverhaltens der LED
	/// Paramter der Off-/Ontime knnen erst nach Auslsen des Blinkmodus gesetzt werden
	/// Das Beenden des Blinkmodus (led=off) setzt beide Parameter auf 500 ms zurck
	/// </summary>
	bool setLedDelay(int _led, const std::string& _delayofftime_ms, const std::string& _delayontime_ms);

	/// <summary>
	/// Setzen der IP-Adresse eines Eth-Devices
	/// Name des Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1.1"
	/// IPv4 als allgemein lesbaren string, z.Bsp: "192.168.168.168"
	/// </summary>
	int ConfigureIpAddr(const std::string& _ethname, const std::string& _ipv4addr);	

	/// <summary>
	/// Entfernen der IP-Adresse eines Eth-Devices / NetLink API
	/// Name des Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1.1"
	/// IPv4als allgemein lesbaren string, z.Bsp: "192.168.168.168"
	/// PrefixLen: Netmaske/Bitanteil d. Prefix (24 fr 255.255.255.0, 16 fr 255.255.0.0 etc)
	/// </summary>
	int nl_RemoveIpAddr(const std::string& _ethname, const std::string& _ipv4addr, const uint32_t _prefixlen);

	/// <summary>
	/// Setzen der IP-Adresse eines Eth-Devices / NetLink API
	/// Name des Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1.1"
	/// IPv4/Broadcast als allgemein lesbaren string, z.Bsp: "192.168.168.168"
	/// PrefixLen: Netmaske/Bitanteil d. Prefix (24 fr 255.255.255.0, 16 fr 255.255.0.0 etc)
	/// </summary>
	int nl_ConfigureIpAddr(const std::string& _ethname, const std::string& _ipv4addr, const uint32_t _prefixlen, const std::string& _bcastAddr);

	/// <summary>
	/// Create a VLAN Interface
	/// Name des **PHYSICAL** Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1"
	/// VLAN ID: VLAN Id z.B. 1, 2 (resultiert in Interface eth0.VlanID bzw. "eth1.VlanID"
	/// </summary>
	int AddVLAN(const std::string& _physInterface, const int& _vlanID);

	/// <summary>
	/// Delete a VLAN Interface
	/// Name des **VLAN** Devices in Kleinbuchstaben, z.Bsp: "eth0.1", "eth1.1"
	/// </summary>
	int DelVLAN(const std::string& _vlanName);

	/// <summary>
	/// Setzen der MAC-Adresse eines Eth-Devices
	/// Name des Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1.1"
	/// MAC als HEX, GROBUCHSTABEN, 6 Bytes, durch ':' getrennt, z.Bsp: "00:50:56:3f:3f:3f"
	/// Virtuelle netdevices erben die MAC vom physikalischem device
	/// anderweitige Konfiguration ist mglich, versetzt die phys. eth aber im Sniffermode
	/// </summary>
	int ConfigureIpHw(const std::string& _ethname, const std::string& _macaddr);

	/// <summary>
	/// Setzen der Netzmaske eines Eth-Devices
	/// Name des Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1.1"
	/// Netzmaske als allgemein lesbaren string in IPv4, z.Bsp: "255.255.0.0"
	/// </summary>
	int ConfigureIpNetmask(const std::string& _ethname, const std::string& _netmask);

	/// <summary>
	/// Setzen der Broadcast IP-Adresse eines Eth-Devices
	/// Name des Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1.1"
	/// IPv4 Broadcastadresse als allgemein lesbaren string, z.Bsp: "192.168.255.255"
	/// </summary>
	int ConfigureIpBroadcast(const std::string& _ethname, const std::string& _broadcast);

	/// <summary>
	/// Setting of the MTU of an ETH Device
	/// Devicename is lowercase (e.g. "eth0", "eth1")
	/// MTU as integer
	/// </summary>
	int ConfigureMTU(const std::string& _ethname, const int _mtu);


	/// <summary>
	/// Setzen einer Netzwerkroute
	/// </summary>
	int ConfigureIPGateway(const std::string& _network, const std::string& _netmask, const std::string& _gateway, const std::string& _interface, const uint32_t& _metric, const bool& _valid);

	/// <summary>
	/// Setzen der Adresse des Default-Gateways, Zuordnung zum eth-Device erfolgt automatisch durch OS
	/// equivalent zu: route add default gw %_gateway%
	/// </summary>
	int ConfigureIpDefaultGateway(const std::string& _gateway, uint32_t _metric=0);

	/// <summary>
	/// z.Zt. ohne Funktion
	/// </summary>
	int RemoveIPGateway(const std::string& _network, const std::string& _netmask, const std::string& _gateway, const bool& _valid);
	//int RemoveIPDefaultGateway();

	/// <summary>
	/// z.Zt. ohne Funktion
	/// </summary>
	int RemoveIPDefaultGateway(const std::string& _gateway);
	//int RemoveIPDefaultGateway();

	/// <summary>
	/// Aktivieren/Deaktivieren eines netdevices
	/// Name des Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1.1"
	/// "true" aktiviert, "false" deaktiviert das netdevice
	/// </summary>
	int ConfigureIpUpDown(const std::string& _ethname, bool _ipupdown);

	/// <summary>
	/// Aktivieren/Deaktivieren eines netdevices
	/// Name des Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1.1"
	/// "true" aktiviert, "false" deaktiviert das netdevice
	/// </summary>
	int ConfigureIpMulticast(const std::string& _ethname, bool _multicast);

	/// <summary>
	/// Abfrage, ob auf Schicht des physical layers eine Verbindung zum eth-device besteht
	/// Name des Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1.1"
	/// "true" Link ist UP, "false" Link ist DOWN
	/// </summary>
	bool GetLinkState(const std::string& _ethname);

	/// <summary>
	/// Abfrage, ob das eth-device aus Sicht des OS aktiv oder inaktiv ist
	/// Name des Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1.1"
	/// "true" device ist UP, "false" device ist DOWN
	/// </summary>
	bool GetIPDeviceState(const std::string& _ethname);

	/// <summary>
	/// Request Interface RX/TX/Error statistics
	/// Name of the Devices is lower case e.g.: "eth0", "eth1.1"
	/// </summary>
	cIFStatistics GetIFStatistics(const std::string& _ethname);

	/// <summary>
	/// Gibt die aktuell vergebene IPv4-Adresse des eth-devices als _ipv4addr zurck
	/// Name des Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1.1"	
	/// </summary>
	int GetIpv4Addr(const std::string& _ethname, std::string& _ipv4addr);

	/// <summary>
	/// Gibt die aktuell vergebene Netzmaske des eth-devices als _netmask zurck
	/// Name des Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1.1"	
	/// </summary>
	int GetNetmask(const std::string& _ethname, std::string& _netmask);

	/// <summary>
	/// Gibt die aktuell vergebene Broadcast-Adresse des eth-devices als _broadcast zurck
	/// Name des Devices in Kleinbuchstaben, z.Bsp: "eth0", "eth1.1"	
	/// </summary>
	int GetBroadcast(const std::string& _ethname, std::string& _broadcast);

	/// <summary>
	/// Gibt die Routing Information fr eine bestimmte Zieladresse zurck
	/// </summary>
	int GetRoute(const std::string& _dst, std::string& _gateway, std::string& _ethname, std::string& _src);

	/// <summary>
	/// Setzt DNS Server Adressen in /etc/resolv.conf
	/// </summary>
	int SetDNS(const std::list<std::string>& _dns, uint32_t _SetDnsMode = SET_DNS_MODE_OVERWRITE);
	
	/// <summary>
	/// TBD, perhaps	
	/// </summary>
	int GetIpGateway(const std::string& _ethname, std::string& _gateway);
	
	/// <summary>
	/// Reset versetzt den Switch in den Default-Zustand
	/// Switch wird dann als unmanaged switch betrieben 	
	/// </summary>
	int ResetSwitch();

	/// <summary>
	/// Switch wird als managed switch konfiguriert
	/// Port1: vlan_id=1, Port2: vlan_id=2
	/// </summary>
	int ConfigureSwitch();

	/// <summary>
	/// Enable/Disable SwitchPort
	/// </summary>
	int SetSwitchPortEnabled(uint32_t _port, bool _enabled);

	/// <summary>
	/// Set Switch Port Mode
	//#define DATA_LAN_PORT_MODE_AUTO		1
	//#define DATA_LAN_PORT_MODE_100FDX	2
	//#define DATA_LAN_PORT_MODE_100HDX	3
	//#define DATA_LAN_PORT_MODE_10FDX	4
	//#define DATA_LAN_PORT_MODE_10HDX	5

	///  Mode 1: Auto
	///  Mode 2: 100-FDX
	///  Mode 3: 100-HDX
	///  Mode 4: 10-FDX
	///  Mode 5: 10-HDX
	/// </summary>
	int SetSwitchPortMode(uint32_t _port, uint32_t _mode);

	/// <summary>
	/// Get Switch Port Mode
	/// Returns 0 on success, -1 on error
	///  Mode 0: Undefined
	///  Mode 2: 100-FDX
	///  Mode 3: 100-HDX
	///  Mode 4: 10-FDX
	///  Mode 5: 10-HDX
	int GetSwitchPortMode(uint32_t _port, uint32_t& _modeResult);

	/// <summary>
	/// Get Switch Port Link-State
	/// Returns 0 on success, -1 on error
	///  0: Link Down
	///  1: Link Up
	///  2: Undefined / Still in Autonegotiation
	int GetSwitchPortLinkState(uint32_t _port, uint32_t& _linkState);

	/// <summary>
	/// Set Switch Port to Strip VLAN Tags before Packets leave the Port
	/// </summary>
	int SetSwitchVLANTagStripping(uint32_t _port, bool _strip);

	/// <summary>
	/// Set Switch Port to enable VLAN TAG Filtering
	/// </summary>
	int SetSwitchVLANTagEnforcing(uint32_t _port, bool _enforce);

	/// <summary>
	/// Flush Switch's VLAN Table
	/// </summary>
	int FlushVLANTable();

	/// <summary>
	/// Set Port's default VLAN Id and Force-VLan Bit
	/// </summary>
	int SetDefaultVLANId(uint32_t _port, uint32_t _vlanId, bool _forceVLAN);

	/// <summary>
	/// Set VLAN Membership Status ...
	/// </summary>
	int SetVLANMembership(uint32_t _vlanId, uint8_t _lan1Membership,
		uint8_t _lan2Membership,
		uint8_t _lan3Membership,
		uint8_t _lan4Membership,
		uint8_t _lan5Membership,
		uint8_t _lan6Membership);

	/// <summary>
	/// Set Switch Register to Value
	/// </summary>
	int ModSwitchReg(uint32_t _port, uint32_t _reg, uint16_t _setMask, uint16_t _clearMask);

	/// <summary>
	/// Set Switch Register to Value
	/// </summary>
	int SetSwitchReg(uint32_t _port, uint32_t _reg, uint32_t _val);

	/// <summary>
	/// Read Switch Register
	/// </summary>
	int GetSwitchReg(uint32_t _port, uint32_t _reg, uint32_t& _result);

	/// <summary>
	/// Schaltet das GSM-Modul ein (onoff=true) bzw. ab (onoff=false)
	/// </summary>
	bool PowerGSM(bool onoff);

	/// <summary>
	/// Powers Switch
	/// </summary>
	bool PowerSwitch(bool onoff);

	/// <summary>
	/// Setting of GSM-Emergency Pin
	/// </summary>
	bool GSMEmergency(bool onoff);

	/// <summary>
	/// Get Mobile Power Indicator Line
	/// </summary>
	bool GetPowerInd(void);

	/// <summary>
	/// Get Mobile Ring Indicator Line
	/// </summary>
	bool GetGSMRing(void);

	/// <summary>
	/// Get System Input Port
	/// </summary>
	bool GetInputPort(void);

	/// <summary>
	/// Get Service Input Port
	/// </summary>
	bool GetServicePort(void);

	/// <summary>
	/// Set System Output Port
	/// </summary>
	bool SetOutputPort(bool onoff);

	/// <summary>
	/// Select Mobile SIM Card Slot (1 or 2)
	/// </summary>
	bool SelSIM(bool bSlot);

	/// <summary>
	/// Set GSM Module Ignition Line
	/// </summary>
	bool GSMIgnition(bool bIgnition);

	/// <summary>
	/// Schaltet das WMBUS-Modul ein (onoff=true) bzw. ab (onoff=false)
	/// </summary>
	bool PowerWMBUS(bool onoff);

	/// <summary>
	/// Schaltet die +12V am Meter-Bus ein (onoff=true) bzw. ab (onoff=false)
	/// </summary>
	bool PowerMBUS12V(bool onoff);

	/// <summary>
	/// Schaltet den spezifizierten Port des Switches (1 oder 2) an (onoff=true) oder aus (onoff=false)
	/// </summary>
	bool PowerSwitchPort(unsigned int portnr, bool onoff);

	/// <summary>
	/// Alle eingehenden Datenpakete des Devices _ethname werden gedroppt (silently)
	/// equivalent zu: iptables -I INPUT 1 -i %_ethname% -j DROP
	/// </summary>	
	int IPTablesBlockInputDevice(const std::string& _ethname);

	/// <summary>
	/// akzeptiert Pakete fr die Zieladdresse _ipv4addr mit Zielport _port
	/// equivalent zu: iptables -I INPUT 1 -d %_ipv4addr% -p tcp --dport %_port% -j ACCEPT
	/// </summary>	
	//int IPTablesOpenPort(const std::string& _ipv4addr, unsigned short _port);

	/// <summary>
	/// akzeptiert Pakete fr Ziel-eth _device mit Zielport _port fr _protokoll=udp|tcp
	/// equivalent zu: iptables -I INPUT 1 -i %_device% -p %_protokoll% --dport %_port% -j ACCEPT
	/// </summary>	
	int IPTablesOpenPort(const std::string& _device, const std::string& _port, const std::string& _protokoll);

	/// <summary>
	/// lscht die Regel zum ffnen eines Ports aus der Kette INPUT
	/// equivalent zu: iptables -D INPUT -d %_ipv4addr% -p tcp --dport %_port% -j ACCEPT
	/// </summary>	
	//int IPTablesClosePort(const std::string& _ipv4addr, unsigned short _port);
	int IPTablesClosePort(const std::string& _device, const std::string& _port, const std::string& _protokoll);

	/// <summary>
	/// lscht die Regel zum ffnen eines Ports aus der Kette INPUT
	/// equivalent zu: iptables -D INPUT -i %_device% -p %_protokoll% --dport %_port% -j ACCEPT
	/// </summary>

	/// <summary>
	/// Lscht alle Regeln der Kette INPUT
	/// equivalent zu: iptables -F INPUT
	/// </summary>	
	int IPTablesFlush();


	/// <summary>
	/// Set EEPROM Boot Bit Informations
	/// </summary>	
	bool EepromBootConfSetBit(uint8_t bitnr);

	/// <summary>
	/// Clear EEPROM Boot Bit Informations
	/// </summary>	
	bool EepromBootConfClrBit(uint8_t bitnr);

	/// <summary>
	/// Read from EEPROM the Boot Bit Information
	/// </summary>	
	bool EepromReadBootConf(uint8_t& byte_bootregister);

	/// <summary>
	/// Write to EEPROM Boot Bit Informations
	/// </summary>	
	bool EepromWriteBootConf(const uint8_t byte_bootregister);

	/// <summary>
	/// GetBootConf
	/// </summary>	
	bool GetBootConf(uint8_t& _bootByteEeprom, uint8_t& _bootByteCPU);

	/// <summary>
	/// _readBootConfFromCPU
	/// </summary>	
	bool _readBootConfFromCPU(uint8_t& boot_byte);

	/// <summary>
	/// _setBootConfBitToCPU
	/// </summary>	
	bool _setBootConfBitToCPU(int32_t bitnr, bool _value);

	/// <summary>
	/// WriteBootConfToCPU
	/// </summary>	
	bool WriteBootConfToCPU(uint8_t& boot_byte);

	/// <summary>
	/// Lscht das gesamte EEPROM
	/// </summary>	
	bool EepromErase();

	/// <summary>
	/// Tainy V4 EEPROM Variables
	///		[ver]			Version of the eeprom data file stream
	///		[hwid]			Hardware - ID(Identification)
	///		[hwver]			Hardware Version
	///		[osc]			Operation Second Counter
	///		[bc]			Boot Counter, Counts every boot
	///		[mac0]			MAC Address 0
	///		[mac1]			MAC Address 1
	///		[mac2]			MAC Address 2
	///		[mac3]			MAC Address 3
	///		[mac4]			MAC Address 4
	///		[mac5]			MAC Address 5
	///		[sn]			The device serial Number
	///		[devid]			Device ID
	///		[pdate]			Production Date
	///		[oem]			The OEM Name i.e.Dr.Neuhaus
	///		[sig]			Signature
	/// Data size name and properties:
	///		[ver]			3 - Bytes
	///		[hwid]			6 - Bytes
	///		[hwver]			5 - Bytes
	///		[osc]			10 - Bytes
	///		[bc]			10 - Bytes
	///		[mac0]			17 - Bytes
	///		[mac1]			17 - Bytes
	///		[mac2]			17 - Bytes
	///		[mac3]			17 - Bytes
	///		[mac4]			17 - Bytes
	///		[mac5]			17 - Bytes
	///		[sn]			16 - Bytes
	///		[devid]			18 - Bytes
	///		[pdate]			8 - Bytes
	///		[oem]			48 - Bytes
	///		[sig]			32 - Bytes
	/// </summary>	
	bool EepromRead(std::map<std::string, std::string>& data);
	/// <summary>
	/// Create the eeprom Signature
	/// </summary>	
	bool EepromCreateSignature(void);
	/// <summary>
	/// Verifys the eeprom Signature
	/// </summary>	
	bool EepromVerifySignature(void);
	/// <summary>
	/// Speichert Daten im EEPROM entsprechend den keys und values von data 	
	/// </summary>	
	bool EepromWrite(std::map<std::string, std::string>& data);

	/// <summary>
	/// Create a file and write _content (text) to it
	/// </summary>
	int WriteToFile(const std::string _fileName, const std::string _content, bool _append=false);

	/// <summary>
	/// Read a TextFile into a String
	/// </summary>
	int ReadFromFile(const std::string _fileName, std::string& _content, uint32_t _maxLen=10000);

	/// <summary>
	/// Dient der Abfrage der OS-Version, Kernel-Release, installierte Updates und opkg-Pakete
	/// mode="single":
	///		Abfrage einzelner oder mehrerer Versionsnummern
	///     first="Os"		-> OS-Version, z.Bsp second="DNT8209-09"
	///     first="Kernel"  -> Kernel-Version und Build-Nr., z.Bsp second="Linux version 3.9.11 #16"
	///     first="%Paketname%"	-> Version eines Userland-Paketes
	///     first="smgw"		-> Version der Applikation, z.Bsp second="DNT8209-SMGW-0.1-546"
	/// mode="allpackages":
	///		Beschreibt die map mit den Paketnamen und Versionsnummern aller auf dem Target installierten Paketen
	/// mode="patches":
	///		Beschreibt die map mit den installierten Patchen
	/// </summary>	
	bool QuerySystemVersion(const std::string& _mode, std::map<std::string, std::string>& data, std::string& _error);

	/// <summary>
	/// Spezielle Updatemassnahmen jenseits des ordinren Updateverfahrens	
	/// </summary>	
	bool SystemUpdateRequest(const std::string& _pathfilename);

	/// <summary>
	/// Gibt den freien Speicher in Bytes fr das durch den Pfad und/oder Dateinamen definierte Filesystem an
	/// </summary>	
	long long int GetFreeDiskSpace(const std::string& _pathfilename);

	/// <summary>
	/// Gibt die Gre des Filesystems in Bytes an
	/// </summary>	
	long long int GetDiskSize(const std::string& _pathfilename);

	/// <summary>
	/// Gre des belegten Speichers eines Speicherdevices
	/// </summary>	
	long long int GetUsedDiskSpace(const std::string& _pathfilename);

	/// <summary>
	/// Setzen des DTR-Signals, versetzt das GSM-Modul zurck in den Command-Mode und beendet pppd
	/// </summary>	
	bool SetDTR();
	bool ResetDTR();
	
	/// <summary>
	/// Abfrage des Tamper-Kontaktes
	/// Return-Werte:
	///		0x01: Tamper OK
	///		0x02: Tamper released
	///		0x03: Tamper OK, but released in the past
	///		other: ERROR
	/// </summary>	
	int ReadTamperStatus();

	/// <summary>
	/// z.Zt. ohne Funktion
	/// </summary>	
	int TamperReboot();

	/// <summary>
	/// Gibt die PID des pppd bei aktivier Schnittstelle ppp0 zurck, im Fehlerfall: 0
	/// </summary>	
	unsigned int getPppdPid();

	/// <summary>
	/// Lst das RESET fr das WMBUS-Modul aus
	/// </summary>	
	bool ResetWMBUS();

	/// <summary>
	/// Lst das RESET fr das GSM-Modul aus
	/// </summary>	
	bool ResetGSM();

	/// <summary>
	/// Gibt den Zustand von DSR des GSM-Moduls zurck (HIGH = Command Mode, LOW = DATA-Mode)
	/// </summary>	
	bool GetDSRPegel();

	/// <summary>
	/// Gibt den Zustand von DCD des GSM-Moduls zurck 
	/// </summary>	
	bool GetDCDPegel();

	/// <summary>
	/// returns the value of the switch interrupt line
	/// </summary>	
	bool GetSwitchInterrupt();

	/// <summary>
	/// Gibt true zurck, sofern das angegebene Device fr die Platform existiert
	/// </summary>	
	bool EthExists(std::string &_networkDevice);
		
	/// <summary>
	/// Gibt die Spannung der Kondensatoren in mV zurck
	/// </summary>	
	unsigned int GetBattVoltage_mv();

	/// <summary>
	/// Gibt die Spannung der Kondensatoren in Prozent (100% ca. 4,22V) zurck
	/// </summary>	
	unsigned int GetBattVoltage_percent();

	/// <summary>
	/// Gibt den Status der Synchronisation via NTP zurck
	/// Kriterien:	- Leap indicator != 3
	///				- Ein NTP Server ist zur Zeitsynchronisation erreichbar (reach)
	///				- Der offset (Zeitdifferenz ntp<->local) liegt im limit (default = 2000 ms)
	/// </summary>		
	bool IsNTPsynchronized(uint16_t _port = 123, uint32_t _maxOffset_ms = 2000);	

	/// <summary>
	/// Add a GRE Tunnel
	/// </summary>	
	int AddGRETunnel(const std::string& _ethname, const std::string& _key, const std::string& _ttl, const std::string& _localAddr, const std::string& _remoteAddr);

	/// <summary>
	/// Remove a GRE Tunnel
	/// </summary>	
	int RemoveGRETunnel(const std::string& _ethname);

	/// <summary>
	/// Flush NHRP Config
	/// </summary>	
	int FLushNHRPConfig();

	/// <summary>
	/// Add NHRP Network
	/// </summary>	
	int AddNHRPInterface(
		const std::string& _ethname, 
		const std::string& _network, 
		const std::string& _nhs, 
		const std::string& _holdingtime, 
		bool _multicast,
		bool _hubmode,
		bool _ciscoauth,
		const std::string& _ciscokey
		);

	/// <summary>
	/// Flush IPSec Policy
	/// </summary>	
	int FlushIPSecPolicy();

	/// <summary>
	/// Prepare IPSec Policy
	/// </summary>	
	int PrepareIPSecPolicy(
		const std::string& _sourcenet, 
		const std::string& _destnet, 
		const std::string& _prot, 
		const std::string& _direction,
		const std::string& _mode,
		const std::string& _endpoints,
		const std::string& _policy
		);

	/// <summary>
	/// Flush IPSec Policy
	/// </summary>	
	int SetIPSecPolicy();

	/// <summary>
	/// Initialize the racoon.conf
	/// </summary>	
	int InitIPSecConfig(
		unsigned int _retrycounter,
		unsigned int _resendinterval,
		unsigned int _ph1timeout,
		unsigned int _ph2timeout,
		unsigned int _nattkeepalive
		);

	/// <summary>
	/// Add Anonymous IPSec Gateway with Pre Shared Keys
	/// </summary>	
	int AddAnonymousIPSecGateway(
		const std::string& _mode,
		unsigned int _lifetime,
		const std::string& _identifier,
		const std::string& _natt,
		const std::string& _encryption,
		const std::string& _hash,
		unsigned int _dhgroup,
		bool _dpd,
		unsigned int _dpddelay,
		unsigned int _dpdretry,
		unsigned int _dpdmaxfail
		);

	/// <summary>
	/// Add Anonymous IPSec SA Info
	/// </summary>	
	int AddAnonymousIPSecSAInfo(
		unsigned int _pfsgroup,
		unsigned int _lifetime,
		const std::string& _encryption,
		const std::string& _hash
		);

	/// <summary>
	/// Establish IPSec SA
	/// </summary>	
	int EstablishIPSecSA(
		const std::string& _satype,
		const std::string& _sourceaddr,
		const std::string& _destaddr,
		const std::string& _prot
		);

	/// <summary>
	/// Delete IPSec SA
	/// </summary>	
	int DeleteIPSecSA(
		const std::string& _satype,
		const std::string& _sourceaddr,
		const std::string& _destaddr
		);

	/// <summary>
	/// Flush IPSec PSK File
	/// </summary>	
	int FlushIPSecPSKs();

	/// <summary>
	/// Add Pre Shared Key for IPSec
	/// </summary>	
	int AddIPSecPSK(const std::string& _identity, const std::string& _psk);


	/// <summary>
	/// Enable IP Forwarding
	/// </summary>	
	int EnableForwarding(bool _enabled = true);


	// ----------------------------------------- EEPROM Definitionen --------------------------------------
	//#define EEPROM_WORK_PATH	"/tmp/"
	#define EEPROM_DEBUG_OUT	1
	#define EEPROM_WORK_PATH	"/etc/key/"

	#define FCS16_LENGTH	2
	#define LENGTH_BYTE		1

	#define VERSION_ADDR	0
	#define VERSION_LENGTH	3	//	0..999

	#define HWID_ADDR		LENGTH_BYTE + VERSION_ADDR + VERSION_LENGTH + FCS16_LENGTH
	#define HWID_LENGTH		6	// 00-000..99-999

	#define HWVER_ADDR		LENGTH_BYTE + HWID_ADDR + HWID_LENGTH + FCS16_LENGTH
	#define HWVER_LENGTH	5	// 0..99999

	#define OSC_ADDR	LENGTH_BYTE + HWVER_ADDR + HWVER_LENGTH + FCS16_LENGTH
	#define OSC_LENGTH	10		// 0..2^32

	#define BC_ADDR	LENGTH_BYTE + OSC_ADDR + OSC_LENGTH + FCS16_LENGTH
	#define BC_LENGTH	10		// 0..2^32

	#define MAC_LENGTH	17		// typ. 01:02:03:04:05:06
	#define MAC0_ADDR	LENGTH_BYTE + BC_ADDR + BC_LENGTH + FCS16_LENGTH
	#define MAC1_ADDR	LENGTH_BYTE + MAC0_ADDR + MAC_LENGTH + FCS16_LENGTH
	#define MAC2_ADDR	LENGTH_BYTE + MAC1_ADDR + MAC_LENGTH + FCS16_LENGTH
	#define MAC3_ADDR	LENGTH_BYTE + MAC2_ADDR + MAC_LENGTH + FCS16_LENGTH
	#define MAC4_ADDR	LENGTH_BYTE + MAC3_ADDR + MAC_LENGTH + FCS16_LENGTH
	#define MAC5_ADDR	LENGTH_BYTE + MAC4_ADDR + MAC_LENGTH + FCS16_LENGTH

	#define SN_ADDR		LENGTH_BYTE + MAC5_ADDR + MAC_LENGTH + FCS16_LENGTH
	#define SN_LENGTH	16		// typ. "0123456789ABCDEF"

	#define DEVID_ADDR	LENGTH_BYTE + SN_ADDR + SN_LENGTH + FCS16_LENGTH
	#define DEVID_LENGTH 18		// typ. "0E444E540101020304"

	#define PDATE_ADDR	LENGTH_BYTE + DEVID_ADDR + DEVID_LENGTH + FCS16_LENGTH
	#define PDATE_LENGTH 8		// typ. 20121231

	#define OEM_ADDR	LENGTH_BYTE + PDATE_ADDR + PDATE_LENGTH + FCS16_LENGTH
	#define OEM_LENGTH	20		// typ. "Dr.Neuhaus__________"

	#define BOOTCONF_ADDR LENGTH_BYTE + OEM_ADDR + OEM_LENGTH + FCS16_LENGTH
	#define BOOTCONF_LENGTH		1	// persistierte Bootkonfiguration binr

	#define MOUNTERRORINIT_ADDR LENGTH_BYTE + BOOTCONF_ADDR + BOOTCONF_LENGTH + FCS16_LENGTH 
	#define MOUNTERRORINIT_LENGTH	6		// Fehlercounter: Fehler beim Mounten einer Partition innerhalb der initramfs

	#define MOUNTERRORRUN_ADDR LENGTH_BYTE + MOUNTERRORINIT_ADDR + MOUNTERRORINIT_LENGTH + FCS16_LENGTH// Fehlercounter: Mountfehler einer Partition im laufenden Betrieb
	#define MOUNTERRORRUN_LENGTH	6		// Wert: 0..999999

	#define SIG_ADDR	LENGTH_BYTE + MOUNTERRORRUN_ADDR + MOUNTERRORRUN_LENGTH + FCS16_LENGTH
	#define SIG_LENGTH	256	// typ. "0000000000000000"
	// ----------------------------------------- TAMPER Definitionen --------------------------------------

	#define UTIME	1000
	#define LOW		false
	#define HIGH	true
	#define TAMPER_CMD_READ         "00100000"		
	#define TAMPER_READ_ACK_OK		"00001111"	// Tamper OK, switch ist gedrckt und wurde noch nie zuvor released
	#define TAMPER_READ_ACK_SWITCH	"10001111"  // switch ist released -> Gehuse ist geffnet -> Alarm
	#define TAMPER_READ_ACK_EEPROM	"01001111"	// switch ist gedrckt, wurde aber in der Vergangenheit released -> stiller Alarm
	#define TAMPER_CMD_REBOOT_REQ	"00110100"	// Neustart der Tamper Firmware
	#define TAMPER_CMD_REBOOT_ACK	"00001100"	// 0xC0


}	// namespace hal


#endif	//__HAL__
