#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdint.h>

#include <string>
#include <map>
#include <sys/sysinfo.h>
#include <errno.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>

#include <sys/time.h>
#include <error.h>

#include <unistd.h>

// ----------------------------------------- EEPROM Definitionen --------------------------------------

#define FCS16_LENGTH	2
#define LENGTH_BYTE		1

#define VERSION_ADDR	0
#define VERSION_LENGTH	3	//	0..999

#define HWID_ADDR		LENGTH_BYTE + VERSION_ADDR + VERSION_LENGTH + FCS16_LENGTH
#define HWID_LENGTH		6	// 00-000..99-999

#define SWID_ADDR	LENGTH_BYTE + HWID_ADDR + HWID_LENGTH + FCS16_LENGTH
#define SWID_LENGTH		6	// 00-000..99-999

#define HWVER_ADDR	LENGTH_BYTE + SWID_ADDR + SWID_LENGTH + FCS16_LENGTH
#define HWVER_LENGTH	5	// 0..99999

#define OSC_ADDR	LENGTH_BYTE + HWVER_ADDR + HWVER_LENGTH + FCS16_LENGTH
#define OSC_LENGTH	10		// 0..2^32

#define BC_ADDR	LENGTH_BYTE + OSC_ADDR + OSC_LENGTH + FCS16_LENGTH
#define BC_LENGTH	10		// 0..2^32

#define MAC0_ADDR	LENGTH_BYTE + BC_ADDR + BC_LENGTH + FCS16_LENGTH
#define MAC_LENGTH	17		// typ. 01:02:03:04:05:06

#define MAC1_ADDR	LENGTH_BYTE + MAC0_ADDR + MAC_LENGTH + FCS16_LENGTH

#define MAC2_ADDR	LENGTH_BYTE + MAC1_ADDR + MAC_LENGTH + FCS16_LENGTH

#define SN_ADDR		LENGTH_BYTE + MAC2_ADDR + MAC_LENGTH + FCS16_LENGTH
#define SN_LENGTH	16		// typ. "0123456789ABCDEF"

#define DEVID_ADDR	LENGTH_BYTE + SN_ADDR + SN_LENGTH + FCS16_LENGTH
#define DEVID_LENGTH 18		// typ. "0E444E540101020304"

#define PDATE_ADDR	LENGTH_BYTE + DEVID_ADDR + DEVID_LENGTH + FCS16_LENGTH
#define PDATE_LENGTH 8		// typ. 20121231

#define KEY_ADDR	LENGTH_BYTE + PDATE_ADDR + PDATE_LENGTH + FCS16_LENGTH
#define KEY_LENGTH	12		// typ. "000000000000"

#define OEM_ADDR	LENGTH_BYTE + KEY_ADDR + KEY_LENGTH + FCS16_LENGTH
#define OEM_LENGTH	48		// typ. "Dr. Neuhaus"

#define WMBUS_ADDR	LENGTH_BYTE + OEM_ADDR + OEM_LENGTH + FCS16_LENGTH
#define WMBUS_LENGTH	16	// typ. "0000000000000000"

#define MOUNTERRORINIT_ADDR LENGTH_BYTE + WMBUS_ADDR + WMBUS_LENGTH + FCS16_LENGTH 
#define MOUNTERRORINIT_LENGTH	6		// Fehlercounter: Fehler beim Mounten einer Partition innerhalb der initramfs

#define MOUNTERRORRUN_ADDR LENGTH_BYTE + MOUNTERRORINIT_ADDR + MOUNTERRORINIT_LENGTH + FCS16_LENGTH// Fehlercounter: Mountfehler einer Partition im laufenden Betrieb
#define MOUNTERRORRUN_LENGTH	6		// Wert: 0..999999

#define BOOTCONF_ADDR LENGTH_BYTE + MOUNTERRORRUN_ADDR + MOUNTERRORRUN_LENGTH + FCS16_LENGTH
#define BOOTCONF_LENGTH		1	// persistierte Bootkonfiguration binaer

#define ROMSIG_ADDR BOOTCONF_ADDR + BOOTCONF_LENGTH + FCS16_LENGTH + 1 // Ein Byte Abstand zur Bootconf zwecks Erweiterung
#define ROMSIG_LENGTH 256

// Hashwert ganz nach hinten
#define PWHASH_ADDR LENGTH_BYTE + ROMSIG_LENGTH + FCS16_LENGTH#define PWHASH_LENGTH 32 // ?
#define PWHASH_LENGTH	32	// ?-------------------------------------------------------------------------------------------

#define HIGH	true
#define LOW		false

using namespace std;

static const unsigned short _fcs16tab[256] = {
	0x0000, 0x1189, 0x2312, 0x329b, 0x4624, 0x57ad, 0x6536, 0x74bf,
	0x8c48, 0x9dc1, 0xaf5a, 0xbed3, 0xca6c, 0xdbe5, 0xe97e, 0xf8f7,
	0x1081, 0x0108, 0x3393, 0x221a, 0x56a5, 0x472c, 0x75b7, 0x643e,
	0x9cc9, 0x8d40, 0xbfdb, 0xae52, 0xdaed, 0xcb64, 0xf9ff, 0xe876,
	0x2102, 0x308b, 0x0210, 0x1399, 0x6726, 0x76af, 0x4434, 0x55bd,
	0xad4a, 0xbcc3, 0x8e58, 0x9fd1, 0xeb6e, 0xfae7, 0xc87c, 0xd9f5,
	0x3183, 0x200a, 0x1291, 0x0318, 0x77a7, 0x662e, 0x54b5, 0x453c,
	0xbdcb, 0xac42, 0x9ed9, 0x8f50, 0xfbef, 0xea66, 0xd8fd, 0xc974,
	0x4204, 0x538d, 0x6116, 0x709f, 0x0420, 0x15a9, 0x2732, 0x36bb,
	0xce4c, 0xdfc5, 0xed5e, 0xfcd7, 0x8868, 0x99e1, 0xab7a, 0xbaf3,
	0x5285, 0x430c, 0x7197, 0x601e, 0x14a1, 0x0528, 0x37b3, 0x263a,
	0xdecd, 0xcf44, 0xfddf, 0xec56, 0x98e9, 0x8960, 0xbbfb, 0xaa72,
	0x6306, 0x728f, 0x4014, 0x519d, 0x2522, 0x34ab, 0x0630, 0x17b9,
	0xef4e, 0xfec7, 0xcc5c, 0xddd5, 0xa96a, 0xb8e3, 0x8a78, 0x9bf1,
	0x7387, 0x620e, 0x5095, 0x411c, 0x35a3, 0x242a, 0x16b1, 0x0738,
	0xffcf, 0xee46, 0xdcdd, 0xcd54, 0xb9eb, 0xa862, 0x9af9, 0x8b70,
	0x8408, 0x9581, 0xa71a, 0xb693, 0xc22c, 0xd3a5, 0xe13e, 0xf0b7,
	0x0840, 0x19c9, 0x2b52, 0x3adb, 0x4e64, 0x5fed, 0x6d76, 0x7cff,
	0x9489, 0x8500, 0xb79b, 0xa612, 0xd2ad, 0xc324, 0xf1bf, 0xe036,
	0x18c1, 0x0948, 0x3bd3, 0x2a5a, 0x5ee5, 0x4f6c, 0x7df7, 0x6c7e,
	0xa50a, 0xb483, 0x8618, 0x9791, 0xe32e, 0xf2a7, 0xc03c, 0xd1b5,
	0x2942, 0x38cb, 0x0a50, 0x1bd9, 0x6f66, 0x7eef, 0x4c74, 0x5dfd,
	0xb58b, 0xa402, 0x9699, 0x8710, 0xf3af, 0xe226, 0xd0bd, 0xc134,
	0x39c3, 0x284a, 0x1ad1, 0x0b58, 0x7fe7, 0x6e6e, 0x5cf5, 0x4d7c,
	0xc60c, 0xd785, 0xe51e, 0xf497, 0x8028, 0x91a1, 0xa33a, 0xb2b3,
	0x4a44, 0x5bcd, 0x6956, 0x78df, 0x0c60, 0x1de9, 0x2f72, 0x3efb,
	0xd68d, 0xc704, 0xf59f, 0xe416, 0x90a9, 0x8120, 0xb3bb, 0xa232,
	0x5ac5, 0x4b4c, 0x79d7, 0x685e, 0x1ce1, 0x0d68, 0x3ff3, 0x2e7a,
	0xe70e, 0xf687, 0xc41c, 0xd595, 0xa12a, 0xb0a3, 0x8238, 0x93b1,
	0x6b46, 0x7acf, 0x4854, 0x59dd, 0x2d62, 0x3ceb, 0x0e70, 0x1ff9,
	0xf78f, 0xe606, 0xd49d, 0xc514, 0xb1ab, 0xa022, 0x92b9, 0x8330,
	0x7bc7, 0x6a4e, 0x58d5, 0x495c, 0x3de3, 0x2c6a, 0x1ef1, 0x0f78
};

unsigned short _createCRC16(const char* data, register int len, unsigned short fcs)
{			
	while(len--) {		
		fcs = (fcs >> 8) ^ _fcs16tab[(fcs ^ *data++) & 0x0ff];
	}
	return fcs ^ 0x0ffff;
}

int _get_eeprom_size()
{		
	int eepromSize = 1;

	char eepromName[5];
	memset(eepromName,0,5);
	FILE* fp=fopen("/sys/bus/i2c/devices/0-0050/name","r");
	if(fp != NULL)
	{			
		fscanf(fp,"%s",eepromName);
		fclose(fp);			
		eepromSize = (strcmp(eepromName,"24c32") == 0) ? 4096 : -1;
	}

	return eepromSize;
}

bool _writeEepromEntry( const unsigned int _addr, const unsigned int _valuelength, const std::string& _value )
{
	bool ret = false;

	const unsigned int blocksize = _valuelength + LENGTH_BYTE + FCS16_LENGTH;
	int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDWR);

	if ( fd < 0 )
	{		
		return false;
	}
	else
	{
		char wbuf[blocksize];
		char rbuf[blocksize];
		::memset(wbuf,0,blocksize);
		::memset(rbuf,0,blocksize);

		if (_value.length() > _valuelength)
		{			
			close(fd);
			return false;
		}

		wbuf[0] = _value.length();			// Lngenbyte setzen
		memcpy(wbuf+1, _value.c_str(),_valuelength);		// _value in den Schreibpuffer ablegen		

		unsigned short fcs16_calc = _createCRC16(wbuf, blocksize, 0xffff);	// Berechnung der CRC16 ber den GESAMTEN Schreibpuffer	

		wbuf[blocksize-2] = (fcs16_calc >> 8) & 0x00FF;		// Eintragen der CRC16 in den Schreibpuffer
		wbuf[blocksize-1] = (fcs16_calc) & 0x00FF;

		// 3x retry Buffer schreiben und anschliessend verifizieren		
		for (int i=0; i<3; i++)
		{
			::lseek( fd, _addr, SEEK_SET);				// Deskriptor auf Position _addr setzen			

			// Schreiben des gesamten Buffers schlgt in manchen Fllen fehl, deshalb byteweise schreiben			
			for (unsigned int j = 0 ; j < blocksize ; j++) ::write(fd, wbuf+j, 1);						
			//::write( fd, wbuf, blocksize);				// Block schreiben
			::lseek( fd, _addr, SEEK_SET);				// Deskriptor zurcksetzen
			::read( fd, rbuf, blocksize);				// Block lesen			
			if (memcmp(wbuf,rbuf,blocksize) == 0)
			{
				ret = true;
				break;
			}			

		}		
		close (fd);
	}

	return ret;
}

bool _readEepromEntry( const unsigned int _addr, const unsigned int _valuelength, std::string& _value )
{
	bool ret = false;

	const unsigned int blocksize = _valuelength + LENGTH_BYTE + FCS16_LENGTH;

	int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDONLY);

	if ( fd < 0 )
	{			
		return false;
	}
	else
	{
		char rbuf[blocksize];				
		::memset(rbuf,0,blocksize);

		for (int i=0; i < 3; i++)
		{				
			::lseek(fd, _addr, SEEK_SET);
			::read(fd, rbuf,blocksize);

			unsigned short fcs16_readed = (rbuf[blocksize - 2] << 8) | rbuf[blocksize - 1];	
			// CRC (die letzten beiden bytes) aus dem readbuffer lschen
			rbuf[blocksize - 2] = 0;
			rbuf[blocksize - 1] = 0;
			unsigned short fcs16_calc = _createCRC16(rbuf, blocksize, 0xffff);
			//unsigned short fcs16_calc = _createCRC16(rbuf, blocksize, 0xffff);
			//printf("fcs16_readed: %x	fcs16_calc: %x \n",fcs16_readed, fcs16_calc);		

			if(fcs16_calc == fcs16_readed)
			{
				_value.assign(rbuf+1,_valuelength);
				ret=true;
				break;
			}
			else 
			{
				_value="n/a";
			}
		}

		close(fd);
	}

	return ret;
}

bool EepromErase()
{
	bool ret = false;
	int eepromSize =  _get_eeprom_size();
	if (eepromSize < 0)
	{			
		return false;
	}

	int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_WRONLY);

	if ( fd < 0 )
	{			
		return false;
	}
	else
	{
		char wbuf[eepromSize];
		memset(wbuf,0,eepromSize);

		for (int i = 0; i < 3 && !ret; i++)
		{		
			int wlen=::write(fd,wbuf,eepromSize);
			ret = wlen == eepromSize;
		}

		close(fd);
	}

	return ret;
}

bool EepromRead( std::map<std::string, std::string>& data )
{
	bool ret	= true;	// Globaler RC
	bool rc		= true; // RC einer Einzelabfrage

	std::map<std::string, std::string>::iterator it;
	std::string eepromEntry = "";

	for(std::map<std::string, std::string>::iterator it=data.begin(); it!=data.end(); it++)
	{				
		eepromEntry = "";
		if (it->first == "ver")					rc = _readEepromEntry(VERSION_ADDR, VERSION_LENGTH, eepromEntry);
		else if (it->first == "hwid")			rc = _readEepromEntry(HWID_ADDR, HWID_LENGTH, eepromEntry);
		else if (it->first == "bc")				rc = _readEepromEntry(BC_ADDR, BC_LENGTH, eepromEntry);
		else if (it->first == "devid")			rc = _readEepromEntry(DEVID_ADDR, DEVID_LENGTH, eepromEntry);
		else if (it->first == "hwver")			rc = _readEepromEntry(HWVER_ADDR, HWVER_LENGTH, eepromEntry);
		else if (it->first == "key")			rc = _readEepromEntry(KEY_ADDR, KEY_LENGTH, eepromEntry);
		else if (it->first == "mac0")			rc = _readEepromEntry(MAC0_ADDR, MAC_LENGTH, eepromEntry);
		else if (it->first == "mac1")			rc = _readEepromEntry(MAC1_ADDR, MAC_LENGTH, eepromEntry);
		else if (it->first == "mac2")			rc = _readEepromEntry(MAC2_ADDR, MAC_LENGTH, eepromEntry);
		else if (it->first == "mnterrinit")		rc = _readEepromEntry(MOUNTERRORINIT_ADDR, MOUNTERRORINIT_LENGTH, eepromEntry);
		else if (it->first == "mnterrrun")		rc = _readEepromEntry(MOUNTERRORRUN_ADDR, MOUNTERRORRUN_LENGTH, eepromEntry);
		else if (it->first == "oem")			rc = _readEepromEntry(OEM_ADDR, OEM_LENGTH, eepromEntry);
		else if (it->first == "osc")			rc = _readEepromEntry(OSC_ADDR, OSC_LENGTH, eepromEntry);
		else if (it->first == "pdate")			rc = _readEepromEntry(PDATE_ADDR, PDATE_LENGTH, eepromEntry);
		else if (it->first == "sn")				rc = _readEepromEntry(SN_ADDR, SN_LENGTH, eepromEntry);
		else if (it->first == "swid")			rc = _readEepromEntry(SWID_ADDR, SWID_LENGTH, eepromEntry);
		else if (it->first == "wmbus")			rc = _readEepromEntry(WMBUS_ADDR, WMBUS_LENGTH, eepromEntry);
		else if (it->first == "romsig")                  rc = _readEepromEntry(ROMSIG_ADDR, ROMSIG_LENGTH, eepromEntry);

		if (!rc) ret = false;
		if (!eepromEntry.empty()) it->second.assign(eepromEntry);			
	}

	return ret;
}

	//-----------------------------------------------------------------------------------------------------------

bool EepromWrite( std::map<std::string, std::string>& data )
{
	bool ret = true;

	std::map<std::string, std::string>::iterator it;
		
	it=data.find("ver");		
	if ( it != data.end() && !_writeEepromEntry(VERSION_ADDR, VERSION_LENGTH, it->second)) ret=false;

	it=data.find("hwid");		
	if ( it != data.end() && !_writeEepromEntry(HWID_ADDR, HWID_LENGTH, it->second)) ret=false;

	it=data.find("swid");		
	if ( it != data.end() && !_writeEepromEntry(SWID_ADDR, SWID_LENGTH, it->second)) ret=false;

	it=data.find("hwver");		
	if ( it != data.end() && !_writeEepromEntry(HWVER_ADDR, HWVER_LENGTH, it->second)) ret=false;

	it=data.find("osc");		
	if ( it != data.end() && !_writeEepromEntry(OSC_ADDR, OSC_LENGTH, it->second)) ret=false;

	it=data.find("bc");		
	if ( it != data.end() && !_writeEepromEntry(BC_ADDR, BC_LENGTH, it->second)) ret=false;

	it=data.find("mac0");		
	if ( it != data.end() && !_writeEepromEntry(MAC0_ADDR, MAC_LENGTH, it->second)) ret=false;

	it=data.find("mac1");		
	if ( it != data.end() && !_writeEepromEntry(MAC1_ADDR, MAC_LENGTH, it->second)) ret=false;

	it=data.find("mac2");		
	if ( it != data.end() && !_writeEepromEntry(MAC2_ADDR, MAC_LENGTH, it->second)) ret=false;

	it=data.find("sn");		
	if ( it != data.end() && !_writeEepromEntry(SN_ADDR, SN_LENGTH, it->second)) ret=false;

	it=data.find("devid");		
	if ( it != data.end() && !_writeEepromEntry(DEVID_ADDR, DEVID_LENGTH, it->second)) ret=false;
		
	it=data.find("pdate");		
	if ( it != data.end() && !_writeEepromEntry(PDATE_ADDR, PDATE_LENGTH, it->second)) ret=false;

	it=data.find("key");		
	if ( it != data.end() && !_writeEepromEntry(KEY_ADDR, KEY_LENGTH, it->second)) ret=false;

	it=data.find("oem");		
	if ( it != data.end() && !_writeEepromEntry(OEM_ADDR, OEM_LENGTH, it->second)) ret=false;

	it=data.find("wmbus");		
	if ( it != data.end() && !_writeEepromEntry(WMBUS_ADDR, WMBUS_LENGTH, it->second)) ret=false;

	it=data.find("mnterrinit");		
	if ( it != data.end() && !_writeEepromEntry(MOUNTERRORINIT_ADDR, MOUNTERRORINIT_LENGTH, it->second)) ret=false;

	it=data.find("mnterrrun");		
	if ( it != data.end() && !_writeEepromEntry(MOUNTERRORRUN_ADDR, MOUNTERRORRUN_LENGTH, it->second)) ret=false;

	it=data.find("romsig");
        if ( it != data.end() && !_writeEepromEntry(ROMSIG_ADDR, ROMSIG_LENGTH, it->second)) ret=false;

	return ret;
}

	//-----------------------------------------------------------------------------------------------------------

bool EepromReadBootConf(uint8_t& byte_bootregister)
{
	bool ret = false;

	const unsigned int blocksize = 3;		// ein byte Bindaten + 2 byte CRC16

	int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDONLY);

	if ( fd < 0 )
	{			
		return ret;
	}
	else
	{
		char rbuf[blocksize];				
		::memset(rbuf,0,blocksize);

		for (int i=0; i < 3; i++)
		{				
			::lseek(fd, BOOTCONF_ADDR, SEEK_SET);
			::read(fd, rbuf, blocksize);

			unsigned short fcs16_readed = (rbuf[blocksize - 2] << 8) | rbuf[blocksize - 1];					
			unsigned short fcs16_calc = _createCRC16(rbuf, 1, 0xffff);		// Berechnung der CRC16 ber das gelesene Binrbyte
			//unsigned short fcs16_calc = _createCRC16(rbuf, blocksize, 0xffff);
			//printf("fcs16_readed: %x	fcs16_calc: %x \n",fcs16_readed, fcs16_calc);		

			if(fcs16_calc == fcs16_readed)
			{
				byte_bootregister = rbuf[0];
				ret=true;
				break;
			}
		}

		close(fd);
	}

	return ret;
}

	//-----------------------------------------------------------------------------------------------------------
		
bool EepromWriteBootConf(const uint8_t byte_bootregister)
{
	bool ret = false;

	const unsigned int blocksize = 3;	// ein byte Bindaten + 2 byte CRC16
	int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDWR);

	if ( fd < 0 )
	{		
		return ret;
	}
	else
	{
		char wbuf[blocksize];
		char rbuf[blocksize];
		::memset(wbuf,0,blocksize);
		::memset(rbuf,0,blocksize);
						
		wbuf[0] = byte_bootregister;
		unsigned short fcs16_calc = _createCRC16(wbuf, 1, 0xffff);	// Berechnung der CRC16 ber das Binrbyte	

		wbuf[blocksize-2] = (fcs16_calc >> 8) & 0x00FF;		// Eintragen der CRC16 in den Schreibpuffer
		wbuf[blocksize-1] = (fcs16_calc) & 0x00FF;
			
		// 3x retry Buffer schreiben und anschliessend verifizieren		
		for (int i=0; i<3; i++)
		{
			::lseek( fd, BOOTCONF_ADDR, SEEK_SET);				// Deskriptor auf Position _addr setzen			

			// Schreiben des gesamten Buffers schlgt in manchen Fllen fehl, deshalb byteweise schreiben			
			for (unsigned int j = 0 ; j < blocksize ; j++) ::write(fd, wbuf+j, 1);				
			::lseek( fd, BOOTCONF_ADDR, SEEK_SET);				// Deskriptor zurcksetzen
			::read( fd, rbuf, blocksize);				// Block lesen			
			if (memcmp(wbuf,rbuf,blocksize) == 0)
			{
				ret = true;
				break;
			}
			/*else
			{
				printf("Writeerror !\n");
			}*/
		}		
		close (fd);
	}

	return ret;
}

int _getRawBattVoltage()
{
	int ret = 0;

	unsigned short value;
	FILE *fp=::fopen("/sys/bus/iio/devices/iio:device0/in_voltage7_raw", "r");
	if(fp != NULL)
	{
		if(fscanf(fp,"%hu",&value) != 1) ret=-1;
		else ret = value > 1160 ? 1160 : value;	// Begrenzung

		fclose(fp);
	}
	else
	{
		ret=-1;
	}

	return ret;
}

unsigned int GetBattVoltage_mv()
{
	unsigned int ret = 1;
	int rawBattVoltage = _getRawBattVoltage();

	if (rawBattVoltage > 0)
	{
		ret = ( rawBattVoltage * 4240 ) / 1160;
	}

	return ret;
}
		
unsigned int GetBattVoltage_percent()
{
	unsigned int ret = 0;

	int rawBattVoltage = _getRawBattVoltage();

	if (rawBattVoltage >= 0)
	{
		ret = ( rawBattVoltage * 100 / 1160 );
	}

	return ret;
}

bool _readBootConfBitFromCPU(int32_t bitnr)
{
	int32_t fd;
	char buf[1];

	switch (bitnr)
	{
	case 0:	fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_0", O_RDONLY);
		break;
	case 1: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_1", O_RDONLY);
		break;
	case 2: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_2", O_RDONLY);
		break;
	case 3: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_3", O_RDONLY);
		break;
	case 4: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_4", O_RDONLY);
		break;
	case 5: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_5", O_RDONLY);
		break;
	case 6: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_6", O_RDONLY);
		break;
	case 7: fd = ::open("/sys/devices/platform/imx28_regs.0/rom_secondary_boot", O_RDONLY);
		break;
	default: return false;
		break;
	}

	::read(fd, buf, 1);
	::close(fd);

	return (buf[0] == '1' ?  true : false);
}

bool _setBootConfBitToCPU(int32_t bitnr, bool _value)
{
	int32_t fd;

	switch (bitnr)
	{
	case 0:	fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_0", O_RDWR);
		break;
	case 1: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_1", O_RDWR);
		break;
	case 2: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_2", O_RDWR);
		break;
	case 3: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_3", O_RDWR);
		break;
	case 4: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_4", O_RDWR);
		break;
	case 5: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_5", O_RDWR);
		break;
	case 6: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_6", O_RDWR);
		break;
	case 7: fd = ::open("/sys/devices/platform/imx28_regs.0/rom_secondary_boot", O_RDWR);
		break;
	default: return false;
		break;
	}

	if ( fd <=0 ) return false;

	if (_value)
	{
		::write(fd, "1", 1);
	}
	else
	{
		::write(fd, "0", 1);
	}
	::close(fd);		

	return true;
}

bool _readBootConfFromCPU(uint8_t& boot_byte)
{
	int32_t i;
	// Zugriff auf die CPU-Register mglich ? 
	int32_t fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_0", O_RDWR);
	if (fd <= 0) return false;		
	close(fd);

	boot_byte = 0;

	for (i = 0; i < 8; i++ )
	{
		boot_byte |= (_readBootConfBitFromCPU(i) ? (0x01 << i) : 0x00);
	}

	return true;		
}

bool WriteBootConfToCPU(uint8_t& boot_byte)
{
	int32_t i;
	// Zugriff auf die CPU-Register mglich ? 
	int32_t fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_0", O_RDWR);
	if (fd <= 0) return false;		
	close(fd);

	for (i = 0; i < 8; i++ )
	{			
		if ((boot_byte & (0x01 << i)) > 0)
		{				
			_setBootConfBitToCPU(i, HIGH);
		}
		else
		{
			_setBootConfBitToCPU(i, LOW);
		}			
	}

	return true;	
}

bool GetBootConf(uint8_t& _bootByteEeprom, uint8_t& _bootByteCPU)
{
	//  Legende der persistenten Bits (Bootkonfiguration im EEPROM, BIT0 = LSB)
	//		Bit0 0 = rootfs0, 1 = rootfs1
	//		Bit1 Updated rootfs
	//		Bit2 Updated BootStream
	//		Bit3 Update Failed
	//		Bit4 previous rootsfs updated
	//		Bit5 previous BootStream updated
	//		Bit6 BootSelector Soll		
	if ((!EepromReadBootConf(_bootByteEeprom)) || (!_readBootConfFromCPU(_bootByteCPU))) return false;		

	return true;
}

bool _eepromBootBitManipulate(bool setter, uint8_t bitnr)
{
	uint8_t boot_byte_eeprom;

	if ((!EepromReadBootConf(boot_byte_eeprom)) || (bitnr > 7)) return false;
	if (setter)
	{
		boot_byte_eeprom |= (0x01 << bitnr);
	}
	else
	{
		boot_byte_eeprom &= ~(0x01 << bitnr);
	}

	return EepromWriteBootConf(boot_byte_eeprom);
}

bool EepromBootConfSetBit(uint8_t bitnr)
{
	return _eepromBootBitManipulate(true, bitnr);
}

bool EepromBootConfClrBit(uint8_t bitnr)
{
	return _eepromBootBitManipulate(false, bitnr);	
}


void outputHelpscreen()
{
	printf("Usage:\n");	
	printf("halcl eeprom <clean|read|write> [-ver <ver> |& -hwid <hwid> |& -swid <swid> |& -hwver <hwver>\n");
	printf("                                [-osc <osc> |& -bc <bc> |& -mac0 <mac0> |& -mac1 <mac1> |& -mac2 <mac2>\n");
	printf("                                [-sn <sn> |& -devid <devid> |& -pdate <pdate> |& -key <key> |& -oem <oem>\n");
	printf("                                [-wmbus <wmbus> |& -mnterrinit <err_cnt> |& -mnterrrun <err_cnt> |& -romsig <signature>]\n");
	printf("halcl eeprom <bootread|bootwrite> [<byte_value(dec)>]\n");	
	printf("halcl getbatt\n");	
	printf("halcl bootconf\n");
	printf("halcl bootbit <set|clr> <bit_no>\n");
	printf("halcl setcpuboot <byte_value(dec)>\n");
}

int eeprom_boot(int argcnt, char* romarg[])
{
	uint8_t boot_byte;

	if (strcmp(romarg[2],"bootread") == 0)
	{
		if (EepromReadBootConf(boot_byte))
		{
			printf("Bootbyte: 0x%02X\n", boot_byte);
		}
		else 
		{
			return -1;
		}
	}
	else if ((strcmp(romarg[2],"bootwrite") == 0) && (argcnt >= 4))
	{
		boot_byte = (uint8_t)atoi(romarg[3]);
		if (EepromWriteBootConf(boot_byte))
		{
			printf("written Bootbyte: 0x%02X\n", boot_byte);
		}
		else 
		{
			return -1;
		}
	}
	return boot_byte;
}

int eeprom(int argcnt, char* romarg[])
{
	int i;
	bool ret = false;
	std::map<std::string, std::string> data;

	if (strcmp(romarg[2],"clean") == 0)
	{
		ret = EepromErase();
	}
	else if (strcmp(romarg[2],"read") == 0)
	{		
		if ( argcnt==3 )				// read all values
		{
			data["ver"]="";	data["hwid"]=""; data["swid"]=""; data["hwver"]=""; data["osc"]="";
			data["bc"]=""; data["mac0"]=""; data["mac1"]=""; data["mac2"]=""; data["sn"]="";
			data["devid"]=""; data["pdate"]=""; data["key"]=""; data["oem"]=""; data["wmbus"]="";
			data["mnterrinit"]=""; data["mnterrrun"]=""; data["romsig"]="";
		}
		else if ( argcnt > 3 )			// selektives Lesen einer oder mehrerer values
		{
			for (i=3; i<argcnt; i++)										
			{
				if (strcmp(romarg[i],"-ver")==0)				data["ver"]="";
				else if (strcmp(romarg[i],"-hwid")==0)			data["hwid"]="";
				else if (strcmp(romarg[i],"-swid")==0)			data["swid"]="";				
				else if (strcmp(romarg[i],"-hwver")==0)			data["hwver"]="";
				else if (strcmp(romarg[i],"-osc")==0)			data["osc"]="";
				else if (strcmp(romarg[i],"-bc")==0)			data["bc"]="";
				else if (strcmp(romarg[i],"-mac0")==0)			data["mac0"]="";
				else if (strcmp(romarg[i],"-mac1")==0)			data["mac1"]="";
				else if (strcmp(romarg[i],"-mac2")==0)			data["mac2"]="";
				else if (strcmp(romarg[i],"-sn")==0)			data["sn"]="";
				else if (strcmp(romarg[i],"-devid")==0)			data["devid"]="";
				else if (strcmp(romarg[i],"-pdate")==0)			data["pdate"]="";
				else if (strcmp(romarg[i],"-key")==0)			data["key"]="";
				else if (strcmp(romarg[i],"-oem")==0)			data["oem"]="";
				else if (strcmp(romarg[i],"-wmbus")==0)			data["wmbus"]="";
				else if (strcmp(romarg[i],"-mnterrinit")==0)	data["mnterrinit"]="";
				else if (strcmp(romarg[i],"-mnterrrun")==0)		data["mnterrrun"]="";
				else if (strcmp(romarg[i],"-romsig")==0)             	data["romsig"]="";
			}
		}
		ret = EepromRead(data);

		for(std::map<std::string, std::string>::iterator it=data.begin();it!=data.end();it++)
		{
			printf("%s\t%s\n",it->first.c_str(),it->second.c_str());
		}
	}
	else if (strcmp(romarg[2],"write") == 0)
	{
		for (i=3; i<argcnt-1; i++)
		{
			if (strcmp(romarg[i],"-ver")==0)
			{
				data["ver"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-hwid")==0)
			{
				data["hwid"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-swid")==0)
			{
				data["swid"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-hwver")==0)
			{
				data["hwver"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-osc")==0)
			{
				data["osc"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-bc")==0)
			{
				data["bc"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-mac0")==0)
			{
				data["mac0"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-mac1")==0)
			{
				data["mac1"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-mac2")==0)
			{
				data["mac2"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-sn")==0)
			{
				data["sn"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-devid")==0)
			{
				data["devid"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-pdate")==0)
			{
				data["pdate"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-key")==0)
			{
				data["key"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-oem")==0)
			{
				data["oem"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-wmbus")==0)
			{
				data["wmbus"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-mnterrinit")==0)
			{
				data["mnterrinit"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-mnterrrun")==0)
			{
				data["mnterrrun"] = romarg[i+1];
			}
			else if (strcmp(romarg[i],"-romsig")==0)
                        {
                                data["romsig"] = romarg[i+1];
                        }
		}

		ret = EepromWrite(data);
	}

	return ((ret) ? 0 : -1);
}

int get_BattVoltage()
{
	int ret = 0;
	unsigned int battVolt_mv = GetBattVoltage_mv();
	unsigned int battVolt_percent = GetBattVoltage_percent();

	printf("Battery voltage: %d mV (%d %%)\n", battVolt_mv, battVolt_percent);

	return ret;
}

int get_bootconf()
{
	//  Legende der persistenten Bits (Bootkonfiguration im EEPROM, BIT0 = LSB)
	//		Bit0 0 = rootfs0, 1 = rootfs1
	//		Bit1 Updated rootfs
	//		Bit2 Updated BootStream
	//		Bit3 Update Failed
	//		Bit4 previous rootsfs updated
	//		Bit5 previous BootStream updated
	//		Bit6 BootSelector Soll

	uint8_t boot_byte_eeprom, boot_byte_imx28;
	bool match = true;

	if (!GetBootConf(boot_byte_eeprom, boot_byte_imx28)) return -1;

	if (boot_byte_eeprom == boot_byte_imx28)
	{
		printf("Bootconf match:\t\tEeprom-Bootconf=0x%02X\t\tiMX28-Bootconf=0x%02X\n", boot_byte_eeprom, boot_byte_imx28);
	}
	else
	{
		printf("Bootconf dismatch:\t\tEeprom-Bootconf=0x%02X\t\tiMX28-Bootconf=0x%02X\n", boot_byte_eeprom, boot_byte_imx28);
		match = false;
	}

	printf("iMx28 Register settings:\n");
	printf(((boot_byte_imx28 & (0x01)) == 0x01 ? "Active rootfs: rootfs1\n" : "Active rootfs: rootfs0\n"));
	printf(((boot_byte_imx28 & (0x02)) == 0x02 ? "rootfs updated\n" : "rootfs not updated\n"));
	printf(((boot_byte_imx28 & (0x04)) == 0x04 ? "bootstream updated\n" : "bootstream not updated\n"));
	printf(((boot_byte_imx28 & (0x08)) == 0x08 ? "Update failed\n" : "Update didnt failed\n"));
	printf(((boot_byte_imx28 & (0x10)) == 0x10 ? "previous rootsfs updated\n" : "previous rootsfs not updated\n"));
	printf(((boot_byte_imx28 & (0x20)) == 0x20 ? "previous BootStream updated\n" : "previous BootStream not updated\n"));
	printf(((boot_byte_imx28 & (0x40)) == 0x40 ? "Should be bootstream: 1\n" : "Should be bootstream: 0\n"));
	printf(((boot_byte_imx28 & (0x80)) == 0x80 ? "Active bootstream: 1\n" : "Active bootstream: 0\n"));

	if (!match)
	{
		printf("\nEEPROM Register settings:\n");
		printf(((boot_byte_eeprom & (0x01)) == 0x01 ? "Active rootfs: rootfs1\n" : "Active rootfs: rootfs0\n"));
		printf(((boot_byte_eeprom & (0x02)) == 0x02 ? "rootfs updated\n" : "rootfs not updated\n"));
		printf(((boot_byte_eeprom & (0x04)) == 0x04 ? "bootstream updated\n" : "bootstream not updated\n"));
		printf(((boot_byte_eeprom & (0x08)) == 0x08 ? "Update failed\n" : "Update didnt failed\n"));
		printf(((boot_byte_eeprom & (0x10)) == 0x10 ? "previous rootsfs updated\n" : "previous rootsfs not updated\n"));
		printf(((boot_byte_eeprom & (0x20)) == 0x20 ? "previous BootStream updated\n" : "previous BootStream not updated\n"));
		printf(((boot_byte_imx28 & (0x40)) == 0x40 ? "Should be bootstream: 1\n" : "Should be bootstream: 0\n"));
		printf(((boot_byte_eeprom & (0x80)) == 0x80 ? "Active bootstream: 1\n" : "Active bootstream: 0\n"));
	}

	return 0;
}

int BootBitManipulate(char* bootarg[])
{	
	bool ret;

	uint8_t bitnr = atoi(bootarg[3]);

	if (strcmp(bootarg[2],"set")==0)
	{
		ret = EepromBootConfSetBit(bitnr);
	}
	else
	{
		ret = EepromBootConfClrBit(bitnr);
	}
	return (ret ? 0 : -1);
}

int setcpuboot(char* bootarg[])
{
	uint8_t boot_byte_imx28 = atoi(bootarg[2]);

	if (!WriteBootConfToCPU(boot_byte_imx28))
	{
		return -1;
	}

	return 0;
}

int main (int argc, char *argv[])
{
	int ret = -1;	

	// Wenn kein Kommando oder -? bergeben wurde, Anzeige der Hilfeseite
	if ((argc<=1) || (strcmp(argv[1],"-?")==0))
	{
		outputHelpscreen();
		return 0;
	}

	if ((strcmp(argv[1],"eeprom")==0))
	{		 
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		}
		else
		{			
			if ((strcmp(argv[2],"bootread")==0) || (strcmp(argv[2],"bootwrite")==0))
			{
				ret = eeprom_boot(argc, argv);		// EEPROM Bootbyte Handling
			}
			else
			{
				ret = eeprom(argc, argv);			// allg. Lesen/Schreiben der Gatewaywerte
			}
		}
		// Muster zum Schreiben:
		// ./halcl eeprom write -ver 1 -hwid 00-000 -swid 00-000 -hwver 12345 -osc 0 -bc 0 -mac0 02:08:02:C0:FF:EE -mac1 00:08:02:C0:FF:ED -sn 0123456789ABCDEF -pdate 20131014 -key 00000000 -oem "Dr. Neuhaus" -wmbus 0123456789ABCDEF
	}

	else if ((strcmp(argv[1],"getbatt")==0))
	{
		ret = get_BattVoltage();
	}

	else if ((strcmp(argv[1],"bootconf")==0))
	{
		ret = get_bootconf();
	}
	else if ((strcmp(argv[1],"bootbit")==0))
	{
		if (argc<4)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=BootBitManipulate(argv);
		}		
	}
	else if (strcmp(argv[1],"setcpuboot") == 0)
	{
		if (argc<3)
		{
			printf("Command: %s: Zuwenig Parameter\n",argv[1]);
			ret=-1;
		} 
		else 
		{		
			ret=setcpuboot(argv);
		}	
	}
	printf("ret=%d\n",ret);	// returncode der Ausfhrung

	return ret;
}
