/*****************************************************************
*
* HAL - Hardware Abstraction Layer implementation
*
* Copyright (c) 2013 Dr. Neuhaus Telekommunikation GmbH
*
*****************************************************************/

#include "stdafx.h"
#include "hal.h"

#ifdef LINUX
	#include <sys/sysinfo.h>
	//#include <sys/statvfs.h>
	#include <sys/vfs.h>
	#include <arpa/inet.h>
	#include <net/if.h>
	#include <net/if_arp.h>
	#include <sys/ioctl.h>
	#include <fcntl.h>
	#include <signal.h>
	#include <termios.h>
	#include <errno.h>
	#include <sys/types.h>
	#include <net/route.h>
	#include "../ksz8863/spi-ksz8863.h"

	//#include <linux/crc16.h>
	
	#include <sys/time.h>
	#include <error.h>

	// iptables incs --------------------------------------------------------------
	#include "libiptc/libiptc.h"	
	#include "linux/netfilter/x_tables.h"
	#include "linux/netfilter/xt_limit.h"
	#include "linux/netfilter/xt_physdev.h"

	// selinux incs ---------------------------------------------------------------
	//#include "semanage/modules.h"
	//#include "semanage/handle.h"

	// selftests
	#include <sys/klog.h>
	#include <dirent.h>	
	// test_sm
    /*#include <a80sm_ex.h>
	#include <a80sm_api.h>
	#include <sci2c.h>
	#include <smComSocket_linux.h>
	#include <smComI2c.h>
	#include <sm_pace.h>
	#include <serial.h>
	#include <timer.h>
	#include <global_platf.h>*/
#endif

#ifdef WIN32
 #include <windows.h>
 #include <Psapi.h>
 //warning C4996:: Funktion wurde als veraltet deklariert
 #pragma warning (disable: 4996)
#endif 

using namespace std;

namespace hal
{
	//-----------------------------------------------------------------------------------------------------------

	bool halInit()
	{
		return true;
	}

	void halDeinit()
	{
	}

	//-----------------------------------------------------------------------------------------------------------

	vector<string> GetStringTokens(const string& s, const string& sep, const unsigned int count = 0)
	{
		unsigned int cnt = 0;
		vector<string> toks;

		if(!s.empty() && !sep.empty()) {

			int pos1 = 0; int pos2 = 0;
			while( (pos2 = s.find(sep, pos1)) != -1) 
			{
				cnt++;
				if (count != 0 && cnt == count)
				{
					break;
				}

				toks.push_back(s.substr(pos1, pos2-pos1));
				pos1 = pos2 + sep.length();
			}

			toks.push_back(s.substr(pos1));
		}

		return toks;
	}

	//-----------------------------------------------------------------------------------------------------------

	static const unsigned short _fcs16tab[256] = {
		0x0000, 0x1189, 0x2312, 0x329b, 0x4624, 0x57ad, 0x6536, 0x74bf,
		0x8c48, 0x9dc1, 0xaf5a, 0xbed3, 0xca6c, 0xdbe5, 0xe97e, 0xf8f7,
		0x1081, 0x0108, 0x3393, 0x221a, 0x56a5, 0x472c, 0x75b7, 0x643e,
		0x9cc9, 0x8d40, 0xbfdb, 0xae52, 0xdaed, 0xcb64, 0xf9ff, 0xe876,
		0x2102, 0x308b, 0x0210, 0x1399, 0x6726, 0x76af, 0x4434, 0x55bd,
		0xad4a, 0xbcc3, 0x8e58, 0x9fd1, 0xeb6e, 0xfae7, 0xc87c, 0xd9f5,
		0x3183, 0x200a, 0x1291, 0x0318, 0x77a7, 0x662e, 0x54b5, 0x453c,
		0xbdcb, 0xac42, 0x9ed9, 0x8f50, 0xfbef, 0xea66, 0xd8fd, 0xc974,
		0x4204, 0x538d, 0x6116, 0x709f, 0x0420, 0x15a9, 0x2732, 0x36bb,
		0xce4c, 0xdfc5, 0xed5e, 0xfcd7, 0x8868, 0x99e1, 0xab7a, 0xbaf3,
		0x5285, 0x430c, 0x7197, 0x601e, 0x14a1, 0x0528, 0x37b3, 0x263a,
		0xdecd, 0xcf44, 0xfddf, 0xec56, 0x98e9, 0x8960, 0xbbfb, 0xaa72,
		0x6306, 0x728f, 0x4014, 0x519d, 0x2522, 0x34ab, 0x0630, 0x17b9,
		0xef4e, 0xfec7, 0xcc5c, 0xddd5, 0xa96a, 0xb8e3, 0x8a78, 0x9bf1,
		0x7387, 0x620e, 0x5095, 0x411c, 0x35a3, 0x242a, 0x16b1, 0x0738,
		0xffcf, 0xee46, 0xdcdd, 0xcd54, 0xb9eb, 0xa862, 0x9af9, 0x8b70,
		0x8408, 0x9581, 0xa71a, 0xb693, 0xc22c, 0xd3a5, 0xe13e, 0xf0b7,
		0x0840, 0x19c9, 0x2b52, 0x3adb, 0x4e64, 0x5fed, 0x6d76, 0x7cff,
		0x9489, 0x8500, 0xb79b, 0xa612, 0xd2ad, 0xc324, 0xf1bf, 0xe036,
		0x18c1, 0x0948, 0x3bd3, 0x2a5a, 0x5ee5, 0x4f6c, 0x7df7, 0x6c7e,
		0xa50a, 0xb483, 0x8618, 0x9791, 0xe32e, 0xf2a7, 0xc03c, 0xd1b5,
		0x2942, 0x38cb, 0x0a50, 0x1bd9, 0x6f66, 0x7eef, 0x4c74, 0x5dfd,
		0xb58b, 0xa402, 0x9699, 0x8710, 0xf3af, 0xe226, 0xd0bd, 0xc134,
		0x39c3, 0x284a, 0x1ad1, 0x0b58, 0x7fe7, 0x6e6e, 0x5cf5, 0x4d7c,
		0xc60c, 0xd785, 0xe51e, 0xf497, 0x8028, 0x91a1, 0xa33a, 0xb2b3,
		0x4a44, 0x5bcd, 0x6956, 0x78df, 0x0c60, 0x1de9, 0x2f72, 0x3efb,
		0xd68d, 0xc704, 0xf59f, 0xe416, 0x90a9, 0x8120, 0xb3bb, 0xa232,
		0x5ac5, 0x4b4c, 0x79d7, 0x685e, 0x1ce1, 0x0d68, 0x3ff3, 0x2e7a,
		0xe70e, 0xf687, 0xc41c, 0xd595, 0xa12a, 0xb0a3, 0x8238, 0x93b1,
		0x6b46, 0x7acf, 0x4854, 0x59dd, 0x2d62, 0x3ceb, 0x0e70, 0x1ff9,
		0xf78f, 0xe606, 0xd49d, 0xc514, 0xb1ab, 0xa022, 0x92b9, 0x8330,
		0x7bc7, 0x6a4e, 0x58d5, 0x495c, 0x3de3, 0x2c6a, 0x1ef1, 0x0f78
	};

	//-----------------------------------------------------------------------------------------------------------

	static std::string Eepromver		= "";
	static std::string Eepromhwid		= "";
	static std::string Eepromswid		= "";
	static std::string Eepromhwver		= "";
	static std::string Eepromosc		= "";
	static std::string Eeprombc			= "";
	static std::string Eeprommac0		= "";
	static std::string Eeprommac1		= "";
	static std::string Eeprommac2		= "";
	static std::string Eepromsn			= "";
	static std::string Eepromdevid		= "";
	static std::string Eeprompdate		= "";
	static std::string Eepromkey		= "";
	static std::string Eepromoem		= "";
	static std::string Eepromwmbus		= "";
	static std::string Eeprommnterrinit = "";
	static std::string Eeprommnterrrun	= "";
	static std::string Eepromromsig		= "";
	
	//-----------------------------------------------------------------------------------------------------------

	static const char *_LedPathFilenamesBrigthness[] = {
		"/sys/class/leds/led0/brightness",
		"/sys/class/leds/led1/brightness",
		"/sys/class/leds/led2/brightness",
		"/sys/class/leds/led3/brightness",
		"/sys/class/leds/led4/brightness",
		"/sys/class/leds/led5/brightness",
	};

	//-----------------------------------------------------------------------------------------------------------

	static const char *_LedPathFilenamesTrigger[] = {
		"/sys/class/leds/led0/trigger",
		"/sys/class/leds/led1/trigger",
		"/sys/class/leds/led2/trigger",
		"/sys/class/leds/led3/trigger",
		"/sys/class/leds/led4/trigger",
		"/sys/class/leds/led5/trigger",
	};

	//-----------------------------------------------------------------------------------------------------------

	static const char *_LedPathFilenamesDelayOn[] = {
		"/sys/class/leds/led0/delay_on",
		"/sys/class/leds/led1/delay_on",
		"/sys/class/leds/led2/delay_on",
		"/sys/class/leds/led3/delay_on",
		"/sys/class/leds/led4/delay_on",
		"/sys/class/leds/led5/delay_on",
	};

	//-----------------------------------------------------------------------------------------------------------

	static const char *_LedPathFilenamesDelayOff[] = {
		"/sys/class/leds/led0/delay_off",
		"/sys/class/leds/led1/delay_off",
		"/sys/class/leds/led2/delay_off",
		"/sys/class/leds/led3/delay_off",
		"/sys/class/leds/led4/delay_off",
		"/sys/class/leds/led5/delay_off",
	};

	//-----------------------------------------------------------------------------------------------------------

	unsigned short _createCRC16(const char* data, register int len, unsigned short fcs)
	{			
		while(len--) {		
			fcs = (fcs >> 8) ^ _fcs16tab[(fcs ^ *data++) & 0x0ff];
		}
		return fcs ^ 0x0ffff;
	}

	//-----------------------------------------------------------------------------------------------------------
		
	int _write_switch_register(const int fd,const unsigned char address, const unsigned char value)
	{
		int ret = 0;

#ifdef LINUX
		unsigned char buf[2];

		buf[0]=address;
		buf[1]=value;
		ret = ioctl(fd, IOCTL_KSZ8863_WRITE, buf);

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	unsigned char _read_switch_register(const int fd,const unsigned char address)
	{
		unsigned char ret = 0;

#ifdef LINUX
		unsigned char buf[2];

		buf[0]=address;
		buf[1]=0;
		ioctl(fd, IOCTL_KSZ8863_READ, buf);

		ret = buf[1];

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _get_eeprom_size()
	{		
		int eepromSize = 1;

#ifdef LINUX
		// Gre des EEPROM bestimmen ber dessen Namen

		char eepromName[5];
		memset(eepromName,0,5);
		FILE* fp=fopen("/sys/bus/i2c/devices/0-0050/name","r");
		if(fp != NULL)
		{			
			fscanf(fp,"%s",eepromName);
			fclose(fp);			
			eepromSize = (strcmp(eepromName,"24c32") == 0) ? 4096 : -1;
		}
#else
		
#endif
		return eepromSize;
	}

	//-----------------------------------------------------------------------------------------------------------

//	bool _writeEepromEntry( const unsigned int _addr, const unsigned int _valuelength, const std::string& _value )
//	{
//		bool ret = true;
//
//#ifdef LINUX
//
//		size_t wlen = 0;
//		//size_t rlen = 0;
//		int i,j;
//		int blocksize = _valuelength + LENGTH_BYTE + FCS16_LENGTH;
//
//		int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDWR);
//
//		if ( fd <= 0 )
//		{
//			printf("ERROR open EEPROM device !\n");
//			return false;
//		}		
//		char wbuf[blocksize];
//		char rbuf[blocksize];		
//		::memset(wbuf,0x0,blocksize);
//
//		if (_value.length() > _valuelength)
//		{
//			printf("ERROR: value with length %d exceeds max. dimension of %d bytes !\n", _value.length(), _valuelength);
//			close(fd);
//			return false;
//		}
//
//		wbuf[0] = _value.length();			// Lngenbyte setzen
//		strcpy(&wbuf[1], _value.c_str());	// _value in den Schreibpuffer ablegen		
//
//		unsigned short fcs16 = _createCRC16(wbuf, blocksize, 0xffff);	// Berechnung der CRC16 ber den Schreibpuffer
//		//unsigned short fcs16=0x4142;
//
//		//printf("calculated CRC16: %d\n", fcs16);
//
//		wbuf[blocksize-2] = (fcs16 >> 8) & 0x00FF;		// Eintragen der CRC16 in den Schreibpuffer
//		wbuf[blocksize-1] = (fcs16) & 0x00FF;
//
//		// 3x retry Buffer schreiben und anschliessend verifizieren
//		for (i=0; i<3; i++)
//		{
//			::lseek( fd, _addr, SEEK_SET);					// Deskriptor auf Position _addr setzen
//			//wlen = ::write(fd, wbuf, blocksize);			// Schreiben des gesamten Buffers schlgt in manchen Fllen fehl, deshalb:
//			for (j=0; j<blocksize; j++)						// Byteweises Schreiben des Buffers
//			{
//				wlen+=::write(fd, &wbuf[j], 1);
//			}
//			//printf("wlen: %d\n",wlen);
//
//			::lseek( fd, _addr, SEEK_SET);				// Deskriptor zurcksetzen
//			::read( fd, rbuf, blocksize);				// Block lesen
//			//printf("rlen: %d\n",rlen);
//
//			if ( ::memcmp(wbuf,rbuf,blocksize)==0 )			// Vergleich Schreibpuffer = Lesepuffer ?
//			{
//				//printf("Writing successfull...\n");
//				ret = true;
//				break;
//			}
//			else
//			{
//				//printf("ERROR writing to EEPROM \n");
//				wlen=0;
//				ret = false;
//			}
//		}		
//		close (fd);
//#endif
//		return ret;
//	}

	bool _writeEepromEntry( const unsigned int _addr, const unsigned int _valuelength, const std::string& _value )
	{
		bool ret = false;
		
#ifdef LINUX
	
		const unsigned int blocksize = _valuelength + LENGTH_BYTE + FCS16_LENGTH;
		int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDWR);

		if ( fd < 0 )
		{		
			return false;
		}
		else
		{
			char wbuf[blocksize];
			char rbuf[blocksize];
			::memset(wbuf,0,blocksize);
			::memset(rbuf,0,blocksize);

			if (_value.length() > _valuelength)
			{			
				close(fd);
				return false;
			}

			wbuf[0] = _value.length();			// Lngenbyte setzen
			memcpy(wbuf+1, _value.c_str(),_valuelength);		// _value in den Schreibpuffer ablegen		

			unsigned short fcs16_calc = _createCRC16(wbuf, blocksize, 0xffff);	// Berechnung der CRC16 ber den GESAMTEN Schreibpuffer	

			wbuf[blocksize-2] = (fcs16_calc >> 8) & 0x00FF;		// Eintragen der CRC16 in den Schreibpuffer
			wbuf[blocksize-1] = (fcs16_calc) & 0x00FF;

			// 3x retry Buffer schreiben und anschliessend verifizieren		
			for (int i=0; i<3; i++)
			{
				::lseek( fd, _addr, SEEK_SET);				// Deskriptor auf Position _addr setzen			

				// Schreiben des gesamten Buffers schlgt in manchen Fllen fehl, deshalb byteweise schreiben			
				for (unsigned int j = 0 ; j < blocksize ; j++) ::write(fd, wbuf+j, 1);						
				//::write( fd, wbuf, blocksize);				// Block schreiben
				::lseek( fd, _addr, SEEK_SET);				// Deskriptor zurcksetzen
				::read( fd, rbuf, blocksize);				// Block lesen			
				if (memcmp(wbuf,rbuf,blocksize) == 0)
				{
					ret = true;
					break;
				}			

			}		
			close (fd);
		}
		
#endif		

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

//	bool _readEepromEntry( const unsigned int _addr, const unsigned int _valuelength, std::string& _value )
//	{
//		bool ret = true;
//
//#ifdef LINUX
//
//		//size_t rlen;
//		int i;
//		unsigned int blocksize = _valuelength + LENGTH_BYTE + FCS16_LENGTH;
//
//		int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDONLY);
//
//		if ( fd <= 0 )
//		{
//			printf("ERROR open EEPROM device !\n");
//			return false;
//		}
//
//		char rbuf[blocksize];		
//		char fcs16buf[FCS16_LENGTH];
//		char cstring[_valuelength+1];
//		::memset(rbuf,0,blocksize);
//		::memset(rbuf,0,FCS16_LENGTH);
//		unsigned short fcs16_readed;
//
//		for (i=0; i<3; i++)										// 3x retry
//		{
//			ret = true;
//			::lseek(fd, _addr, SEEK_SET);
//			::read(fd, &rbuf[0],1);								// Lngenbyte lesen
//			if ((rbuf[0]>0) && (rbuf[0]<=_valuelength))			// Bereichsprfung Lngenbyte
//			{
//				::read(fd, &rbuf[1], rbuf[0]);							// value auslesen in der Lnge des Lngenbytes
//				::lseek(fd, _addr+LENGTH_BYTE+_valuelength, SEEK_SET);
//				::read(fd, fcs16buf, FCS16_LENGTH);						// CRC16 einlesen
//				fcs16_readed = fcs16buf[0];
//				fcs16_readed = (fcs16_readed << 8) | fcs16buf[1];
//
//				unsigned short fcs16_calc = _createCRC16(rbuf, blocksize, 0xffff);		// CRC16 berechnen ber rbuf
//				//unsigned short fcs16_calc=0x4142;			
//
//				if (fcs16_calc == fcs16_readed)						// Vergleich zwischen eingelesener und berechneter CRC16
//				{
//					memcpy(cstring, &rbuf[1], rbuf[0]);
//					cstring[(unsigned char)rbuf[0]]='\0';
//					_value = cstring;					
//					break;
//				}
//				else
//				{
//					ret = false;
//					//printf("ERROR: CRC16 dismatch: calculated CRC16: %d stored CRC16: %d at address: %d\n", fcs16_calc, fcs16_readed, _addr);
//				}				
//			} else {
//				if ( rbuf[0]==0 )
//				{
//					//printf("Data empty...\n");
//					_value = "n/a";
//				}
//				else
//				{
//					//printf("ERROR: invalid length byte stored: %d, should between 1..%d	at address: %d\n", rbuf[0], _valuelength, _addr);
//					ret = false;
//				}								
//			}			
//		}
//
//		close(fd);
//		return ret;
//#else
//		return false;
//#endif
//	}


	bool _readEepromEntry(const unsigned int _addr, const unsigned int _valuelength, std::string& _value)
	{
		bool ret = false;

#ifdef LINUX

		const unsigned int blocksize = _valuelength + LENGTH_BYTE + FCS16_LENGTH;

		int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDONLY);

		if ( fd < 0 )
		{			
			return false;
		}
		else
		{
			char rbuf[blocksize];				
			::memset(rbuf,0,blocksize);

			for (int i=0; i < 3; i++)
			{				
				::lseek(fd, _addr, SEEK_SET);
				::read(fd, rbuf,blocksize);

				unsigned short fcs16_readed = (rbuf[blocksize - 2] << 8) | rbuf[blocksize - 1];	
				// CRC (die letzten beiden bytes) aus dem readbuffer lschen
				rbuf[blocksize - 2] = 0;
				rbuf[blocksize - 1] = 0;
				unsigned short fcs16_calc = _createCRC16(rbuf, blocksize, 0xffff);
				//unsigned short fcs16_calc = _createCRC16(rbuf, blocksize, 0xffff);
				//printf("fcs16_readed: %x	fcs16_calc: %x \n",fcs16_readed, fcs16_calc);		

				if(fcs16_calc == fcs16_readed)
				{
					_value.assign(rbuf+1,_valuelength);
					ret=true;
					break;
				}
				else 
				{
					_value="n/a";
				}
			}

			close(fd);
		}

		if (ret)
		{
			switch (_addr)
			{
			case VERSION_ADDR:		Eepromver = _value;
				break;
			case HWID_ADDR:			Eepromhwid = _value;
				break;
			case SWID_ADDR:			Eepromswid = _value;
				break;
			case HWVER_ADDR:		Eepromhwver = _value;
				break;
			case OSC_ADDR:			Eepromosc = _value;
				break;
			case BC_ADDR:			Eeprombc = _value;
				break;
			case MAC0_ADDR:			Eeprommac0 = _value;
				break;
			case MAC1_ADDR:			Eeprommac1 = _value;
				break;
			case MAC2_ADDR:			Eeprommac2 = _value;
				break;
			case DEVID_ADDR:		Eepromdevid = _value;
				break;
			case PDATE_ADDR:		Eeprompdate = _value;
				break;
			case KEY_ADDR:			Eepromkey = _value;
				break;
			case OEM_ADDR:			Eepromoem = _value;
				break;
			case WMBUS_ADDR:		Eepromwmbus = _value;
				break;
			case MOUNTERRORINIT_ADDR:	Eeprommnterrinit = _value;
				break;
			case MOUNTERRORRUN_ADDR:	Eeprommnterrrun = _value;
				break;
			case ROMSIG_ADDR:		Eepromromsig = _value;
				break;
			case SN_ADDR:			Eepromsn = _value;
				break;
			default: break;
			}
		}
		
		
		return ret;
#else
		switch (_addr)
		{
		case VERSION_ADDR:		_value = "1";
			break;
		case HWID_ADDR:			_value = "11-001";
			break;
		case SWID_ADDR:			_value = "09-001";
			break;
		case HWVER_ADDR:		_value = "2.0";
			break;
		case OSC_ADDR:			_value = "1";
			break;
		case BC_ADDR:			_value = "0";
			break;
		case MAC0_ADDR:			_value = "02:08:02:C0:FF:EE";
			break;
		case MAC1_ADDR:			_value = "00:08:02:C0:FF:ED";
			break;
		case MAC2_ADDR:			_value = "00:08:02:C0:FF:EF";
			break;
		case DEVID_ADDR:		_value = "0E444E540101020304";
			break;
		case PDATE_ADDR:		_value = "20131014";
			break;
		case KEY_ADDR:			_value = "00000000";
			break;
		case OEM_ADDR:			_value = "Dr. Neuhaus";
			break;
		case WMBUS_ADDR:		_value = "0123456789ABCDEF";
			break;
		case MOUNTERRORINIT_ADDR:	_value = "0";
			break;
		case MOUNTERRORRUN_ADDR:	_value = "0";
			break;
		case ROMSIG_ADDR:		_value = "ojvocC3kzmC/bNwVuN9P6MxTBgX0Dxu1uymmv7/nXvpYIhDermZW0aLuNvds5Sa3dbGAC6DoyDvXWtnKY71to/uFZ/mBsPGtw3ThQgKECrlAMDuziXwiApZKao0VoD3soohL/V4RoTRuOm7X12MA/4lSGStfTfcVwbPI+2qAe9A=";
			break;
		case SN_ADDR:			_value = "12345678";
			break;
		default: break;
		}		
		
		return true;
#endif
	}

	//-----------------------------------------------------------------------------------------------------------
	//	NTP Statusabfrage:	- erfolgt ber NTP Control Message Protokoll (mode 6)
	//						- Mechanismus ist von ntpstat bernommen worden
	

#define RMASK 0x80  /* bit mask for the response bit in Status Byte2 */
#define EMASK 0x40  /* bit mask for the error bit in Status Byte2 */
#define MMASK 0x20  /* bit mask for the more bit in Status Byte2 */
#define PAYLOADSIZE 468 /* size in bytes of the message payload string */
//#define MAX_OFFSET	2000	// maximal zulssiger Offset in ms

	struct  NtpControlMessageHeader					// RFC-1305 NTP control message format Appendix B
	{
		// Bit 0: MSB des jeweiligen Registers
		unsigned char byte1;	// Bit 0,1:		00
								// Bit 2-4:		NTP-Version, Version: 2 (010)
								// Bit 5-7:		Mode, 6 - NTP control message
		unsigned char byte2;	// Bit 0:		Response Bit
								// Bit 1:		Error Bit
								// Bit 2:		More Bit
								// Bit 3-7:		Operation Code:
								//					1 - read status
								//					2 - read variables
								//					4 - read clock	
		unsigned short sequence;
		unsigned char  status1;		// Bit 0-1: Leap indicator: 
									//				00 - No warning, 
									//				01 - last minute has 61 seconds
									//				10 - last minute has 59 seconds
									//				11 - alarm condition (clock not synchronized)
									// Bit 2-7:	Clock Source
		unsigned char  status2;		// Bit 0-3: System Event Counter
									// Bit 4-7: System Event Code
		unsigned short AssocID;	
		unsigned short Offset;
		unsigned short Count;
		char payload[PAYLOADSIZE];
		char authenticator[96];
	};

	// Senden/Empfangen des UDP-Paketes zum/vom ntpd
	bool _sendReceive_NTPControlMessage(NtpControlMessageHeader &_ntpmsg, const sockaddr_in &_sock, int& sd, fd_set &fds, string &_errortext)
	{
		bool ret = true;
		
#ifdef LINUX
		struct timeval	tv;

		// Timeout = 1s
		tv.tv_sec = 1;
		tv.tv_usec = 0;

		// socket aufbauen
		if(sd == 0)
		{
			if ((sd = socket (PF_INET, SOCK_DGRAM, 0)) < 0)
			{
				_errortext = "unable to open socket";
				return false;
			}    

			if (connect(sd, (struct sockaddr *)&_sock, sizeof(_sock)) < 0) 
			{
				perror ("On connect");
				_errortext = "unable to connect to socket";
				return false;	
			}
		}

		FD_SET(sd, &fds);

		// Sendet die Command-Message
		if (send(sd, &_ntpmsg, sizeof(_ntpmsg), 0) < 0) {
			perror ("On send");
			_errortext = "unable to send command to NTP port";
			return false;
		}

		int n = select (sd+1, &fds, (fd_set *)0, (fd_set *)0 , &tv);

		if (n == 0)
		{
			_errortext = "timeout";
			return false;
		}   

		if (n == -1)
		{
			_errortext = "error on select";
			return false;
		}

		if ((n = recv (sd, &_ntpmsg, sizeof(_ntpmsg), 0)) < 0)
		{
			_errortext = "Unable to talk to NTP daemon. Is it running?";
			return false;
		}
#endif
		return ret;
	}

	// berprft, ob das Response-Paket valid ist
	bool _checkResponseMsg( const NtpControlMessageHeader &_ntpmsg, unsigned char _opcode, string &_errortext )
	{
		bool ret = true;

		unsigned char byte1ok = ((_ntpmsg.byte1&0x3F) == 0x16);
		unsigned char byte2ok = ((_ntpmsg.byte2 & ~MMASK) == (_opcode|RMASK));
  
		if (!(byte1ok && byte2ok)) {
			fprintf (stderr,"status word is 0x%02x%02x\n", _ntpmsg.byte1,_ntpmsg.byte2 );
			printf("return data appears to be invalid based on status word\n");
			return false;	
		}

		if (!(_ntpmsg.byte2 | EMASK)) {
			fprintf (stderr,"status byte2 is %02x\n", _ntpmsg.byte2 );
			printf("error bit is set in reply\n");
			return false;	    
		}

		if (!(_ntpmsg.byte2 | MMASK)) {
			fprintf (stderr,"status byte2 is %02x\n", _ntpmsg.byte2 );
			fprintf (stderr,"More bit unexpected in reply");
		}

		return ret;
	}

	bool _isTimeSynchronized( std::map<std::string, std::string> &_ntpParams, unsigned int _maxOffset, string &_errortext )
	{
		bool ret = true;
		int64_t offset=0;
		
		std::map<std::string, std::string>::iterator it;		
		
		// Der leap indicator im status1 byte wird u.U. erst Stunden spter gesetzt
		///* if the leap indicator (LI), which is the two most significant bits
		//in status byte1, are both one, then the clock is not synchronised. */
		//if ((_ntpmsg.status1 >> 6) == 3) 
		//{
		//	ret = false;			
		//	/* look at the system event code and see if indicates system restart */
		//	if ((_ntpmsg.status2 & 0x0F) == 1)
		//	{
		//		_errortext = "time server re-starting";
		//	}
		//	else
		//	{
		//		_errortext = "unsynchronized";
		//	}				
		//}

		// Kriterien fr eine bestehende Synchronisation der Zeit:
		//	1. Leap indicator != 3 (unsynchronized), 1,2 - warning, 0 - synchronized
		//	2. reach > 0, sonst besteht keine Konnektivitt zwischen ntpd und einem Zeitserver
		//	3. abs(offset) < 2s

		it = _ntpParams.find("leap");
		if ((it != _ntpParams.end()) && (it->second == "3"))
		{
			_errortext = "unsynchronized";
			return false;
		}

		it = _ntpParams.find("reach");
		if ((it != _ntpParams.end()) && (it->second == "0x0"))
		{
			_errortext = "reach level zero or not found";
			return false;
		}

		it = _ntpParams.find("offset");		
		if (it != _ntpParams.end())
		{				
			if(sscanf(it->second.c_str(), "%lld", &offset) == 1)
			{					
				if (llabs(offset) > _maxOffset)
				{
					_errortext = "time offset exceeds maximum value";
					return false;
				}
				else 
				{			
					ret = true;					
				}
			}
			//printf("offset: %d\n", offset);			
		}
		else 
		{			
			_errortext = "offset value unknown";
			return false;		
		}

		return ret;
	}	

	// Hilfsfunktion zum Entfernen von strenden Zeichen
	void StringTrimLeft(std::string& str, const string& chars2remove)
	{
		if (!str.empty()) {
			std::string::size_type pos = str.find_first_not_of(chars2remove);
			if (pos != std::string::npos)
				str.erase(0,pos);
			else
				str.erase( str.begin() , str.end() ); // make empty
		}
	}

	// Parsen des Response und Ablegen der Werte in einer cMap
	void _readNtpParams( const NtpControlMessageHeader &_ntpmsg , std::map<std::string, std::string> &_ntpParams )
	{
		string msgbuf = _ntpmsg.payload;		
		
		//printf("Payload: \n%s\n", msgbuf.c_str());
		
		// clocksource bestimmen
		int clksrc = (_ntpmsg.status1 & 0x3F);
		
		switch (clksrc)
		{
		case 0:		_ntpParams["Clocksource"] = "unspecified";
			break;
		case 1:		_ntpParams["Clocksource"] = "atomic clock";
			break;
		case 2:		_ntpParams["Clocksource"] = "VLF radio";
			break;
		case 3:		_ntpParams["Clocksource"] = "HF radio";
			break;
		case 4:		_ntpParams["Clocksource"] = "UHF radio";
			break;
		case 5:		_ntpParams["Clocksource"] = "local net";
			break;
		case 6:		_ntpParams["Clocksource"] = "NTP server";
			break;
		case 7:		_ntpParams["Clocksource"] = "UDP/TIME";
			break;
		case 8:		_ntpParams["Clocksource"] = "wristwatch";
			break;
		case 9:		_ntpParams["Clocksource"] = "modem";
			break;
		default:	_ntpParams["Clocksource"] = "unknown source";
			break;
		}
				
		// Zerlegen des Payload in Key=value Paare
		vector<string> params = GetStringTokens(msgbuf, ",", 0 );

		vector<string> keyvalue;
		string key, value;
				
		// Auftrennen von key/value und Eintragen in die Map
		for(vector<string>::iterator it=params.begin(); it!=params.end(); it++)
		{				
			keyvalue = GetStringTokens(it->c_str(), "=", 0);	// Trennung bei "="			
			StringTrimLeft(keyvalue[0], " ");					// fhrende Leerzeichen entfernen
			StringTrimLeft(keyvalue[0], "\r\n");				// Zeilenumbrche entfernen
			_ntpParams[keyvalue[0]] = keyvalue[1];				// Eintragen in die Map
		}
	}

	// Ausgabe von Parametern, nur zum Debuggen
	void _printNtpParams( std::map<std::string, std::string> &_ntpParams )
	{
		std::map<std::string, std::string>::iterator it;

		/*for(it=_ntpParams.begin(); it!=_ntpParams.end(); it++)
		{			
			printf("%s\t%s\n",it->first.c_str(),it->second.c_str());
		}*/

		it = _ntpParams.find("leap");
		if (it != _ntpParams.end())
		{			
			printf("%s\t%s\n",it->first.c_str(),it->second.c_str());
		}
		it = _ntpParams.find("reach");
		if (it != _ntpParams.end())
		{			
			printf("%s\t%s\n",it->first.c_str(),it->second.c_str());
		}
		it = _ntpParams.find("offset");
		if (it != _ntpParams.end())
		{			
			printf("%s\t%s\n",it->first.c_str(),it->second.c_str());
		}
	}


	int _ntpstat (unsigned short _ntp_port, unsigned int _maxOffset) 
	{
		int								rc = 0;        //  return code
	
#ifdef LINUX
		struct sockaddr_in				sock;
		struct in_addr					address;
		int								sd = 0;
		fd_set							fds;     
		struct NtpControlMessageHeader	ntpmsg;
		string							errortext;
		string							payload_string;

		std::map<std::string, std::string> NtpParams;  

		// File descriptor set
		FD_ZERO(&fds);

		inet_aton("127.0.0.1", &address);
		sock.sin_family = AF_INET;
		sock.sin_addr = address;
		sock.sin_port = htons(_ntp_port);

		//---------------------------------------------------------------------------------------------------
		// Aufbau der ersten ntpd Abfrage (nur Status, Association ID wird bentigt)

		memset ( &ntpmsg, 0, sizeof(ntpmsg));
		// 0x1601  
		ntpmsg.byte1 = 0x16;
		ntpmsg.byte2 = 0x01;    
		ntpmsg.sequence=htons(1);  

		if (!_sendReceive_NTPControlMessage(ntpmsg, sock, sd, fds, errortext))
		{
			// Fehler beim Senden
			::close(sd);
			printf("%s\n", errortext.c_str());
			return 2;
		}

		if (!_checkResponseMsg(ntpmsg, 0x01, errortext))
		{
			::close(sd);
			// Fehlerhafte Antwort erhalten
			printf("%s\n", errortext.c_str());
			return 2;
		}
	
		// Auslesen der Association ID aus der ersten Statusantwort
		char assoID1 = ntpmsg.payload[0];
		char assoID2 = ntpmsg.payload[1];

		//---------------------------------------------------------------------------------------------------
		// Aufbau der zweiten ntpd Abfrage (Abfrage der Variablen)

		memset ( &ntpmsg, 0, sizeof(ntpmsg));
		ntpmsg.byte1 = 0x16;
		ntpmsg.byte2 = 0x02;  
		ntpmsg.sequence=htons(2);
		// Setzen der Association ID, wird fr den erweiterten Report bentigt
		ntpmsg.AssocID = (assoID2 << 8);
		ntpmsg.AssocID |= assoID1;
 
		if (!_sendReceive_NTPControlMessage(ntpmsg, sock, sd, fds, errortext))
		{
			::close(sd);
			// Fehler beim Senden
			printf("%s\n", errortext.c_str());
			return 2;
		}  

		if (!_checkResponseMsg(ntpmsg, 0x02, errortext))
		{
			::close(sd);
			// Fehlerhafte Antwort erhalten
			printf("%s\n", errortext.c_str());
			return 2;
		}

		::close(sd);

		// Payload der zweiten ntp Statusabfrage Bsp:
		/* srcadr=172.24.1.0, srcport=123, dstadr=172.24.100.24, dstport=123,
			leap=0, stratum=4, precision=-6, rootdelay=2839.066, rootdisp=189.651,
			refid=172.23.1.151, reftime=0xd6bee212.ff9d4017,
			rec=0xd6beec04.43396326, reach=0xff, unreach=0, hmode=3, pmode=4,
			hpoll=3, ppoll=3, headway=6, flash=0x400, keyid=0, offset=846.895,
			delay=3.991, dispersion=15.684, jitter=5.191, xleave=3.315,
			filtdelay= 3.99 4.18 4.22 4.05 2.81 1.52 4.10 4.31,*/
  
		_readNtpParams( ntpmsg , NtpParams );
		_printNtpParams( NtpParams );

		if (_isTimeSynchronized(NtpParams, _maxOffset, errortext))
		{
			rc = 0;
		}
		else
		{
			rc = 1;
			printf("%s\n", errortext.c_str());
		} 
#endif

		return rc;
	}

	//-----------------------------------------------------------------------------------------------------------
	
	int ResetSwitch()
	{
		int ret = 0;
#ifdef LINUX
		int fd = open("/dev/ksz8863", O_RDWR);
		if (fd < 0)
		{
				printf("%s: can't open /dev/ksz8863\n", __FUNCTION__);
				ret = fd;				
		}
		else
		{
			// addr: 0x43, set 0x11 total reset to the switch	
			ret = _write_switch_register(fd,0x43,0x11);		
			close(fd);
		}
		
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int ConfigureSwitch()
	{	
		int ret = 0;

#ifdef LINUX
		int fd = open("/dev/ksz8863", O_RDWR);
		if (fd < 0)
		{
			printf("%s: can't open /dev/ksz8863\n", __FUNCTION__);
			ret = fd;
			return ret;
		}
		else
		{
			// ###########  Schreiben der Register des Switches Micrel KSZ8863

			// cpu interface clock selection
			// supported spi clock is now below 6 MHz (power saving)	
			ret = _write_switch_register(fd,0x0B,0x08);

			// assign port 1 & port 3 to vlan 1	
			ret = _write_switch_register(fd,0x81,0x0D);		// mbr
			ret = _write_switch_register(fd,0x82,0x10);		// fid
			ret = _write_switch_register(fd,0x83,0x01);		// vid
			ret = _write_switch_register(fd,0x79,0x04);		// write vlan table selected
			ret = _write_switch_register(fd,0x7A,0x01);		// trigger write operation

			// assign port 2 & port 3 to vlan 2
			ret = _write_switch_register(fd,0x81,0x0E);		// mbr
			ret = _write_switch_register(fd,0x82,0x20);		// fid
			ret = _write_switch_register(fd,0x83,0x02);		// vid
			ret = _write_switch_register(fd,0x79,0x04);		// write vlan table selected
			ret = _write_switch_register(fd,0x7A,0x02);		// trigger write operation

			// configure default vlan id
			ret = _write_switch_register(fd,0x14,0x01);		// port 1, vlan_id=1
			ret = _write_switch_register(fd,0x24,0x02);		// port 2, vlan_id=2

			// discard non pvid packets
			ret = _write_switch_register(fd,0x12,0x46);
			ret = _write_switch_register(fd,0x22,0x46);

			// enable tag insertion on port 3
			// remove vlan-tags on port 1 & 2
			ret = _write_switch_register(fd,0x30,0x04);
			ret = _write_switch_register(fd,0x10,0x02);
			ret = _write_switch_register(fd,0x20,0x02);

			// insert src pvid
			ret = _write_switch_register(fd,0xC2,0x14);
			//ret = _write_switch_register(fd,0xC2,0x3F);

			// enable vlan
			ret = _write_switch_register(fd,0x05,0x80);

			// ###################################################################		
			close(fd);
		}
		
		
#endif

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	void _parse_mac(const char* mac_char, unsigned char* mac_byte)
	{
		for(int i=0;i<16;i+=3)
		{
			if(sscanf(mac_char + i,"%c",mac_byte++) != 1) return;
		}
	}

	//-----------------------------------------------------------------------------------------------------------

#ifdef LINUX
	static void _SetDirIN(int dev)
	{
		::lseek(dev, 0, SEEK_SET); 
		::write(dev, "in", 2); 		
	}

	static void _SetDirOUT(int dev)
	{
		::lseek(dev, 0, SEEK_SET); 
		::write(dev, "out", 3); 		
	}

	void _writeGPIO(bool logic, int fd)
	{
		::lseek(fd, 0, SEEK_SET);
		::write(fd, logic ? "1" : "0", 1);
	}

	char _readGPIO(int fd)
	{
		char buf;

		::lseek(fd, 0, SEEK_SET); 
		::read(fd, &buf, 1);
		return buf;		
	}

	//-----------------------------------------------------------------------------------------------------------

	int _TamperRequest(const char* request, char* response)
	{
		int ret = 0;
		int i;

		int dev_CLK_IO   = ::open("/sys/class/gpio/gpio42/value", O_RDWR);
		int dev_CLK_DIR  = ::open("/sys/class/gpio/gpio42/direction", O_RDWR);
		int dev_DATA_IO  = ::open("/sys/class/gpio/gpio44/value", O_RDWR);
		int dev_DATA_DIR = ::open("/sys/class/gpio/gpio44/direction", O_RDWR);

		if(dev_CLK_IO == -1 || dev_CLK_DIR == -1 || dev_DATA_IO == -1 || dev_DATA_DIR == -1)
		{
			printf("sendTamperRequest open device error \n");	
			::close(dev_CLK_IO); ::close(dev_CLK_DIR); ::close(dev_DATA_IO); ::close(dev_DATA_DIR);
			return -1;
		}
		else
		{
			_SetDirOUT(dev_CLK_DIR);			// auf request vorbereiten
			_SetDirOUT(dev_DATA_DIR);		
			usleep(UTIME);

			for (i=0; i<8; i++)
			{
				if (*request=='0')
				{
					_writeGPIO(LOW, dev_DATA_IO);
				}
				else
				{
					_writeGPIO(HIGH, dev_DATA_IO);
				}			
				_writeGPIO(HIGH, dev_CLK_IO);
				usleep(UTIME);
				_writeGPIO(LOW, dev_CLK_IO);
				usleep(UTIME);
				request++;
			}

			usleep(UTIME);
			_SetDirIN(dev_DATA_DIR);
			usleep(UTIME);			

			for (i=0; i<8; i++)
			{
				_writeGPIO(HIGH, dev_CLK_IO);
				usleep(UTIME);			
				*response = _readGPIO(dev_DATA_IO);
				response++;				
				_writeGPIO(LOW, dev_CLK_IO);
				usleep(UTIME);
			}

			::close(dev_CLK_IO); ::close(dev_CLK_DIR); ::close(dev_DATA_IO); ::close(dev_DATA_DIR);

		}

		
		return ret;
	}

#endif
		
	//-----------------------------------------------------------------------------------------------------------

	bool SwitchLed(int _led, bool _on)
	{						
		int wlen = 0;

#ifdef LINUX
		int device = ::open(_LedPathFilenamesBrigthness[_led] , O_WRONLY);

		if (device < 0) 
		{
				return false;
		}
		else
		{            			
			if (_on==false) 
			{	
				// berprfung, ob entsprechende LED im Blink-Modus ist
				// Denn dann sind die Dateien delay_on, delay_off vorhanden
				// Um eine blinkende LED auszuschalten muss sie zuerst eingeschaltet werden
				// dank der Low-Active Logik bei DNT
				int delay=::open(_LedPathFilenamesDelayOn[_led], O_WRONLY);

				if (delay > 0)
				{
					// LED einschalten
					::write(device, "0", 1);	
					// Datei "delay_on" schlieen
					::close(delay);
					// reopen the device file
					if(::close(device) == 0)			
					{
						device = ::open(_LedPathFilenamesBrigthness[_led] , O_WRONLY);
						
						if(device)
						{
							// LED ausschalten
							wlen = ::write(device, "255", 3);		
						}
					}					
				}
				else
				{
					// LED ausschalten
					wlen = ::write(device, "255", 3);		
				}
			} 
			else 
			{
				// LED einschalten
				wlen = ::write(device, "0", 1);			
			}		

			::close(device);
			return (wlen > 0);
		}
		
		
		
#else
		return true; //dummy
#endif
	}

	//-----------------------------------------------------------------------------------------------------------

	bool BlinkLed(int _led)
	{
		bool ret = true;		
		int wlen = 0;
				
#ifdef LINUX
		int fd = ::open(_LedPathFilenamesTrigger[_led], O_WRONLY);
		if (fd > 0)
		{
			wlen = ::write(fd, "timer", 5);		// startet den Blinkmodus, Intervall On zu Off=500 ms
			
			::close(fd);
		} 

		ret = wlen == 5;
#endif

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool setLedDelay(int _led, const std::string& _delayofftime_ms, const std::string& _delayontime_ms)
	{
		bool ret = true;

#ifdef LINUX		
		int fd_delay_on = ::open(_LedPathFilenamesDelayOn[_led], O_WRONLY);
		int fd_delay_off = ::open(_LedPathFilenamesDelayOff[_led], O_WRONLY);

		if ((fd_delay_on * fd_delay_off) > 0)	// hat das ffnen beider Dateien funktioniert ?
		{			
			::write(fd_delay_off, _delayofftime_ms.c_str(), _delayofftime_ms.length());
			::write(fd_delay_on, _delayontime_ms.c_str(), _delayontime_ms.length());

			::close(fd_delay_off);
			::close(fd_delay_on);
		} 
		else 
		{
			ret = false;
		}
#endif		
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int ConfigureIpAddr(const string& _ethname, const string& _ipv4addr)
	{
		int ret = 0;

#ifdef LINUX
		struct ifreq ifr;

		int fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
		if(fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ifr.ifr_addr.sa_family=AF_INET;
			struct sockaddr_in* addr = (struct sockaddr_in*)&ifr.ifr_addr;
			//inet_pton(AF_INET, ipv4_addr, ifr.ifr_addr.sa_data+2);
			inet_pton(AF_INET, _ipv4addr.c_str(), &addr->sin_addr);
			ret = ioctl(fd,SIOCSIFADDR,&ifr);
			close(fd);
		}
		
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int ConfigureIpHw(const std::string& _ethname, const std::string& _macaddr)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;
		unsigned char mac_byte[6];	

		_parse_mac(_macaddr.c_str(), mac_byte);
		
		int fd=socket(AF_INET, SOCK_DGRAM, 0);
		if(fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ifr.ifr_hwaddr.sa_family=ARPHRD_ETHER;
			memcpy(ifr.ifr_hwaddr.sa_data,mac_byte,6);
			ret = ioctl(fd,SIOCSIFHWADDR,&ifr);
			close(fd);		
		}
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	
	int ConfigureIpNetmask(const std::string& _ethname, const std::string& _netmask)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;

		int fd=socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
		if(fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ifr.ifr_addr.sa_family=AF_INET;
			inet_pton(AF_INET, _netmask.c_str(), ifr.ifr_addr.sa_data+2);	
			ret = ioctl(fd,SIOCSIFNETMASK, &ifr);
			close(fd);
		}
		
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int ConfigureIpBroadcast(const std::string& _ethname, const std::string& _broadcast)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;

		int fd=socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
		if (fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ifr.ifr_addr.sa_family=AF_INET;
			inet_pton(AF_INET, _broadcast.c_str(), ifr.ifr_addr.sa_data+2);	
			ret = ioctl(fd,SIOCSIFBRDADDR,&ifr);
			close(fd);
		}

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

#define SET_SA_FAMILY(addr, family)                     \
	memset ((char *) &(addr), '\0', sizeof(addr));      \
	addr.sa_family = (family);

#define SIN_ADDR(x)     (((struct sockaddr_in *) (&(x)))->sin_addr.s_addr)

	bool _configGw(const std::string& _SetRemove, const std::string& _gateway, std::string& _errortext)
	{
		bool ret = true;
#ifdef LINUX
		int32_t rc;

		struct rtentry rt;
		struct sockaddr_in *addr;

		int32_t sock_fd = socket(AF_INET, SOCK_DGRAM,0);

		if ( sock_fd <= 0)
		{
			_errortext="Unable to open Socket";
			return false;
		}

		memset (&rt, '\0', sizeof (rt));

		SET_SA_FAMILY (rt.rt_dst,  AF_INET);

		// gateway adresse setzen
		addr = (struct sockaddr_in*)&rt.rt_gateway;
		addr->sin_family=AF_INET;
		addr->sin_addr.s_addr = inet_addr(_gateway.c_str());

		SET_SA_FAMILY (rt.rt_genmask, AF_INET);
		SIN_ADDR(rt.rt_genmask) = 0L;

		if (_SetRemove == "set")
		{
			rt.rt_flags = RTF_UP | RTF_GATEWAY;
			rt.rt_metric = 0;
			rc = ioctl(sock_fd, SIOCADDRT, &rt);
		}
		else
		{
			rt.rt_flags = RTF_UP;
			rc = ioctl(sock_fd, SIOCDELRT, &rt);
		}	

		if (rc != 0)
		{
			_errortext = strerror(errno);
			ret = false;
		}
		close(sock_fd);
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool ConfigureIpDefaultGateway(const std::string& _gateway, std::string& _errortext)
	{

		return _configGw("set", _gateway, _errortext);
	}

	//-----------------------------------------------------------------------------------------------------------

	bool RemoveIPDefaultGateway(const std::string& _gateway, std::string& _errortext)
	{
		return _configGw("remove", _gateway, _errortext);
	}

	//-----------------------------------------------------------------------------------------------------------
	
	int ConfigureIpUpDown(const std::string& _ethname, bool _ipupdown)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;

		int fd=socket(AF_INET, SOCK_DGRAM, 0);

		if (fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ret = ioctl(fd, SIOCGIFFLAGS, &ifr);
			
			if (_ipupdown == false)
			{
				//ifr.ifr_flags |= (IFF_UP | IFF_RUNNING); 
				ifr.ifr_flags &= ~(0x41); 
			} 
			else 
			{		
				ifr.ifr_flags |= 0x41; 
			}
			ret = ioctl(fd, SIOCSIFFLAGS, &ifr);
			close(fd);
		}

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool GetLinkState(const std::string& _ethname)
	{
		bool ret = false;

		if (_ethname == "eth0")
		{
#ifdef LINUX
			FILE* fp = ::fopen("/sys/class/net/eth0/operstate", "r");
			if(fp !=NULL)
			{
				char read_state[4];
				memset(read_state,0,4);
				if(fscanf(fp,"%s",read_state) == 1)
				
				ret = strcmp(read_state,"up") == 0;

				fclose(fp);
			}			
#endif
		}
		else if ((_ethname == "eth1.1") || (_ethname == "eth1.2"))
		{
#ifdef LINUX
			int fd = open("/dev/ksz8863", O_RDWR);
			if (fd < 0)
			{
					printf("%s: can't open /dev/ksz8863\n", __FUNCTION__);
					ret = false;					
			} 
			else 
			{				
				unsigned char switch_reg_addr = (_ethname == "eth1.1") ? 0x1E : 0x2E;
				unsigned char switch_reg_value = _read_switch_register(fd,switch_reg_addr);			
				
				// Bit 5 = LINK GOOD				
				ret = (switch_reg_value & 0x20) ? UP : DOWN;
				
				close(fd);				
			}			
#endif
		}
		else // unbekanntes Device
		{
			printf("unknown eth-device !\n");
		}

#ifdef WIN32
		ret = UP;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool GetIPDeviceState(const std::string& _ethname)
	{
		bool ret = DOWN;
#ifdef LINUX
		struct ifreq ifr;

		int fd=socket(AF_INET, SOCK_DGRAM, 0);
		if (fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ioctl(fd, SIOCGIFFLAGS, &ifr);
	
			ret = (ifr.ifr_flags & IFF_UP);

			close(fd);
		}

#else
		ret = UP;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int GetIpv4Addr(const std::string& _ethname, std::string& _ipv4addr)
	{
		int ret = 0;

#ifdef LINUX
		struct ifreq ifr;

		int fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);

		if (fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ret = ioctl(fd,SIOCGIFADDR,&ifr);
			struct sockaddr_in* addr = (struct sockaddr_in*)&ifr.ifr_addr;
			_ipv4addr = inet_ntoa(addr->sin_addr);	
			close(fd);
		}
#else
		_ipv4addr = "192.168.168.168";
#endif

		return ret;
	}
	
	//-----------------------------------------------------------------------------------------------------------

	int GetNetmask(const std::string& _ethname, std::string& _netmask)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;

		int fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
		if (fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ret = ioctl(fd,SIOCGIFNETMASK,&ifr);
			struct sockaddr_in* addr = (struct sockaddr_in*)&ifr.ifr_addr;
			_netmask = inet_ntoa(addr->sin_addr);
			close(fd);
		}

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int GetBroadcast(const std::string& _ethname, std::string& _broadcast)
	{
		int ret = 0;
#ifdef LINUX
		struct ifreq ifr;

		int fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_IP);
		if (fd > 0)
		{
			strncpy(ifr.ifr_name, _ethname.c_str(), IFNAMSIZ);
			ret = ioctl(fd,SIOCGIFBRDADDR,&ifr);
			struct sockaddr_in* addr = (struct sockaddr_in*)&ifr.ifr_addr;
			_broadcast = inet_ntoa(addr->sin_addr);
			close(fd);
		}

#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int GetIpGateway(const std::string& _ethname, std::string& _gateway)
	{
		int ret = 0;

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool getIpStatistics(const std::string& _ethname, uint32_t& _rxbytes, uint32_t& _txbytes, uint32_t& _rxpackets, uint32_t& _txpackets, std::string& _errortext)
	{	
		if (_ethname.empty())
		{
			_errortext = "No netdevice specified !";
			return false;
		}
#ifdef LINUX
		FILE* file_rxbytes;
		FILE* file_txbytes;
		FILE* file_rxpackets;
		FILE* file_txpackets;

		std::string pathfilename_rxbytes	= "/sys/class/net/" + _ethname + "/statistics/rx_bytes";
		std::string pathfilename_txbytes	= "/sys/class/net/" + _ethname + "/statistics/tx_bytes";
		std::string pathfilename_rxpackets	= "/sys/class/net/" + _ethname + "/statistics/rx_packets";
		std::string pathfilename_txpackets	= "/sys/class/net/" + _ethname + "/statistics/tx_packets";
		
		file_rxbytes	= ::fopen(pathfilename_rxbytes.c_str(), "r");		
		file_txbytes	= ::fopen(pathfilename_txbytes.c_str(), "r");
		file_rxpackets	= ::fopen(pathfilename_rxpackets.c_str(), "r");
		file_txpackets	= ::fopen(pathfilename_txpackets.c_str(), "r");

		if ((file_rxbytes == NULL) || (file_txbytes == NULL) || (file_rxpackets == NULL) || (file_txpackets == NULL))
		{
			_errortext = "Could not open ip statistic file !";			
		}
		else
		{			
			if ((fscanf(file_rxbytes, "%u", &_rxbytes) <= 0) ||
				(fscanf(file_txbytes, "%u", &_txbytes) <= 0) ||
				(fscanf(file_rxpackets, "%u", &_rxpackets) <= 0) ||
				(fscanf(file_txpackets, "%u", &_txpackets) <= 0))
			{
				_errortext = "Failed to read ip statistic values !";
			}
		}
		if (file_rxbytes != NULL)	fclose(file_rxbytes);
		if (file_txbytes != NULL)	fclose(file_txbytes);
		if (file_rxpackets != NULL) fclose(file_rxpackets);
		if (file_txpackets != NULL) fclose(file_txpackets);
#endif
#ifdef WIN32
		_rxbytes		= 45433;
		_rxpackets		= 414;
		_txbytes		= 163044;
		_txpackets		= 3882;
#endif
		return (_errortext.empty() ? true : false);		
	}

	//-----------------------------------------------------------------------------------------------------------

	int TriggerSysWD()
	{
		int ret = 0;
#ifdef LINUX
		// Zuerst wird versucht den Software-Watchdog (Heartbeat=20s) zu triggern
		int32_t fd_softdog = ::open("/dev/watchdog1", O_RDWR);
		if (fd_softdog > 0)			// Sollte kein zweiter watchdog im system vorhanden sein, so wird versucht den Originren zu triggern
		{
			::write(fd_softdog, "1", 2);		// Watchdog triggern
			::close(fd_softdog);
			//printf("/dev/watchdog1 triggerd...\n");
			int32_t fd_harddog = ::open("/dev/watchdog0", O_RDWR);		// Neben den Soft-Watchdog, muss auch ein Hardware-Watchdog vorhanden sein
			if (fd_harddog > 0)
			{
				::write(fd_harddog, "1", 2);	// Watchdog triggern
				::close(fd_harddog);
				//printf("/dev/watchdog0 triggerd...\n");
			}
			else
			{
				ret = -1;						// Fehlerfall: Hardwarewatchdog lsst sich nicht triggern
			}
		}
		else	
		{
			int32_t fd_harddog = ::open("/dev/watchdog", O_RDWR);		// Single-Watchdog System: Hardware watchdog
			if (fd_harddog > 0)
			{
				::write(fd_harddog, "1", 2);	// Watchdog triggern
				::close(fd_harddog);
				//printf("/dev/watchdog triggerd...\n");
			}
			else
			{
				ret = -1;						// Fehlerfall: Hardwarewatchdog lsst sich nicht triggern
			}
		}		
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool PowerGSM(bool onoff, std::string &_modell)
	{
		bool ret = true;
#ifdef LINUX
		int fd_gsmOn = ::open("/sys/class/gpio/gpio102/value", O_RDWR);
		int fd_PokIn = ::open("/sys/class/gpio/gpio125/value", O_RDWR);
		if ((fd_gsmOn<0) || (fd_PokIn<0))
		{
			ret = false;			
		} 
		else 
		{
			if (onoff == ON)
			{
				::write(fd_gsmOn, "1", 1);	// VBATT am GSM Modul zuschalten
				if (_modell == "PLS8")
				{
					usleep(100000);
					::write(fd_PokIn, "0", 1);	// LTE-Modul, IGT auf 0 setzen
					usleep(200000);				// mind. 100 ms IGT auf 0 lassen
					::write(fd_PokIn, "1", 1);	// Final IGT auf 1 setzen
				}
				else
				{
					// BGS2
					::write(fd_PokIn, "0", 1);	// GSM Modul starten
				}
			} 
			else 
			{
				::write(fd_gsmOn, "0", 1);	// VBATT am GSM Modul ausschalten
			}
		}
		close(fd_gsmOn);
		close(fd_PokIn);
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool PowerWMBUS(bool onoff)
	{
		bool ret = true;
#ifdef LINUX
		// Deprecated
		//int fd = ::open("/sys/class/gpio/gpio111/value", O_RDWR);
		//if (fd<0)
		//{
		//	ret = false;
		//} 
		//else 
		//{
		//	if (onoff == ON)
		//	{
		//		::write(fd, "0", 1);	// Versorgungsspannung am WMBUS-Modul zuschalten (default=ON)
		//	} 
		//	else 
		//	{
		//		::write(fd, "1", 1);	// Versorgungsspannung am WMBUS-Modul abschalten
		//	}
		//	::close(fd);
		//}
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool ResetWMBUS()
	{
		bool ret = false;
#ifdef LINUX		 
		int fd = ::open("/sys/class/gpio/gpio126/value", O_RDWR);
		if (fd>0)
		{
			::write(fd, "1", 1);	// Reset auslsen
			usleep(500000);
			::write(fd, "0", 1);	// Reset aufheben
			::close(fd);
			ret = true;
		}	
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool ResetGSM()
	{
		bool ret = false;
		
#ifdef LINUX		 
		int fd = ::open("/sys/class/gpio/gpio76/value", O_RDWR);
		if (fd>0)
		{
			::write(fd, "1", 1);	// Reset auslsen
			usleep(300000);
			::write(fd, "0", 1);	// Reset aufheben
			::close(fd);
			ret = true;
		}	
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool PowerMBUS12V(bool onoff)
	{
		bool ret = false;
#ifdef LINUX
		int fd = ::open("/sys/class/gpio/gpio67/value", O_RDWR);
		if (fd<0)
		{
			ret = false;
		} 
		else 
		{
			// +12V am Meter-Bus zuschalten
			// +12V am Meter-Bus abschalten
			::write(fd, (onoff ? "1" : "0"), 1);
			
			::close(fd);
			ret = true;
		}
#endif
		return ret;
	}	

	//-----------------------------------------------------------------------------------------------------------

	bool PowerSwitchPort(unsigned int portnr, bool onoff)
	{
		bool ret = false;
#ifdef LINUX
		unsigned char addr;
		unsigned char value;
		int fd;

		fd = open("/dev/ksz8863", O_RDWR);
		
		if (fd < 0)
		{		
			return ret;			
		}
		else
		{
			switch (portnr)
			{
			case 1:	addr = 0x1D;			
				break;
			case 2: addr = 0x2D;			
				break;
			default: return ret;			
				break;
			}

			value = _read_switch_register(fd, addr);

			if (onoff == OFF)
			{
				value |= 0x08;			// Das Setzen von Bit 3 schaltet den Port ab 
			}
			else
			{
				value &= ~(0x08);		// Das Lschen von Bit 3 schaltet den Port an
			}
		
			if (_write_switch_register(fd, addr, value) >= 0)
			{
				ret = true;			
			}
			close (fd);
		}
		
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool SetDTR()
	{
		bool ret = true;
#ifdef LINUX
		int fd = ::open("/sys/class/gpio/gpio116/value", O_RDWR);
		if (fd<0)
		{
			ret = false;
		} 
		else 
		{
			::write(fd, "1", 1);			
			::close(fd);
			ret = true;
		}
#endif
		return ret;
	}	

	//-----------------------------------------------------------------------------------------------------------

	bool GetSysMemory(uint64_t& _total_KB, uint64_t& _avail_KB, uint64_t& _process_KB)
	{	
		bool ret = false;

#ifdef iMX28
		FILE* fp = ::fopen( "/proc/self/statm", "r" );
		if(fp != NULL)
		{
			// 481 209 158 15 0 90 0
			// parsen der ersten 2 Zahlen 481 209						
			// erste Zahl - Gesamtgre des Programmes
			// zweite Zahl - im RAM befindlicher Teil des Programmes - RSS - Resident Set Size
			// die Angabe der Zahlen ist in Pages, so dass diese mit der Pagesize multipliziert wird
			// das Ergebnis ist ein Wert in Bytes
			// die Schnittstelle ist in KBytes, so dass durch 1024 dividiert == >> 10 wird
			long int tmp_process_KB[2]; 					
			if(fscanf(fp,"%ld %ld",tmp_process_KB,tmp_process_KB+1) == 2)
			{
				_process_KB = tmp_process_KB[1] * ::sysconf(_SC_PAGESIZE) >> 10;
				
				struct sysinfo si; 
				ret=::sysinfo(&si) == 0;

				if(ret)
				{
					_total_KB = si.totalram >> 10;
					_avail_KB = si.freeram >> 10;
				}
			}
			/*
			char line[128];
			if(::fgets(line, sizeof(line)-1, fp) != NULL)
			{
				string sline(line);
				vector<string> tok = GetStringTokens(sline, " ");
				if(tok.size() > 1)
				{
					_process_KB = (size_t)atol(tok[1].c_str()) * (size_t)::sysconf(_SC_PAGESIZE) / 1024;
					ret = true;
				}
			}
			*/
			::fclose(fp);
		}



		
#endif

#ifdef WIN32
		MEMORYSTATUSEX statex;
		statex.dwLength = sizeof (statex);

		ret = ::GlobalMemoryStatusEx(&statex)==TRUE;
		if(ret)
		{
			_total_KB = statex.ullTotalPhys >> 10;
			_avail_KB = statex.ullAvailPhys >> 10;
		}

		PROCESS_MEMORY_COUNTERS info;
		::GetProcessMemoryInfo(GetCurrentProcess(), &info, sizeof(info));
		_process_KB = info.WorkingSetSize >> 10;
#endif		

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int IPTablesBlockInputDevice(const std::string& _ethname)
	{			
		int ret = 0;
		const char* const arguments[]= { "iptables","-I","INPUT","1","-i",_ethname.c_str(),"-j","DROP",NULL };		
				
#ifdef LINUX
		pid_t child_pid;
		child_pid=fork();

		if (child_pid==0)
		{			
			execv ("/usr/sbin/iptables", const_cast<char* const*>(arguments));
			// sollte execv hierher zurckkehren, ist ein Fehler aufgetreten
			ret = -1;
		}
#endif	
		return ret;
	}	
	
	//-----------------------------------------------------------------------------------------------------------
	
	bool _IPTablesOpenPort(const std::string& _device, uint16_t _port, const std::string& _protokoll, std::string& _errortext, const std::string& _chain)
	{
		bool ret = true;
#ifdef LINUX

		struct xtc_handle *h;						// Handler
		struct ipt_entry *entry;					// Schale, beinhaltet alle weiteren structures
		struct xt_standard_target *entry_target;	// Target: ACCESS, DROP etc.
		struct ipt_entry_match *match_proto;		// match-Struktur, beinhaltet erweiterte Filtereigenschaften
		struct ipt_tcp *tcpinfo;					// TCP Port-Filter
		struct ipt_udp *udpinfo;					// UDP Port-Filter
				
		const char *table = "filter";				// default-Tabelle: filter
		const char *chain = _chain.c_str();			// iptables-Kette
						
		// Grenbestimmung der einzelnen structs: ein reines sizeof liefert hier falsche Werte
		uint32_t size_ipt_tcp			= XT_ALIGN(sizeof(struct ipt_tcp));
		uint32_t size_ipt_udp			= XT_ALIGN(sizeof(struct ipt_udp));
		uint32_t size_ipt_entry_match	= XT_ALIGN(sizeof(struct ipt_entry_match));
		uint32_t size_ipt_entry			= XT_ALIGN(sizeof(struct ipt_entry));
		uint32_t size_ipt_target		= XT_ALIGN(sizeof(struct xt_standard_target));
				
		uint32_t total_length;			// total_length: Summe aller Strukturgren
		uint32_t target_offset;			// Einhngepunkt fr das abschlieende struct (relativ zu entry)
		uint32_t match_size;

		uint16_t proto;
		
		if (_protokoll == "tcp")
		{
			total_length = size_ipt_entry + size_ipt_entry_match + size_ipt_target + size_ipt_tcp;
			target_offset = size_ipt_entry + size_ipt_entry_match + size_ipt_tcp;
			match_size = size_ipt_entry_match + size_ipt_tcp;
			proto = IPPROTO_TCP;
		}
		else if (_protokoll == "udp")
		{
			total_length = size_ipt_entry + size_ipt_entry_match + size_ipt_target + size_ipt_udp;
			target_offset = size_ipt_entry + size_ipt_entry_match + size_ipt_udp;
			match_size = size_ipt_entry_match + size_ipt_udp;
			proto = IPPROTO_UDP;
		}
		else
		{
			//printf("Invalid protokoll (only tcp/udp supported): %s\n", _protokoll.c_str());
			_errortext = "Invalid protokoll (only tcp/udp supported): " + _protokoll;
			return false;
		}
		
		// iptables init
		h = iptc_init (table);
		if (!h)
		{			
			//printf ("Could not init IPTC library: %s\n", iptc_strerror (errno));			
			_errortext = "Could not init IPTC library: " + (string)iptc_strerror (errno);
			return false;
		}
				
		// Speicher allozieren fr die Aufnahme aller netfilter-Regeln
		entry = (struct ipt_entry*)calloc(1, total_length);
		if(entry == NULL)
		{
			//printf("malloc failure...\n");
			_errortext = "malloc failure...";
			return false;
		}

		// Daten zu ipt_entry:				
		entry->target_offset = target_offset;		// target struct begining
		entry->next_offset = total_length;			// Ende des iptables Eintrages

		strcpy(entry->ip.iniface, _device.c_str());
		// IPv4 source & destination: anywhere
		entry->ip.src.s_addr  = INADDR_ANY;
		entry->ip.dst.s_addr  = INADDR_ANY;
		//inet_pton (AF_INET, "255.255.255.255", &src);
		//inet_pton (AF_INET, "255.255.255.255", &dest);
		entry->ip.proto = proto;

		// Daten zu xt_standard_target		
		entry_target = (struct xt_standard_target *)(((char*)entry) + entry->target_offset);		
		entry_target->target.u.user.target_size = size_ipt_target;		
		strncpy (entry_target->target.u.user.name, "ACCEPT", 6);
				
		// Daten zu ipt_entry_match
		match_proto = (struct ipt_entry_match*)(entry->elems);
		match_proto->u.match_size = match_size;
		strcpy(match_proto->u.user.name, _protokoll.c_str());	//set name of the module, we will use in this match		

		if (_protokoll == "tcp")
		{
			tcpinfo = (struct ipt_tcp *)(match_proto->data);
			if (_chain == "INPUT")					// ffne Eingang fr port
			{
				tcpinfo->spts[0] = ntohs(0);		// source-port: Alle erlaubt
				tcpinfo->spts[1] = ntohs(0xFFFF);			
				tcpinfo->dpts[0] = _port;
				tcpinfo->dpts[1] = _port;
			}
			else			
			{
				tcpinfo->spts[0] = _port;
				tcpinfo->spts[1] = _port;
				tcpinfo->dpts[0] = ntohs(0);			// destination-port: Alle erlaubt
				tcpinfo->dpts[1] = ntohs(0xFFFF);			
			}
			
		}
		else
		{
			udpinfo = (struct ipt_udp *)(match_proto->data);
			if (_chain == "INPUT")					// ffne Eingang fr port
			{				
				udpinfo->spts[0] = ntohs(0);		// source-port: Alle erlaubt
				udpinfo->spts[1] = ntohs(0xFFFF);
				udpinfo->dpts[0] = _port;
				udpinfo->dpts[1] = _port;
			}
			else
			{
				udpinfo->spts[0] = _port;
				udpinfo->spts[1] = _port;
				udpinfo->dpts[0] = ntohs(0);		// source-port: Alle erlaubt
				udpinfo->dpts[1] = ntohs(0xFFFF);
			}
			
		}
				
		if (!iptc_insert_entry(chain, (struct ipt_entry*) entry, 0, h))
		{
			//fprintf (stderr, "Could not insert a rule in iptables (table %s): %s\n", table, iptc_strerror (errno));
			_errortext = "Could not insert a rule in iptables: " + (string)iptc_strerror (errno);
			ret = false;
		}
		else if (!iptc_commit (h))
		{
			//fprintf (stderr, "Could not commit changes in iptables (table %s): %s\n", table, iptc_strerror (errno));
			_errortext = "Could not commit changes in iptables: " + (string)iptc_strerror (errno);
			ret = false;
		}

		if (h) iptc_free (h);
			
#endif		
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool IPTablesOpenPort(const std::string& _device, uint16_t _port, const std::string& _protokoll, std::string& _errortext)
	{
		if (_IPTablesOpenPort(_device, _port, _protokoll, _errortext, "INPUT"))
		{
			if (_IPTablesOpenPort(_device, _port, _protokoll, _errortext, "OUTPUT"))
			{
				return true;
			}
			else
			{
				return false;
			}
		}		
		return false;
	}
	
	//-----------------------------------------------------------------------------------------------------------

	bool _IPTablesClosePort(const std::string& _device, uint16_t _port, const std::string& _protokoll, std::string& _errortext, const std::string& _chain)
	{
		bool ret = true;	
#ifdef LINUX	
		uint32_t entrynr = 0;
		bool entry_found = false;
		//uint16_t proto;						
		//uint16_t dport0, dport1, sport0, sport1;
		uint16_t port0, port1;						// in Abhngigkeit von _chain: destination (INPUT) oder source port (OUTPUT)

		uint16_t MAX_SEARCH_DEPTH = 128;			// max. Anzahl der zu iterierenden rules

		struct xtc_handle *h;						// Handler

		struct ipt_entry_match *match_proto;		// match-Struktur, beinhaltet erweiterte Filtereigenschaften
		struct ipt_tcp *tcpinfo;					// TCP Port-Filter
		struct ipt_udp *udpinfo;					// UDP Port-Filter
		
		const char *table = "filter";				// default-Tabelle: filter
		const char *chain = _chain.c_str();			// iptables-Kette

		string proto_str;
		string iniface_str;
		
		if ((_protokoll != "tcp") && (_protokoll != "udp"))
		{
			//printf("Invalid protokoll (only tcp/udp supported): %s\n", _protokoll.c_str());
			_errortext = "Invalid protokoll (only tcp/udp supported): " + _protokoll;
			return false;
		}
						
		// iptables init
		h = iptc_init (table);
		if (!h)
		{		
			//printf ("Could not init IPTC library: %s\n", iptc_strerror (errno));
			_errortext = "Could not init IPTC library: " + (string)iptc_strerror (errno);
			return false;
		}

		const struct ipt_entry* entry = iptc_first_rule(chain,h);
		
		if (entry == NULL)
		{
			//printf("No iptables entry ...\n");
			_errortext = "No iptables entry...";
			return true;								// wird nicht als Fehler gewertet
		}

		while ((entry != NULL) && (!entry_found) && (entrynr < MAX_SEARCH_DEPTH))
		{
			entrynr++;

			match_proto = (struct ipt_entry_match*)(entry->elems);			
			iniface_str = entry->ip.iniface;
			proto_str = match_proto->u.user.name;
						
			if (proto_str == "tcp")
			{
				tcpinfo = (struct ipt_tcp *)(match_proto->data);
				if (_chain == "INPUT")
				{
					port0 = tcpinfo->dpts[0];
					port1 = tcpinfo->dpts[1];
				}
				else
				{
					port0 = tcpinfo->spts[0];
					port1 = tcpinfo->spts[1];
				}				
			}
			else if (proto_str == "udp")
			{
				udpinfo = (struct ipt_udp *)(match_proto->data);
				if (_chain == "INPUT")
				{
					port0 = udpinfo->dpts[0];
					port1 = udpinfo->dpts[1];
				}
				else
				{
					port0 = udpinfo->spts[0];
					port1 = udpinfo->spts[1];
				}				
			} 
			else
			{
				continue;
			}

			//printf("entry No.: %d\n", entrynr);
			//printf("----------------\n");
			//printf("entry.ip.iniface: %s\n", entry->ip.iniface);
			//printf("entry.ip.proto: %d\n", entry->ip.proto);
			//printf("match_proto->u.user.name: %s\n", match_proto->u.user.name);
			////printf("sport0: %d\t\tsport1: %d\t\tdport0: %d\t\tdport1: %d\n", sport0, sport1, dport0, dport1);
			//printf("\n");

			// Ruleeintrag erkennen:			
			// destinationport0/1 = _port 
			// entry->ip.proto=match_proto->u.user.name = _protokoll
			// entry.ip.iniface = _device
			if ((port0 == _port) && (port1 == _port) && (proto_str == _protokoll) && (iniface_str == _device))
			{
				// Eintrag lschen
				entry_found = true;
				//printf("Matching rule: %d\n", entrynr);
				
				if (!iptc_delete_num_entry(chain, entrynr-1, h))
				{
					//printf ("Could not delete rule no.: %d	reason: %s\n", entrynr, iptc_strerror (errno));					
					_errortext = "Could not delete rule in iptables: " + (string)iptc_strerror (errno);
					ret = false;
				}

				if (!iptc_commit (h))
				{
					//fprintf (stderr, "Could not commit changes in iptables (table %s): %s\n", table, iptc_strerror (errno));
					_errortext = "Could not commit changes in iptables: " + (string)iptc_strerror (errno);
					ret = false;
				}
			}
			else
			{
				entry = iptc_next_rule(entry, h);
			}			

		}

		if (!entry_found) _errortext = "No matching rule found in iptables";

		if (h) iptc_free (h);
	
#endif
		
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool IPTablesClosePort(const std::string& _device, uint16_t _port, const std::string& _protokoll, std::string& _errortext)
	{
		if (_IPTablesClosePort(_device, _port, _protokoll, _errortext, "INPUT"))
		{
			if (_IPTablesClosePort(_device, _port, _protokoll, _errortext, "OUTPUT"))
			{
				return true;
			}
			else
			{
				return false;
			}
		}		
		return false;
	}

	//-----------------------------------------------------------------------------------------------------------

	int IPTablesFlush()
	{			
		int ret = 0;
		const char* const arguments[]= { "iptables", "-F", "INPUT", NULL };
				
#ifdef LINUX
		pid_t child_pid;
		child_pid=fork();

		if (child_pid==0)
		{
			execv ("/usr/sbin/iptables", const_cast<char* const*>(arguments));

			// sollte execv hierher zurckkehren, ist ein Fehler aufgetreten
			ret = -1;
		}		
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool EepromErase()
	{
		bool ret = false;
		int eepromSize =  _get_eeprom_size();
		if (eepromSize < 0)
		{			
			return false;
		}

#ifdef LINUX
							
		int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_WRONLY);

		if ( fd < 0 )
		{			
			return false;
		}
		else
		{
			char wbuf[eepromSize];
			memset(wbuf,0,eepromSize);

			for (int i = 0; i < 3 && !ret; i++)
			{		
				int wlen=::write(fd,wbuf,eepromSize);
				ret = wlen == eepromSize;
			}

			close(fd);
		}
		
#endif

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool EepromRead( std::map<std::string, std::string>& data )
	{
		bool ret	= true;	// Globaler RC
		bool rc		= true; // RC einer Einzelabfrage

		std::map<std::string, std::string>::iterator it;
		std::string eepromEntry = "";

		for(std::map<std::string, std::string>::iterator it=data.begin(); it!=data.end(); it++)
		{				
			eepromEntry = "";
			if (it->first == "ver")					rc = _readEepromEntry(VERSION_ADDR, VERSION_LENGTH, eepromEntry);
			else if (it->first == "hwid")			rc = _readEepromEntry(HWID_ADDR, HWID_LENGTH, eepromEntry);
			else if (it->first == "bc")				rc = _readEepromEntry(BC_ADDR, BC_LENGTH, eepromEntry);
			else if (it->first == "devid")			rc = _readEepromEntry(DEVID_ADDR, DEVID_LENGTH, eepromEntry);
			else if (it->first == "hwver")			rc = _readEepromEntry(HWVER_ADDR, HWVER_LENGTH, eepromEntry);
			else if (it->first == "key")			rc = _readEepromEntry(KEY_ADDR, KEY_LENGTH, eepromEntry);
			else if (it->first == "mac0")			rc = _readEepromEntry(MAC0_ADDR, MAC_LENGTH, eepromEntry);
			else if (it->first == "mac1")			rc = _readEepromEntry(MAC1_ADDR, MAC_LENGTH, eepromEntry);
			else if (it->first == "mac2")			rc = _readEepromEntry(MAC2_ADDR, MAC_LENGTH, eepromEntry);
			else if (it->first == "mnterrinit")		rc = _readEepromEntry(MOUNTERRORINIT_ADDR, MOUNTERRORINIT_LENGTH, eepromEntry);
			else if (it->first == "mnterrrun")		rc = _readEepromEntry(MOUNTERRORRUN_ADDR, MOUNTERRORRUN_LENGTH, eepromEntry);
			else if (it->first == "oem")			rc = _readEepromEntry(OEM_ADDR, OEM_LENGTH, eepromEntry);
			else if (it->first == "osc")			rc = _readEepromEntry(OSC_ADDR, OSC_LENGTH, eepromEntry);
			else if (it->first == "pdate")			rc = _readEepromEntry(PDATE_ADDR, PDATE_LENGTH, eepromEntry);
			else if (it->first == "sn")				rc = _readEepromEntry(SN_ADDR, SN_LENGTH, eepromEntry);
			else if (it->first == "swid")			rc = _readEepromEntry(SWID_ADDR, SWID_LENGTH, eepromEntry);
			else if (it->first == "wmbus")			rc = _readEepromEntry(WMBUS_ADDR, WMBUS_LENGTH, eepromEntry);
			else if (it->first == "romsig")			rc = _readEepromEntry(ROMSIG_ADDR, ROMSIG_LENGTH, eepromEntry);
			
			if (!rc) ret = false;
			if (!eepromEntry.empty()) it->second.assign(eepromEntry);			
		}		
			
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	//bool _retWin32Eprom();

	bool getEepromVer(std::string& _ver)
	{
		if (!Eepromver.empty())
		{
			_ver = Eepromver;
			return true;
		}
		return _readEepromEntry(VERSION_ADDR, VERSION_LENGTH, _ver);
	}
	
	bool getEepromHwid(std::string& _hwid)
	{
		if (!Eepromhwid.empty())
		{
			_hwid = Eepromhwid;
			return true;
		}
		return _readEepromEntry(HWID_ADDR, HWID_LENGTH, _hwid);		
	}

	bool getEepromSwid(std::string& _swid)
	{
		if (!Eepromswid.empty())
		{
			_swid = Eepromswid;
			return true;
		}
		return _readEepromEntry(SWID_ADDR, SWID_LENGTH, _swid);
	}

	bool getEepromHwver(std::string& _hwver)
	{
		if (!Eepromhwver.empty())
		{
			_hwver = Eepromhwver;
			return true;
		}
		return _readEepromEntry(HWVER_ADDR, HWVER_LENGTH, _hwver);
	}
	
	bool getEepromMac0(std::string& _mac0)
	{
		if (!Eeprommac0.empty())
		{
			_mac0 = Eeprommac0;
			return true;
		}
		return _readEepromEntry(MAC0_ADDR, MAC_LENGTH, _mac0);
	}

	bool getEepromMac1(std::string& _mac1)
	{
		if (!Eeprommac1.empty())
		{
			_mac1 = Eeprommac1;
			return true;
		}
		return _readEepromEntry(MAC1_ADDR, MAC_LENGTH, _mac1);
	}

	bool getEepromMac2(std::string& _mac2)
	{
		if (!Eeprommac2.empty())
		{
			_mac2 = Eeprommac2;
			return true;
		}
		return _readEepromEntry(MAC2_ADDR, MAC_LENGTH, _mac2);
	}

	bool getEepromDevid(std::string& _devid)
	{
		if (!Eepromdevid.empty())
		{
			_devid = Eepromdevid;
			return true;
		}
		return _readEepromEntry(DEVID_ADDR, DEVID_LENGTH, _devid);
	}

	bool getEepromPdate(std::string& _pdate)
	{
		if (!Eeprompdate.empty())
		{
			_pdate = Eeprompdate;
			return true;
		}
		return _readEepromEntry(PDATE_ADDR, PDATE_LENGTH, _pdate);
	}
	
	bool getEepromOem(std::string& _oem)
	{
		if (!Eepromoem.empty())
		{
			_oem = Eepromoem;
			return true;
		}
		return _readEepromEntry(OEM_ADDR, OEM_LENGTH, _oem);
	}

	bool getEepromWmbus(std::string& _wmbus)
	{
		if (!Eepromwmbus.empty())
		{
			_wmbus = Eepromwmbus;
			return true;
		}
		return _readEepromEntry(WMBUS_ADDR, WMBUS_LENGTH, _wmbus);
	}
	
	bool getEepromSn(std::string& _sn)
	{
		if (!Eepromsn.empty())
		{
			_sn = Eepromsn;
			return true;
		}
		return _readEepromEntry(SN_ADDR, SN_LENGTH, _sn);
	}

	// Weitere Eintrge im EEPROM, deren Getter jedoch nicht notwendig sind:
	/*bool getEepromOsc(std::string& _osc)
	{
		if (!Eepromosc.empty())
		{
			_osc = Eepromosc;
			return true;
		}
		return _readEepromEntry(OSC_ADDR, OSC_LENGTH, _osc);
	}

	bool getEepromBc(std::string& _bc)
	{
		if (!Eeprombc.empty())
		{
			_bc = Eeprombc;
			return true;
		}
		return _readEepromEntry(BC_ADDR, BC_LENGTH, _bc);
	}

	bool getEepromKey(std::string& _key)
	{
		if (!Eepromkey.empty())
		{
			_key = Eepromkey;
			return true;
		}
		return _readEepromEntry(KEY_ADDR, KEY_LENGTH, _key);
	}

	bool getEepromMnterrinit(std::string& _mnterrinit)
	{
		if (!Eeprommnterrinit.empty())
		{
			_mnterrinit = Eeprommnterrinit;
			return true;
		}
		return _readEepromEntry(MOUNTERRORINIT_ADDR, MOUNTERRORINIT_LENGTH, _mnterrinit);
	}

	bool getEepromMnterrrun(std::string& _mnterrrun)
	{
		if (!Eeprommnterrrun.empty())
		{
			_mnterrrun = Eeprommnterrrun;
			return true;
		}
		return _readEepromEntry(MOUNTERRORRUN_ADDR, MOUNTERRORRUN_LENGTH, _mnterrrun);
	}

	bool getEepromRomsig(std::string& _romsig)
	{
		if (!Eepromromsig.empty())
		{
			_romsig = Eepromromsig;
			return true;
		}
		return _readEepromEntry(ROMSIG_ADDR, ROMSIG_LENGTH, _romsig);
	}*/

	//-----------------------------------------------------------------------------------------------------------

	bool EepromWrite( std::map<std::string, std::string>& data )
	{
		bool ret = true;

		std::map<std::string, std::string>::iterator it;
		
		it=data.find("ver");		
		if ( it != data.end() && !_writeEepromEntry(VERSION_ADDR, VERSION_LENGTH, it->second)) ret=false;

		it=data.find("hwid");		
		if ( it != data.end() && !_writeEepromEntry(HWID_ADDR, HWID_LENGTH, it->second)) ret=false;

		it=data.find("swid");		
		if ( it != data.end() && !_writeEepromEntry(SWID_ADDR, SWID_LENGTH, it->second)) ret=false;

		it=data.find("hwver");		
		if ( it != data.end() && !_writeEepromEntry(HWVER_ADDR, HWVER_LENGTH, it->second)) ret=false;

		it=data.find("osc");		
		if ( it != data.end() && !_writeEepromEntry(OSC_ADDR, OSC_LENGTH, it->second)) ret=false;

		it=data.find("bc");		
		if ( it != data.end() && !_writeEepromEntry(BC_ADDR, BC_LENGTH, it->second)) ret=false;

		it=data.find("mac0");		
		if ( it != data.end() && !_writeEepromEntry(MAC0_ADDR, MAC_LENGTH, it->second)) ret=false;

		it=data.find("mac1");		
		if ( it != data.end() && !_writeEepromEntry(MAC1_ADDR, MAC_LENGTH, it->second)) ret=false;

		it=data.find("mac2");		
		if ( it != data.end() && !_writeEepromEntry(MAC2_ADDR, MAC_LENGTH, it->second)) ret=false;

		it=data.find("sn");		
		if ( it != data.end() && !_writeEepromEntry(SN_ADDR, SN_LENGTH, it->second)) ret=false;

		it=data.find("devid");		
		if ( it != data.end() && !_writeEepromEntry(DEVID_ADDR, DEVID_LENGTH, it->second)) ret=false;
		
		it=data.find("pdate");		
		if ( it != data.end() && !_writeEepromEntry(PDATE_ADDR, PDATE_LENGTH, it->second)) ret=false;

		it=data.find("key");		
		if ( it != data.end() && !_writeEepromEntry(KEY_ADDR, KEY_LENGTH, it->second)) ret=false;

		it=data.find("oem");		
		if ( it != data.end() && !_writeEepromEntry(OEM_ADDR, OEM_LENGTH, it->second)) ret=false;

		it=data.find("wmbus");		
		if ( it != data.end() && !_writeEepromEntry(WMBUS_ADDR, WMBUS_LENGTH, it->second)) ret=false;

		it=data.find("mnterrinit");		
		if ( it != data.end() && !_writeEepromEntry(MOUNTERRORINIT_ADDR, MOUNTERRORINIT_LENGTH, it->second)) ret=false;

		it=data.find("mnterrrun");		
		if ( it != data.end() && !_writeEepromEntry(MOUNTERRORRUN_ADDR, MOUNTERRORRUN_LENGTH, it->second)) ret=false;

		it=data.find("romsig");		
		if ( it != data.end() && !_writeEepromEntry(ROMSIG_ADDR, ROMSIG_LENGTH, it->second)) ret=false;
		
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool EepromReadBootConf(uint8_t& byte_bootregister)
	{
		bool ret = false;

#ifdef LINUX
		const unsigned int blocksize = 3;		// ein byte Bindaten + 2 byte CRC16

		int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDONLY);

		if ( fd < 0 )
		{			
			return ret;
		}
		else
		{
			char rbuf[blocksize];				
			::memset(rbuf,0,blocksize);

			for (int i=0; i < 3; i++)
			{				
				::lseek(fd, BOOTCONF_ADDR, SEEK_SET);
				::read(fd, rbuf, blocksize);

				unsigned short fcs16_readed = (rbuf[blocksize - 2] << 8) | rbuf[blocksize - 1];					
				unsigned short fcs16_calc = _createCRC16(rbuf, 1, 0xffff);		// Berechnung der CRC16 ber das gelesene Binrbyte
				//unsigned short fcs16_calc = _createCRC16(rbuf, blocksize, 0xffff);
				//printf("fcs16_readed: %x	fcs16_calc: %x \n",fcs16_readed, fcs16_calc);		

				if(fcs16_calc == fcs16_readed)
				{
					byte_bootregister = rbuf[0];
					ret=true;
					break;
				}
			}

			close(fd);
		}
#else
		byte_bootregister = 0x00;
		return true;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
		
	bool EepromWriteBootConf(const uint8_t byte_bootregister)
	{
		bool ret = false;
#ifdef LINUX
		const unsigned int blocksize = 3;	// ein byte Bindaten + 2 byte CRC16
		int fd = ::open("/sys/bus/i2c/devices/0-0050/eeprom", O_RDWR);

		if ( fd < 0 )
		{		
			return ret;
		}
		else
		{
			char wbuf[blocksize];
			char rbuf[blocksize];
			::memset(wbuf,0,blocksize);
			::memset(rbuf,0,blocksize);
						
			wbuf[0] = byte_bootregister;
			unsigned short fcs16_calc = _createCRC16(wbuf, 1, 0xffff);	// Berechnung der CRC16 ber das Binrbyte	

			wbuf[blocksize-2] = (fcs16_calc >> 8) & 0x00FF;		// Eintragen der CRC16 in den Schreibpuffer
			wbuf[blocksize-1] = (fcs16_calc) & 0x00FF;
			
			// 3x retry Buffer schreiben und anschliessend verifizieren		
			for (int i=0; i<3; i++)
			{
				::lseek( fd, BOOTCONF_ADDR, SEEK_SET);				// Deskriptor auf Position _addr setzen			

				// Schreiben des gesamten Buffers schlgt in manchen Fllen fehl, deshalb byteweise schreiben			
				for (unsigned int j = 0 ; j < blocksize ; j++) ::write(fd, wbuf+j, 1);				
				::lseek( fd, BOOTCONF_ADDR, SEEK_SET);				// Deskriptor zurcksetzen
				::read( fd, rbuf, blocksize);				// Block lesen			
				if (memcmp(wbuf,rbuf,blocksize) == 0)
				{
					ret = true;
					break;
				}
				/*else
				{
					printf("Writeerror !\n");
				}*/
			}		
			close (fd);
		}
#else
		return true;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _getOsVersion(std::string& _value)
	{
		int ret = 0;		
		string bspname = "";
		string bspversion = "";

#ifdef LINUX
		FILE *osreleasefile;
		char buffer[255];		
		char* start;
		char* stop;

		if ((osreleasefile=fopen("/etc/os-release","r")) == NULL)
		{
			printf("ERROR open os-release file !\n");
			return -1;
		}
		else
		{
			while (fgets(buffer,255,osreleasefile))
			{
				if (strstr(buffer, "PTXDIST_BSP_NAME"))
				{
					start= strstr(buffer,"\"") + 1;
					stop = strstr(start,"\"");
					bspname.assign(start,stop-start);
				}
				else if (strstr(buffer, "PTXDIST_BSP_VERSION"))
				{
					start= strstr(buffer,"\"") + 1;
					stop = strstr(start,"\"");
					bspversion.assign(start,stop-start);
				}
			}
			fclose(osreleasefile);
		}
		
		

#else
		bspname = "DNT8209";
		bspversion = "-88";		
		
#endif		
		_value = bspname + bspversion;
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _getKernelVersion(std::string& _value)
	{
		int ret = 0;
		std::string kversion = "";
		std::string kbuild = "";

#ifdef LINUX
		FILE *kversionfile=fopen("/proc/version","r");
		char buffer[255];
		char* start;
		char* stop;

		if (kversionfile == NULL)
		{
			printf("ERROR open kernel version file !\n");
			return -1;
		}
		else
		{
			if (fgets(buffer, 255, kversionfile))
			{
				start = &buffer[0];
				stop = strchr(buffer, '(');
				kversion.assign(start,stop-start);

				start = strchr(buffer, '#');
				stop = strstr(buffer, "PREEMPT");
				kbuild.assign(start,stop-start);
			}
			else
			{
				printf("ERROR reading kernel version file !\n");
				ret = -1;
			}

			fclose(kversionfile);
		}

		

#else
		kversion = "Windows XXX";
		kbuild = "x.y";
#endif		
		_value = kversion + kbuild;

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _getAppVersion(std::string& _value)
	{
		int ret = 0;

		_value = "0.1-546";

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _getPacketVersion(std::string& _packetname, std::string& _value)
	{
		int ret = 0;
		_value = "";

#ifdef LINUX
		FILE *opkfile=fopen("/usr/lib/opkg/status","r");
		char buffer[255];
		char* start;
		char* stop;
		char* checkpoint;
		bool isComplete=false;

		if (opkfile == NULL)
		{
			printf("ERROR open opkg-status file !\n");
			return -1;
		}
		else
		{
			while ((fgets(buffer,255,opkfile)) && (!isComplete))
			{			
				start = strstr(buffer, _packetname.c_str());
				if (start)										// Package-Name wurde gefunden
				{
					if (strstr(buffer, "Package"))				// tritt der Name als Paketname auf oder als dependency
					{
						checkpoint = strrchr(buffer, ':');
						if ((checkpoint+2)==start)				// steht der Paketname allein dort
						{
							stop = strrchr(buffer,'\n');

							//printf("stop-start: %d	_packetname.length: %d\n", stop-start, _packetname.length());
							if ((unsigned int)(stop-start)==_packetname.length())
							{
								fgets(buffer,255,opkfile);
								start = strrchr(buffer,':')+2;
								stop = strrchr(buffer,'\n');
								_value.assign(start,stop-start);
								isComplete = true;
							}
						}
					}				
				}
			}
			fclose(opkfile);
		}

		
#else		
		_value = "1.0";
		if (_packetname == "smgw") _value = "1.0.1066";
#endif

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _getAllPacketVersions(std::map<std::string, std::string>& data)
	{
		int ret = 0;

#ifdef LINUX
		FILE *opkfile=fopen("/usr/lib/opkg/status","r");
		char buffer[255];
		char* start;
		char* stop;

		std::string pname = "";
		std::string pversion = "";

		if (opkfile == NULL)
		{
			printf("ERROR open opkg-status file !\n");
			return -1;
		}
		else
		{
			while (fgets(buffer,255,opkfile))
			{
				if(strstr(buffer,"Package"))
				{
					start = strrchr(buffer,':')+2;
					stop = strrchr(buffer,'\n');
					pname.assign(start,stop-start);

					fgets(buffer,255,opkfile);
					start = strrchr(buffer,':')+2;
					stop = strrchr(buffer,'\n');
					pversion.assign(start,stop-start);

					//printf("Paket: %s	Version: %s\n", pname.c_str(), pversion.c_str());
					data[pname] = pversion;
				}
			}

			fclose(opkfile);
		}

		
#else
		data["glib"] = "2.3.4";
		data["zlib"] = "1.2";
		data["openssl"] = "1.0.1e";
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int _getPatches(std::map<std::string, std::string>& data)
	{
		int ret = 0;		

#ifdef LINUX

		FILE *opkfile=fopen("/usr/lib/opkg/status","r");
		char buffer[255];
		char* start;
		char* stop;

		std::string pname = "";
		std::string pversion = "";

		if (opkfile == NULL)
		{
			printf("ERROR open opkg-status file !\n");
			return -1;
		}
		else
		{
			while (fgets(buffer,255,opkfile))
			{
				if(strstr(buffer,"Package"))
				{
					if (strstr(buffer, "patch"))
					{
						start = strrchr(buffer,':')+2;
						stop = strrchr(buffer,'\n');
						pname.assign(start,stop-start);

						fgets(buffer,255,opkfile);
						start = strrchr(buffer,':')+2;
						stop = strrchr(buffer,'\n');
						pversion.assign(start,stop-start);

						//printf("Paket: %s	Version: %s\n", pname.c_str(), pversion.c_str());
						data[pname] = pversion;
					}								
				}
			}

			fclose(opkfile);
		}

		
#else
		data["patch1"] = "1.0";
		data["patch2"] = "1.0";
		data["patch4"] = "1.0";
#endif

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------
	
	bool QuerySystemVersion(const std::string& _mode, std::map<std::string, std::string>& data, string& _error)
	{
		bool ret = true;
		int retint = 0;
		int retprev = 0;
		string key;

		//printf("mode: %s	\n", _mode.c_str());
		
		std::map<std::string, std::string>::iterator i;

		if (_mode == "single")
		{							
			for (i = data.begin(); i!=data.end(); i++)
			{
				key = i->first;			
				retprev = retint;							// Error detector

				if (key == "os")
				{
					if (_getOsVersion(data["os"]) != 0)
					{				
						retint--;
					}
				}
				else if (key == "kernel")
				{				
					if (_getKernelVersion(data["kernel"]) != 0)
					{				
						retint--;
					}
				}
				else
				{				
					if (_getPacketVersion(key, i->second) != 0)
					{
						retint--;
					}
				}

				if (retint != retprev)				// es ist ein Fehler aufgetreten
				{
					_error = "QuerySystemVersion failed: reading Version for '" + key + "=" + i->second + "' ";
					ret = false;
				}
			}			
		} 
		else if (_mode == "allpackages")
		{
			_getAllPacketVersions(data);
		}
		else if (_mode == "patches")
		{
			_getPatches(data);
		}
		else
		{
			_error = "QuerySystemVersion: invalid mode '" + _mode + "'";
			ret = false;
		}

		if (retint < 0)
		{
			ret = false;
		}

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool SystemUpdateRequest(const std::string& _pathfilename)
	{
		bool ret = true;

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	long long int _statFS(const std::string& _typ, const std::string& _pathfilename)
	{
		long long int ret = 0;
#ifdef LINUX

		struct statfs buf;

		if (!statfs(_pathfilename.c_str(),&buf))
		{
			long long int blksize, blocks, disk_size, used, free, bavail;
			blksize = buf.f_bsize;
			blocks = buf.f_blocks;
			//freeblks = buf.f_bfree;
			bavail = buf.f_bavail;

			disk_size = blocks*blksize;
			//free = freeblks*blksize;
			free = bavail*blksize;
			used = disk_size - free;

			//printf("blksize: %lli	blocks: %lli	freeblks: %lli disk_size: %lli free: %lli used: %lli\n", blksize,blocks,freeblks,disk_size,free,used);
			
			if (_typ=="free")
			{
				ret = free;
			}
			else if (_typ=="disksize")
			{
				ret = disk_size;
			}
			else if (_typ=="used")
			{
				ret = used;
			}
		}
		else
		{
			ret = -1;
		}
		

#else
		ret = 123456;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	long long int GetFreeDiskSpace(const std::string& _pathfilename)
	{
		long long int ret = 0;

		ret = _statFS("free",_pathfilename);

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	long long int GetDiskSize(const std::string& _pathfilename)
	{
		long long int ret = 0;

		ret = _statFS("disksize",_pathfilename);

		return ret;
	}
	
	//-----------------------------------------------------------------------------------------------------------

	long long int GetUsedDiskSpace(const std::string& _pathfilename)
	{
		long long int ret = 0;

		ret = _statFS("used",_pathfilename);

		return ret;
	}	

	//-----------------------------------------------------------------------------------------------------------

	int ReadTamperStatus()
	{
		int ret = 0;		

#ifdef LINUX
				
		char tamperResponse[8];	
		int retry;
		
		for (retry=0; retry<3; retry++)
		{			
			_TamperRequest(TAMPER_CMD_READ,tamperResponse);

			if (strcmp(tamperResponse, TAMPER_READ_ACK_OK)==0)
			{
				ret = 0x01;
				break;
			}
			else if (strcmp(tamperResponse, TAMPER_READ_ACK_SWITCH)==0)
			{
				ret = 0x02;
				break;
			}
			else if (strcmp(tamperResponse, TAMPER_READ_ACK_EEPROM)==0)
			{
				ret = 0x03;
				break;
			}
			else
			{
				ret = -1;			
			}
		}		
#else
		ret = 0x01;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	int TamperReboot()
	{
		int ret = 0;
		

#ifdef LINUX

		int retry;
		
		char tamperResponse[8];

		for (retry=0; retry<3; retry++)
		{
			_TamperRequest(TAMPER_CMD_REBOOT_REQ, tamperResponse);			

			printf("Tamper response: %s\n", tamperResponse);

			if (strcmp(tamperResponse, TAMPER_CMD_REBOOT_ACK)==0)
			{
				ret = 0;
				break;
			}			
			else
			{
				ret = -1;			
			}
		}		
#else
		ret = 0x01;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	unsigned int getPppdPid()
	{
		unsigned int ret = 0;

#ifdef LINUX
		FILE *ppp0_pid_file=fopen("/var/run/ppp0.pid","r");
		//char buffer[255];
		
		if (ppp0_pid_file == NULL)
		{		
			return 0;
		}
		else
		{
			if(fscanf(ppp0_pid_file,"%d",&ret) != 1) ret=0;

			fclose(ppp0_pid_file);
		}		

#else
		ret = 100;
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool GetDSRPegel()
	{
		bool ret = false;
#ifdef LINUX

		char value;
		FILE *fp=::fopen("/sys/class/gpio/gpio117/value", "r");
		if(fp != NULL)
		{
			fscanf(fp,"%c",&value);			
			ret = value == '1';

			fclose(fp);
		}
		
#endif		
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool EthExists(std::string &_networkDevice)
	{		
		std::string dummy;		
		return (GetIpv4Addr(_networkDevice, dummy) == 0);
	}

	//-----------------------------------------------------------------------------------------------------------

	int _getRawBattVoltage()
	{
		int ret = 0;
#ifdef LINUX

		unsigned short value;
		FILE *fp=::fopen("/sys/bus/iio/devices/iio:device0/in_voltage7_raw", "r");
		if(fp != NULL)
		{
			if(fscanf(fp,"%hu",&value) != 1) ret=-1;
			else ret = value > 1160 ? 1160 : value;	// Begrenzung

			fclose(fp);
		}
		else
		{
			ret=-1;
		}
		
#else
		ret = 1160;		// fr Windows: stets voller Ladezustand
#endif
		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	unsigned int GetBattVoltage_mv()
	{
		unsigned int ret = 1;
		int rawBattVoltage = _getRawBattVoltage();

		if (rawBattVoltage > 0)
		{
			ret = ( rawBattVoltage * 4240 ) / 1160;
		}

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	unsigned int GetBattVoltage_percent()
	{
		unsigned int ret = 0;

		int rawBattVoltage = _getRawBattVoltage();

		if (rawBattVoltage >= 0)
		{
			ret = ( rawBattVoltage * 100 / 1160 );
		}

		return ret;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool IsNTPsynchronized(uint16_t _port, uint32_t _maxOffset_ms)
	{
#ifdef WIN32
		return true;
#endif

		// rc von ntpstat:
		//		0:   Uhr ist synchronisiert
		//		1:   Uhr ist nicht synchronisiert, Eines der Kriterien wurde nicht erfllt
		//		2:   Status der Synchronisation konnte nicht ermittelt werden (ntpd nicht erreichbar)

		return (_ntpstat(_port, _maxOffset_ms) == 0 ? true : false);		
	}

	//-----------------------------------------------------------------------------------------------------------

	bool _readBootConfBitFromCPU(int32_t bitnr)
	{
#ifdef LINUX
		int32_t fd;
		char buf[1];

		switch (bitnr)
		{
		case 0:	fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_0", O_RDONLY);
			break;
		case 1: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_1", O_RDONLY);
			break;
		case 2: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_2", O_RDONLY);
			break;
		case 3: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_3", O_RDONLY);
			break;
		case 4: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_4", O_RDONLY);
			break;
		case 5: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_5", O_RDONLY);
			break;
		case 6: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_6", O_RDONLY);
			break;
		case 7: fd = ::open("/sys/devices/platform/imx28_regs.0/rom_secondary_boot", O_RDONLY);
			break;
		default: return false;
			break;
		}

		::read(fd, buf, 1);
		::close(fd);

		return (buf[0] == '1' ?  true : false);
#else
		return false;
#endif
	}

	//-----------------------------------------------------------------------------------------------------------

	bool _setBootConfBitToCPU(int32_t bitnr, bool _value)
	{
#ifdef LINUX
		int32_t fd;
		//char buf[1];

		switch (bitnr)
		{
		case 0:	fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_0", O_RDWR);
			break;
		case 1: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_1", O_RDWR);
			break;
		case 2: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_2", O_RDWR);
			break;
		case 3: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_3", O_RDWR);
			break;
		case 4: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_4", O_RDWR);
			break;
		case 5: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_5", O_RDWR);
			break;
		case 6: fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_6", O_RDWR);
			break;
		case 7: fd = ::open("/sys/devices/platform/imx28_regs.0/rom_secondary_boot", O_RDWR);
			break;
		default: return false;
			break;
		}

		if ( fd <=0 ) return false;

		if (_value)
		{
			::write(fd, "1", 1);
		}
		else
		{
			::write(fd, "0", 1);
		}
		::close(fd);		
#endif
		return true;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool _readBootConfFromCPU(uint8_t& boot_byte)
	{
		int32_t i;
		// Zugriff auf die CPU-Register mglich ? 
#ifdef LINUX
		int32_t fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_0", O_RDONLY);
		if (fd <= 0) return false;		
		close(fd);
#endif
		boot_byte = 0;

		for (i = 0; i < 8; i++ )
		{
			boot_byte |= (_readBootConfBitFromCPU(i) ? (0x01 << i) : 0x00);
		}

		return true;		
	}

	//-----------------------------------------------------------------------------------------------------------

	bool WriteBootConfToCPU(uint8_t& boot_byte)
	{		
#ifdef LINUX
		int32_t i;
		// Zugriff auf die CPU-Register mglich ? 
		int32_t fd = ::open("/sys/devices/platform/imx28_regs.0/user_bit_0", O_RDWR);
		if (fd <= 0) return false;		
		close(fd);
	
		for (i = 0; i < 8; i++ )
		{			
			if ((boot_byte & (0x01 << i)) > 0)
			{				
				_setBootConfBitToCPU(i, HIGH);
			}
			else
			{
				_setBootConfBitToCPU(i, LOW);
			}			
		}
#endif

		return true;	
	}

	//-----------------------------------------------------------------------------------------------------------

	bool GetBootConf(uint8_t& _bootByteEeprom, uint8_t& _bootByteCPU)
	{
		//  Legende der persistenten Bits (Bootkonfiguration im EEPROM, BIT0 = LSB)
		//		Bit0 0 = rootfs0, 1 = rootfs1
		//		Bit1 Updated rootfs
		//		Bit2 Updated BootStream
		//		Bit3 Update Failed
		//		Bit4 previous rootsfs updated
		//		Bit5 previous BootStream updated
		//		Bit6 BootSelector Soll		
#ifdef LINUX		
		if ((!EepromReadBootConf(_bootByteEeprom)) || (!_readBootConfFromCPU(_bootByteCPU))) return false;		
#endif
		return true;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool _eepromBootBitManipulate(bool setter, uint8_t bitnr)
	{
		uint8_t boot_byte_eeprom;

		if ((!EepromReadBootConf(boot_byte_eeprom)) || (bitnr > 7)) return false;
		if (setter)
		{
			boot_byte_eeprom |= (0x01 << bitnr);
		}
		else
		{
			boot_byte_eeprom &= ~(0x01 << bitnr);
		}
		
		return EepromWriteBootConf(boot_byte_eeprom);
	}

	//-----------------------------------------------------------------------------------------------------------

	bool EepromBootConfSetBit(uint8_t bitnr)
	{
		return _eepromBootBitManipulate(true, bitnr);
	}

	//-----------------------------------------------------------------------------------------------------------

	bool EepromBootConfClrBit(uint8_t bitnr)
	{
		return _eepromBootBitManipulate(false, bitnr);	
	}

	//-----------------------------------------------------------------------------------------------------------

	bool ActivatePowerlossDetection()
	{
#ifdef LINUX
		if ((open("/sys/kernel/debug/vdd5v", O_RDONLY)) <= 0) return false;
#endif
		return true;
	}

	//-----------------------------------------------------------------------------------------------------------

#ifdef WIN32
	#include <strsafe.h>
#endif

	bool SetTimeZone(const std::string& _timezoneinfo, std::string& _errortext)
	{
#ifdef LINUX
		string pathname = "/usr/share/zoneinfo/" + _timezoneinfo;
		int32_t fd = open(pathname.c_str(), O_RDONLY);		

		if (fd < 0)
		{
			_errortext = "Timezone info not exists !";
			return false;
		}
		close(fd);
		unlink("/etc/localtime");
		int32_t ret = symlink(pathname.c_str(), "/etc/localtime");
		if (ret != 0)
		{
			string error = strerror(errno);
			_errortext = "Error creating symlink: " + error;
			return false;
		}
#elif defined WIN32

		// Disabled fr Windows: Setzen des Standardnamens ist nicht ausreichend

		/*DYNAMIC_TIME_ZONE_INFORMATION tzinfo;

		ZeroMemory(&tzinfo, sizeof(tzinfo));

		std:wstring widestr = std::wstring(_timezoneinfo.begin(), _timezoneinfo.end());
		wcscpy(tzinfo.StandardName, widestr.c_str());
				
		
		
		if( !SetDynamicTimeZoneInformation( &tzinfo ) ) 
		{
			printf("SDTZI failed (%d)\n", GetLastError());
			return false;
		}*/

#endif
		return true;
	}

	//-----------------------------------------------------------------------------------------------------------

	bool GetTimeZone(std::string& _timezoneinfo, std::string& _errortext)
	{
#ifdef LINUX
		char buf[1024];
		ssize_t len;

		if ((len = readlink("/etc/localtime", buf, sizeof(buf)-1)) != -1)
		{
			buf[len] = '\0';
		}
		else
		{
			string error = strerror(errno);
			_errortext = "Error reading link \"/etc/localtime\": " + error;
			return false;
		}

		_timezoneinfo = buf;
		StringTrimLeft(_timezoneinfo, "../usr/share/zoneinfo/");
#elif defined WIN32
		char ch[32];
		char defChar=' ';		
		DYNAMIC_TIME_ZONE_INFORMATION tzinfo;
		
		DWORD ret = GetDynamicTimeZoneInformation(&tzinfo);
		
		switch (ret)
		{
			case TIME_ZONE_ID_STANDARD: WideCharToMultiByte(CP_ACP,0,tzinfo.StandardName, -1, ch, 32, &defChar, NULL);
				break;
			case TIME_ZONE_ID_DAYLIGHT: WideCharToMultiByte(CP_ACP,0,tzinfo.DaylightName, -1, ch, 32, &defChar, NULL);
				break;
			case TIME_ZONE_ID_UNKNOWN:	_errortext = "TIME ZONE ID UNKNOWN";
			default: return false;
		}
		
		// mit den deutschen Umlauten gibts leider Probleme, Lsung ?
		std::string tmp(ch);
		_timezoneinfo = tmp;

		//printf("TimeZoneKeyName: %s\n", tmp.c_str());

		/*wprintf(L"tzinfo.TimeZoneKeyName: %s\n", tzinfo.TimeZoneKeyName);						
		wprintf(L"tzinfo->DaylightName: %s\n", tzinfo.DaylightName);
		wprintf(L"tzinfo->StandardName: %s\n", tzinfo.StandardName);
		printf("tzinfo->StandardBias: %d\n", tzinfo.StandardBias);
		printf("tzinfo->DaylightBias: %d\n", tzinfo.DaylightBias);
		printf("tzinfo->Bias: %d\n", tzinfo.Bias);		*/		
#else
		_timezoneinfo = "CET";
#endif
		return true;
	}

	//-----------------------------------------------------------------------------------------------------------

	uint32_t GetCurrent12v_mA()
	{		
		uint32_t result;
#ifdef LINUX
		bool ret = false;
		char value;
		uint32_t lradc3_raw;

		// Ist Ua=12V aktiviert ?, sonst (auch Fehlerfall) return 0
		FILE *fp=::fopen("/sys/class/gpio/gpio67/value", "r");
		if(fp != NULL)
		{
			fscanf(fp,"%c",&value);
			ret = value == '1';
			fclose(fp);
		}

		if (!ret) return 0;

		// Auslesen des raw-Wertes aus dem LRADC3
		FILE *flradc=::fopen("/sys/bus/iio/devices/iio:device0/in_voltage3_raw", "r");
		if (flradc != NULL)
		{
			fscanf(flradc, "%d", &lradc3_raw);
			fclose(flradc);
		}

		//printf("lradc3_raw: %d\n", lradc3_raw);
		// Durch Messung ermittelt: y(mA) = (7/50) * x + 10
		result = (14 * lradc3_raw)/100 + 10;
#else
		result = 22;
#endif
		return result;
	}

	//-----------------------------------------------------------------------------------------------------------

//	bool GetPolicyModuleVersion(std::string& _version, std::string& _errortext)
//	{
//#ifdef LINUX
//		semanage_module_info_t *modinfo;
//		semanage_handle_t *sh = NULL;
//		int32_t num_modules;
//		//int32_t result;		
//
//		sh = semanage_handle_create();
//		if (!sh) {
//			_errortext = "Could not create semanage handle";
//			return false;
//		}
//				
//		if ( semanage_connect(sh) < 0 )
//		{
//			_errortext = "Could not connect to policy handler !";
//			return false;
//		}		
//
//		if (semanage_module_list(sh, &modinfo, &num_modules) >= 0) 
//		{
//			int32_t i;
//			if (num_modules == 0) 
//			{
//				_errortext = "No policy modules found.";				
//			}
//			for (i = 0; i < num_modules; i++) 
//			{
//				semanage_module_info_t *m =	semanage_module_list_nth(modinfo, i);
//				string modname = semanage_module_get_name(m);
//				if (modname == "smgw") _version = semanage_module_get_version(m);
//				//printf("%s\t%s\t%s\n", semanage_module_get_name	(m), semanage_module_get_version (m), (semanage_module_get_enabled(m) ? "" : "Disabled"));
//				semanage_module_info_datum_destroy(m);
//			}
//			free(modinfo);
//		}
//		else
//		{
//			_errortext = "Could not get the module list !";
//		}
//		if (!_errortext.empty()) return false;		
//#else
//		_version = "1.0";
//#endif
//		return true;
//	}

	//-----------------------------------------------------------------------------------------------------------

// Gre des LogPuffers: 128 KiB
// Definiert in der Kernel-Konfiguration unter: LOG_BUF_SHIFT
#define LOG_BUF_SHIFT	131072
// Maximale Lnge eines abzuspeichernden Kernellog-Eintrages
#define MAX_MSG_LENGTH	255

	bool selfTestFlash(std::string& _errortext)
	{		
#ifdef LINUX
		char log[LOG_BUF_SHIFT];					// Buffer fr den Kernellog
		
		// Kernellog in den Buffer bertragen, rc liefert die Anzahl der gelesenen bytes, im Fehlerfall rc=-1		
		if (klogctl(3, log, LOG_BUF_SHIFT) < 0)
		{
			_errortext = strerror(errno);
			return false;
		}
		
		char* ubi_error = NULL;
		char* ubifs_error = NULL;
		char* ubi_error_parseStop;
		char* ubifs_error_parseStop;
		
		int32_t length_ubi_error_msg = 0;
		int32_t length_ubifs_error_msg = 0;				

		// Kernellog wird nach folgenden zwei Strings durchsucht
		char Errorstring_Ubi[]		= "UBI error";
		char Errorstring_Ubifs[]	= "UBIFS error";
		//char Errorstring_Ubi[]	= "UBI:";
		//char Errorstring_Ubifs[]	= "UBIFS:";
				
		// erste Suche starten
		ubi_error	= strstr(log, Errorstring_Ubi);
		ubifs_error = strstr(log, Errorstring_Ubifs);

		// weiteres Handling, falls die erste Suche erfolgreich
		while ((ubi_error != NULL) || (ubifs_error != NULL))
		{
			if (ubi_error != NULL)									// ein Fehler in UBI wurde gefunden
			{
				ubi_error_parseStop = strchr(ubi_error, '\n');		// Position des nchsten Zeilenumbruchs ermitteln
				if (ubi_error_parseStop != NULL)					// Zu jedem Kernellog-Eintrag muss ein Zeilenumbruch existieren
				{					
					// Lnge des Kernellog-Eintrages ermitteln
					length_ubi_error_msg = ubi_error_parseStop - ubi_error;
					// Lngenbegrenzung auf MAX_MSG_LENGTH (255 bytes)
					if (length_ubi_error_msg > MAX_MSG_LENGTH) length_ubi_error_msg = MAX_MSG_LENGTH;
					// Kernellog-Zeichenkette an _errortext anfgen					
					_errortext.append(ubi_error, length_ubi_error_msg);
					// Neue Suche starten
					ubi_error = strstr(ubi_error_parseStop, Errorstring_Ubi);
				}
				else
				{
					// wird kein begrenzender Zeilenumbruch gefunden, gilt die Suche als beendet
					ubi_error = NULL;
				}				
			}
			if (ubifs_error != NULL)								// ein Fehler in UBIFS wurde gefunden
			{
				ubifs_error_parseStop = strchr(ubifs_error, '\n');
				if (ubifs_error_parseStop != NULL)
				{					
					length_ubifs_error_msg = ubifs_error_parseStop - ubifs_error;
					if (length_ubifs_error_msg > MAX_MSG_LENGTH) length_ubifs_error_msg = MAX_MSG_LENGTH;					
					_errortext.append(ubifs_error, length_ubifs_error_msg);
					ubifs_error = strstr(ubifs_error_parseStop, Errorstring_Ubifs);
				}
				else
				{
					ubifs_error = NULL;
				}
			}
		}
#endif
		// Sind keine Eintrge in _errortext enthalten, so gilt der Test als bestanden		
		return (_errortext.empty() ? true : false);
	}

	//-----------------------------------------------------------------------------------------------------------

	void _ramtestRandMov(char* bp1, char *bp2, uint64_t chunksize, std::string& _errortext)
	{		
		char* p1;
		char* p2;
		uint64_t i;
				
		//printf("Randomize and MOV comparison test: Setting...\n");
		p1 = bp1; p2 = bp2;
		for (i = 0; i < chunksize; i++)
		{
			*p1++ = *p2++ = rand();
		}
		//printf("Testing...\n");
		p1 = bp1; p2 = bp2;
		for (i = 0; i < chunksize; i++, p1++, p2++)
		{			
			if (*p1 != *p2)
			{
				//printf("FAILURE: 0x%x != 0x%x at offset 0x%x.\n", *p1, *p2, i);
				_errortext = "Randomize and MOV comparison test failed.";
				break;
			}			
		}		
		//printf("Passed.\n");
	}

	void _ramtestXor(char* bp1, char *bp2, uint64_t chunksize, std::string& _errortext)
	{
		char q;
		char* p1;
		char* p2;
		uint64_t i;

		//printf("XOR comparison test: Setting...\n");
		p1 = bp1; p2 = bp2;
		q = rand();
		for (i = 0; i < chunksize; i++)
		{
			*p1++ ^= q;
			*p2++ ^= q;
		}
		//printf("Testing...\n");
		p1 = bp1; p2 = bp2;
		for (i = 0; i < chunksize; i++, p1++, p2++)
		{
			if (*p1 != *p2)
			{
				//printf("FAILURE: 0x%x != 0x%x at offset 0x%x.\n", *p1, *p2, i);
				_errortext = "XOR comparison test failed.";
				break;
			}
		}
		//printf("Passed.\n");
	}

	void _ramtestSub(char* bp1, char *bp2, uint64_t chunksize, std::string& _errortext)
	{
		char q;
		char* p1;
		char* p2;
		uint64_t i;

		//printf("SUB comparison test: Setting...\n");
		p1 = bp1; p2 = bp2;
		q = rand();
		for (i = 0; i < chunksize; i++)
		{
			*p1++ -= q;
			*p2++ -= q;
		}
		//printf("Testing...\n");
		p1 = bp1; p2 = bp2;
		for (i = 0; i < chunksize; i++, p1++, p2++)
		{
			if (*p1 != *p2)
			{
				//printf("FAILURE: 0x%x != 0x%x at offset 0x%x.\n", *p1, *p2, i);
				_errortext = "SUB comparison test failed.";
				break;
			}
		}
		//printf("Passed.\n");
	}

	void _ramtestMul(char* bp1, char *bp2, uint64_t chunksize, std::string& _errortext)
	{
		char q;
		char* p1;
		char* p2;
		uint64_t i;
		
		//printf("MUL comparison test: Setting...\n");
		p1 = bp1; p2 = bp2;
		q = rand();
		for (i = 0; i < chunksize; i++)
		{
			*p1++ *= q;
			*p2++ *= q;			
		}
		//printf("Testing...\n");
		p1 = bp1; p2 = bp2;
		for (i = 0; i < chunksize; i++, p1++, p2++)
		{
			if (*p1 != *p2)
			{
				//printf("FAILURE: 0x%x != 0x%x at offset 0x%x.\n", *p1, *p2, i);
				_errortext = "MUL comparison test failed.";
				break;
			}
		}
		//printf("Passed.\n");
	}

	void _ramtestDiv(char* bp1, char *bp2, uint64_t chunksize, std::string& _errortext)
	{
		char q;
		char* p1;
		char* p2;
		uint64_t i;

		//printf("DIV comparison test: Setting...\n");
		p1 = bp1; p2 = bp2;
		q = rand();
		for (i = 0; i < chunksize; i++)
		{
			if (!q) q++;
			*p1++ /= q;
			*p2++ /= q;
		}
		//printf("Testing...\n");
		p1 = bp1; p2 = bp2;
		for (i = 0; i < chunksize; i++, p1++, p2++)
		{
			if (*p1 != *p2)
			{
				//printf("FAILURE: 0x%x != 0x%x at offset 0x%x.\n", *p1, *p2, i);
				_errortext = "DIV comparison test failed.";
				break;
			}
		}
		//printf("Passed.\n");
	}

	void _ramtestOr(char* bp1, char *bp2, uint64_t chunksize, std::string& _errortext)
	{
		char q;
		char* p1;
		char* p2;
		uint64_t i;

		//printf("OR comparison test: Setting...\n");
		p1 = bp1; p2 = bp2;
		q = rand();
		for (i = 0; i < chunksize; i++)
		{
			*p1++ |= q;
			*p2++ |= q;
		}
		//printf("Testing...\n");
		p1 = bp1; p2 = bp2;
		for (i = 0; i < chunksize; i++, p1++, p2++)
		{
			if (*p1 != *p2)
			{
				//printf("FAILURE: 0x%x != 0x%x at offset 0x%x.\n", *p1, *p2, i);
				_errortext = "OR comparison test failed.";
				break;
			}
		}
		//printf("Passed.\n");
	}

	void _ramtestAnd(char* bp1, char *bp2, uint64_t chunksize, std::string& _errortext)
	{
		char q;
		char* p1;
		char* p2;
		uint64_t i;

		//printf("AND comparison test: Setting...\n");
		p1 = bp1; p2 = bp2;
		q = rand();
		for (i = 0; i < chunksize; i++)
		{
			*p1++ &= q;
			*p2++ &= q;
		}
		//printf("Testing...\n");
		p1 = bp1; p2 = bp2;
		for (i = 0; i < chunksize; i++, p1++, p2++)
		{
			if (*p1 != *p2)
			{
				//printf("FAILURE: 0x%x != 0x%x at offset 0x%x.\n", *p1, *p2, i);
				_errortext = "AND comparison test failed.";
				break;
			}
		}
		//printf("Passed.\n");
	}

	bool selfTestRam(const std::list<std::string>& _testAlgos, std::string& _errortext)
	{				
		if (_testAlgos.empty())
		{
			_errortext = "No RAM test algorithm specified !";
			return false;
		}		

		// Gre des freien RAM-Speichers ermitteln
		uint64_t total_KB, avail_KB, process_KB;
		if (!GetSysMemory(total_KB, avail_KB, process_KB))
		{
			_errortext = "Could not determine free RAM space !";
			return false;
		}

		//printf("total_KB: %llu, avail_KB: %llu, process_KB: %llu\n", total_KB, avail_KB, process_KB);

		// Kriterien fr den Start eines RAM-Tests: mindestens noch 5 MiB frei
		if (avail_KB < (5 * 1204))
		{
			_errortext = "Not enough RAM free for running RAM Test !";
			return false;
		}

		// Gre des zu testenden RAM-Speichers ermitteln: (Freier Speicher - 5MiB) dividiert durch 4 ?
		//uint64_t testingKB = (avail_KB - (5 * 1024)) / 4;
		uint64_t testingKB = 5 * 1024;
		uint64_t testingBytes = testingKB * 1024;

		//printf("Zu testender Speicher: %llu Bytes (%llu KiB)\n", testingBytes, testingKB);

		/////////////////////////////////// Memtester ////////////////////////////////////////////////////
		/// Sources are taken from:	Simon Kirby <sim@stormix.com> <sim@neato.org>, 1999/10/21
		///
		char *buf, *bp1, *bp2;
		uint64_t chunksize;
						
		buf = (char *)malloc((size_t)testingBytes);
		if (!buf)
		{
			_errortext = "Unable to malloc: ";
			_errortext.append(strerror(errno));
			return false;			
		}		

		chunksize = testingBytes >> 1;			// Testspeicher halbieren
		bp1 = buf;								// pointer 1: Anfang des Testspeichers
		bp2 = buf + (chunksize);				// pointer 2: Mitte vom Testspeicher
		
		// Aufruf der einzelnen RAM-Speicher Tests
		for (std::list<std::string>::const_iterator it = _testAlgos.begin(); it != _testAlgos.end(); it++)
		{
			if		(*it == "rand")		_ramtestRandMov(bp1, bp2, chunksize, _errortext);
			else if (*it == "sub")		_ramtestSub(bp1, bp2, chunksize, _errortext);
			else if (*it == "and")		_ramtestAnd(bp1, bp2, chunksize, _errortext);
			else if (*it == "div")		_ramtestDiv(bp1, bp2, chunksize, _errortext);
			else if (*it == "mul")		_ramtestMul(bp1, bp2, chunksize, _errortext);
			else if (*it == "or")		_ramtestOr(bp1, bp2, chunksize, _errortext);
			else if (*it == "xor")		_ramtestXor(bp1, bp2, chunksize, _errortext);
			else
			{
				_errortext = "Unknown Testalgorithm: ";
				_errortext.append(*it);
			}
			if (!_errortext.empty()) break;
		}

		//////////////////////////////////////////////////////////////////////////////////////////////////
		
		return (_errortext.empty() ? true : false);
	}

	//-----------------------------------------------------------------------------------------------------------

#ifdef WIN32
	void _getWinProcessNameAndPid(std::map<std::string, std::string>& _processList, DWORD processID)
	{
		TCHAR szProcessName[MAX_PATH] = TEXT("<unknown>");		

		// Handle zum Prozess anlegen
		HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION |
			PROCESS_VM_READ,
			FALSE, processID);

		// Name des Prozesses ermitteln
		if (NULL != hProcess)
		{
			HMODULE hMod;
			DWORD cbNeeded;

			if (EnumProcessModules(hProcess, &hMod, sizeof(hMod),
				&cbNeeded))
			{
				GetModuleBaseName(hProcess, hMod, szProcessName,
					sizeof(szProcessName) / sizeof(TCHAR));
			}
		}

		// PID und Prozessname in Map eintragen
		_processList[to_string(processID)] = szProcessName;
		CloseHandle(hProcess);
	}
#endif

#ifdef LINUX
	void getLinuxProcessNameAndPid(std::map<std::string, std::string>& _processList) 
	{
		DIR *dir_p;
		struct dirent *dir_entry_p;
		char dir_name[255];				// procfs-Ordner eines Prozesses, z.Bsp.: /proc/107/
		
		FILE *statfile;
		std::string statfilename;		// Dateipfad zum statfile, z.Bsp.: /proc/107/stat
		std::string pname;				// Prozessname, wird aus dem statfile ausgelesen
		char buffer[256];				// Inhalt des statfile

		char* pname_start;
		char* pname_stop;
		
		dir_p = opendir("/proc/"); 								// ffnet das /proc Verzeichnis
		
		if (dir_p == NULL) return;
		while (NULL != (dir_entry_p = readdir(dir_p)))			// Einlesen der /proc/ Eintrge
		{			
			if (strspn(dir_entry_p->d_name, "0123456789") == strlen(dir_entry_p->d_name))		// Verzeichnisname besteht nur aus Ziffern
			{		
				strcpy(dir_name, "/proc/");
				strcat(dir_name, dir_entry_p->d_name);
				strcat(dir_name, "/"); 						// Aufbau des kompletten Pfades: /proc/107/ 
				statfilename = dir_name;
				statfilename.append("stat");					// vollstndiger Dateipfad als string
				statfile = fopen(statfilename.c_str(), "r");	// ffnen des statfiles				
				if (statfile != NULL)
				{					
					fgets(buffer, 255, statfile);				// Einlesen des statfiles
					pname_start = strchr(buffer, '(') + 1;		// Suche nach dem Prozessnamen, Start ist '('
					pname_stop = strchr(buffer, ')');			// Ende der Zeichenkette wird durch '(' markiert
					if ((pname_start != NULL) && (pname_stop != NULL))	// Wurde der Prozessname eingegrenzt...
					{
						pname.assign(pname_start, pname_stop - pname_start);
						_processList[dir_entry_p->d_name] = pname;		// Eintragen der PID und zugehrigem Prozessnamen in die Map
					}
					fclose(statfile);
				}
				//printf("dir_name: %s\td_name: %s\t\tstatfilename: %s\t\tpname: %s\n", dir_name, dir_entry_p->d_name, statfilename.c_str(), pname.c_str());				
			}
		}
		if (dir_p != NULL) closedir(dir_p);		
	}
#endif

	bool selfTestListRunningProcesses(std::map<std::string, std::string>& _processList, std::string& _errortext)
	{	
		
#ifdef WIN32				
		DWORD aProcesses[1024], cbNeeded, cProcesses;
		unsigned int i;

		if (!EnumProcesses(aProcesses, sizeof(aProcesses), &cbNeeded))
		{
			_errortext = "Could not enumerating the processes !";
			return false;
		}
		// Calculate how many process identifiers were returned.
		cProcesses = cbNeeded / sizeof(DWORD);

		for (i = 0; i < cProcesses; i++)
		{
			if (aProcesses[i] != 0)
			{				
				_getWinProcessNameAndPid(_processList, aProcesses[i]);
			}
		}
#endif

#ifdef LINUX
		getLinuxProcessNameAndPid(_processList);
#endif

		return (!_processList.empty() ? true : false);
	}

	//-----------------------------------------------------------------------------------------------------------
	// Aus Grnden der Kompatibilitt mit V.1.0 deaktiviert (sm-hostlib: #ifdef cplusplus fehlt in sci2c.h)
	bool selfTestSm(std::string& _errortext)
	{
#ifdef LINUX
		// OpenI2C1 Bus device
		//if (I2C_Open((I2C_BUS_NR_t)1) <= 0)
		//{
		//	_errortext = "Open i2c1 device failed";
		//}
		//// WakeUp + Softreset + ATR + ParameterExchange
		//else if (sci2c_Init() != 0)
		//{
		//	_errortext = "Init sm failed";
		//	I2C_Close();
		//}		
#endif
		return (_errortext.empty() ? true : false);
	}

	//-----------------------------------------------------------------------------------------------------------
	
	bool KillProcessByName(const std::string& _processName, int32_t _signal, std::string& _errortext)
	{
#ifdef LINUX
		std::map<std::string, std::string> processList;
		uint32_t pid;
		bool isFound = false;

		if (!selfTestListRunningProcesses(processList, _errortext)) return false;

		for (std::map<std::string, std::string>::const_iterator it = processList.begin(); it != processList.end(); it++)
		{
			if (it->second == _processName)
			{
				isFound = true;
				sscanf(it->first.c_str(), "%u", &pid);
				if (kill((pid_t)pid, _signal) != 0) _errortext = strerror(errno);
				//printf("process %s with pid: %u killed !\n", it->second.c_str(), pid);				
			}
		}
		if (!isFound) _errortext = "Process not running";
#endif
		return (_errortext.empty() ? true : false);
	}

	bool KillProcessByName(const std::string& _processName, std::string& _errortext)
	{
#ifdef LINUX
		return KillProcessByName(_processName, SIGKILL, _errortext);
#else
		return true;
#endif		
	}

}	//namespace hal

