#include <linux/slab.h>
#include <linux/device.h>
#include <linux/module.h>
#include <linux/err.h>
#include <linux/interrupt.h>
#include <linux/completion.h>
#include <linux/platform_device.h>
#include <linux/io.h>
#include <linux/of.h>
#include <linux/of_device.h>

#include <linux/sched.h>
#include <linux/debugfs.h>

#include <linux/jiffies.h>
#include <linux/timer.h>

#ifdef REPORT_IRQ_ON_WRITE

// interrupt events will be reported on file_read() function

#include <linux/poll.h>

#define EVT_VDD5V_IRQ		0x01

struct _instance_data
{
    unsigned int value;
    int pending_value;
    int sleeping;
};
static DECLARE_WAIT_QUEUE_HEAD (wq);
static atomic_t data_ready;
static atomic_t vdd5v_value;

#else

// interrupt events will be reported by Sending Signals from the Kernel to the User Space

#include <asm/siginfo.h>	//siginfo
#include <linux/rcupdate.h>	//rcu_read_lock

#define SIGVDD5V 44

static atomic_t pid_value;

#endif

// ************************************************************

#define DRIVER_NAME "imx28_power"

#define MXS_POWER_CTRL			(0x00)
#define MXS_POWER_CTRL_SET		(0x04)
#define MXS_POWER_CTRL_CLR		(0x08)

#define BP_POWER_CTRL_RSRVD2	28
#define BM_POWER_CTRL_RSRVD2	0xF0000000
#define BF_POWER_CTRL_RSRVD2(v) \
		(((v) << 28) & BM_POWER_CTRL_RSRVD2)
#define BM_POWER_CTRL_PSWITCH_MID_TRAN	0x08000000
#define BP_POWER_CTRL_RSRVD1	25
#define BM_POWER_CTRL_RSRVD1	0x06000000
#define BF_POWER_CTRL_RSRVD1(v)  \
		(((v) << 25) & BM_POWER_CTRL_RSRVD1)
#define BM_POWER_CTRL_DCDC4P2_BO_IRQ	0x01000000
#define BM_POWER_CTRL_ENIRQ_DCDC4P2_BO	0x00800000
#define BM_POWER_CTRL_VDD5V_DROOP_IRQ	0x00400000
#define BM_POWER_CTRL_ENIRQ_VDD5V_DROOP	0x00200000
#define BM_POWER_CTRL_PSWITCH_IRQ	0x00100000
#define BM_POWER_CTRL_PSWITCH_IRQ_SRC	0x00080000
#define BM_POWER_CTRL_POLARITY_PSWITCH	0x00040000
#define BM_POWER_CTRL_ENIRQ_PSWITCH	0x00020000
#define BM_POWER_CTRL_POLARITY_DC_OK	0x00010000
#define BM_POWER_CTRL_DC_OK_IRQ	0x00008000
#define BM_POWER_CTRL_ENIRQ_DC_OK	0x00004000
#define BM_POWER_CTRL_BATT_BO_IRQ	0x00002000
#define BM_POWER_CTRL_ENIRQBATT_BO	0x00001000
#define BM_POWER_CTRL_VDDIO_BO_IRQ	0x00000800
#define BM_POWER_CTRL_ENIRQ_VDDIO_BO	0x00000400
#define BM_POWER_CTRL_VDDA_BO_IRQ	0x00000200
#define BM_POWER_CTRL_ENIRQ_VDDA_BO	0x00000100
#define BM_POWER_CTRL_VDDD_BO_IRQ	0x00000080
#define BM_POWER_CTRL_ENIRQ_VDDD_BO	0x00000040
#define BM_POWER_CTRL_POLARITY_VBUSVALID	0x00000020
#define BM_POWER_CTRL_VBUSVALID_IRQ	0x00000010
#define BM_POWER_CTRL_ENIRQ_VBUS_VALID	0x00000008
#define BM_POWER_CTRL_POLARITY_VDD5V_GT_VDDIO	0x00000004
#define BM_POWER_CTRL_VDD5V_GT_VDDIO_IRQ	0x00000002
#define BM_POWER_CTRL_ENIRQ_VDD5V_GT_VDDIO	0x00000001

#define MXS_POWER_5VCTRL		(0x10)
#define MXS_POWER_5VCTRL_SET	(0x14)
#define MXS_POWER_5VCTRL_CLR	(0x18)

#define BP_POWER_5VCTRL_RSRVD6	30
#define BM_POWER_5VCTRL_RSRVD6	0xC0000000
#define BF_POWER_5VCTRL_RSRVD6(v) \
		(((v) << 30) & BM_POWER_5VCTRL_RSRVD6)
#define BP_POWER_5VCTRL_VBUSDROOP_TRSH	28
#define BM_POWER_5VCTRL_VBUSDROOP_TRSH	0x30000000
#define BF_POWER_5VCTRL_VBUSDROOP_TRSH(v)  \
		(((v) << 28) & BM_POWER_5VCTRL_VBUSDROOP_TRSH)
#define BM_POWER_5VCTRL_RSRVD5	0x08000000
#define BP_POWER_5VCTRL_HEADROOM_ADJ	24
#define BM_POWER_5VCTRL_HEADROOM_ADJ	0x07000000
#define BF_POWER_5VCTRL_HEADROOM_ADJ(v)  \
		(((v) << 24) & BM_POWER_5VCTRL_HEADROOM_ADJ)
#define BP_POWER_5VCTRL_RSRVD4	22
#define BM_POWER_5VCTRL_RSRVD4	0x00C00000
#define BF_POWER_5VCTRL_RSRVD4(v)  \
		(((v) << 22) & BM_POWER_5VCTRL_RSRVD4)
#define BP_POWER_5VCTRL_PWD_CHARGE_4P2	20
#define BM_POWER_5VCTRL_PWD_CHARGE_4P2	0x00300000
#define BF_POWER_5VCTRL_PWD_CHARGE_4P2(v)  \
		(((v) << 20) & BM_POWER_5VCTRL_PWD_CHARGE_4P2)
#define BP_POWER_5VCTRL_RSRVD3	18
#define BM_POWER_5VCTRL_RSRVD3	0x000C0000
#define BF_POWER_5VCTRL_RSRVD3(v)  \
		(((v) << 18) & BM_POWER_5VCTRL_RSRVD3)
#define BP_POWER_5VCTRL_CHARGE_4P2_ILIMIT	12
#define BM_POWER_5VCTRL_CHARGE_4P2_ILIMIT	0x0003F000
#define BF_POWER_5VCTRL_CHARGE_4P2_ILIMIT(v)  \
		(((v) << 12) & BM_POWER_5VCTRL_CHARGE_4P2_ILIMIT)
#define BM_POWER_5VCTRL_RSRVD2	0x00000800
#define BP_POWER_5VCTRL_VBUSVALID_TRSH	8
#define BM_POWER_5VCTRL_VBUSVALID_TRSH	0x00000700
#define BF_POWER_5VCTRL_VBUSVALID_TRSH(v)  \
		(((v) << 8) & BM_POWER_5VCTRL_VBUSVALID_TRSH)
#define BM_POWER_5VCTRL_PWDN_5VBRNOUT	0x00000080
#define BM_POWER_5VCTRL_ENABLE_LINREG_ILIMIT	0x00000040
#define BM_POWER_5VCTRL_DCDC_XFER	0x00000020
#define BM_POWER_5VCTRL_VBUSVALID_5VDETECT	0x00000010
#define BM_POWER_5VCTRL_VBUSVALID_TO_B	0x00000008
#define BM_POWER_5VCTRL_ILIMIT_EQ_ZERO	0x00000004
#define BM_POWER_5VCTRL_PWRUP_VBUS_CMPS	0x00000002
#define BM_POWER_5VCTRL_ENABLE_DCDC	0x00000001

#define MXS_POWER_VDDDCTRL		(0x40)

#define BP_POWER_VDDDCTRL_LINREG_OFFSET	16
#define BM_POWER_VDDDCTRL_LINREG_OFFSET	0x00030000
#define BF_POWER_VDDDCTRL_LINREG_OFFSET(v)  \
		(((v) << 16) & BM_POWER_VDDDCTRL_LINREG_OFFSET)

#define MXS_POWER_VDDACTRL		(0x50)

#define BP_POWER_VDDACTRL_LINREG_OFFSET	12
#define BM_POWER_VDDACTRL_LINREG_OFFSET	0x00003000
#define BF_POWER_VDDACTRL_LINREG_OFFSET(v)  \
		(((v) << 12) & BM_POWER_VDDACTRL_LINREG_OFFSET)

#define MXS_POWER_VDDIOCTRL		(0x60)

#define BM_POWER_VDDIOCTRL_LINREG_OFFSET	0x00003000
#define BF_POWER_VDDIOCTRL_LINREG_OFFSET(v)  \
		(((v) << 12) & BM_POWER_VDDIOCTRL_LINREG_OFFSET)

#define VBUSVALID_THRESH_2_90V		0x0
#define VBUSVALID_THRESH_4_00V		0x1
#define VBUSVALID_THRESH_4_10V		0x2
#define VBUSVALID_THRESH_4_20V		0x3
#define VBUSVALID_THRESH_4_30V		0x4
#define VBUSVALID_THRESH_4_40V		0x5
#define VBUSVALID_THRESH_4_50V		0x6
#define VBUSVALID_THRESH_4_60V		0x7

#define LINREG_OFFSET_STEP_BELOW	0x2

// First adress byte changed because the kernel doesn´t know the real register addresses
#define HW_ENET_MAC0_ECR_ADDR		0xF50F0024
#define HW_ENET_MAC1_ECR_ADDR		0xF50F4024

#define HW_UARTAPP0_CTRL2_CLR_ADDR	0xF506A028
#define HW_UARTAPP1_CTRL2_CLR_ADDR	0xF506C028
#define HW_UARTAPP2_CTRL2_CLR_ADDR	0xF506E028
#define HW_UARTAPP3_CTRL2_CLR_ADDR	0xF5070028
#define HW_UARTAPP4_CTRL2_CLR_ADDR	0xF5072028

#define HW_POWER_MINPWR_SET_ADDR	0xF5044024
#define HW_POWER_MINPWR_CLR_ADDR	0xF5044028

struct mxs_power_dev {
	struct device *dev;
	void __iomem *regs;
	struct dentry *debug_file;
    pid_t pid;
};

//	Disable the MAC0 and MAC1 Controller
static void mxs_power_disableMacs(void)
{		
	void __iomem* reg_mac0 = (void*)HW_ENET_MAC0_ECR_ADDR;
	void __iomem* reg_mac1 = (void*)HW_ENET_MAC1_ECR_ADDR;
	uint32_t mac0_ecr = readl(reg_mac0);	
	uint32_t mac1_ecr = readl(reg_mac1);

	// Setting Bit 3 to sleep the MAC
	mac0_ecr |= 0x08;
	mac1_ecr |= 0x08;

	writel(mac0_ecr, reg_mac0);
	writel(mac1_ecr, reg_mac1);
}

// Disable all Application UARTs by clearing Bit 0 of the register
static void mxs_power_diableUarts(void)
{
	writel(1, (void*)HW_UARTAPP0_CTRL2_CLR_ADDR);
	writel(1, (void*)HW_UARTAPP1_CTRL2_CLR_ADDR);
	writel(1, (void*)HW_UARTAPP2_CTRL2_CLR_ADDR);
	writel(1, (void*)HW_UARTAPP3_CTRL2_CLR_ADDR);
	writel(1, (void*)HW_UARTAPP4_CTRL2_CLR_ADDR);
}

// SetUp for minimum power consumption
static void mxs_power_SetMinPower(void)
{
	// Bit 6: Disable the use of double sized power transistors
	writel(0x40, (void*)HW_POWER_MINPWR_CLR_ADDR);
	// Bit 0: Slow Down the DC-DC clock from 1,5 MHz to 750 kHz
	writel(1, (void*)HW_POWER_MINPWR_SET_ADDR);
	// Bit 4: Reduce DC-DC analog Bias
	writel(0x10, (void*)HW_POWER_MINPWR_SET_ADDR);
	// Bit 11: Change the reference used in the dcdc converter
	writel(0x800, (void*)HW_POWER_MINPWR_SET_ADDR);
}

static void mxs_power_disable_5v_connection_irq(struct mxs_power_dev *power)
{
	writel(BM_POWER_CTRL_ENIRQ_VBUS_VALID, power->regs + MXS_POWER_CTRL_CLR);
	writel(BM_POWER_CTRL_ENIRQ_VDD5V_GT_VDDIO, power->regs + MXS_POWER_CTRL_CLR);
}

//	time between powerloss detection and commit reboot command
#define WAITTOREBOOT	6
//static char * reboot_argv[] = { "/sbin/reboot", NULL };
static char * reboot_argv[] = { "/bin/systemctl", "--force", "reboot" };

static void RebootTimerFire(unsigned long data);

static struct timer_list rebootTimer = TIMER_INITIALIZER(RebootTimerFire, 0, 0);

//	Timer expired
static void RebootTimerFire(unsigned long data)
{
	del_timer(&rebootTimer);
	printk(KERN_INFO "%s: Execute Reboot...\n", __func__);
	call_usermodehelper(reboot_argv[0], reboot_argv, NULL, UMH_NO_WAIT);
}

//	Start the timer after powerloss detection
static void StartRebootCountDown(void)
{
	mod_timer(&rebootTimer, jiffies+(WAITTOREBOOT*HZ));
}

static irqreturn_t mxs_power_vdd5v_isr(int this_irq, void *dev_id)
{
#ifndef REPORT_IRQ_ON_WRITE
	pid_t pid = atomic_read(&pid_value);
#endif

	struct mxs_power_dev *power = dev_id;
	u32 stat = readl(power->regs + MXS_POWER_CTRL);

	if (stat & BM_POWER_CTRL_VDD5V_GT_VDDIO_IRQ)
	{
		printk(KERN_INFO "%s: detect VDD5V_GT_VDDIO_IRQ\n", __func__);
		mxs_power_disable_5v_connection_irq(power);

		mxs_power_disableMacs();
		mxs_power_diableUarts();
		mxs_power_SetMinPower();

		StartRebootCountDown();

#ifdef REPORT_IRQ_ON_WRITE
		wake_up_interruptible (&wq);
		atomic_set(&data_ready, 1);
		atomic_set(&vdd5v_value, 1);
#else
		if (pid > 0)
		{
			struct siginfo		sinfo;
			struct task_struct	*task;

			memset(&sinfo, 0, sizeof(struct siginfo));
			sinfo.si_signo = SIGVDD5V;
			sinfo.si_code  = SI_USER;

			rcu_read_lock();
			task = pid_task(find_vpid(pid), PIDTYPE_PID);
			rcu_read_unlock();
			if (task)
				send_sig_info (SIGVDD5V, &sinfo, task);
			else
				printk(KERN_INFO "%s: Cannot find pid from user program\n", __func__);
		}
#endif
	}
	return IRQ_HANDLED;
}

static void mxs_power_enable_5v_disconnect_detection(struct mxs_power_dev *power)
{
	uint32_t val;

	// Disable hardware power down when 5V is inserted or removed
	writel(BM_POWER_5VCTRL_PWDN_5VBRNOUT,
		power->regs + MXS_POWER_5VCTRL_CLR);
	//printk("pointer power->regs + MXS_POWER_5VCTRL_CLR: %p\n", power->regs + MXS_POWER_5VCTRL_CLR);
	/*
	 * Enabling VBUSVALID hardware detection even if VDD5V_GT_VDDIO
	 * is the detection method being used for 5V status (hardware
	 * or software).  This is in case any other drivers (such as
	 * USB) are specifically monitoring VBUSVALID status
	 */
	writel(BM_POWER_5VCTRL_VBUSVALID_5VDETECT,
		power->regs + MXS_POWER_5VCTRL_SET);

	// Set 5V detection threshold to 4.2V for VBUSVALID. */
	writel(BM_POWER_5VCTRL_VBUSVALID_TRSH,
		power->regs + MXS_POWER_5VCTRL_CLR);
	writel(BF_POWER_5VCTRL_VBUSVALID_TRSH(VBUSVALID_THRESH_4_20V),
		power->regs + MXS_POWER_5VCTRL_SET);

	// gotta set LINREG_OFFSET to STEP_BELOW according to manual
	val = readl(power->regs + MXS_POWER_VDDIOCTRL);
	val &= ~(BM_POWER_VDDIOCTRL_LINREG_OFFSET);
	val |= BF_POWER_VDDIOCTRL_LINREG_OFFSET(LINREG_OFFSET_STEP_BELOW);
	writel(val, power->regs + MXS_POWER_VDDIOCTRL);

	val = readl(power->regs + MXS_POWER_VDDACTRL);
	val &= ~(BM_POWER_VDDACTRL_LINREG_OFFSET);
	val |= BF_POWER_VDDACTRL_LINREG_OFFSET(LINREG_OFFSET_STEP_BELOW);
	writel(val, power->regs + MXS_POWER_VDDACTRL);

	val = readl(power->regs + MXS_POWER_VDDDCTRL);
	val &= ~(BM_POWER_VDDDCTRL_LINREG_OFFSET);
	val |= BF_POWER_VDDDCTRL_LINREG_OFFSET(LINREG_OFFSET_STEP_BELOW);
	writel(val, power->regs + MXS_POWER_VDDDCTRL);

	// Clear interrupt flag
	writel(BM_POWER_CTRL_VBUSVALID_IRQ, power->regs + MXS_POWER_CTRL_CLR);
	writel(BM_POWER_CTRL_VDD5V_GT_VDDIO_IRQ, power->regs + MXS_POWER_CTRL_CLR);

	// Set polarity for disconnect detection
	writel(BM_POWER_CTRL_POLARITY_VBUSVALID,	power->regs + MXS_POWER_CTRL_CLR);
	writel(BM_POWER_CTRL_POLARITY_VDD5V_GT_VDDIO, power->regs + MXS_POWER_CTRL_CLR);

#ifdef REPORT_IRQ_ON_WRITE
	atomic_set(&data_ready, 0);
	atomic_set(&vdd5v_value, 0);
#endif

	// Enable 5V detection interrupt
	printk(KERN_INFO "%s: enable VDD5V_GT_VDDIO_IRQ\n", __func__);
	writel(BM_POWER_CTRL_ENIRQ_VDD5V_GT_VDDIO,
	power->regs + MXS_POWER_CTRL_SET);
}

static int file_open(struct inode *inode, struct file *filp)
{
#ifdef REPORT_IRQ_ON_WRITE
	struct _instance_data *iptr = (struct _instance_data *) kmalloc(sizeof(struct _instance_data), GFP_KERNEL);
	if (!iptr)
		return -ENOMEM;
	iptr->value = 0;
	iptr->pending_value = 0;
	iptr->sleeping = 0;
	filp->private_data = (void *)iptr;
#else
	/*
	 * In order to be able to send a signal from kernel space to user space,
	 * the kernel needs to know the pid of the user space process.
	 */
	atomic_set(&pid_value, current->pid);
#endif
	printk(KERN_DEBUG "%s: process %i (%s) ref %ld\n", __func__, current->pid, current->comm, module_refcount (THIS_MODULE));
	return 0;
}

static int file_release (struct inode *inode, struct file *filp)
{
#ifdef REPORT_IRQ_ON_WRITE
	if (filp->private_data)
		kfree(filp->private_data);
#else
	atomic_set(&pid_value, 0);
#endif
	printk(KERN_DEBUG "%s: process %i (%s) ref %ld\n", __func__, current->pid, current->comm, module_refcount (THIS_MODULE));
	return 0;
}

static ssize_t file_read(struct file *filp, char __user *buf, size_t count, loff_t *offp)
{
#ifdef REPORT_IRQ_ON_WRITE
	int result = 0;
	struct _instance_data *data = (struct _instance_data *)filp->private_data;

	if (data->pending_value)
	{
		data->pending_value = 0;
		if (put_user(data->value, (int *)buf))
			return -EFAULT;
		return sizeof(data->value);
	}

	if (!atomic_read (&data_ready))
	{
		printk(KERN_DEBUG "process %i (%s) going to sleep\n", current->pid, current->comm);
		wait_event_interruptible (wq, (atomic_read (&data_ready)));
		printk(KERN_DEBUG "process %i (%s) awakening\n", current->pid, current->comm);
	}
	atomic_set(&data_ready, 0);

	if (atomic_read (&vdd5v_value))
		result |= EVT_VDD5V_IRQ;

	if (put_user(result, (int *)buf))
		return -EFAULT;

	return sizeof(result);
#else
	return 0;
#endif
}

#ifdef REPORT_IRQ_ON_WRITE
/*
 * polling does not supported by debugfs
 */
static unsigned int file_poll (struct file *filp, poll_table * wait)
{
	struct _instance_data *data = (struct _instance_data *)filp->private_data;

	poll_wait (filp, &wq, wait);

	if (data->sleeping)
	{
		data->value = 0;
		if (atomic_read (&vdd5v_value))
			data->value |= EVT_VDD5V_IRQ;
		data->pending_value = 1;
		data->sleeping = 0;
		return POLLIN | POLLRDNORM;
	}
	else
		data->sleeping = 1;

	return 0;
}
#endif

static const struct file_operations mxs_power_debug_fops =
{
	.owner = THIS_MODULE,
	.open = file_open,
	.release = file_release,
	.read = file_read,
#ifdef REPORT_IRQ_ON_WRITE
	.poll = file_poll,
#endif
};

static int mxs_power_probe(struct platform_device *pdev)
{
	struct device *dev = &pdev->dev;
	struct mxs_power_dev *power;
	struct resource *res;
	resource_size_t res_size;
	int err, irq;

	power = devm_kzalloc(dev, sizeof(struct mxs_power_dev), GFP_KERNEL);
	if (!power)
		return -ENOMEM;

	res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	// BM_POWER_CTRL_VDD5V_GT_VDDIO_IRQ corresponds to dts interrupt source 0
	irq = platform_get_irq(pdev, 0);
	if (!res || irq < 0)
		return -ENOENT;

	res_size = resource_size(res);
	if (!devm_request_mem_region(dev, res->start, res_size, res->name))
		return -EBUSY;

	power->regs = devm_ioremap_nocache(dev, res->start, res_size);
	if (!power->regs)
		return -EBUSY;

	mxs_power_disable_5v_connection_irq(power);

	err = devm_request_irq(dev, irq, mxs_power_vdd5v_isr, 0, dev_name(dev), power);
	if (err)
		return err;

	power->dev = dev;

	atomic_set(&pid_value, 0);
	power->debug_file = debugfs_create_file("vdd5v", 0666, NULL, power, &mxs_power_debug_fops);

	platform_set_drvdata(pdev, power);
	mxs_power_enable_5v_disconnect_detection(power);
	return 0;
}

static int mxs_power_remove(struct platform_device *pdev)
{
	struct mxs_power_dev *power = platform_get_drvdata(pdev);
	mxs_power_disable_5v_connection_irq(power);
	debugfs_remove(power->debug_file);
	return 0;
}

static const struct of_device_id mxs_power_dt_ids[] = {
	{ .compatible = "fsl,imx28-power", },
	{ /* sentinel */ }
};
MODULE_DEVICE_TABLE(of, mxs_power_dt_ids);

static struct platform_driver mxs_power_driver = {
	.driver = {
		.name = DRIVER_NAME,
		.owner = THIS_MODULE,
		.of_match_table = mxs_power_dt_ids,
	},
	.remove = mxs_power_remove,
};

static int __init mxs_power_init(void)
{
	return platform_driver_probe(&mxs_power_driver, mxs_power_probe);
}
subsys_initcall(mxs_power_init);

static void __exit mxs_power_exit(void)
{
	platform_driver_unregister(&mxs_power_driver);
}
module_exit(mxs_power_exit);

MODULE_AUTHOR("B.Bauch");
MODULE_DESCRIPTION("MXS IMX28 Power Driver");
MODULE_LICENSE("GPL");
MODULE_ALIAS("platform:" DRIVER_NAME);
