/*****************************************************************
 *
 * KSZ8863 - Demo Application
 *
 * Copyright (c) 2012 Birger Bauch, Dr. Neuhaus Telekommunikation GmbH
 *
 *****************************************************************/

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>
#include <termios.h>
#include <errno.h>

#include <sys/ioctl.h>
#include <sys/types.h>
#include <syslog.h>

#ifdef MEMLIB_ENABLED
#include "memlib.h"
#endif

#include <assert.h>

#include "./spi-ksz8863.h"

/****************************************************************/

#ifndef VERSION
#define VERSION "v0.01 - "__DATE__" "__TIME__
#endif

#ifndef false
#define	false	0
#endif

#ifndef true
#define	true	1
#endif

/****************************************************************/

static void ksz8863_dump()
{
	int fd = open("/dev/ksz8863", O_RDWR);
	if (fd < 0)
	{
		printf("%s: can't open /dev/ksz8863, try 'insmod ksz8863.ko'\n", __FUNCTION__);
		return;
	}
	else
	{
		unsigned char buf[255];
		read(fd, buf, sizeof(buf));
	}
	close(fd);
}

static void ksz8863_read(unsigned char address)
{
	int rc, fd = open("/dev/ksz8863", O_RDWR);
	if (fd < 0)
	{
		printf("%s: can't open /dev/ksz8863, try 'insmod ksz8863.ko'\n", __FUNCTION__);
		return;
	}
	else
	{
		unsigned char buf[2];
		buf[0] = address;
		if (0 > (rc =  ioctl(fd, IOCTL_KSZ8863_READ, buf)))
			printf("%s: ioctl error %d\n", __FUNCTION__, rc);
		else
			printf("%s: address=%02x : %02x\n", __func__, buf[0], buf[1]);
	}
	close(fd);
}

static void ksz8863_write(unsigned char address, unsigned char value)
{
	int rc, fd = open("/dev/ksz8863", O_RDWR);
	if (fd < 0)
	{
		printf("%s: can't open /dev/ksz8863, try 'insmod ksz8863.ko'\n", __FUNCTION__);
		return;
	}
	else
	{
		unsigned char buf[2];
		buf[0] = address;
		buf[1] = value;
		if (0 > (rc =  ioctl(fd, IOCTL_KSZ8863_WRITE, buf)))
			printf("%s: ioctl error %d\n", __FUNCTION__, rc);
	}
	close(fd);
}

#if 0
typedef enum
{
	e_TableSelect_StaticMAC,
	e_TableSelect_VLAN,
	e_TableSelect_dynamicAddress,
	e_TableSelect_MIBcounter
} t_eTableSelect;

static void ksz8863_read_indirect(const t_eTableSelect tsel, const unsigned short entry, unsigned char* dest)
{
	int i, rc, fd = open("/dev/ksz8863", O_RDWR);
	if (fd < 0)
	{
		printf("%s: can't open /dev/ksz8863, try 'insmod ksz8863.ko'\n", __FUNCTION__);
		return;
	}
	else
	{
		unsigned char buf[2];
		buf[0] = 110;
		buf[1] = ((entry >> 8) & 0x03) | 0x10;	//Bit 9-8 of indirect address, read cycle
		switch (tsel)
		{
			default:
			case e_TableSelect_StaticMAC:
				break;
			case e_TableSelect_VLAN:
				buf[1] |= 0x04;
				break;
			case e_TableSelect_dynamicAddress:
				buf[1] |= 0x08;
				break;
			case e_TableSelect_MIBcounter:
				buf[1] |= 0x0c;
				break;
		}
		if (0 > (rc =  ioctl(fd, IOCTL_KSZ8863_WRITE, buf)))
			printf("%s: ioctl error %d\n", __FUNCTION__, rc);

		//trigger the read operation
		buf[0] = 111;
		buf[1] = entry & 0xff;					//Bit 7-0 of indirect address
		if (0 > (rc =  ioctl(fd, IOCTL_KSZ8863_WRITE, buf)))
			printf("%s: ioctl error %d\n", __FUNCTION__, rc);

		//read register 112..120
		for (i=0; i < 9; i++)
		{
			buf[0] = 112 + i;
			if (0 > (rc =  ioctl(fd, IOCTL_KSZ8863_READ, buf)))
				printf("%s: ioctl error %d\n", __FUNCTION__, rc);
			else
				dest[i] = buf[1];
		}
	}
	close(fd);
}

static void ksz8863_write_indirect(const t_eTableSelect tsel, const unsigned short entry, const unsigned char* src)
{
	int i, rc, fd = open("/dev/ksz8863", O_RDWR);
	if (fd < 0)
	{
		printf("%s: can't open /dev/ksz8863, try 'insmod ksz8863.ko'\n", __FUNCTION__);
		return;
	}
	else
	{
		unsigned char buf[2];
		buf[0] = 110;
		buf[1] = ((entry >> 8) & 0x03) | 0x10;	//Bit 9-8 of indirect address, read cycle
		switch (tsel)
		{
			default:
			case e_TableSelect_StaticMAC:
				break;
			case e_TableSelect_VLAN:
				buf[1] |= 0x04;
				break;
			case e_TableSelect_dynamicAddress:
				buf[1] |= 0x08;
				break;
			case e_TableSelect_MIBcounter:
				buf[1] |= 0x0c;
				break;
		}
		if (0 > (rc =  ioctl(fd, IOCTL_KSZ8863_WRITE, buf)))
			printf("%s: ioctl error %d\n", __FUNCTION__, rc);

		//write register 112..120
		for (i=0; i < 9; i++)
		{
			buf[0] = 112 + i;
			buf[1] = src[i];
			if (0 > (rc =  ioctl(fd, IOCTL_KSZ8863_WRITE, buf)))
				printf("%s: ioctl error %d\n", __FUNCTION__, rc);
		}

		buf[0] = 110;
		buf[1] = ((entry >> 8) & 0x03);		//Bit 9-8 of indirect address, write cycle
		switch (tsel)
		{
			default:
			case e_TableSelect_StaticMAC:
				break;
			case e_TableSelect_VLAN:
				buf[1] |= 0x04;
				break;
			case e_TableSelect_dynamicAddress:
				buf[1] |= 0x08;
				break;
			case e_TableSelect_MIBcounter:
				buf[1] |= 0x0c;
				break;
		}
		if (0 > (rc =  ioctl(fd, IOCTL_KSZ8863_WRITE, buf)))
			printf("%s: ioctl error %d\n", __FUNCTION__, rc);

		//trigger the write operation
		buf[0] = 111;
		buf[1] = entry & 0xff;					//Bit 7-0 of indirect address
		if (0 > (rc =  ioctl(fd, IOCTL_KSZ8863_WRITE, buf)))
			printf("%s: ioctl error %d\n", __FUNCTION__, rc);
	}
	close(fd);
}

static void ksz8863_read_mac(unsigned short entry)
{
	int i;
	unsigned char buf[9];	//register 112..120
	ksz8863_read_indirect(e_TableSelect_StaticMAC, entry, buf);
	printf("tab=%4d :", entry);	//ignore register 112
	for (i=1; i<9; i++)
		printf(" %02x", buf[i]);
	printf("\n");
}

static unsigned short ksz8863_get_vlan_table_entry(const int index, const unsigned char* src)
{
	//15 14 13 12 11 10  9  8  7  6  5  4  3  2  1  0
	// 0  0  0 en ------MBR----- --------FID---------
	switch (index)
	{
		default:
		case 0:
			return ((src[7] & 0x1f) << 8) + src[8];
		case 1:
			return ((src[5] & 0x03) << 11) + (src[6] << 3) + ((src[7] & 0xe0) >> 5);
		case 2:
			return ((src[4] & 0x7f) << 6) + ((src[5] & 0xfc) >> 2);
		case 3:
			return ((src[2] & 0x0f) << 9) + (src[3] << 1) + ((src[4]  & 0x80) >> 7);
	}
}

static void ksz8863_set_vlan_table_entry(const int index, unsigned char* dst, const unsigned short entry)
{
	switch (index)
	{
		default:
		case 0:
			dst[7] &= 0xe0;
			dst[7] |= (entry >> 8) & 0x1f;
			dst[8] = entry & 0xff;
			break;
		case 1:
			dst[7] &= 0x1f;
			dst[7] |= (entry << 5) & 0xe0;
			dst[6] = entry >> 3;
			dst[5] &= 0xfc;
			dst[5] |= (entry >> 11) & 0x03;
			break;
		case 2:
			dst[5] &= 0x03;
			dst[5] |= (entry << 2) & 0xfc;
			dst[4] &= 0x80;
			dst[4] |= (entry >> 6) & 0x7f;
			break;
		case 3:
			dst[4] &= 0x7f;
			dst[4] |= (entry << 7) & 0x80;
			dst[3] = entry >> 1;
			dst[2] &= 0xf0;
			dst[2] |= (entry >> 9) & 0x0f;
			break;
	}
}

static void ksz8863_read_vtable(unsigned short entry)
{
	unsigned short i;
	unsigned char buf[9];	//register 112..120
	ksz8863_read_indirect(e_TableSelect_VLAN, entry, buf);
	printf("tab=%4d :", entry);	//ignore register 112, 113
	for (i=2; i<9; i++)
		printf(" %02x", buf[i]);
	printf("\n");

	i = ksz8863_get_vlan_table_entry(0, buf);
	printf("vid=%4d : %04x : valid[%d] mbr[%02x] fid[%3d]\n", entry*4+0, i, (i & 0x1000)?1:0, (i >> 7) & 0x1f, i & 0x7f);
	i = ksz8863_get_vlan_table_entry(1, buf);
	printf("vid=%4d : %04x : valid[%d] mbr[%02x] fid[%3d]\n", entry*4+1, i, (i & 0x1000)?1:0, (i >> 7) & 0x1f, i & 0x7f);
	i = ksz8863_get_vlan_table_entry(2, buf);
	printf("vid=%4d : %04x : valid[%d] mbr[%02x] fid[%3d]\n", entry*4+2, i, (i & 0x1000)?1:0, (i >> 7) & 0x1f, i & 0x7f);
	i = ksz8863_get_vlan_table_entry(3, buf);
	printf("vid=%4d : %04x : valid[%d] mbr[%02x] fid[%3d]\n", entry*4+3, i, (i & 0x1000)?1:0, (i >> 7) & 0x1f, i & 0x7f);
}

static void ksz8863_update_vtable(const unsigned int vid, const int valid, const unsigned char mbr, const unsigned int fid)
{
	unsigned short entry = (vid & 0xfff) >> 2;
	unsigned short index = (vid & 0x03);
	unsigned char buf[9];	//register 112..120
	ksz8863_read_indirect(e_TableSelect_VLAN, entry, buf);

	unsigned short i = (fid & 0x7f) | ((mbr & 0x1f) << 7);
	if (valid)
		i |= 0x1000;
	ksz8863_set_vlan_table_entry(index, buf, i);
	ksz8863_write_indirect(e_TableSelect_VLAN, entry, buf);
}

static void ksz8863_read_dynamicAddress(unsigned short entry)
{
	int i;
	unsigned char buf[9];	//register 112..120
	ksz8863_read_indirect(e_TableSelect_dynamicAddress, entry, buf);
	printf("tab=%4d : %x", entry, buf[0]);
	for (i=1; i<9; i++)
		printf(" %02x", buf[i]);
	printf("\n");
}
#endif

static int print_help(void)
{
	printf("\nKSZ8863 - Demo Application\n");
	printf("\n");
	printf("\t-dump_reg\n");
#if 0
	printf("\t-dump_mac <entry> (0..31)\n");
	printf("\t-dump_vtab <entry> (0..1023)\n");
	printf("\t-dump_dynamic <entry> (0..1023)\n");
	printf("\t-add_vlan <vid> (0..4095) <fid> (0..127) <mbr> (0x00..0x1f)\n");
	printf("\t-del_vlan <vid> (0..4095)\n");
#endif
	printf("\t-rd <value> (0x00..0xff)\n");
	printf("\t-wr <address> (0x00..0xff) <value> (0x00..0xff)\n");
	printf("\n");
	printf("\t-v version\n");
	printf("\t-? help\n");
	printf("\n");

	return EXIT_SUCCESS;
}

static int print_error(const char *Text)
{
	printf("error: %s\n", Text);
	return EXIT_SUCCESS;
}

static void get_mobile_copyright(char *cText, const unsigned int size)
{
	char tmpText[120];

	if (size < 120)
	{
		cText[0] = 0x00;
		return;
	}
	strcpy(cText, "\nKZS8863 Demo Application\n");

#ifdef __GNUC__
	snprintf(tmpText, sizeof(tmpText), "\n%s\nGCC %s", VERSION, __VERSION__);
#elif defined _MSC_VER
	snprintf(tmpText, sizeof(tmpText), "\n%s\nMSC %d", VERSION, _MSC_VER);
#else
	snprintf(tmpText, sizeof(tmpText), "\n%s", VERSION);
#endif
	if ((strlen(cText) + strlen(tmpText)) < size)
		strcat(cText, tmpText);

	strcpy(tmpText, "\nCopyright (c) 2012 Dr.Neuhaus Telekommunikation GmbH\n");
	if ((strlen(cText) + strlen(tmpText)) < size)
		strcat(cText, tmpText);

	strcpy(tmpText, "\n");
	if ((strlen(cText) + strlen(tmpText)) < size)
		strcat(cText, tmpText);
}

int main(int argc, char **argv)
{
	int arg_index = 1;
	int value = 0;
	int address = 0;
#if 0
	int entry = 0;
	int unsigned vid = 0;
	int vid_valid = 0;
	unsigned char vid_mbr = 0;
	unsigned int vid_fid = 0;
#endif
	enum
	{
		UNDEFINED, DUMP, DUMP_MAC, DUMP_VTAB, DUMP_DYNAMIC_MAC, UPDATE_VID, READ, WRITE
	} action = UNDEFINED;

	while (arg_index < argc)
	{
		if (0 == strcmp(argv[arg_index], "-v"))
		{
			char *cText;
#			define COPYRIGHT_SIZE	1024
			if (NULL != (cText = malloc(COPYRIGHT_SIZE)))
			{
				get_mobile_copyright(cText, COPYRIGHT_SIZE);
				printf("%s", cText);
				free(cText);
			}
			return EXIT_SUCCESS;
		}
		else if (0 == strcmp(argv[arg_index], "-?"))
			return print_help();
		else if (0 == strcmp(argv[arg_index], "-h"))
			return print_help();
		else if (0 == strcmp(argv[arg_index], "-dump_reg"))
		{
			action = DUMP;
			break;
		}
#if 0
		else if (0 == strcmp(argv[arg_index], "-dump_mac"))
		{
			arg_index++;
			if (arg_index < argc)
			{
				entry = atoi(argv[arg_index]);
				action = DUMP_MAC;
				break;
			}
		}
		else if (0 == strcmp(argv[arg_index], "-dump_vtab"))
		{
			arg_index++;
			if (arg_index < argc)
			{
				entry = atoi(argv[arg_index]);
				action = DUMP_VTAB;
				break;
			}
		}
		else if (0 == strcmp(argv[arg_index], "-dump_dynamic"))
		{
			action = DUMP_DYNAMIC_MAC;
			break;
		}
		else if (0 == strcmp(argv[arg_index], "-add_vlan"))
		{
			arg_index++;
			if (arg_index < argc)
			{
				vid = atoi(argv[arg_index]);

				arg_index++;
				if (arg_index < argc)
				{
					vid_fid = atoi(argv[arg_index]);

					arg_index++;
					if (arg_index < argc)
					{
						vid_mbr = strtoul(argv[arg_index], NULL, 16);

						vid_valid = 1;
						action = UPDATE_VID;
						break;
					}
				}
			}
		}
		else if (0 == strcmp(argv[arg_index], "-del_vlan"))
		{
			arg_index++;
			if (arg_index < argc)
			{
				vid = atoi(argv[arg_index]);
				action = UPDATE_VID;
				break;
			}
		}
#endif
		else if (0 == strcmp(argv[arg_index], "-rd"))
		{
			arg_index++;
			if (arg_index < argc)
			{
				address = strtoul(argv[arg_index], NULL, 16);
				action = READ;
				break;
			}
		}
		else if (0 == strcmp(argv[arg_index], "-wr"))
		{
			arg_index++;
			if (arg_index < argc)
			{
				address = strtoul(argv[arg_index], NULL, 16);

				arg_index++;
				if (arg_index < argc)
				{
					value = strtoul(argv[arg_index], NULL, 16);
					action = WRITE;
					break;
				}
				else
					return print_error("missing value, try -? for help");
			}
			else
				return print_error("missing address, try -? for help");
		}
		arg_index++;
	}

	switch (action)
	{
		default:
			return print_error("missing parameter, try -? for help");
		case DUMP:
			ksz8863_dump();
			break;
#if 0
		case DUMP_MAC:
			ksz8863_read_mac(entry & 0x1f);
			break;
		case DUMP_VTAB:
			ksz8863_read_vtable(entry & 0x3ff);
			break;
		case DUMP_DYNAMIC_MAC:
			ksz8863_read_dynamicAddress(entry & 0x3ff);
			break;
		case UPDATE_VID:
			ksz8863_update_vtable(vid, vid_valid, vid_mbr, vid_fid);
			break;
#endif
		case READ:
			ksz8863_read(address);
			break;
		case WRITE:
			ksz8863_write(address, value);
			break;
	}
	return EXIT_SUCCESS;
}
