
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

//type definitions
typedef unsigned char 		u8;
typedef unsigned short 		u16;
typedef unsigned int 		u32;
#ifdef __HAS_LONG_LONG__
typedef unsigned long long 	u64;
#endif
#ifndef VERSION
#define VERSION "v0.0.1 - "__DATE__" "__TIME__
#endif

/* FEC MII MMFR bits definition */
#define FEC_MMFR_ST		(1 << 30)
#define FEC_MMFR_OP_READ	(2 << 28)
#define FEC_MMFR_OP_WRITE	(1 << 28)
#define FEC_MMFR_PA(v)		((v & 0x1f) << 23)
#define FEC_MMFR_RA(v)		((v & 0x1f) << 18)
#define FEC_MMFR_TA		(2 << 16)
#define FEC_MMFR_DATA(v)	(v & 0xffff)

#define FEC_MII_TIMEOUT		30000 /* us */

#define HW_ENET_MAC_MSCR 0x800F0044
#define MSCR_SPEED  0x0000043C // HOLD Time and Speed
#define MSCR_DISABLE  0x00000000


#define HW_ENET_MAC_MMFR 0x800F0040

static int print_help(void)
{
	printf("\nMII - memtool  Application %s\n", VERSION);
	printf("\n");
	printf("\t-rd <index> (0x0..0x1f) <address> (0x00..0x1f)\n");
	printf("\t-wr <index> (0x0..0x1f) <address> (0x00..0x1f) <value> (0x0000..0xffff)\n");
	printf("\n");
	printf("\t-? help\n");
	printf("\n");
	return 0;
}

static int print_error(const char *Text)
{
	printf("error: %s\n", Text);
	return 0;
}

static int disable_clock()
{
    char cmd[50];
    snprintf(cmd, sizeof(cmd), "/usr/bin/memtool 0x%x=0x%x", HW_ENET_MAC_MSCR, MSCR_DISABLE);
    printf("cmd= %s\n", cmd );
    int ret = system(cmd);

	return ret;
}

static int enable_clock()
{
    char cmd[50];
    snprintf(cmd, sizeof(cmd), "/usr/bin/memtool 0x%x=0x%x", HW_ENET_MAC_MSCR, MSCR_SPEED);
    printf("cmd= %s\n", cmd );
    int ret = system(cmd);

	return ret;
}


int main(int argc, char **argv)
{

    int arg_index = 1;

	while (arg_index < argc)
	{
		if (0 == strcmp(argv[arg_index], "-?"))
			return print_help();
		else if (0 == strcmp(argv[arg_index], "-h"))
			return print_help();
		else if (0 == strcmp(argv[arg_index], "-enable"))
		{
                    enable_clock();
		}

		else if (0 == strcmp(argv[arg_index], "-disable"))
		{
                   disable_clock();
				   break;
		}
		arg_index++;
   }

    return 0;
}
