/*****************************************************************
 *
 * MII network interface - Demo Application
 *
 * Copyright (c) 2013 Birger Bauch, Dr. Neuhaus Telekommunikation GmbH
 *
 *****************************************************************/

#include "mii_demo.h"

#ifndef VERSION
#define VERSION "v0.02 - "__DATE__" "__TIME__
#endif

/****************************************************************/

cMDIO::cMDIO(unsigned int phy_index, const char* name)
{
	idx = phy_index;

	if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
		printf("%s: can't open socket\n", __func__);
	else
	{
		strncpy(ifr.ifr_name, name, IFNAMSIZ);
		/*
        printf("%s: phy_index on '%i' \n", __func__, phy_index);
		printf("%s: ifr.ifr_name on '%s' \n", __func__, ifr.ifr_name);
		printf("%s: SIOCGMIIPHY on '0x%x' \n", __func__, SIOCGMIIPHY);
		printf("%s: SIOCGMIIPHY2 on '0x%x' \n", __func__, SIOCGMIIPHY2);
    	*/
        // Get address of MII PHY in use
		if (ioctl(sock, SIOCGMIIPHY, &ifr) < 0)
		{
			if (errno != ENODEV)
				printf("%s: SIOCGMIIPHY on '%s' failed: %s\n", __func__, ifr.ifr_name, strerror(errno));
			close(sock);
			sock = -1;
		}
	}
}

cMDIO::~cMDIO()
{
	if (sock >= 0)
	{
		close(sock);
		sock = -1;
	}
}

int cMDIO::mdio_read(unsigned int phy_id, unsigned int location)
{
	if (sock >= 0)
	{
		struct mii_data *mii = (struct mii_data *) &ifr.ifr_data;
		mii->phy_id=phy_id;
		mii->reg_num = location;
		/*
        printf("%s: idx on '%i' \n", __func__, idx);
        printf("%s: ifr.ifr_name on '%s' \n", __func__, ifr.ifr_name);
		printf("%s: SIOCGMIIPHY on '0x%x' \n", __func__, SIOCGMIIPHY);
		printf("%s: SIOCGMIIPHY2 on '0x%x' \n", __func__, SIOCGMIIPHY2);
		*/

		if (ioctl(sock, SIOCGMIIREG, &ifr) < 0)
		{
			printf("SIOCGMIIREG on %s failed: %s\n", ifr.ifr_name, strerror(errno));
			return -1;
		}
		return mii->val_out;
	}
	return -1;
}

int cMDIO::mdio_write(unsigned int phy_id, unsigned int location, int value)
{
	if (sock >= 0)
	{   /*
		printf("%s: idx on '%i' \n", __func__, idx);
        printf("%s: ifr.ifr_name on '%s' \n", __func__, ifr.ifr_name);
		printf("%s: SIOCGMIIPHY on '0x%x' \n", __func__, SIOCGMIIPHY);
		printf("%s: SIOCGMIIPHY2 on '0x%x' \n", __func__, SIOCGMIIPHY2);
		*/
		struct mii_data *mii = (struct mii_data *) &ifr.ifr_data;
		mii->phy_id=phy_id;
        mii->reg_num = location;
		mii->val_in = value;
		if (ioctl(sock, SIOCSMIIREG, &ifr) < 0)
		{
			printf("SIOCGMIIREG on %s failed: %s\n", ifr.ifr_name, strerror(errno));
			return -1;
		}
		return EXIT_SUCCESS;
	}
	return -1;
}

void cMDIO::dump(unsigned int phy_id)
{
	//printf("%s: phy_id on '%i' \n", __func__, phy_id);
	int i, mii_val[32];
	for (i = 0; i < 32; i++)
		mii_val[i] = mdio_read(phy_id, i);

    //printf("%s: mii_val[MII_BMCR] on '%x' \n", __func__, mii_val[MII_BMCR]);

	if (mii_val[MII_BMCR] == -1)
	{
		printf("%s: No MII transceiver present\n", __func__);
		return;
	}
	for (i = 0; i < 32; i++)
	{
		printf(" 0x%04X", mii_val[i]);
		if ((i == 7) || (i == 15) || (i == 23) || (i == 31))
			printf("\n");
	}
}

int cMDIO::read(unsigned int phy_id, unsigned int location)
{

/*
	printf("%s: phy_id on '%i' \n", __func__, phy_id);
	printf("%s: location on '%i' \n", __func__, location);
*/
	if (location > 32)
		location = 32;
	int mii_val = mdio_read(phy_id, location);
	printf(" 0x%04X\n", mii_val);
	return mii_val;
}

int cMDIO::write(unsigned int phy_id, unsigned int location, int value)
{
/*
    printf("%s: phy_id on '%i' \n", __func__, phy_id);
	printf("%s: location on '%i' \n", __func__, location);
	printf("%s: register value on '%x' \n", __func__, value);
*/
	if (location > 32)
		location = 32;
	return mdio_write(phy_id, location, value);
}

static int print_help(void)
{
	printf("\nMII - Demo Application %s\n", VERSION);
	printf("\n");
	printf("\t-dump <device> (eth0..eth1) <index> (0..31)\n");
	printf("\t-rd <device> (eth0..eth1) <index> (0..31) <address> (0x00..0x1f)\n");
	printf("\t-wr <device> (eth0..eth1) <index> (0..31) <address> (0x00..0x1f) <value> (0x0000..0xffff)\n");
	printf("\n");
	printf("\t-? help\n");
	printf("\n");
	return EXIT_SUCCESS;
}

static int print_error(const char *Text)
{
	printf("error: %s\n", Text);
	return EXIT_SUCCESS;
}

int main(int argc, char **argv)
{
	int arg_index = 1;
	char device[80] = "";
	int phy_index = 0;
	int phy_address = 0;
	int phy_value = 0;

	enum
	{
		UNDEFINED, DUMP, READ, WRITE
	} action = UNDEFINED;

	while (arg_index < argc)
	{
		if (0 == strcmp(argv[arg_index], "-?"))
			return print_help();
		else if (0 == strcmp(argv[arg_index], "-h"))
			return print_help();
		else if (0 == strcmp(argv[arg_index], "-dump"))
		{
			arg_index++;
			if (arg_index < argc)
			{
				strncpy(device, argv[arg_index], sizeof(device) - 1);

				arg_index++;
				if (arg_index < argc)
				{
					phy_index = atoi(argv[arg_index]);
					action = DUMP;
					break;
				}
				else
					return print_error("missing parameter, try -? for help");
			}
			else
				return print_error("missing parameter, try -? for help");
		}
		else if (0 == strcmp(argv[arg_index], "-rd"))
		{
			arg_index++;
			if (arg_index < argc)
			{
				strncpy(device, argv[arg_index], sizeof(device) - 1);

				arg_index++;
				if (arg_index < argc)
				{
					phy_index = atoi(argv[arg_index]);

					arg_index++;
					if (arg_index < argc)
					{
						phy_address = strtoul(argv[arg_index], NULL, 16);
						action = READ;
						break;
					}
					else
						return print_error("missing value, try -? for help");
				}
				else
					return print_error("missing address, try -? for help");
			}
			else
				return print_error("missing address, try -? for help");
		}
		else if (0 == strcmp(argv[arg_index], "-wr"))
		{
			arg_index++;
			if (arg_index < argc)
			{
				strncpy(device, argv[arg_index], sizeof(device) - 1);

				arg_index++;
				if (arg_index < argc)
				{
					phy_index = atoi(argv[arg_index]);

					arg_index++;
					if (arg_index < argc)
					{
						phy_address = strtoul(argv[arg_index], NULL, 16);

						arg_index++;
						if (arg_index < argc)
						{
							phy_value = strtoul(argv[arg_index], NULL, 16);
							action = WRITE;
							break;
						}
						else
							return print_error("missing value, try -? for help");
					}
					else
						return print_error("missing value, try -? for help");
				}
				else
					return print_error("missing value, try -? for help");
			}
			else
				return print_error("missing address, try -? for help");
		}
		arg_index++;
	}

	cMDIO mdio(phy_index, device);
	switch (action)
	{
		default:
			return print_error("missing parameter, try -? for help");
		case DUMP:
			mdio.dump(phy_index);
			break;
		case READ:
			mdio.read(phy_index, phy_address);
			break;
		case WRITE:
			mdio.write(phy_index, phy_address, phy_value);
			break;
	}
	return EXIT_SUCCESS;
}
