
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

//type definitions
typedef unsigned char 		u8;
typedef unsigned short 		u16;
typedef unsigned int 		u32;
#ifdef __HAS_LONG_LONG__
typedef unsigned long long 	u64;
#endif
#ifndef VERSION
#define VERSION "v0.0.1 - "__DATE__" "__TIME__
#endif

/* FEC MII MMFR bits definition */
#define FEC_MMFR_ST		(1 << 30)
#define FEC_MMFR_OP_READ	(2 << 28)
#define FEC_MMFR_OP_WRITE	(1 << 28)
#define FEC_MMFR_PA(v)		((v & 0x1f) << 23)
#define FEC_MMFR_RA(v)		((v & 0x1f) << 18)
#define FEC_MMFR_TA		(2 << 16)
#define FEC_MMFR_DATA(v)	(v & 0xffff)

#define FEC_MII_TIMEOUT		30000 /* us */

#define HW_ENET_MAC_MSCR 0x800F0044
#define MSCR_SPEED  0x0000043C // HOLD Time and Speed
#define MSCR_DISABLE  0x00000000


#define HW_ENET_MAC_MMFR 0x800F0040

static int print_help(void)
{
	printf("\nMII - memtool  Application %s\n", VERSION);
	printf("\n");
	printf("\t-rd <index> (0x0..0x1f) <address> (0x00..0x1f)\n");
	printf("\t-wr <index> (0x0..0x1f) <address> (0x00..0x1f) <value> (0x0000..0xffff)\n");
	printf("\n");
	printf("\t-? help\n");
	printf("\n");
	return 0;
}

static int print_error(const char *Text)
{
	printf("error: %s\n", Text);
	return 0;
}

static int disable_clock()
{
    char cmd[50];
    snprintf(cmd, sizeof(cmd), "/usr/bin/memtool 0x%x=0x%x", HW_ENET_MAC_MSCR, MSCR_DISABLE);
//  printf("cmd= %s\n", cmd );
    int ret = system(cmd);

	return ret;
}

static int enable_clock()
{
    char cmd[50];
    snprintf(cmd, sizeof(cmd), "/usr/bin/memtool 0x%x=0x%x", HW_ENET_MAC_MSCR, MSCR_SPEED);
//  printf("cmd= %s\n", cmd );
    int ret = system(cmd);

	return ret;
}



static int read(unsigned phy_address, unsigned regnum)
{
    //enable_clock();
    printf("phy_address=0x%x\n", phy_address);
    printf("regnum=0x%x\n", regnum);
    //printf("data=0x%x\n", data);
    int data = 0x0000;

	/* start a read op */
	unsigned int hw_enet_mac_mmfr_packet = (FEC_MMFR_ST | FEC_MMFR_OP_READ | FEC_MMFR_PA(phy_address) | FEC_MMFR_RA(regnum) | FEC_MMFR_TA | FEC_MMFR_DATA(data));

    printf("hw_enet_mac_mmfr=0x%x\n", hw_enet_mac_mmfr_packet);

    //int ret = system("/usr/bin/memtool  1");

    char cmd[50];
    //snprintf(cmd, sizeof(cmd), "/usr/bin/memtool 0x%x %x", HW_ENET_MAC_MMFR , 1);
    snprintf(cmd, sizeof(cmd), "/usr/bin/memtool 0x%x=0x%x", HW_ENET_MAC_MMFR, hw_enet_mac_mmfr_packet);
    printf("cmd= %s\n", cmd );
    int ret = system(cmd);
    usleep(300000);

    char cmd2[50];
    snprintf(cmd2, sizeof(cmd2), "/usr/bin/memtool 0x%x %x", HW_ENET_MAC_MMFR , 1);
    printf("cmd2= %s\n", cmd2 );
    int ret2 = system(cmd2);

   // disable_clock();
	return ret2;
}

static int write(unsigned phy_address, unsigned regnum, unsigned data)
{
    //enable_clock();
    printf("phy_address=0x%x\n", phy_address);
    printf("regnum=0x%x\n", regnum);
    printf("data=0x%x\n", data);

	/* start a write op */
	unsigned int hw_enet_mac_mmfr_packet2 = (FEC_MMFR_ST | FEC_MMFR_OP_WRITE | FEC_MMFR_PA(phy_address) | FEC_MMFR_RA(regnum) | FEC_MMFR_TA | FEC_MMFR_DATA(data));

    printf("hw_enet_mac_mmfr=0x%x\n", hw_enet_mac_mmfr_packet2);

    char cmd[50];
    snprintf(cmd, sizeof(cmd), "/usr/bin/memtool 0x%x=0x%x", HW_ENET_MAC_MMFR, hw_enet_mac_mmfr_packet2);
    printf("cmd= %s\n", cmd );
    int ret = system(cmd);

    //disable_clock();
	return ret;
}


int main(int argc, char **argv)
{

    int arg_index = 1;
	int phy_address = 0;
	int regnum = 0;
    int data = 0x0000;

	while (arg_index < argc)
	{
		if (0 == strcmp(argv[arg_index], "-?"))
			return print_help();
		else if (0 == strcmp(argv[arg_index], "-h"))
			return print_help();
		else if (0 == strcmp(argv[arg_index], "-rd"))
		{
			arg_index++;
			if (arg_index < argc)
			{
				phy_address = strtoul(argv[arg_index], NULL, 16);
			}
                arg_index++;
				if (arg_index < argc)
				{
					regnum = strtoul(argv[arg_index], NULL, 16);
                    read(phy_address, regnum);
					break;
				}
			else
				return print_error("missing value, try -? for help");
        }

		else if (0 == strcmp(argv[arg_index], "-wr"))
		{
			        arg_index++;
					if (arg_index < argc)
					{
						phy_address = strtoul(argv[arg_index], NULL, 16);

						arg_index++;
						if (arg_index < argc)
						{
							regnum = strtoul(argv[arg_index], NULL, 16);
						}
                            arg_index++;
						    if (arg_index < argc)
						    {
							    data = strtoul(argv[arg_index], NULL, 16);
                                write(phy_address, regnum, data);
							    break;
						    }

						else
							return print_error("missing value, try -? for help");
					}
					else
						return print_error("missing value, try -? for help");
			}
			else
				return print_error("missing address, try -? for help");
		}
		arg_index++;
/*
    printf("phy_address=0x%x\n", phy_address);
    printf("regnum=0x%x\n", regnum);
    printf("data=0x%x\n", data);
*/
    //int mii_id;

	/* start a read op */
	//unsigned int hw_enet_mac_mmfr = (FEC_MMFR_ST | FEC_MMFR_OP_READ | FEC_MMFR_PA(phy_address) | FEC_MMFR_RA(regnum) | FEC_MMFR_TA | FEC_MMFR_DATA(data));

    //printf("hw_enet_mac_mmfr=0x%x\n", hw_enet_mac_mmfr);

    //int ret = system("/usr/bin/memtool  1");

    //char cmd[50];
    //snprintf(cmd, sizeof(cmd), "/usr/bin/memtool %i %i", 1 , hw_enet_mac_mmfr);
    //int ret = system(cmd);

	/* wait for end of transfer */
	/*
    time_left = wait_for_completion_timeout(&fep->mdio_done,
			usecs_to_jiffies(FEC_MII_TIMEOUT));
	if (time_left == 0) {
		fep->mii_timeout = 1;
		printk(KERN_ERR "FEC: MDIO read timeout\n");
		return -ETIMEDOUT;
	}
*/

    /* start a write op */
	//unsigned int hw_enet_mac_mmfr2 = (FEC_MMFR_ST | FEC_MMFR_OP_WRITE | FEC_MMFR_PA(phy_address) | FEC_MMFR_RA(regnum) | FEC_MMFR_TA | FEC_MMFR_DATA(data));
    //printf("hw_enet_mac_mmfr2=0x%x\n", hw_enet_mac_mmfr2);

	/* wait for end of transfer */
/*
	time_left = wait_for_completion_timeout(&fep->mdio_done,
			usecs_to_jiffies(FEC_MII_TIMEOUT));
	if (time_left == 0) {
		fep->mii_timeout = 1;
		printk(KERN_ERR "FEC: MDIO write timeout\n");
		return -ETIMEDOUT;
	}
*/


    //printf("\n exit program \n");
    return 0;
}
