/*
 * Copyright (C) 2006-2008 Nokia Corporation

 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/err.h>
#include <linux/mtd/mtd.h>
#include <linux/slab.h>
#include <linux/sched.h>

//#define PRINT_PREF KERN_INFO "mtd_readtest: "
#define PRINT_PREF KERN_INFO ""

//+++++++++++++++++++++++++++++ Module Parameter Definition ++++++++++++++++++++

static int dev=999;				
static int ob;				
static int ops,ope;	
static int ErrBlock;
static int ErrPage;
static int RepBlock;
static int layout;
static int peb;
static int arep;
static int vformat=999;

// V3
static int mb=0;

//-------------------------------------------------
module_param(dev, int, S_IRUGO);
module_param(ob, int, S_IRUGO);
module_param(ops, int, S_IRUGO);
module_param(ope, int, S_IRUGO);
module_param(RepBlock, int, S_IRUGO);
module_param(ErrBlock, int, S_IRUGO);
module_param(ErrPage, int, S_IRUGO);
module_param(layout, int, S_IRUGO);
module_param(peb, int, S_IRUGO);
module_param(arep, int, S_IRUGO);
module_param(vformat, int, S_IRUGO);

// V3
module_param(mb, int, S_IRUGO);
//-------------------------------------------------

MODULE_PARM_DESC(dev, "MTD device number to use");
MODULE_PARM_DESC(ob, "Eraseblock number for output");
MODULE_PARM_DESC(ops, "Start pagenumber of the block for output");
MODULE_PARM_DESC(ope, "End pagenumber of the block for output");
MODULE_PARM_DESC(RepBlock, "Eraseblocknumber for rewriting");
MODULE_PARM_DESC(ErrBlock, "Eraseblocknumber to provoke an ECC error");
MODULE_PARM_DESC(ErrPage, "Pagenumber of ErrBlock for creating the ECC error");
MODULE_PARM_DESC(layout, "1 to output the UBI Layout Volume Information");
MODULE_PARM_DESC(peb, "1 to output additional information of each eraseblock");
MODULE_PARM_DESC(arep, "1 to activate the automatic repair feature");
MODULE_PARM_DESC(vformat, "Volume ID to format all blocks of this");

// V3
MODULE_PARM_DESC(mb, "Move the specified eraseblock to the last empty block");
//-------------------------------------------------

//++++++++++++++++++++++++++ Prototypes +++++++++++++++++++++++++++

void Format_Ubi_Volume(int ebnum);
static int read_eraseblock_by_page(int ebnum);
static int Erase_Block(int ebnum);
static void Rep_Block(int ebnum);
static void Error_Write_Page(void);
static int is_block_bad(int ebnum);
void Load_Layout_Volume(void);
static int scan_for_bad_eraseblocks(void);
void Move_Block(void);

//++++++++++++++++++++++++++ MTD Structure +++++++++++++++++++++++++++

static struct mtd_info *mtd;

//+++++++++++++++++++++++++++ Buffers ++++++++++++++++++++++++++++++++

static unsigned char *iobuf;		// DATA-Buffer
static unsigned char *iobuf1;		// OOB-Buffer
static unsigned char *iobuf_tmp;	// 0xFF-Buffer 
static unsigned char *bbt;			// Bad Block List

//++++++++++++++++++++++++ Global Variables +++++++++++++++++++++++++++

static int pgsize;
static int ebcnt;
static int pgcnt;

static int cnt_UBI_blocks;
static int cnt_UBI_empty_blocks;
static int cnt_unknown_blocks;
static int cnt_Layout_Volumes=0;
static int cnt_Flash_Errors=0;
static int cnt_Block_vformat=0;
static int cnt_Block_rep=0;
static int Error_Block=0;
static int Last_Empty_Block=0;

static int Layout_Volume_PEB=0;

//++++++++++++++++++++++++++++++++++ UBI Volume Table Record ++++++++++++

static struct ubi_vtbl_record {			// structure is not really necessary, but nice
	unsigned int  reserved_pebs;
	unsigned int  alignment;
	unsigned int	data_pad;
	unsigned char    vol_type;
	unsigned char    upd_marker;
	unsigned short  name_len;
	unsigned char    name[128];
	unsigned char    flags;
	unsigned char    padding[23];
	unsigned int  crc;
} lv;

/*
*	Format_Ubi_Volume() will erase each block of a specified UBI Volume ID.
*   The first page (Erase Counter Header) will be written back after erasing.
*   This function will be called from read_eraseblock_by_page() if the eraseblock has
*	the same volume identifier as specified with the vformat parameter.
*	All DATA of the block will be deleted.
*/
void Format_Ubi_Volume(int ebnum)
{
	int ret=0;
	size_t write = 0;
	void *buf = iobuf;
	loff_t addr = ebnum * mtd->erasesize;
	
	ret = Erase_Block(ebnum);						// Erase the Block
	
	if (ret!=(mtd->erasesize*0xFF))					// verify the erasing
	{
		printk( "ERROR: Bad checksum of erased Block -> %d\n",ret);		
	} 

	cnt_Block_vformat++;
	
	ret = mtd_write(mtd, addr, pgsize, &write, buf);	
	
	printk(PRINT_PREF "Empty Block %d created with RC %d at addr: %05X\n", ebnum,ret, (unsigned int)addr);
}

/*
*	Move_Block() reads the DATA from eraseblock mb and erase it after that. 
*	Page 0 (EC Header) will be written back, so it becomes an empty block. 
*	The DATA will be written to an empty block, beginning at page 1. A free 
*	eraseblock was found by the readtest.
*/
void Move_Block(void)
{
	size_t read = 0;
	size_t write = 0;
	int i=0;
	int j;
	int ret=0;
	int err = 0;
	loff_t addr = mb * mtd->erasesize;				// Source Block address		
	void *buf = iobuf;								// Pointer for reading
	void *write_buf = iobuf;						// Pointer for writing
	unsigned int pg_add;							// DATA Page checksum
	unsigned int data_pg=0;							// number of DATA Pages
		
	
	if (Last_Empty_Block==0)						// pre-running readtest failed or no empty eraseblock was found
	{
		printk(PRINT_PREF "ERROR: No Empty Block found !\n");
		return;
	}
	
	printk(PRINT_PREF "Moving DATA from Block %d to Empty Block %d...\n", mb, Last_Empty_Block);
	
	do												// read out the DATA only
	{				
		pg_add=0;
		memset(buf, 0 , pgsize);
				
		ret = mtd_read(mtd, addr, pgsize, &read, buf);		// reading page
		
		if (ret == -EUCLEAN)						// read ERROR Handling
		{
			ret = 0;
		} else {
			err=ret;
		}
		if (ret || read != pgsize) {
			printk(PRINT_PREF "error: read failed in Block %d, Page %d at %#llx\n", mb, i,
			       (long long)addr);			
			if (!err)
				err = -EINVAL;
		} 
		for (j=0; j<pgsize;j++)						// Calculate the DATA Page checksum
		{
			pg_add+=iobuf[i*pgsize+j];
		}		
		printk( "Read page: %d from Block: %d with RC %d, pg_add= %d\n",i, mb, ret, pg_add);
		i++;
		
		addr+=pgsize;										// pointer incresing for page address & read buffer
		buf+=pgsize;		
	} while ((i<pgcnt) && (pg_add<522240) && (ret!=-74));	// read cancel criteria: Block end reached, 
															// DATA page checksum = 0xFF * pagesize or Error in page
	if ((i==pgcnt) && (pg_add<522240) && (ret!=-74))
	{
		data_pg=i;
	} else {
		data_pg=i-1;
	}
		
	addr=mb * mtd->erasesize;					// reset the address to Block address
	
	ret = Erase_Block(mb);						// Erase the source block
	
	ret = mtd_write(mtd, addr, pgsize, &write, write_buf);		// Write back the EC Header
	write_buf+=pgsize;
	printk(PRINT_PREF "Write Page %d on Block %d with RC %d at addr: %05X\n", 0,mb,ret, (unsigned int)addr);
	
	addr= Last_Empty_Block * mtd->erasesize + pgsize;			// Destination block address
					
	for (i=1;i<data_pg;i++)						// write back all DATA pages to the destination block
	{
		ret = mtd_write(mtd, addr, pgsize, &write, write_buf);		
		printk(PRINT_PREF "Write Page %d on Block %d with RC %d at addr: %05X\n", i,Last_Empty_Block,ret, (unsigned int)addr);
		write_buf+=pgsize;
		addr+=pgsize;	
	}	
}

/*
*	This function read out a whole eraseblock page-by-page and stores all 
*	bytes into the buffer iobuf. The OOB bytes will be stored in buffer iobuf1.
*	Read Errors will be recognized and increses the errorcounter. For statistical 
*	information and special operations, UBI header information will be processed here.
*	A functionality for output of some pages of a eraseblock is also part of this function.
*/
static int read_eraseblock_by_page(int ebnum)
{
	size_t read = 0;
	int i,j;
	int ret = 0;
	int err = 0;
	loff_t addr = ebnum * mtd->erasesize;
	void *buf = iobuf;
	void *oobbuf = iobuf1;
	
	unsigned int pg_add=0;						// Block Checksum DATA
	unsigned int oob_add=0;						// Block Checksum OOB
	int oobsize=mtd->oobsize;
	unsigned int blocksize = mtd->erasesize;
	
	unsigned int Layout_Volume_LEB=999;
	
	char line[128];								// output string
	unsigned int vid_offset=0;					// UBI Header Data
	unsigned int data_offset=0;
	unsigned int DATA_Size=0;	
	unsigned int volume_id=0;

	for (i = 0; i < pgcnt; i++) {				// Readtest, page-by-page
				
		memset(buf, 0 , pgcnt);
		ret = mtd_read(mtd, addr, pgsize, &read, buf);		// Reading a whole page

		if (ret == -EUCLEAN)					// No Read Error
			ret = 0;
		if (ret || read != pgsize) 				// an Error occurs
		{
			printk(PRINT_PREF "error %d: read failed in Block %d, Page %d at %#llx\n", ret, ebnum, i,(long long)addr);
			if (ret==-74)						// only ECC Errors will be handled
			{
				cnt_Flash_Errors++;
				Error_Block=ebnum;				// save the block nr. of a defect
			}
			if (!err)
				err = ret;
			if (!err)
				err = -EINVAL;
		} 		
		if (mtd->oobsize) 						// OOB read test, if supported from the NAND Flash
		{
			struct mtd_oob_ops ops;

			ops.mode      = MTD_OPS_PLACE_OOB;
			ops.len       = 0;
			ops.retlen    = 0;
			ops.ooblen    = mtd->oobsize;
			ops.oobretlen = 0;
			ops.ooboffs   = 0;
			ops.datbuf    = NULL;
			ops.oobbuf    = oobbuf;
			ret = mtd_read_oob(mtd, addr, &ops);			
			if (ret || ops.oobretlen != mtd->oobsize) {
				printk(PRINT_PREF "error: read oob failed at "
						  "%#llx\n", (long long)addr);
				if (!err)
					err = ret;
				if (!err)
					err = -EINVAL;
			}			
			oobbuf += mtd->oobsize;
		}
		addr += pgsize;							// set address pointer to the next page
		buf += pgsize;							// DATA Buffer (iobuf) pointer increase
	}	
	for (j=0;j<blocksize;j++)					// Calculate the DATA Block checksum
	{
		pg_add+=(unsigned int)iobuf[j];					
	}
	for (j=0;j<oobsize*pgcnt;j++)				// Calculate the OOB Block checksum
	{
		oob_add+=(unsigned int)iobuf1[j];
	}
	
	for (i=0;i<4;i++)							// Read VID Offset & DATA Offset from the UBI Erase Counter Header
	{
		vid_offset = (vid_offset<<8);
		vid_offset |= iobuf[16+i];		
		data_offset = (data_offset<<8);
		data_offset |= iobuf[20+i];		
	}	
		
	// Block identification.... at first: UBI Block ? (begins with Magic number and the VID Offset)
	if ((iobuf[0]==0x55) && (iobuf[1]==0x42) && (iobuf[2]==0x49) && (iobuf[3]==0x23) && (vid_offset==pgsize))
	{		
			
		for (i=0;i<4;i++)							// Read DATA SIZE Field from UBI VID Header
		{											// Empty Blocks returns a -1 in reason of 0xFFFFFFFF....
			DATA_Size = (DATA_Size<<8);
			DATA_Size |= iobuf[vid_offset+20+i];		
		}		
		for (i=0;i<4;i++)							// Read Volume ID
		{
			volume_id = (volume_id<<8);
			volume_id |= iobuf[vid_offset+8+i];		
		}		
		if (volume_id==0x7FFFEFFF)					// This Volume ID marks this Block for UBI Layout Volume information
		{
			Layout_Volume_PEB=ebnum;
			cnt_Layout_Volumes++;
			Layout_Volume_LEB=0;
			for (i=0;i<4;i++)
			{
				Layout_Volume_LEB = (Layout_Volume_LEB<<8);
				Layout_Volume_LEB |= iobuf[vid_offset+12+i];		
			}
			if (peb==1) printk("Volume Block %d mit LEB-Nr: %d\n",ebnum,Layout_Volume_LEB);
		}		
		
		// Detects a UBI Block, filled with DATA, by reading VID Header 
		if ((iobuf[vid_offset]==0x55) && (iobuf[vid_offset+1]==0x42) && (iobuf[vid_offset+2]==0x49) && (iobuf[vid_offset+3]==0x21))
		{	
			// UBI DATA Block
			if (peb==1) printk("Block: %d pg_add: %d oob_add: %d, Volume-ID: %d, DATASIZE: %d\n",ebnum,pg_add,oob_add, volume_id, DATA_Size);			
			cnt_UBI_blocks++;
			if (vformat==volume_id) Format_Ubi_Volume(ebnum);
		} else {
			// UBI Empty Block
			if (peb==1) printk("EMPTY UBI Block: %d pg_add: %d oob_add: %d DATASIZE: %d\n",ebnum,pg_add,oob_add, DATA_Size );
			cnt_UBI_empty_blocks++;
			if (ebnum>Last_Empty_Block) Last_Empty_Block=ebnum;			// Stores the last Empty Block
		}		
	} else {
		// Not a UBI Block
		if (peb==1) printk("Block: %d KEIN UBI BLOCK mit pg_add: %d oob_add: %d\n",ebnum,pg_add,oob_add);
		cnt_unknown_blocks++;
	}	
	
	// output some pages of a defined Block
	if ((ebnum==ob) && (ob>0))
	{		
		printk("Block: %d pg_add: %d oob_add: %d\n",ebnum,pg_add,oob_add);
		for (i = ops*pgsize; i < ope*pgsize;) 
		{
			char *p = line;
			if (i%pgsize==0)
				printk("--------------------------- Block: %d Page: %d ----------------------------\n",ebnum,i/pgsize);
			p += sprintf(p, "%05x: ", i);
			for (j = 0; j < 32 && i < blocksize; j++, i++)
			{
				if (j%4==0) p += sprintf(p, " ");
				p += sprintf(p, "%02X", (unsigned int)iobuf[i]);
			}
			printk(PRINT_PREF "%s\n", line);
			cond_resched();
		}		
	}
	return err;
}

/*
*	This function erase the specified eraseblock ebnum. To verify this operation, 
*	the whole eraseblock will be read out after this and the content will be compared 
*	with its have to be, every byte should be a 0xFF. This function will be called by 
*	Rep_Block() and Format_Ubi_Volume() in the first step.
*/
static int Erase_Block(int ebnum)
{
	int ret=0;
	void *tmp_buf = iobuf_tmp;						// DATA Buffer
	struct erase_info ei;							// Block Erase structure
	int i,j;
	loff_t addr = ebnum * mtd->erasesize;			// Block address
	size_t read = 0;
	unsigned int pg_add=0;							// DATA Block checksum
	
	memset(&ei, 0, sizeof(struct erase_info));		// setup the Erase structure
	ei.mtd  = mtd;
	ei.addr = addr;
	ei.len  = mtd->erasesize;

	ret = mtd_erase(mtd, &ei);						// Erasing......
	
	mtd_sync(mtd);									// waiting for completing
	
	printk(PRINT_PREF "Erase-State: %d\n", ei.state);				// puts the ERASE State out, 8 = Finished
	printk(PRINT_PREF "RC %d while erasing EB %d\n", ret, ebnum);

	if (unlikely(ret)) {							// Erase Error Handling
		printk(PRINT_PREF "error %d while erasing EB %d\n", ret, ebnum);
		return ret;
	}
	if (unlikely(ei.state == MTD_ERASE_FAILED)) {
		printk(PRINT_PREF "some erase error occurred at EB %d\n",
		       ebnum);
		return -EIO;
	}
	
	for (i=0; i<pgcnt;i++)							// Verifying the erased Block by reading page-by-page
	{												// and calculating the DATA Block checksum and checks every Byte
		memset(tmp_buf, 0 , pgcnt);
		ret = mtd_read(mtd, addr, pgsize, &read, tmp_buf);
		
		if (ret == -EUCLEAN)
			ret = 0;
		if (ret || read != pgsize) {
			printk(PRINT_PREF "error: read failed in erased Block %d, Page %d at %#llx\n", ebnum, i, (long long)addr);			
		} 
		for (j=0; j<pgsize;j++)
		{
			pg_add+=iobuf_tmp[i*pgsize+j];
			if (iobuf_tmp[i*pgsize+j]!=0xFF)		// Every Byte in the erased Block has to be 0xFF
			{
				printk(PRINT_PREF "Read %02X, should be 0xFF at addr: %#llx\n", iobuf_tmp[i*pgsize+j], (long long)(addr+j));			
			}			
		}
		
		addr+=pgsize;
		if (i<pgcnt-1) tmp_buf+=pgsize;				// after the last page: dont increase the buffer pointer
	}	
	printk(PRINT_PREF "Block %d erased, Checksum: %d \n", ebnum, pg_add);
	return pg_add;									// returns the DATA Block checksum
}

/*
*	This function will be used to renew the specific eraseblock ebnum. At first all DATA (headers & data of files) 
*	of the block will be read out and stored in ram till a empty page or a ECC read-error occur. Then the block will 
*	be erased and the data will be written back to the eraseblock. Blocks with ECC read-errors in empty space will 
*	be repaired by this method. 
*	When using this function to eraseblocks filled with data from the first to the last page, file data will be lost 
*	because the last page will never be written back ! In this scenario the UBI Filesystem of the volume will be 
*	detroyed too.
*/
static void Rep_Block(int ebnum)
{
	size_t read = 0;
	size_t write = 0;
	int i=0;
	int j;
	int ret=0;
	int err = 0;
	loff_t addr = ebnum * mtd->erasesize;			// Block address
	void *buf = iobuf;								// Pointer for reading
	void *write_buf = iobuf;						// Pointer for writing
	unsigned int pg_add;							// DATA Page checksum
	unsigned int data_pg=0;							// number of DATA Pages
	
	do
	{				
		pg_add=0;
		memset(buf, 0 , pgcnt);
		
		ret = mtd_read(mtd, addr, pgsize, &read, buf);		// reading page
		
		if (ret == -EUCLEAN)						// read ERROR Handling
		{
			ret = 0;
		} else {
			err=ret;
		}
		if (ret || read != pgsize) {
			printk(PRINT_PREF "error: read failed in Block %d, Page %d at %#llx\n", ebnum, i,
			       (long long)addr);			
			if (!err)
				err = -EINVAL;
		} 
		for (j=0; j<pgsize;j++)						// Calculate the DATA Page checksum
		{
			pg_add+=iobuf[i*pgsize+j];
		}		
		printk( "Read page: %d from Block: %d with RC %d, pg_add= %d\n",i, ebnum, ret, pg_add);
		i++;
		
		addr+=pgsize;								// pointer incresing for page address & read buffer
		buf+=pgsize;		
	} while ((i<pgcnt) && (pg_add<522240) && (ret!=-74));	// read cancel criteria: Block end reached, 
															// DATA page checksum = 0xFF * pagesize or Error in page
	
	data_pg=i-1;									// last readed page will never written
	addr=ebnum * mtd->erasesize;					// reset the address to Block address
	
	ret = Erase_Block(ebnum);						// Erase the Block
	
	if (ret!=(mtd->erasesize*0xFF))					// verify the erasing
	{
		printk( "ERROR: Bad checksum of erased Block -> %d\n",ret);
	}
			
	for (i=0;i<data_pg;i++)							// write back all DATA pages
	{
		ret = mtd_write(mtd, addr, pgsize, &write, write_buf);		
		printk(PRINT_PREF "Write Page %d on Block %d with RC %d at addr: %05X\n", i,ebnum,ret, (unsigned int)addr);
		write_buf+=pgsize;
		addr+=pgsize;	
	}
	cnt_Block_rep++;
}

/*
*	If the parameters ErrBlock and ErrPage are set, this function will provoke an ECC read-error 
*	at this memory location. This location should ever be a free page, otherwise stored data will be lost.
*	The error will be created of writing of two different data pages to the same position. 
*	This method is not legitim and the second write cycle will provoke a bitflip. 
*/
static void Error_Write_Page(void)
{	
	size_t write = 0;
	int ret = 0;
	int i;
	loff_t addr = (ErrBlock * mtd->erasesize) + (ErrPage * pgsize);		// page address
	void *buf = iobuf;	

	memset(buf, 0 , pgcnt);
	for (i=0; i<pgsize;i++)				// Fill up the write buffer with 0xFF
	{
		iobuf[i]=0xFF;
	}
		
	iobuf[0]=0xAF;						// put 0xAFFE into the write buffer to the beginning of the page
	iobuf[1]=0xFE;

	ret = mtd_write(mtd, addr, pgsize, &write, buf);		// write page, legal write
	
	printk(PRINT_PREF "Written 0xAFFE at addr: %05X (Block %d, Page %d) with RC:%d\n", (unsigned int)addr, ErrBlock, ErrPage,ret);
	
	iobuf[0]=0xE5;						// put 0xE5E1 into the buffer
	iobuf[1]=0xE1;

	ret = mtd_write(mtd, addr, pgsize, &write, buf);		// illegal write to the same position -> ERROR -74
															// result: 0xA5E0 is stored
	
	printk(PRINT_PREF "Written 0xE5E1 at addr: %05X (Block %d, Page %d) with RC:%d\n", (unsigned int)addr, ErrBlock, ErrPage,ret);	
}

/*
*	This functions checks the specific block ebnum for bad block markering.
*/
static int is_block_bad(int ebnum)
{
	int ret;

	loff_t addr = ebnum * mtd->erasesize;	

	ret = mtd_block_isbad(mtd, addr);
	
	if (ret)
	{
		printk(PRINT_PREF "block %d is bad\n", ebnum);
	} 
	return ret;
}

/*
*	This function will be called after all eraseblocks will be checked and the parameter layout is set. 
*	Here, one of the two Layout Volume blocks will be red out and the Volume information will be printed out.
*	Only just for info !
*/
void Load_Layout_Volume(void)
{	
	int i,ret,err;
	loff_t addr = Layout_Volume_PEB * mtd->erasesize;	// address of Eraseblock which consists the Layout Volume information
	size_t read;
	void *buf = iobuf;
	int cnt=0;
	unsigned int start_addr;	

	for (i = 0; i < pgcnt; i++) 				// reads the whole block page-by-page
	{		
		memset(buf, 0 , pgcnt);
		ret = mtd_read(mtd, addr, pgsize, &read, buf);
		
		if (ret == -EUCLEAN)
			ret = 0;
		if (ret || read != pgsize) {
			printk(PRINT_PREF "error: read failed in Block , Page %d at %#llx\n", Layout_Volume_PEB,
			       (long long)addr);
			if (!err)
				err = ret;
			if (!err)
				err = -EINVAL;
		}
		addr+=pgsize;
		buf+=pgsize;
	}

	i=0;

	do 											// processing of the Layout Volume Table
	{		
		start_addr=(172*cnt)+0x1000;
		for (i=0;i<4;i++)
		{
			lv.reserved_pebs = (lv.reserved_pebs<<8);
			lv.reserved_pebs |= iobuf[start_addr+i];

			lv.alignment = (lv.alignment<<8);
			lv.alignment |= iobuf[start_addr+4+i];

			lv.data_pad = (lv.data_pad<<8);
			lv.data_pad |= iobuf[start_addr+8+i];
		}
				
		lv.name_len = iobuf[start_addr+14];
		lv.name_len = (lv.name_len<<8);
		lv.name_len |= iobuf[start_addr+15];

		for (i=0;i<127;i++)
		{
			lv.name[i]=iobuf[start_addr+16+i];
		}
		lv.name[16]='\0';
		if (lv.name_len!=0)					// output of the volume, when it given a name
		{
			printk(PRINT_PREF "-----------------------------UBI Volume %d------------------------------------------------- \n ",cnt);
			printk(PRINT_PREF "lv.reserved_pebs: %d    lv.alignment: %d    lv.data_pad: %d   lv.name_len: %d \n lv.name:%s \n ", lv.reserved_pebs, lv.alignment, lv.data_pad, lv.name_len, lv.name);
			cnt++;
		}

	} while ((lv.reserved_pebs!=0xFFFFFFFF) && (lv.name_len!=0));
}

/*
*	This function localized the bad blocks of the FLASH and stores the blocknumber into the bad block table
*/
static int scan_for_bad_eraseblocks(void)
{
	int i, bad = 0;

	bbt = kzalloc(ebcnt, GFP_KERNEL);
	if (!bbt) {
		printk(PRINT_PREF "error: cannot allocate memory\n");
		return -ENOMEM;
	}

	/* NOR flash does not implement block_isbad */
	if (mtd_block_isbad == NULL)
		return 0;

	printk(PRINT_PREF "scanning for bad eraseblocks\n");
	for (i = 0; i < ebcnt; ++i) {
		bbt[i] = is_block_bad(i) ? 1 : 0;
		if (bbt[i])
			bad += 1;
		cond_resched();
	}
	printk(PRINT_PREF "scanned %d eraseblocks, %d are bad\n", i, bad);
	return 0;
}

/*
*	This is the main function of the kernel module. In the first step, irformation of the memory 
*	technoly device will be get, ram for the buffer will be allocated and parameters will be evaluated.
*	After something to do (tests, error handling etc.) statistical data will be put out and the 
*	program terminates.
*/
static int __init mtd_readtest_init(void)
{
	uint64_t tmp;
	int err, i;
	int ret=0;	

	cnt_UBI_blocks=0;
	cnt_UBI_empty_blocks=0;
	cnt_unknown_blocks=0;

	if (dev==999) dev=1;					// Default MTD: 1
	
	printk(KERN_INFO "\n");
	printk(KERN_INFO "=================================================================\n");
	printk(PRINT_PREF "MTD device: %d\n", dev);
	
	mtd = get_mtd_device(NULL, dev);	// get the MTD structure
	
	if (IS_ERR(mtd)) {
		err = PTR_ERR(mtd);
		printk(PRINT_PREF "error: Cannot get MTD device\n");
		return err;
	}

	if (mtd->writesize == 1) {			// special case: NO NAND-FLASH
		printk(PRINT_PREF "not NAND flash, assume page size is 512 "
				"bytes.\n");
		pgsize = 512;
	} else
		pgsize = mtd->writesize;

	tmp = mtd->size;						
	do_div(tmp, mtd->erasesize);
	ebcnt = tmp;							// i.e. 1024 eraseblocks
	pgcnt = mtd->erasesize / pgsize;		// i.e. 64 pages

	printk(PRINT_PREF "MTD device size %llu, eraseblock size %u, "
			"page size %u, count of eraseblocks %u, pages per "
			"eraseblock %u, OOB size %u\n",
			(unsigned long long)mtd->size, mtd->erasesize,
			pgsize, ebcnt, pgcnt, mtd->oobsize);

	err = -ENOMEM;
	iobuf = kmalloc(mtd->erasesize, GFP_KERNEL);		// Allocation of RAM for read/write-buffers
	if (!iobuf) {
		printk(PRINT_PREF "error: cannot allocate memory\n");
		goto out;
	}
	iobuf1 = kmalloc(mtd->erasesize, GFP_KERNEL);
	if (!iobuf1) {
		printk(PRINT_PREF "error: cannot allocate memory\n");
		goto out;
	}
	iobuf_tmp = kmalloc(mtd->erasesize, GFP_KERNEL);
	if (!iobuf_tmp) {
		printk(PRINT_PREF "error: cannot allocate memory\n");
		goto out;
	}

	if ((ErrBlock>0) && (ErrPage>0)) 					// Parameter for writing an Errorpage are set
	{
		Error_Write_Page();
		err=0;
		goto out;
	}
	else if (RepBlock>0) 								// Parameter to renew a eraseblock is set
	{
		Rep_Block(RepBlock);
		err=0;
		goto out;
	} 
		
	err = scan_for_bad_eraseblocks();					// at first: scanning for Bad Blocks
	
	if (err)
		goto out;

	printk(PRINT_PREF "testing page read\n");
	for (i = 0; i < ebcnt; ++i) {						// readtest: block-by-block
		
		if (bbt[i])
			continue;
		ret = read_eraseblock_by_page(i);
		cond_resched();
	}
	
	if (mb>0)
	{
		Move_Block();
		err=0;
	}
	
	if (layout==1)										// Parameter for output Layout Volume Information is set
	{
		Load_Layout_Volume();
	}
	
	if ((cnt_Flash_Errors>0) && (arep==1))				// Auto-repairing Parameter is set
	{		
		Rep_Block(Error_Block);
		err=Error_Block;
		Error_Block=0;
		ret = read_eraseblock_by_page(err);
		err=0;
		if (Error_Block!=0)								// if the first try of repairing fails..., second try is to copy the data and mark the block
		{
			mb=Error_Block;								// set up the mb parameter for move block operation
			Move_Block();								// call the moving block function
			err = mtd_block_markbad(mtd, Error_Block * mtd->erasesize);	// marks the unrepairable block as bad block
			printk(PRINT_PREF "Block %d at Addr: %05X marked as Bad with RC: %d\n", Error_Block,Error_Block * mtd->erasesize, err);		
		}		
	}	
	
	if (err)
		printk(PRINT_PREF "finished with errors\n");
	else
		printk(PRINT_PREF "finished\n");

out:													// Exit & statistic output

	kfree(iobuf);
	kfree(iobuf1);
	kfree(iobuf_tmp);
	kfree(bbt);
	put_mtd_device(mtd);
	if (err)
	{
		printk(PRINT_PREF "error %d occurred\n", err);
	}
	printk(KERN_INFO "=================================================================\n");

	printk(KERN_INFO "inspected Blocks:\t%d (%d used - %d empty - %d unknown)\n", cnt_UBI_blocks+cnt_UBI_empty_blocks+cnt_unknown_blocks,cnt_UBI_blocks,cnt_UBI_empty_blocks,cnt_unknown_blocks);
	printk(KERN_INFO "ECC Errors found:\t%d \n", cnt_Flash_Errors);
	printk(KERN_INFO "Blocks formatted:\t%d \n", cnt_Block_vformat);
	printk(KERN_INFO "Blocks repaired :\t%d \n", cnt_Block_rep);
	printk(KERN_INFO "=================================================================\n");
	
	return err;
}
module_init(mtd_readtest_init);

static void __exit mtd_readtest_exit(void)
{
	return;
}
module_exit(mtd_readtest_exit);

MODULE_DESCRIPTION("Read test module (modified by DNT)");
MODULE_AUTHOR("Adrian Hunter & Maik Gotzmann");
MODULE_LICENSE("GPL");
