
/***************************************************************************
 *  stp.cc : This file is part of 'stpspoof'
 *
 *  (c) 2004,2005 by Lukasz Tomicki <tomicki@o2.pl>
 *	
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <string.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <asm/types.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>
#include <stdlib.h>
//type definitions
typedef unsigned char 		u8;
typedef unsigned short 		u16;
typedef unsigned int 		u32;
#ifdef __HAS_LONG_LONG__
typedef unsigned long long 	u64;
#endif
#ifndef VERSION
#define VERSION "v0.0.0 - "__DATE__" "__TIME__
#endif
#define DEFAULT_IF  "eth1"
#define DEFAULT_PROTOCOL  "STP" // STP or RSTP
#define DEFAULT_ROOT_BRIDGE_PRIO  32768
#define DEFAULT_ROOT_BRIDGE_SYSTEM_ID 0
#define DEFAULT_TX_HOLD_COUNT 6
#define DEFAULT_ROOT_MAX_AGE 20
#define DEFAULT_MAX_AGE 20
#define DEFAULT_HELLO_TIME 2
#define DEFAULT_FORWARD_DELAY 15
//#define DEFAULT_PORT_ID 0x0280
#define DEFAULT_PORT_ID 0x0B80
#define STP_MULTICAST 	"01-80-c2-00-00-00"
#define PACKET_LENGTH 0x0026
//#define PACKET_LENGTH 0x0034


#define FLAG_TOPOLOGY_CHANGE  1  // 2^0, bit 0
#define FLAG_PROPOSAL         2  // 2^1, bit 1
#define FLAG_ROLE1            4  // 2^2, bit 2
#define FLAG_ROLE2            8  // 2^3, bit 3
#define FLAG_LEARN           16  // 2^4, bit 4
#define FLAG_FORWAR          32  // 2^5, bit 5
#define FLAG_AGREE           64  // 2^6, bit 6
#define FLAG_CHANGE_ACK     128  // 2^7 bit 7

typedef unsigned char byte;
typedef byte u8;
typedef unsigned short int u16;
typedef unsigned int u32;

char *alloc_copy(const char *str, uint extra = 0)
{
    if (!str)
        return 0;

    uint size(strlen(str));

    if (!size)
        return 0;

    char *new_alloc = new char[size + 1 + extra];

    if (!new_alloc)
        return 0;

    memset(new_alloc, 0, size + 1 + extra);
    strncpy(new_alloc, str, size);

    return (new_alloc);
}

struct ether_header
{
    u8  	dhost[6]; 	// destination hdwaddr should be 01-80-C2-00-00-00
    u8  	shost[6];   	// = 0x0000 for our purposes
    u16 	size;       	// = 52  for our purposes
} __attribute__ ((packed));

struct llc_header {
    u8 dsap; 	// = 0x42 for our purposes
    u8 ssap; 	// = 0x42 for our purposes
    u8 func; 	// = 0x03 for our purposes
} __attribute__ ((packed));

struct stp_header {
    struct 	llc_header llc;
    u16 	type; 				// = 0x0000  for our purposes
    u8	version; 			// = 0x00  for our purposes
    u8	config; 			// = 0x00  for our purposes
    u8	flags; 				// = 0x00  for our purposes

    union {
        u8    root_id[8];	
        struct {
            u16	root_priority; 		// 32768 by default (0x8000 in network order)
            u8    	root_hdwaddr[6]; 	// can be almost anything
        } root_data;
    };
    u32	root_path_cost; 	// = 0x00  for our purposes

    union {
        u8    bridge_id[8];	
        struct {
            u16	bridge_priority; 		// 32768 by default (0x8000 in network order)
            u8    	bridge_hdwaddr[6]; 	// can be almost anything
        } bridge_data;
    };

    u16 	port_id; 			// = 0x8002  for our purposes
    u16 	message_age; 		// = 0x0000  for our purposes
    u16 	max_age; 			// 0x0014 = 20 sec 
    u16 	hello_time; 		// 0x0002 =  2 sec
    u16 	forward_delay; 		// 0x000f = 15 sec 
    /*
       u16 	protocol_version; 	
       u16 	tx_hold_count; 		
       u16 	root_maximum_age; 	
       u16 	root_cost; 		    
       u16 	root_port;  // 0x0280 		   
       */
} __attribute__ ((packed));

typedef struct {
    struct ether_header eth;
    struct stp_header stp;
} eth_stp;

typedef int hex; 

hex int2hex (int i){ 
    hex x = (hex)i; 
    return x; 
}


u16 atohex (u8 *hex)
{
    //short int x,y,a,a2=0;
    short int a,a2=0;
    char buf[2];

    char nums[] = {"0123456789abcdef"};

    memcpy(buf, hex, 2);	
    for (int x(0); x < 2; ++x) {
        for (int y(0); y < 16; ++y) {
            if (buf[x] == nums[y]) {
                if (x == 0) 
                    a = (y) * 16;   
                else 
                    a = y;
                a2 +=a;
            }
        }
    }
    return a2;
}

u8 *ascii_to_hwaddr (const char *hwaddr)
{
    u8 t[2];
    u8 y(0);
    static u8 buf[6];
    do {     
        t[0] = *hwaddr++;	
        t[1] = *hwaddr++;
        hwaddr++;
        buf[y] = atohex (t);
        y++;
    } while (y < 6);

    return (buf);
}

const char *fill_stp_header(char *shwaddr, bool topology_change, bool topology_change_ack, char *root_id, 
        u32 forward_delay = 0x000f, u32 max_age = 0x0014, u32 hello_time = 0x0002, 
        u32 port_id = DEFAULT_PORT_ID, const char * protocol_version = DEFAULT_PROTOCOL)
{
    static eth_stp stp_packet;
    unsigned char my_flags;

    memset(&stp_packet, 0, sizeof(stp_packet));
    memcpy (stp_packet.eth.dhost, ascii_to_hwaddr(STP_MULTICAST), 6);
    memcpy (stp_packet.eth.shost, shwaddr, 6);  
    stp_packet.eth.size = htons(PACKET_LENGTH);

    stp_packet.stp.llc.dsap = 0x42;
    stp_packet.stp.llc.ssap = 0x42;
    stp_packet.stp.llc.func = 0x03;

     printf("\n set RSTP Version Flags %s \n", protocol_version);
    if (strcmp(protocol_version,"RSTP")==0)
    {
        //forwarding, Learning, Port Role,  Proposal
        my_flags = FLAG_FORWAR | FLAG_LEARN | FLAG_ROLE1 | FLAG_ROLE2 | FLAG_PROPOSAL; 
        // setting one flag on, leaving the rest untouched
        //my_flags |= FLAG_CHANGE_ACK;
        printf("\n enter RSTP Version Flags\n");
        stp_packet.eth.size = htons(PACKET_LENGTH+1);
        stp_packet.stp.type = 0x0000;
        stp_packet.stp.version = 0x02;
        stp_packet.stp.config = 0x02;
        stp_packet.stp.flags  = my_flags;
    }

    memcpy(stp_packet.stp.root_id, root_id, 8);
    memcpy(stp_packet.stp.bridge_id, root_id, 8);

    my_flags = 0x00;
    stp_packet.stp.port_id = port_id;
    stp_packet.stp.hello_time = hello_time;
    stp_packet.stp.max_age = max_age;
    stp_packet.stp.forward_delay = forward_delay;
    if (topology_change){
        //stp_packet.stp.flags = 0x01;
        my_flags |= FLAG_TOPOLOGY_CHANGE;
        stp_packet.stp.flags = my_flags;
    }
    if (topology_change_ack){
        my_flags |= FLAG_CHANGE_ACK;     
        stp_packet.stp.flags = my_flags;
   } 
    return (const char*) &stp_packet;
}

void make_rand_hwaddr(char *buf)
{
    for (int i(0); i < 6; ++i)
        buf[i] = rand() % 256;
}

int initialise_network_int(const char *name, int protocol)
{
    int sockfd = socket(PF_PACKET, SOCK_RAW, htons(protocol));

    if (sockfd == -1) {
        perror("Unable to create socket: %s");
        return sockfd;
    }

    ifreq ifr;

    memset(&ifr, 0, sizeof(ifr));
    strncpy(ifr.ifr_name, name, strlen(name));

    if (ioctl(sockfd, SIOCGIFINDEX, &ifr) == -1) {
        perror("Unable to get interface index: %s");
        return -1;
    }

    sockaddr_ll sll;

    memset(&sll, 0, sizeof(sll));
    sll.sll_family = AF_PACKET;
    sll.sll_ifindex = ifr.ifr_ifindex;
    sll.sll_protocol = htons(protocol);

    if (bind(sockfd, (sockaddr*) &sll, sizeof(sll)) == -1) {
        perror("Unable to bind socket: %s");
        return -1;
    }

    return sockfd;
}

static int print_help(void)
{
    printf("\nSend Spanning Tree BPDUs - %s\n", VERSION);
    puts("stpspoof - a spanning tree protocol spoofer v0.2");
    puts("(c) 2004,2005 by Lukasz Tomicki <tomicki@o2.pl>");
    puts("    2014 adapted to a general BPDUs sending tool by <ogr@neuhaus.de>\n");
    puts("\tusage: stpspoof <interface> <options>\n");
    puts("\toptions:\n");

    puts("\t-v Protocol Version (STP or RSTP)");
    puts("\t-d <delay between packets> (default: 1s)");
    puts("\t-t announce a topology change (default: no)");
    puts("\t-a ack a topology change (default: no)");
    puts("\t-h hello time (default: 2s (1..10))");
    puts("\t-m max age (default: 20s (6..40))");
    puts("\t-f forward delay (default: 15s (4..30)");
    puts("\t-bridge-prio (0..32768)");
    puts("\t-bridge-id (0..0)");
    puts("\t-tx-hold-count (1..10)");
    puts("\t-root-maximum-age (6..40)");
    puts("\t-root-cost (default 0)");
    puts("\t-root-port (default 0x0280)");
    puts("\t-p randomize port IDs (default: 0x0280)");
    puts("\t-r randomize source MAC addresses (default: no)");

    printf("\t-? help\n");
    printf("\n");
    return EXIT_SUCCESS;
}


int main(int argc, char **argv)
{
    char *interface_name = 0;
    bool topology_change(false);
    bool topology_change_ack(false);
    u32 sleep_delay(1000000);
    bool rand_mac_source(false);
    bool rand_port_id(false);
    u32 portid(DEFAULT_PORT_ID);  		   
    u32 hello_time(DEFAULT_HELLO_TIME);
    u32 max_age(DEFAULT_MAX_AGE);
    u32 forward_delay(DEFAULT_FORWARD_DELAY);
    unsigned int bridge_prio =  DEFAULT_ROOT_BRIDGE_PRIO;
    unsigned int bridge_id =  DEFAULT_ROOT_BRIDGE_SYSTEM_ID;
    const char*  protocol_version = DEFAULT_PROTOCOL;
    
    if (argc == 1) {
        print_help();
        return 0;
    }

    srand(getpid());
    interface_name = alloc_copy(argv[1]);
    float tmp;

    if (argc > 2) {
       //char c;
       wchar_t c;
        while ((c = getopt(argc, argv, "td:rph:m:f:b:v:?")) != -1) {
        printf("\nEnter while loop c == %d\n", c );
            switch (c) {
                case 't':
                    topology_change = true;
                    break;

                case 'a':
                    topology_change_ack = true;
                    break;

                case 'r':
                    rand_mac_source = true;
                    break;

                case 'p':
                    rand_port_id = true;
                    break;

                case 'd':
                    sscanf(optarg, "%f", &tmp);
                    sleep_delay = tmp * 1000000;
                    break;

                case 'h':
                    hello_time = atoi(optarg);
                    break;

                case 'm':
                    max_age = atoi(optarg);
                    break;

                case 'f':
                    forward_delay = atoi(optarg);
                    break;

               case 'v':
                    protocol_version = alloc_copy(argv[3]);
                    if (strcmp(protocol_version,"STP"))  
                        { 
                            printf("RSTP not supported yet \n");
                            //return (EXIT_FAILURE);
                        }
                        else
                        {
                            printf("Only STP supported yet \n");
                        }

                   break;

                case 'b':
                    bridge_prio = atoi(optarg);
                    printf("bridge_prio= %i\n", bridge_prio);
                    break;
                
                    /*
                       case 'bridge-id':
                       bridge_id = atoi(optarg);
                       break;

                       case 'tx-hold-count':
                       tx_hold_count = atoi(optarg);
                       break;

                       case 'root-maximum-age':
                       root_maximum_age = atoi(optarg);
                       break;

                       case 'root-cost':
                       root_cost = atoi(optarg);
                       break;

                       case 'root-port':
                       portid = atoi(optarg);
                       break;
                       */
                case '?':
                    print_help();
                    return EXIT_SUCCESS;
                break;

                case 255:
                    return EXIT_SUCCESS;
                break;

            }

        printf("\nLeave while loop \n");
        }
    }


    ifreq ifr;

    printf("\n Open Socket tmpfd \n");
    int tmpfd = socket (AF_INET, SOCK_DGRAM, 0);

    // Use ioctl() to look up interface name and get its MAC address.
    memset (&ifr, 0, sizeof (ifr));
    snprintf (ifr.ifr_name, sizeof (ifr.ifr_name), "%s", interface_name);
    if (ioctl (tmpfd, SIOCGIFHWADDR, &ifr) < 0) {
        perror ("ioctl() failed to get source MAC address - no network interface given");
        return (EXIT_FAILURE);
    }

    //int x = sizeof(sockaddr_ll);

    char shwaddr[8];

    //printf("bridge_prio= %d\n", bridge_prio);
    sprintf(shwaddr, "%X", DEFAULT_ROOT_BRIDGE_PRIO);
    shwaddr[0] = htons(hex(bridge_prio));
    shwaddr[1] = htons(hex(bridge_id));

    //printf("Hexadecimal value of the decimal value %d is %s.\n", bridge_prio,shwaddr);

    if (!rand_mac_source)
        memcpy(shwaddr + 2, ifr.ifr_hwaddr.sa_data, 6);

    printf("\n Open Socket fd \n");
    int fd = initialise_network_int(interface_name, ETH_P_ALL);
    if (fd == -1) {
        perror("socket:");
        return 0;
    }

    u32 port_id(portid);

    while (true) {
    printf("\n enter while loop \n");
        if (rand_mac_source)
            make_rand_hwaddr(shwaddr + 2);

        if (rand_port_id)
            port_id = rand() % 0xFFFF;

        printf("topology_change=%s , shwaddr + 2 == %s, forward_delay=%i, max_age=%i, hello_time==%i, port_id==0x%x, protocol_version = %s \n", topology_change ? "true" : "false", shwaddr + 2, forward_delay, max_age, hello_time, port_id, protocol_version);
        const char *buf = fill_stp_header(shwaddr + 2, topology_change,topology_change_ack, shwaddr, forward_delay, max_age, hello_time, port_id, protocol_version);

        printf("send_bpdu -> \n");
        if (write(fd, buf, sizeof(eth_stp)) == -1) {
            perror("write:");
            return 0;
        }
        printf("sleep_delay=%i \n", sleep_delay);
        usleep(sleep_delay);

    printf("\n leave while loop \n");
    }

    /* cleanup time */

    close(fd);
    if (interface_name != 0) {
        delete [] interface_name;
        interface_name = 0;
    }

    printf("\n exit program \n");
    return 0;
}
