/*****************************************************************
 *
 * EEPROM - Utilities
 *
 * Copyright (c) 2011 Birger Bauch, Dr. Neuhaus Telekommunikation GmbH
 *
 *****************************************************************/

#ifndef __EEPROM_H_
#define __EEPROM_H_

/***************************************************************************************/

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#ifndef max
#define max(a,b)  (((a) > (b)) ? (a) : (b))
#endif

#ifndef min
#define min(a,b)  (((a) < (b)) ? (a) : (b))
#endif

#define EC_OK 			0
#define EC_UNSPECIFIED	-1
#define EC_FCS16		-2
#define EC_FORMAT		-3
#define EC_HASH			-4

/***************************************************************************************/

class cSysFile
{
private:
	int fd;
public:
	cSysFile(const char *name, mode_t mode);
	~cSysFile();
	size_t rd(char *buf, const size_t len, const off_t offset);
	size_t wr(const char *buf, const size_t len, const off_t offset);
};

class cEEPROM
{
protected:
	//write data to eeprom address, add fcs16
	//returns 0 on success, -1 on unspecified error
	int setUint(const off_t offset, const uint8_t value);
	int setUint(const off_t offset, const uint16_t value);
	int setUint(const off_t offset, const uint32_t value);
	int setData(const off_t offset, const char *value, const size_t length);
	//read data from eeprom address, verify fcs16
	//returns 0 on success, -1 on unspecified error, -2 on fcs16 error
	int getUint(const off_t offset, uint8_t &value);
	int getUint(const off_t offset, uint16_t &value);
	int getUint(const off_t offset, uint32_t &value);
	int getData(const off_t offset, char *value, const size_t length);
	//write/read data from/to eeprom, add/verify fcs16
	int setPwHash(const char *value, const size_t length);
	int getPwHash(char *value, size_t &length);
	void createHash(char* acPwdHash, const char* pDataToHash, const size_t length);

public:
	void print_hex(const char *data, const int len);
	int erase();
	int hexdump();
	const char *ec2str(const int ec);

	//write/read data from/to eeprom, add/verify fcs16
	int setVersion(const uint8_t value);
	int getVersion(uint8_t &value);

	int setBc(const uint32_t value);
	int getBc(uint32_t &value);

	int setOsc(const uint32_t value);
	int getOsc(uint32_t &value);

	int setHwid(const char *value, const size_t length);
	int getHwid(char *value, size_t &length);

	int setSwid(const char *value, const size_t length);
	int getSwid(char *value, size_t &length);

	int setSerial(const char *value, const size_t length);
	int getSerial(char *value, size_t &length);

	int setPdate(const char *value, const size_t length);
	int getPdate(char *value, size_t &length);

	int setHwver(const char *value, const size_t length);
	int getHwver(char *value, size_t &length);

	int setMacEth0(const char *value, const size_t length);
	int getMacEth0(char *value, size_t &length);

	int setMacEth1(const char *value, const size_t length);
	int getMacEth1(char *value, size_t &length);

	int setKey(const char *value, const size_t length);
	int getKey(char *value, size_t &length);

	int setFlag(const char *value, const size_t length);
	int getFlag(char *value, size_t &length);

	int setOem(const char *value, const size_t length);
	int getOem(char *value, size_t &length);

	int setWmbus(const char *value, const size_t length);
	int getWmbus(char *value, size_t &length);

	int setPassword(const char *value, const size_t length);
	int verifyPassword(const char *value, const size_t length);
};

#endif
