/*****************************************************************
 *
 * EEPROM - Generic Class Interface for Hash Calculation
 *
 * Copyright (c) 2008 SSI, Dr. Neuhaus Telekommunikation GmbH
 *
 *****************************************************************/

#ifndef __HASH_CALCULATOR_H_
#define __HASH_CALCULATOR_H_

#include <stdio.h>
#include <string.h>
#include <string>

using namespace std;

class HashCalculator
{
protected:
	int iBlockSize;
	unsigned int* acHashResult;
	int iHashResultLen;
	char* acBlockBuffer;
	int iBufferPos;

	long long llMessageSize;

	virtual void ProcessBlock(unsigned int* BlockData)
	{
	}
	virtual void Finish()
	{
	}

public:
	HashCalculator();
	virtual ~HashCalculator()
	{
	}

	virtual void Init()
	{
	}
	void AddFile(const char* FileName);
	void AddData(const char* Data, int Len);
	void GetHashResult(char* ResultTarget);
	void GetHashResult(string& ResultTarget);
};

class SHA256Calculator: public HashCalculator
{
private:
	unsigned int W[64], a, b, c, d, e, f, g, h, T1, T2;

	//Utility Functions
	unsigned int ROTR(unsigned int X, int n);
	unsigned int SIGMA0(unsigned int X);
	unsigned int SIGMA1(unsigned int X);
	unsigned int Sigma0(unsigned int X);
	unsigned int Sigma1(unsigned int X);
	unsigned int Ch(unsigned int X, unsigned int Y, unsigned int Z);
	unsigned int Maj(unsigned int X, unsigned int Y, unsigned int Z);

	void ProcessBlock(unsigned int* BlockData);
	void Finish();
public:
	SHA256Calculator();
	~SHA256Calculator();

	void Init();
};

#endif
