#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>

#ifndef REPORT_IRQ_ON_WRITE

#define SIGVDD5V 44

static int signal_received = 0;
static void signal_handler(const int sig)
{
	printf("%s received %d\n", __func__, sig);
	switch (sig)
	{
		default:
			break;
		case SIGVDD5V:
			signal_received = 1;
			break;
		case SIGTERM: //kill
			exit(0);
			break;
		case SIGINT: //strg-C
			exit(0);
			break;
	}
}
#endif

int main(int argc, char **argv)
{
	/*
	 * In order to be able to send a signal from kernel space to user space,
	 * the kernel needs to know the pid of the user space process.
	 */
	int fd = open("/sys/kernel/debug/vdd5v", O_RDONLY);
	if (fd > 0)
#ifdef REPORT_IRQ_ON_WRITE
	{
		char result;
		int rc = read(fd, &result, sizeof(result));
		if (rc > 0)
			printf("%s: read 0x%02x\n", __func__, result);
	}
#else
	{
		signal(SIGTERM, signal_handler);
		signal(SIGINT, signal_handler);
		signal(SIGVDD5V, signal_handler);

		while (signal_received == 0)
			sleep(1);

		printf("sleep...");
		sleep(5);
		printf("system halt\n");
		if (0 > system("halt"))
			printf("%s can't execute system call\n", __func__);
	}
#endif
	else
	{
		printf("%s: can't open /sys/kernel/debug/vdd5v, try 'modprobe imx28_power'\n", __FUNCTION__);
		return -1;
	}
	close(fd);
	return EXIT_SUCCESS;
}
