# -*-makefile-*-
#
# Copyright (C) 2015 Dr. Neuhaus Telekommunikation GmbH, Hamburg Germany, Oliver Graute <oliver.graute@neuhaus.de>
#
# See CREDITS for details about who has contributed to this project.
#
# For further information about the PTXdist project and license conditions
# see the README file.
#

#
# We provide this package
#
PACKAGES-$(PTXCONF_MAILDROP) += maildrop

#
# Paths and names
#
MAILDROP_VERSION	:= 2.8.3
MAILDROP_MD5		:= 4e3b2b19b58a76581a486381e4444b5e
MAILDROP		:= maildrop-$(MAILDROP_VERSION)
MAILDROP_SUFFIX		:= tar.bz2
MAILDROP_URL		:= http://downloads.sourceforge.net/project/courier/maildrop/$(MAILDROP_VERSION)/$(MAILDROP).$(MAILDROP_SUFFIX)
MAILDROP_SOURCE		:= $(SRCDIR)/$(MAILDROP).$(MAILDROP_SUFFIX)
MAILDROP_DIR		:= $(BUILDDIR)/$(MAILDROP)
MAILDROP_LICENSE	:= GPLv3

# ----------------------------------------------------------------------------
# Prepare
# ----------------------------------------------------------------------------

MAILDROP_CONF_ENV	:= $(CROSS_ENV)

AUTOCONFIG_POSTFIX_EXTRAS+=\
	ac_cv_func_getpgrp_void=yes \
	ac_cv_func_setpgrp_void=yes \
	ac_cv_func_memcmp_working=yes \
	rb_cv_binary_elf=no \
	rb_cv_negative_time_t=no

#
# autoconf
#
MAILDROP_CONF_TOOL	:= autoconf
MAILDROP_CONF_OPT	:= $(CROSS_AUTOCONF_USR)\
			--with-locking-method=fcntl
#CPPFLAGS="-L/$(PTXDIST_SYSROOT_TARGET)/lib -I $(PTXDIST_SYSROOT_TARGET)/usr/include"

$(STATEDIR)/maildrop.prepare:
	@$(call targetinfo)
	@$(call clean, $(MAILDROP_DIR)/config.cache)
	cd $(MAILDROP_DIR) && \
		$(MAILDROP_PATH) $(MAILDROP_ENV) \
		$(AUTOCONFIG_POSTFIX_EXTRAS) ./configure $(MAILDROP_CONF_OPT)
	@$(call touch)

# ----------------------------------------------------------------------------
# Compile
# ----------------------------------------------------------------------------

$(STATEDIR)/maildrop.compile:
	@$(call targetinfo)
	@$(call world/compile, MAILDROP)
	@$(call touch)

# ----------------------------------------------------------------------------
# Install
# ----------------------------------------------------------------------------

$(STATEDIR)/maildrop.install:
	@$(call targetinfo)
	@$(call world/install, MAILDROP)
	@$(call touch)

# ----------------------------------------------------------------------------
# Target-Install
# ----------------------------------------------------------------------------

$(STATEDIR)/maildrop.targetinstall:
	@$(call targetinfo)

	@$(call install_init, maildrop)
	@$(call install_fixup, maildrop,PRIORITY,optional)
	@$(call install_fixup, maildrop,SECTION,base)
	@$(call install_fixup, maildrop,AUTHOR,"<oliver.graute@neuhaus.de>")
	@$(call install_fixup, maildrop,DESCRIPTION,missing)

	@$(call install_copy, maildrop, 0, 0, 0755, $(MAILDROP_DIR)/libs/maildrop/maildrop, /usr/bin/maildrop)
	@$(call install_copy, maildrop, 0, 0, 0755, $(MAILDROP_DIR)/libs/maildrop/mailbot, /usr/bin/mailbot)
	@$(call install_copy, maildrop, 0, 0, 0755, $(MAILDROP_DIR)/libs/maildrop/reformail, /usr/bin/reformail)
	@$(call install_copy, maildrop, 0, 0, 0755, $(MAILDROP_DIR)/libs/rfc2045/makemime, /usr/bin/makemime)
	@$(call install_copy, maildrop, 0, 0, 0755, $(MAILDROP_DIR)/libs/rfc2045/reformime, /usr/bin/reformime)

	@$(call install_finish, maildrop)

	@$(call touch)

# ----------------------------------------------------------------------------
# Clean
# ----------------------------------------------------------------------------

$(STATEDIR)/maildrop.clean:
	@$(call targetinfo)
	@$(call clean_pkg, MAILDROP)

# vim: syntax=make
