# -*-makefile-*-
#
# Copyright (C) 2002 by Jochen Striepe for Pengutronix e.K., Hildesheim, Germany
#               2003-2008 by Pengutronix e.K., Hildesheim, Germany
#		2009 by Marc Kleine-Budde <mkl@pengutronix.de>
#		2013 by Michael Grzeschik <mgr@pengutronix.de>
#
# See CREDITS for details about who has contributed to this project.
#
# For further information about the PTXdist project and license conditions
# see the README file.
#

#
# We provide this package
#
PACKAGES-$(PTXCONF_OPENSSL110DEV) += openssl110dev

#
# Paths and names
#
OPENSSL110DEV_VERSION	:= 70d416e
OPENSSL110DEV_MD5	:= 66bf6f10f060d561929de96f9dfe5b8c
OPENSSL110DEV		:= openssl-$(OPENSSL110DEV_VERSION)
OPENSSL110DEV_SUFFIX	:= tar.gz
OPENSSL110DEV_URL	:= http://openssl.org/source/$(OPENSSL110DEV).$(OPENSSL110DEV_SUFFIX)
OPENSSL110DEV_SOURCE	:= $(SRCDIR)/$(OPENSSL110DEV).$(OPENSSL110DEV_SUFFIX)
OPENSSL110DEV_DIR	:= $(BUILDDIR)/$(OPENSSL110DEV)
OPENSSL110DEV_LICENSE	:= openssl110dev

# ----------------------------------------------------------------------------
# Prepare
# ----------------------------------------------------------------------------

OPENSSL110DEV_CONF_ENV 	:= $(CROSS_ENV)
OPENSSL110DEV_MAKE_PAR := NO

OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_X86_I386)	+= debian-i386
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_X86_I486)	+= debian-i386-i486
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_X86_I586)	+= debian-i386-i586
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_X86_I686)	+= debian-i386-i686/cmov
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_X86_P2)	+= debian-i386-i686/cmov
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_X86_P3M)	+= debian-i386-i686/cmov
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_M68K)	+= debian-m68k
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_PPC)	+= debian-powerpc
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_SPARC)	+= debian-sparc

ifdef PTXCONF_ENDIAN_LITTLE
#OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_ARM)	+= debian-armel
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_ARM)       += linux-generic32
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_MIPS)	+= debian-mipsel
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_SH_SH3)	+= debian-sh3
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_SH_SH4)	+= debian-sh4
else
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_ARM)	+= debian-armeb
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_MIPS)	+= debian-mips
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_SH_SH3)	+= debian-sh3eb
OPENSSL110DEV_ARCH-$(PTXCONF_ARCH_SH_SH4)	+= debian-sh4eb
endif

ifdef PTXCONF_OPENSSL110DEV
ifndef OPENSSL110DEV_ARCH-y
$(error *** Sorry unsupported ARCH in openssl.make)
endif
endif


#
# autoconf
#
OPENSSL110DEV_CONF_OPT := \
	--prefix=/usr \
	--openssldir=/usr/lib/ssl \
	--install_prefix=$(OPENSSL110DEV_PKGDIR)

ifdef PTXCONF_OPENSSL110DEV_SHARED
OPENSSL110DEV_CONF_OPT += shared
else
OPENSSL110DEV_CONF_OPT += no-shared
endif

$(STATEDIR)/openssl110dev.prepare:
	@$(call targetinfo)
	cd $(OPENSSL110DEV_DIR) && \
		$(OPENSSL110DEV_PATH) $(OPENSSL110DEV_CONF_ENV) \
		./Configure $(OPENSSL110DEV_ARCH-y) $(OPENSSL110DEV_CONF_OPT)
	@$(call touch)

# ----------------------------------------------------------------------------
# Target-Install
# ----------------------------------------------------------------------------

$(STATEDIR)/openssl110dev.targetinstall:
	@$(call targetinfo)

	@$(call install_init, openssl110dev)
	@$(call install_fixup, openssl110dev,PRIORITY,optional)
	@$(call install_fixup, openssl110dev,SECTION,base)
	@$(call install_fixup, openssl110dev,AUTHOR,"Marc Kleine-Budde <mkl@pengutronix.de>")
	@$(call install_fixup, openssl110dev,DESCRIPTION,missing)

ifdef PTXCONF_OPENSSL110DEV_BIN
	@$(call install_copy, openssl110dev, 0, 0, 0755, -, \
		/usr/bin/openssl)
endif
ifdef PTXCONF_OPENSSL110DEV_SHARED
	@$(call install_lib, openssl110dev, 0, 0, 0644, libssl)
	@$(call install_lib, openssl110dev, 0, 0, 0644, libcrypto)
endif
	@$(call install_finish, openssl110dev)

	@$(call touch)

# vim: syntax=make
